

################################################################################
##
##    This program is distributed in the hope that it will be useful,
##    but WITHOUT ANY WARRANTY; without even the implied warranty of
##    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
##
##    Contact help@edautils.com you need more informattion/support
################################################################################


### This routine processes the arguments to prepare
### the arguments for the Java APIs. It will map each
### commandline switches and their respective values
### That boolean switches are processed specially by inserting
### true valuse so that we can pass this value to the Java API.
###
### The  -help switch prints the help message
###
###

proc processArgs {args} {
    ### Remove unnecessary spaces otherwise the split inserts {} which
    ### is unwanted.

    set argList [join $args " "]
    set fields [ split $argList ]
    ### Created proper list without any unwanted {} which comes due to extra
    ### space chars between two elements
 
    set index 0
    set argCount [ llength $fields ]
    set modifiedArgs " "
    while {$index < $argCount} {
        set ele [lindex $fields $index]
        set modifiedArgs "$modifiedArgs  $ele"
        set nextIndex [expr {$index + 1}]
        if {  $nextIndex < $argCount  } {
            set nextEle [lindex $fields [expr {$index + 1}] ]
            ## Check if the next element is an option i.e. if the next 
            ## next element also starts with '-' 
            
            ## Check if the next element is a negative integer e.g. -10 ?
            ## Found bug when param value was negative. Solved by checking
            ## if the substring after the '-' is integer
            set substr [string range $nextEle 1 end ]
            if { [string first - $nextEle 0] == 0 && \
                 [string first - $ele 0] == 0 && \
                 [string is integer $substr] == 0} {
                set modifiedArgs "$modifiedArgs  true"
            }
        } else {
            ## If this is the last element then its considered as
            ## boolean type
            
            ## Check if the next element is a negative integer e.g. -10 ?
            ## Found bug when param value was negative. Solved by checking
            ## if the substring after the '-' is integer

            set substr [string range $ele 1 end ]
            if { [string first - $ele 0] == 0 && \
                 [string is integer $substr] == 0 } {
                set modifiedArgs "$modifiedArgs  true"
            }
        }

        set index [expr {$index + 1}]
    }
    ### Now the modifiedArgs contains key value pairs even
    ### for the boolean switches(, switch without any value).
    ### Remove unnecessary spaces and prepare proper list
    set argList [join $modifiedArgs " "]
    set fields [ split $argList ]
    ### Since we can not return array in Tcl, use the following
    ### trick i.e. create an array and get its contens. This way the
    ### caller will get the content of the array. In this particular
    ### case, it will be returning/displaying the processed arguments
    array set retArray  $fields
    array get retArray
}

proc print_help {msg} {
    if {[info exists mi]} {
    	$mi print 0 $nc "$msg"
	} else {
    	$::msgInst print 0 $::noneCode "$msg"
	}
	
}

set null [java::null] 


