
################################################################################
##
##    This program is distributed in the hope that it will be useful,
##    but WITHOUT ANY WARRANTY; without even the implied warranty of
##    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
##
##    Contact help@edautils.com you need more informattion/support
################################################################################


### This routine processes the arguments to prepare
### the arguments for the Java APIs. It will map each
### commandline switches and their respective values
### That boolean switches are processed specially by inserting
### true valuse so that we can pass this value to the Java API.
###
### The  -help switch prints the help message
###
###

package require java

java::import com.eu.miscedautils.CommonUtils.Utils
set fs [java::call Utils getFilePathSeparator ]

set edautilsroot  [ java::call System getenv "EDAUTILS_ROOT" ]

if {[string equal -nocase $fs "/"]} {
    source $edautilsroot/bin/cmdArgs.tcl
} else {
    source $edautilsroot\\\\bin\\\\cmdArgs.tcl
}
if {[string equal -nocase $fs "/"]} {
    source $edautilsroot/bin/load_designplayer_classes.tcl
} else {
    source $edautilsroot\\\\bin\\\\load_designplayer_classes.tcl
}

global baya


if {[info exists bayaroot] == 0} {
    global bayaroot
    set bayaroot [java::call BayaBackBone instance true]
} else {
    set bayaroot [java::call BayaBackBone instance]
}
set baya $bayaroot


$baya setFile "undefined.v"
$baya setLine 0

set verilogdescription [java::call FVPDescription instance ]
set vhdldescription [java::call FVhPDescription instance ]

proc baya_get_current_module {args} {

    set msg    "
    Description: returns the currently active design.
               "

    if { [ string compare -nocase $args "-help" ] == 0 } {
        print_help $msg
        return 
    }

    upvar #0 baya b
    set ret [$b getCurrentModule]
    return $ret
}

proc baya_get_file {args} {

    set msg    "
    Description: returns the file name where the database needs to be saved
               "

    if { [ string compare -nocase $args "-help" ] == 0 } {
        print_help $msg
        return 
    }

    upvar #0 baya b
    set ret [$b getFile]
    return $ret
}

proc baya_get_line {args} {

    set msg    "
    Description: returns the line number to be saved for the construct(s)
                 to be created by the next set of Baya command(s)
               "

    if { [ string compare -nocase $args "-help" ] == 0 } {
        print_help $msg
        return 
    }

    upvar #0 baya b
    set ret [$b getLine]
    return $ret
}

proc baya_get_root {args} {

    set msg    "
    Description:  returns the root of the design database
               "

    if { [ string compare -nocase $args "-help" ] == 0 } {
        print_help $msg
        return 
    }

    upvar #0 baya b
    set ret [$b getRoot]
    return $ret
}

proc baya_set_current_design {args} {

    set msg    "
    Description: sets the specified module as the currently active module.
    Options    : 
                 -name <module name> , mandatory option to specify the module name. 
    Example    : baya_set_current_design -name foo 
                "

    if { [ string compare -nocase $args "-help" ] == 0 } {
        print_help $msg
        return 
    }

    set argList [join $args " "]
    set args [ split $argList ]

    set ret ""
    ### Get the reference to $baya variable
    upvar #0 baya b
    ### Get the processed arguments
    array set javaArgs [processArgs  $args ]
    ### Call the Java API with the argument values since we can not
    ### pass the switches with dash(, - ) to the Java APIs. 
    if {[info exists javaArgs(-name)]} {
        set ret [$b setCurrentModule $javaArgs(-name) ]
    } else {
        print_help $msg
    }
    return $ret
}

proc baya_set_current_module {args} {

    set msg    "
    Description: sets the specified module as the currently active module.
    Options    : 
                 -name <module name> , mandatory option to specify the module name. 
    Example    : baya_set_current_module -name foo 
                "

    if { [ string compare -nocase $args "-help" ] == 0 } {
        print_help $msg
        return 
    }

    set argList [join $args " "]
    set args [ split $argList ]

    set ret ""
    ### Get the reference to $baya variable
    upvar #0 baya b
    ### Get the processed arguments
    array set javaArgs [processArgs  $args ]
    ### Call the Java API with the argument values since we can not
    ### pass the switches with dash(, - ) to the Java APIs. 
    if {[info exists javaArgs(-name)]} {
        set ret [$b setCurrentModule $javaArgs(-name) ]
    } else {
        print_help $msg
    }
    return $ret
}

proc baya_set_file {args} {

    set msg    "
    Description: Sets the name of the file where the design database will be saved. 
    Options    :  
                 -name <file name> , mandatory option to specify the module name.
    Example    : baya_set_file -name foo.v
                "

    if { [ string compare -nocase $args "-help" ] == 0 } {
        print_help $msg
        return 
    }

    set argList [join $args " "]
    set args [ split $argList ]

    set ret ""
    ### Get the reference to $baya variable
    upvar #0 baya b
    ### Get the processed arguments
    array set javaArgs [processArgs  $args ]
    ### Call the Java API with the argument values since we can not
    ### pass the switches with dash(, - ) to the Java APIs. 
    if {[info exists javaArgs(-name)]} {
        set ret [$b setFile $javaArgs(-name) ]
    } else {
        print_help $msg
    }
    return $ret
}

proc baya_set_line {args} {

    set msg    "
    Description: Sets the number for the objects to be created in the following commands.
                 This command is mostly useful to debug the in memory design database ...
    Options    : 
                 -num <Line number> , mandatory option to specify the line number.
    Example    : baya_set_line -num 10
                "

    if { [ string compare -nocase $args "-help" ] == 0 } {
        print_help $msg
        return 
    }

    set argList [join $args " "]
    set args [ split $argList ]

    set ret ""
    ### Get the reference to $baya variable
    upvar #0 baya b
    ### Get the processed arguments
    array set javaArgs [processArgs  $args ]
    ### Call the Java API with the argument values since we can not
    ### pass the switches with dash(, - ) to the Java APIs. 
    if {[info exists javaArgs(-num)]} {
        set ret [$b setLine $javaArgs(-num) ]
    } else {
        print_help $msg
    }
    return $ret
}



proc baya_add_connection {args} {

    set msg    "
    Description: Creates connection between the source port or pin and the destination 
                 port or pin. The source could be a input port of the top module or
                 output pin of an instance. The destination can be an output of the 
                 top module or input pin of an instance. 
    Options    : 
                 -src <port or pin name> , mandatory option to specify from where the 
                                           connection starts
                 -dest <port or pin name> , mandatory option to specify to where the 
                                            connection ends
    Example    : baya_add_connection -src input1  -dest instance_1.in1
                 baya_add_connection -src inst_1.out1  -dest instance_2.in1\[0\]
                 baya_add_connection -src inst_1.out1  -dest out
                 baya_add_connection -src inst_1.out1(0)  -dest instance_2.in1(3)
                 baya_add_connection -src inst_1.out1(3:1)  -dest instance_2.in1(7:9)
                "

    if { [ string compare -nocase $args "-help" ] == 0 } {
        print_help $msg
        return 
    }

    set argList [join $args " "]
    set args [ split $argList ]

    set ret ""
    upvar #0 baya b
    array set javaArgs [processArgs  $args ]
    if {[info exists javaArgs(-src)] && [info exists javaArgs(-dest)]} {
        set ret [$b addConnection $javaArgs(-src) $javaArgs(-dest) ]
    } else {
        print_help $msg
    }
    return $ret
}

proc baya_add_constant_connection {args} {

    set msg    "
    Description: Drives the input pin of an instance or the output port of the
                 current design/module with the specified constance value.
    Options    : 
                 -value <constant value> , mandatory option to provide the constant value.
                                           It supports constant values in the verilog format.
                                           Example, 3'b101 or 16'hdead .
 
                 -dest <port or pin name> , mandatory option to specify to where the 
                                            constant value should get connected/assigned to 

    Example    : baya_add_constant_connection -value 3'b101  -dest instance_1.in1
                 baya_add_constant_connection -value 16'hdead  -dest instance_2.in1
                 baya_add_constant_connection -value 15.out1  -dest out
                "

    if { [ string compare -nocase $args "-help" ] == 0 } {
        print_help $msg
        return 
    }

    set argList [join $args " "]
    set args [ split $argList ]

    set ret ""
    upvar #0 baya b
    array set javaArgs [processArgs  $args ]
    if {[info exists javaArgs(-value)] && [info exists javaArgs(-dest)]} {
        set ret [$b addConnectionConstantValue $javaArgs(-value) $javaArgs(-dest) ]
    } else {
        print_help $msg
    }
    return $ret
}

proc baya_add_empty_connection {args} {

    set msg    "
    Description: Creates empty/open port connection for an instance
    Options    : 
 
                 -pin <instance pin name> , mandatory option to specify the
                                            instance pin name to keep open

    Example    : baya_add_empty_connection -pin instance_1.in1
                "

    if { [ string compare -nocase $args "-help" ] == 0 } {
        print_help $msg
        return 
    }

    set argList [join $args " "]
    set args [ split $argList ]

    set ret ""
    upvar #0 baya b
    array set javaArgs [processArgs  $args ]
    if {[info exists javaArgs(-pin)]} {
        set ret [$b addEmptyConnection $javaArgs(-pin) ]
    } else {
        print_help $msg
    }
    return $ret
}

proc baya_create_hierarchy {args} {

    set msg    "
    Description: Creates a new hierarchy by enclosing the list of instances in the 
                 given module. It internally creates a new module with the name specified 
                 name and then instantiates the given list of instances in that new module.
                 This way one new hierarchy get created in the given module.

    Options    :  
                 -instances <instance list> , mandatory option to provide  list of instances 
                                              separted with colon(:). These instances must be present
                                              in the given module. 
                 -new_inst_name <name> , mandatory option for the new hierarchy name 
                 -new_mod_name <name> , mandatory option for the new module name enclosing the instances

    Example    : baya_create_hierarchy -instances inst1:in2:inst3 -new_inst_name newInst -new_mod_name wrapperMod
                "

    if { [ string compare -nocase $args "-help" ] == 0 } {
        print_help $msg
        return 
    }

    set argList [join $args " "]
    set args [ split $argList ]

    set ret ""
    ### Get the reference to $baya variable
    upvar #0 baya b
    ### Get the processed arguments
    array set javaArgs [processArgs  $args ]
    ### Call the Java API with the argument values since we can not
    ### pass the switches with dash(, - ) to the Java APIs. 
    if {[info exists javaArgs(-instances)] && [info exists javaArgs(-new_inst_name)] && [ info exists javaArgs(-new_mod_name)]} {
        set ret [$b createHierarchy javaArgs(-instances) javaArgs(-new_inst_name) javaArgs(-new_mod_name) ]
    } else {
        print_help $msg
    }
    return $ret
}

proc baya_create_instance {args} {
    set msg    "
    Description: Instantiates a module inside the current module. The name of the module which
                 gets instantiated is specified with the -master switch. The name of the instance
                 is specified with the -name switch of this this function.
    Options    : 
                 -name <instance name> , the name of the instance to be created. 
                 -master <master module name> , the name of the module which is to be instantiated
    Example    : baya_create_instance -name my_inst_1 -master mod1
                "

    if { [ string compare -nocase $args "-help" ] == 0 } {
        print_help $msg
        return 
    }

    set argList [join $args " "]
    set args [ split $argList ]

    set ret ""
    ### Get the reference to $baya variable
    upvar #0 baya b
    ### Get the processed arguments
    array set javaArgs [processArgs  $args ]
    ### Call the Java API with the argument values since we can not
    ### pass the switches with dash(, - ) to the Java APIs. 
    if {[info exists javaArgs(-name)] && [info exists javaArgs(-master)]} {
        set ret [$b createInstance $javaArgs(-name) $javaArgs(-master) ]
    } else {
        print_help $msg
    }
    return $ret
}

proc baya_create_module {args} {

    set msg    "
    Description: Creates new module with the specified name and sets 
                 as the same current module.
    Options    : 
                 -name <module name> , the name of the module to be created.
                 Note that this module will get set as the currrentt module/design.

    Example    : baya_create_module -name mod1
                "

    if { [ string compare -nocase $args "-help" ] == 0 } {
        print_help $msg
        return 
    }

    set argList [join $args " "]
    set args [ split $argList ]

    set ret ""
    ### Get the reference to $baya variable
    upvar #0 baya b
    ### Get the processed arguments
    array set javaArgs [processArgs  $args ]
    ### Call the Java API with the argument values since we can not
    ### pass the switches with dash(, - ) to the Java APIs. 
    if {[info exists javaArgs(-name)]} {
        set ret [$b createModule $javaArgs(-name) ]
    } else {
        print_help $msg
    }
    return $ret
}

proc baya_create_parameter {args} {

    set msg    "
    Description: Creates a parameter in the current module or the in the module 
                 specified with the -module switch. The name of the parameter and its
                 valuw will be the one passed with the -name and the value switches.
    Options    : 
                 -name <param name> , mandatory input, the name of the parameter to be created.
                 -value <param value> , mandatory input, the value of the parameter to be set to. 
                 -module <module name> , optional input, name of the module where this parameter
                                         is to be created. If not specified then the parameter will 
                                         get created the current module.
    Example    : baya_create_parameter -name param1 -value 10 -module mod1
                "

    if { [ string compare -nocase $args "-help" ] == 0 } {
        print_help $msg
        return 
    }

    set argList [join $args " "]
    set args [ split $argList ]

    set ret ""
    ### Get the reference to $baya variable
    upvar #0 baya b
    ### Get the processed arguments
    array set javaArgs [processArgs  $args ]
    ### Call the Java API with the argument values since we can not
    ### pass the switches with dash(, - ) to the Java APIs. 
    if {[info exists javaArgs(-module)] && [info exists javaArgs(-name)] && [info exists javaArgs(-value)]} {
        set prevScope [baya_get_current_module]
        set ret [$b createParameter $javaArgs(-module) $javaArgs(-name)  $javaArgs(-value)]
        $b setCurrentModule $prevScope
    } else {
        if {[info exists javaArgs(-name)] && [info exists javaArgs(-value)]} {
            set ret [$b createParameter $javaArgs(-name)  $javaArgs(-value)] 
        } else {
            print_help $msg
        }
    }
    return $ret
}

proc baya_create_parameter_map {args} {

    set msg    "
    Description: Overrides a parameter with the new value for the given instance.
                 It creates a parameter map in the given instance.
    Options    : 
                 -inst <inst name> , mandatory input, the name of the instance.
                 -name <param name> , mandatory input, name of the parameter whose 
                                      value to be set to.
                 -value <param value> , mandatory input, the value of the parameter to be set to.
    Example    : baya_create_parameter_map -inst instance_1 -name param1 -value 10
                "

    if { [ string compare -nocase $args "-help" ] == 0 } {
        print_help $msg
        return 
    }

    set argList [join $args " "]
    set args [ split $argList ]

    set ret ""
    ### Get the reference to $baya variable
    upvar #0 baya b
    ### Get the processed arguments
    array set javaArgs [processArgs  $args ]
    ### Call the Java API with the argument values since we can not
    ### pass the switches with dash(, - ) to the Java APIs. 
    if {[info exists javaArgs(-inst)] && [info exists javaArgs(-param)] && [info exists javaArgs(-value)]} {
        set ret [$b createParamMap $javaArgs(-inst) $javaArgs(-param)  $javaArgs(-value)]
    } else {
        print_help $msg
    }
    return $ret
}

proc baya_create_port {args} {

    set msg    "
    Description: Creates a port in the current module or in the module specified module.
    Options    : 
                 -name <port name> , mandatory input, name of the port be created
                 -dir <in|out|inout> , mandatory input, direction of the port be created
                 -range <left-bound:right-bound> , optional input, range for the vector port or bus
                 -module <module name> , optional input, name of the module where this port 
                                         is to be created. If not specified then the port will 
                                         get created in the current module.
    Example    : baya_create_port -module myMod -name port1 -dir in -range 2:0
                "

    if { [ string compare -nocase $args "-help" ] == 0 } {
        print_help $msg
        return 
    }

    set argList [join $args " "]
    set args [ split $argList ]

    set ret ""
    ### Get the reference to $baya variable
    upvar #0 baya b
    ### Get the processed arguments
    global javaArgs
    set javaArgs(-range) ""
    array set javaArgs [processArgs  $args ]
    if { [string compare $javaArgs(-range) ""] != 0 } {
        set rangeBounds [split $javaArgs(-range) ":" ] 
        set left [lindex $rangeBounds 0]
        set right [lindex $rangeBounds 1]
        ### Call the Java API with the argument values since we can not
        ### pass the switches with dash(, - ) to the Java APIs. 
        if {[info exists javaArgs(-module)] && [info exists javaArgs(-name)] && [info exists javaArgs(-dir)]} {
            set prevScope [baya_get_current_module]
            $b setCurrentModule $javaArgs(-module)
            set ret [$b createPort $javaArgs(-name)  $javaArgs(-dir) $left $right ] 
            $b setCurrentModule $prevScope
        } else {
            if {[info exists javaArgs(-name)] && [info exists javaArgs(-dir)]} {
                set ret [$b createPort $javaArgs(-name)  $javaArgs(-dir) $left $right ]
            } else {
                print_help $msg
            }
        }
    } else {
        if {[info exists javaArgs(-module)] && [info exists javaArgs(-name)] && [info exists javaArgs(-dir)]} {
            set prevScope [baya_get_current_module]
            $b setCurrentModule $javaArgs(-module)
            set ret [$b createPort $javaArgs(-name)  $javaArgs(-dir) ] 
            $b setCurrentModule $prevScope
        } else {
            if {[info exists javaArgs(-name)] && [info exists javaArgs(-dir)]} {
                set ret [$b createPort $javaArgs(-name)  $javaArgs(-dir) ]
            } else {
                print_help $msg
            }
        }
    }
    return $ret
}

proc baya_elaborate {args} {

    set msg    "
    Description: Elaborates a module
    Options    : 
                 -module <module name> , optional input, name of the module which 
                                         is to be elaborated. The current module be elaborated
                                         if the this switch is not provided.
    Example    : baya_elaborate -module myMod 
                "

    if { [ string compare -nocase $args "-help" ] == 0 } {
        print_help $msg
        return 
    }

    set argList [join $args " "]
    set args [ split $argList ]

    set ret ""
    ### Get the reference to $baya variable
    upvar #0 baya b
    array set javaArgs [processArgs  $args ]
    set prevScope [baya_get_current_module]
    if {[info exists javaArgs(-module)]} {
        $b setCurrentModule $javaArgs(-module)
    }
    set ret [$b elaborate]
    $b setCurrentModule $prevScope
    return $ret
}

proc baya_elaborate_root {args} {

    set msg    "
    Description: Elaborates a design database root
    Options    : 
    Example    : baya_elaborate_root 
                "

    if { [ string compare -nocase $args "-help" ] == 0 } {
        print_help $msg
        return 
    }

    set argList [join $args " "]
    set args [ split $argList ]

    set ret ""
    ### Get the reference to $baya variable
    upvar #0 baya b
    set ret [$b elaborateRoot]
    return $ret
}

proc baya_find_all_gate_instances {args} {

    set msg    "
    Description: Returns list of verilog primitive gate instances in the current module.
    Options    : 
    Example    : baya_find_all_gate_instances 
                "

    if { [ string compare -nocase $args "-help" ] == 0 } {
        print_help $msg
        return 
    }

    set argList [join $args " "]
    set args [ split $argList ]

    set ret ""
    ### Get the reference to $baya variable
    upvar #0 baya b
    set ret [$b findAllGateInstances]
    return $ret
}

proc baya_find_all_module_instances {args} {

    set msg    "
    Description: Returns list of module instances in the current module.
    Options    : 
    Example    : baya_find_all_module_instances 
                "

    if { [ string compare -nocase $args "-help" ] == 0 } {
        print_help $msg
        return 
    }

    set argList [join $args " "]
    set args [ split $argList ]

    set ret ""
    ### Get the reference to $baya variable
    upvar #0 baya b
    set ret [$b findAllModuleInstances]
    return $ret
}


proc baya_find_all_udp_instances {args} {

    set msg    "
    Description: Returns list of udp instances in the current module.
    Options    : 
    Example    : baya_find_all_udp_instances 
                "

    if { [ string compare -nocase $args "-help" ] == 0 } {
        print_help $msg
        return 
    }

    set argList [join $args " "]
    set args [ split $argList ]

    set ret ""
    ### Get the reference to $baya variable
    upvar #0 baya b
    set ret [$b findAllUDPInstances]
    return $ret
}

proc baya_find_all_modules {args} {

    set msg    "
    Description: Returns list of all modules in the current design database in the memory.
    Options    : 
    Example    : baya_find_all_modules 
                "

    if { [ string compare -nocase $args "-help" ] == 0 } {
        print_help $msg
        return 
    }

    set argList [join $args " "]
    set args [ split $argList ]

    set ret ""
    ### Get the reference to $baya variable
    upvar #0 baya b
    set ret [$b findAllModules]
    return $ret
}

proc baya_find_all_nets {args} {

    set msg    "
    Description: Returns list of nets in the current module.
    Options    : 
    Example    : baya_find_all_nets
                "

    if { [ string compare -nocase $args "-help" ] == 0 } {
        print_help $msg
        return 
    }

    set argList [join $args " "]
    set args [ split $argList ]

    set ret ""
    ### Get the reference to $baya variable
    upvar #0 baya b
    set ret [$b findAllNets]
    return $ret
}

proc baya_find_all_ports {args} {

    set msg    "
    Description: Returns list of ports of the current module.
    Options    : 
    Example    : baya_find_all_ports 
                "

    if { [ string compare -nocase $args "-help" ] == 0 } {
        print_help $msg
        return 
    }

    set argList [join $args " "]
    set args [ split $argList ]

    set ret ""
    ### Get the reference to $baya variable
    upvar #0 baya b
    set ret [$b findAllPorts]
    return $ret
}

proc baya_find_gate_instances {args} {

    set msg    "
    Description: Returns list of veriog primitive instances with name matching 
                 with the regular expression
    Options    : 
                 -name <pattern or regular expression> , mandatory input, to match 
                                                         the instance name 
    Example    : baya_find_gate_instances -name \"/*\" 
                "

    if { [ string compare -nocase $args "-help" ] == 0 } {
        print_help $msg
        return 
    }

    set argList [join $args " "]
    set args [ split $argList ]

    set ret ""
    ### Get the reference to $baya variable
    upvar #0 baya b
    ### Get the processed arguments
    array set javaArgs [processArgs  $args ]
    ### Call the Java API with the argument values since we can not
    ### pass the switches with dash(, - ) to the Java APIs. 
    if {[info exists javaArgs(-name)]} {
        set ret [$b findGateInstances $javaArgs(-name) ]
    } else {
        print_help $msg
    }
    return $ret
}

proc baya_find_hier_gate_instances {args} {

    set msg    "
    Description: Returns list of veriog primitive instances with hierarchical name 
                 matching with the regular expression
    Options    : 
                 -name <pattern or regular expression> , mandatory input, to match the 
                                                         hierarchical instance names
    Example    : baya_find_hier_gate_instances -name \"/*\"
                "

    if { [ string compare -nocase $args "-help" ] == 0 } {
        print_help $msg
        return 
    }

    set argList [join $args " "]
    set args [ split $argList ]

    set ret ""
    ### Get the reference to $baya variable
    upvar #0 baya b
    ### Get the processed arguments
    array set javaArgs [processArgs  $args ]
    ### Call the Java API with the argument values since we can not
    ### pass the switches with dash(, - ) to the Java APIs. 
    if {[info exists javaArgs(-name)]} {
        set ret [$b findHierGateInstances $javaArgs(-name) ]
    } else {
        print_help $msg
    }
    return $ret
}

proc baya_find_hier_module_instances {args} {

    set msg    "
    Description: Returns list of veriog module instances with hierarchical 
                 name matching with the regular expression
    Options    : 
                 -name <pattern or regular expression> , mandatory input, to match 
                                                         the hierarchical instance names
    Example    : baya_find_hier_module_instances -name \"/*\" 
                "

    if { [ string compare -nocase $args "-help" ] == 0 } {
        print_help $msg
        return 
    }

    set argList [join $args " "]
    set args [ split $argList ]

    set ret ""
    ### Get the reference to $baya variable
    upvar #0 baya b
    ### Get the processed arguments
    array set javaArgs [processArgs  $args ]
    ### Call the Java API with the argument values since we can not
    ### pass the switches with dash(, - ) to the Java APIs. 
    if {[info exists javaArgs(-name)]} {
        set ret [$b findHierModuleInstances $javaArgs(-name) ]
    } else {
        print_help $msg
    }
    return $ret
}


proc baya_find_hier_nets {args} {
    set msg    "
    Description: Returns list of hierarchical nets with name matching with the 
                 regular expression
    Options    : 
                 -name <pattern or regular expression> , mandatory input, to match 
                                                         the hierarchical net names
    Example    : baya_find_hier_nets -name \"/*\" 
                "

    if { [ string compare -nocase $args "-help" ] == 0 } {
        print_help $msg
        return 
    }

    set argList [join $args " "]
    set args [ split $argList ]

    set ret ""
    ### Get the reference to $baya variable
    upvar #0 baya b
    ### Get the processed arguments
    array set javaArgs [processArgs  $args ]
    ### Call the Java API with the argument values since we can not
    ### pass the switches with dash(, - ) to the Java APIs. 
    if {[info exists javaArgs(-name)]} {
        set ret [$b findHierNets $javaArgs(-name) ]
    } else {
        print_help $msg
    }
    return $ret
}

proc baya_find_hier_ports {args} {

    set msg    "
    Description: Returns list of hierarchical ports/pin with name matching with 
                 the regular expression
    Options    : 
                 -name <pattern or regular expression> , mandatory input, to match 
                                                         the hierarchical port names
    Example    : baya_find_hier_ports -name \"/*\" 
                "

    if { [ string compare -nocase $args "-help" ] == 0 } {
        print_help $msg
        return 
    }

    set argList [join $args " "]
    set args [ split $argList ]

    set ret ""
    ### Get the reference to $baya variable
    upvar #0 baya b
    ### Get the processed arguments
    array set javaArgs [processArgs  $args ]
    ### Call the Java API with the argument values since we can not
    ### pass the switches with dash(, - ) to the Java APIs. 
    if {[info exists javaArgs(-name)]} {
        set ret [$b findHierPorts $javaArgs(-name) ]
    } else {
        print_help $msg
    }
    return $ret
}

proc baya_find_hier_udp_instances {args} {

    set msg    "
    Description: Returns list of veriog udp instances with hierarchical name 
                 matching with the regular expression
    Options    : 
                 -name <pattern or regular expression> , mandatory input, to match 
                                                         the hierarchical instance names
    Example    : baya_find_hier_udp_instances -name \"/*\" 
                "

    if { [ string compare -nocase $args "-help" ] == 0 } {
        print_help $msg
        return 
    }

    set argList [join $args " "]
    set args [ split $argList ]

    set ret ""
    ### Get the reference to $baya variable
    upvar #0 baya b
    ### Get the processed arguments
    array set javaArgs [processArgs  $args ]
    ### Call the Java API with the argument values since we can not
    ### pass the switches with dash(, - ) to the Java APIs. 
    if {[info exists javaArgs(-name)]} {
        set ret [$b findHierUDPInstances $javaArgs(-name) ]
    } else {
        print_help $msg
    }
    return $ret
}

proc baya_find_gate_instances {args} {

    set msg    "
    Description: Returns list of verilog primitive instances with name matching 
                 with the regular expression
    Options    : 
                 -name <pattern or regular expression> , mandatory input, to match
                                                         the instance names
    Example    : baya_find_gate_instances -name \"/*\"
                "

    if { [ string compare -nocase $args "-help" ] == 0 } {
        print_help $msg
        return 
    }

    set argList [join $args " "]
    set args [ split $argList ]

    set ret ""
    ### Get the reference to $baya variable
    upvar #0 baya b
    ### Get the processed arguments
    array set javaArgs [processArgs  $args ]
    ### Call the Java API with the argument values since we can not
    ### pass the switches with dash(, - ) to the Java APIs. 
    if {[info exists javaArgs(-name)]} {
        set ret [$b findGateInstances $javaArgs(-name) ]
    } else {
        print_help $msg
    }
    return $ret
}

proc baya_find_module_instances {args} {

    set msg    "
    Description: Returns list of verilog module instances with name matching 
                 with the regular expression
    Options    : 
                 -name <pattern or regular expression> , mandatory input, to match
                                                         the instance names
    Example    : baya_find_module_instances -name \"/*\" 
                "

    if { [ string compare -nocase $args "-help" ] == 0 } {
        print_help $msg
        return 
    }

    set argList [join $args " "]
    set args [ split $argList ]

    set ret ""
    ### Get the reference to $baya variable
    upvar #0 baya b
    ### Get the processed arguments
    array set javaArgs [processArgs  $args ]
    ### Call the Java API with the argument values since we can not
    ### pass the switches with dash(, - ) to the Java APIs. 
    if {[info exists javaArgs(-name)]} {
        set ret [$b findModuleInstances $javaArgs(-name) ]
    } else {
        print_help $msg
    }
    return $ret
}


proc baya_find_nets {args} {

    set msg    "
    Description: Returns list of nets with name matching with the regular expression
    Options    : 
                 -name <pattern or regular expression> , mandatory input, to match 
                                                         the net names 
    Example    : baya_find_nets -name \"/*\" 
                "

    if { [ string compare -nocase $args "-help" ] == 0 } {
        print_help $msg
        return 
    }

    set argList [join $args " "]
    set args [ split $argList ]

    set ret ""
    ### Get the reference to $baya variable
    upvar #0 baya b
    ### Get the processed arguments
    array set javaArgs [processArgs  $args ]
    ### Call the Java API with the argument values since we can not
    ### pass the switches with dash(, - ) to the Java APIs. 
    if {[info exists javaArgs(-name)]} {
        set ret [$b findNets $javaArgs(-name) ]
    } else {
        print_help $msg
    }
    return $ret
}

proc baya_find_ports {args} {

    set msg    "
    Description: Returns list of ports with name matching with the regular expression
    Options    :
                 -name <pattern or regular expression> , mandatory input, to match 
                                                         the port names 
    Example    : baya_find_ports -name \"/*\" 
                "

    if { [ string compare -nocase $args "-help" ] == 0 } {
        print_help $msg
        return 
    }

    set argList [join $args " "]
    set args [ split $argList ]

    set ret ""
    ### Get the reference to $baya variable
    upvar #0 baya b
    ### Get the processed arguments
    array set javaArgs [processArgs  $args ]
    ### Call the Java API with the argument values since we can not
    ### pass the switches with dash(, - ) to the Java APIs. 
    if {[info exists javaArgs(-name)]} {
        set ret [$b findPorts $javaArgs(-name) ]
    } else {
        print_help $msg
    }
    return $ret
}

proc baya_find_udp_instances {args} {

    set msg    "
    Description: Returns list of udp instances with name matching with the 
                 regular expression
    Options    : 
                 -name <pattern or regular expression> , mandatory input, to match
                                                         the instance names
    Example    : baya_find_udp_instances -name \"/*\"
                "

    if { [ string compare -nocase $args "-help" ] == 0 } {
        print_help $msg
        return 
    }

    set argList [join $args " "]
    set args [ split $argList ]

    set ret ""
    ### Get the reference to $baya variable
    upvar #0 baya b
    ### Get the processed arguments
    array set javaArgs [processArgs  $args ]
    ### Call the Java API with the argument values since we can not
    ### pass the switches with dash(, - ) to the Java APIs. 
    if {[info exists javaArgs(-name)]} {
        set ret [$b findUDPInstances $javaArgs(-name) ]
    } else {
        print_help $msg
    }
    return $ret
}

proc baya_import_ipxact {args} {

    set msg    "
    Description: Reads the specified IP-XACT component xml file so that the component
                 can be instantiated. 
    Options    : 
                 -xml <IP-XACT component XML file name> , mandatory input, component XML
                                                          file name
    Example    : baya_import_ipxact -xml foo.component.xml 
                "

    if { [ string compare -nocase $args "-help" ] == 0 } {
        print_help $msg
        return 
    }

    set argList [join $args " "]
    set args [ split $argList ]

    set ret ""
    ### Get the reference to $baya variable
    upvar #0 baya b
    ### Get the processed arguments
    array set javaArgs [processArgs  $args ]
    ### Call the Java API with the argument values since we can not
    ### pass the switches with dash(, - ) to the Java APIs. 
    if {[info exists javaArgs(-xml)]} {
        set ret [$b importIPXACTComponent $javaArgs(-xml) ]
    } else {
        print_help $msg
    }
    return $ret
}

proc baya_generate_rsa_keys {args} { 

    set msg    "
    Description: Generate public and private key files
    Options    : 
                 -pvtkey <file name>, input to specify the private key file name
                 -pubkey <file name>, input to specify the public key file name
    Example    : baya_generate_rsa_keys -pvtkey key.pvt.dat -pubkey key.pub.dat
                "

    if { [ string compare -nocase $args "-help" ] == 0 } {
        print_help $msg
        return 
    }

    set argList [join $args " "]
    set args [ split $argList ]
    set javaArgs(-pvtkey) $::null
    set javaArgs(-pubkey) $::null
    upvar #0 baya b
    set ret ""
    ### Get the processed arguments
    array set javaArgs [processArgs  $args ]
    ### Call the Java API with the argument values since we can not
    ### pass the switches with dash(, - ) to the Java APIs. 
    if {[info exists javaArgs(-help)] } {
        print_help $msg
        return
    }
    
    set ret [ $b geterateKeyFiles $javaArgs(-pvtkey) $javaArgs(-pubkey) $::null ] 
    return $ret
}

proc baya_preprocess_verilog {args} {

    set msg    "
    Description: Reads the given input file and writes back the preprocessed
                 output file
    Options    : 
                 -in <input file name> , mandatory input, to specify the
                                         input file name
                 -out <ioutput file name> , mandatory input, to specify the
                                         output file name
                 -incdir <directories> , optional input, directories where include
                                         files to be searched 
                 -definedir <macros to be defined> , optional input, directives to
                                                     be defined 
                 -pubkey <publick key file> input to passs the public keyfile 
                                        with which the data will get decrypted.
                                        This input is mandatory for decryption.
                 -key <key for encrytption> , input to passs the key with which
                                        the data will get encrypted. This input
                                        is must while encryption and not 
                                        required while decrypting an encrypted
                                        file. Howver, the private is must in
                                        order to decrypt a file. 
                 -pvtkey <private keyfile name> , to specify the name of the
                                         private key file. This input is must 
                                         in order to decrypt an encrypted input
                                         file. It is optional when you call this
                                         to encrypt a file. While encryption,
                                         the private key will get dumped in this
                                         file. 
                 -keyowner <Owner of the key> , Optional input to specify 
                                         the name of the key owner. Generally,
                                         name of the company is specified here.
                 -keyname <name of the key> , Optional input to specify 
                                         the name of the key. 
                 -commented , optional boolean switch to specify if 
                              the 'protect' directives should be inside 
                              the '//' instead of the '`pragma'. By default
                              its false.
                 -algo <algo name> , optional input, to passs the name of the
                                     algorithm which will be used to decrypt 
                                     the protected functionality. Default is
                                     value AES. 
    Example (for encryption)   : baya_preprocess_verilog -in foo.v -out processed.v -incdir dir1+dir2  -definedir define+macro1+macro2 -key edautils_dot_com  -pubkey pubkey.dat -keyowner SiliconIndia -keyname SI_001
    Example(for decryption)    : baya_preprocess_verilog -in foo.v -out processed.v -incdir dir1+dir2  -definedir define+macro1+macro2 -pvtkey pvtket.dat -keyowner SiliconIndia -keyname SI_001
                "

    if { [ string compare -nocase $args "-help" ] == 0 } {
        print_help $msg
        return 
    }

    set argList [join $args " "]
    set args [ split $argList ]
    set javaArgs(-key) $::null
    set javaArgs(-pvtkey) $::null
    set javaArgs(-pubkey) $::null
    set javaArgs(-keyowner) $::null
    set javaArgs(-keyname) $::null
    set javaArgs(-incdir) $::null
    set javaArgs(-definedir) $::null
    set javaArgs(-algo) $::null
    set javaArgs(-commented) false

    set ret ""
    ### Get the processed arguments
    array set javaArgs [processArgs  $args ]
    ### Call the Java API with the argument values since we can not
    ### pass the switches with dash(, - ) to the Java APIs. 
    if {[info exists javaArgs(-in)] && [info exists javaArgs(-out)] } {

       set ret [java::call PreProcessVerilog preprocess $javaArgs(-in) $javaArgs(-out) $javaArgs(-incdir)  $javaArgs(-definedir) $javaArgs(-pubkey) $javaArgs(-key) $javaArgs(-pvtkey) $javaArgs(-keyowner) $javaArgs(-keyname) $javaArgs(-commented) $javaArgs(-algo) ]
    } else {
        print_help $msg
    }
    return $ret
}


proc baya_import_verilog {args} {

    set msg    "
    Description: Imports the specified verilog file so that the module can be instantiated.
    Options    : 
                 -file <file name> , mandatory input, file to be read
                 -excludefilelist <exclude.list> , optional input to exclude 
                                                  files listed in the given file
                 -top <module to be imported> , mandatory input, top module in this file
                 -incdir <directories> , optional input, directories where include
                                         files to be searched 
                 -definedir <macros to be defined> , optional input, directives to
                                                     be defined 
                 -sort , optional boolean switch to specify if the inputs needs to
                                         be sorted before parsing
                 -nowildcard , optional boolean switch to disable wildcard processing in file name(s)
                 -key <key for encrytption> , mandatory input while encrypting
                 -pvtkey <private keyfile name> , mandatory input while
                                         while encrypting an input file. This
                                         file can be created if does not exists
                                         with the command baya_generate_rsa_keys
                 -pubkey <public keyfile name> , to specify the name of the
                                         public key file. This input is must 
                                         in order to decrypt an encrypted input
                                         file. 
                 -keyowner <Owner of the key> , Optional input to specify 
                                         the name of the key owner. Generally,
                                         name of the company is specified here.
                 -keyname <name of the key> , Optional input to specify 
                                         the name of the key. 
                 -algo <algo name> , optional input, to passs the name of the
                                     algorithm which will be used to decrypt 
                                     the protected functionality. Default is
                                     value AES. 
    Example    : baya_import_verilog -file foo.v -top myTop -incdir dir1+dir2  -definedir define+macro1+macro2
                "

    if { [ string compare -nocase $args "-help" ] == 0 } {
        print_help $msg
        return 
    }

    set argList [join $args " "]
    set args [ split $argList ]

    upvar #0 null nil
    set ret $nil
    set javaArgs(-incdir) ./
    set javaArgs(-definedir) $nil
    set javaArgs(-key) $nil
    set javaArgs(-pvtkey) $::null
    set javaArgs(-pubkey) $::null
    set javaArgs(-keyowner) $::null
    set javaArgs(-keyname) $::null
    set javaArgs(-algo) $nil
    set javaArgs(-top) $nil
    set javaArgs(-nowildcard) false
    set javaArgs(-excludefilelist) $nil
    set javaArgs(-sort) false


    ### Get the reference to $baya variable
    upvar #0 baya b
    ### Get the processed arguments
    array set javaArgs [processArgs  $args ]
    ### Call the Java API with the argument values since we can not
    ### pass the switches with dash(, - ) to the Java APIs. 
    if { [info exists javaArgs(-file)] } {
        set ret [$b importVerilogModule $javaArgs(-file) $javaArgs(-excludefilelist) $javaArgs(-top) $javaArgs(-incdir)  $javaArgs(-definedir) $javaArgs(-sort) $javaArgs(-nowildcard) $javaArgs(-key) $javaArgs(-pvtkey) $javaArgs(-pubkey) $javaArgs(-keyowner) $javaArgs(-keyname) $javaArgs(-algo)]
    } else {
        print_help $msg
    }
    return $ret
}

proc baya_import_verilog_filelist {args} {

    set msg    "
    Description: Imports the specified multiple verilog files listed in the input file
    Options    : 
                 -filelist <list feil> , mandatory input, file where verilog files
                                          are listed 
                 -excludefilelist <exclude.list> , optional input to exclude 
                                                  files listed in the given file
                 -top <module to be imported> , mandatory input, top module in this file
                 -incdir <directories> , optional input, directories where include
                                         files to be searched 
                 -definedir <macros to be defined> , optional input, directives to
                                         be defined 
                 -sort , optional boolean switch to specify if the inputs needs to
                                         be sorted before parsing
                 -nowildcard , optional boolean switch to disable wildcard processing in file name(s)
                 -key <key for encrytption> , mandatory input while encrypting
                 -pvtkey <private keyfile name> , mandatory input while
                                         while encrypting an input file. This
                                         file can be created if does not exists
                                         with the command baya_create_rsa_keys .
                 -pubkey <public keyfile name> , to specify the name of the
                                         public key file. This input is must 
                                         in order to decrypt an encrypted input
                                         file. 
                 -keyowner <Owner of the key> , Optional input to specify 
                                         the name of the key owner. Generally,
                                         name of the company is specified here.
                 -keyname <name of the key> , Optional input to specify 
                                         the name of the key. 
                 -algo <algo name> , optional input, to passs the name of the
                                     algorithm which will be used to decrypt 
                                     the protected functionality. Default is
                                     value AES. 
    Example    : baya_import_verilog_filelist -filelist infile.list -top myTop -incdir dir1+dir2  -definedir define+macro1+macro2
                "

    if { [ string compare -nocase $args "-help" ] == 0 } {
        print_help $msg
        return 
    }

    set argList [join $args " "]
    set args [ split $argList ]

    upvar #0 null nil
    set ret $nil
    set javaArgs(-incdir) ./
    set javaArgs(-definedir) $nil
    set javaArgs(-key) $nil
    set javaArgs(-pvtkey) $::null
    set javaArgs(-pubkey) $::null
    set javaArgs(-keyowner) $::null
    set javaArgs(-keyname) $::null
    set javaArgs(-algo) $nil
    set javaArgs(-top) $nil
    set javaArgs(-sort) false
    set javaArgs(-nowildcard) false
    set javaArgs(-excludefilelist) $nil


    ### Get the reference to $baya variable
    upvar #0 baya b
    ### Get the processed arguments
    array set javaArgs [processArgs  $args ]
    ### Call the Java API with the argument values since we can not
    ### pass the switches with dash(, - ) to the Java APIs. 
    ### Get the reference to $ipxactroot variable

    if {[info exists javaArgs(-filelist)]} {
        set ret [$b importVerilogFileList  $javaArgs(-filelist) $javaArgs(-excludefilelist) $javaArgs(-top)  $javaArgs(-incdir)  $javaArgs(-definedir) $javaArgs(-sort) $javaArgs(-nowildcard) $javaArgs(-key)  $javaArgs(-pvtkey) $javaArgs(-pubkey) $javaArgs(-keyowner) $javaArgs(-keyname) $javaArgs(-algo) ]
        return $ret
    } else {
        print_help $msg
    }
    return  $ret 
}

proc baya_import_verilog_file_list {args} {

    set msg    "
    Description: Imports the specified multiple verilog files listed in the input file
    Options    : 
                 -file_list <list feil> , mandatory input, file where verilog files
                                          are listed 
                 -excludefilelist <exclude.list> , optional input to exclude 
                                                  files listed in the given file
                 -top <module to be imported> , mandatory input, top module in this file
                 -incdir <directories> , optional input, directories where include
                                         files to be searched 
                 -definedir <macros to be defined> , optional input, directives to
                                         be defined 
                 -sort , optional boolean switch to specify if the inputs needs to
                                         be sorted before parsing
                 -nowildcard , optional boolean switch to disable wildcard processing in file name(s)
                 -key <key for encrytption> , mandatory input while encrypting
                 -pvtkey <private keyfile name> , mandatory input while
                                         while encrypting an input file. This
                                         file can be created if does not exists
                                         with the command baya_create_rsa_keys .
                 -pubkey <public keyfile name> , to specify the name of the
                                         public key file. This input is must 
                                         in order to decrypt an encrypted input
                                         file. 
                 -keyowner <Owner of the key> , Optional input to specify 
                                         the name of the key owner. Generally,
                                         name of the company is specified here.
                 -keyname <name of the key> , Optional input to specify 
                                         the name of the key. 
                 -algo <algo name> , optional input, to passs the name of the
                                     algorithm which will be used to decrypt 
                                     the protected functionality. Default is
                                     value AES. 
    Example    : baya_import_verilog_file_list -file_list infile.list -top myTop -incdir dir1+dir2  -definedir define+macro1+macro2
                "

    if { [ string compare -nocase $args "-help" ] == 0 } {
        print_help $msg
        return 
    }

    set argList [join $args " "]
    set args [ split $argList ]

    upvar #0 null nil
    set ret $nil
    set javaArgs(-incdir) ./
    set javaArgs(-definedir) $nil
    set javaArgs(-key) $nil
    set javaArgs(-pvtkey) $::null
    set javaArgs(-pubkey) $::null
    set javaArgs(-keyowner) $::null
    set javaArgs(-keyname) $::null
    set javaArgs(-algo) $nil
    set javaArgs(-top) $nil
    set javaArgs(-sort) false
    set javaArgs(-nowildcard) false
    set javaArgs(-excludefilelist) $nil


    ### Get the reference to $baya variable
    upvar #0 baya b
    ### Get the processed arguments
    array set javaArgs [processArgs  $args ]
    ### Call the Java API with the argument values since we can not
    ### pass the switches with dash(, - ) to the Java APIs. 
    ### Get the reference to $ipxactroot variable

    if {[info exists javaArgs(-file_list)]} {
        set ret [$b importVerilogFileList  $javaArgs(-file_list) $javaArgs(-excludefilelist) $javaArgs(-top)  $javaArgs(-incdir)  $javaArgs(-definedir) $javaArgs(-sort) $javaArgs(-nowildcard) $javaArgs(-key)  $javaArgs(-pvtkey) $javaArgs(-pubkey) $javaArgs(-keyowner) $javaArgs(-keyname) $javaArgs(-algo) ]
        return $ret
    } else {
        print_help $msg
    }
    return  $ret 
}

proc baya_import_vhdl {args} {

    set msg    "
    Description: Imports the specified VHDL file
    Options    : 
                 -file <VHDL file name> , mandatory input, name of the VHDL to be read
                 -top <entity to be imported> , mandatory input, entity defined in this file 
                 -work <work lib name> , optional input, to specify the work library
                 -mapfile <map file> , optional input, logical to physical library 
                                       map file. Each line in this mapfile should 
                                       be similar to below.

                                        WORK=/home/klg/vhdl_dumps/work
                                        GATE=/home/klg/vhdl_dumps/gatelib
                                        CORE=/home/klg/vhdl_dumps/core

         -excludefilelist <exclude.list> , optional input to exclude 
                                           files listed in the given file
         -nowildcard , optional boolean switch to disable wildcard processing in file name(s)


    Example    : baya_import_vhdl -file foo.vhd -top myTop -work work -mapfile libmap.ini
                "

    if { [ string compare -nocase $args "-help" ] == 0 } {
        print_help $msg
        return 
    }

    set argList [join $args " "]
    set args [ split $argList ]

    set ret ""
    ### Get the reference to $baya variable
    upvar #0 baya b

    upvar #0 null nil
    set javaArgs(-work) $nil
    set javaArgs(-mapfile) $nil
    set javaArgs(-nowildcard) false
    set javaArgs(-excludefilelist) $nil

    ### Get the processed arguments
    array set javaArgs [processArgs  $args ]
    ### Call the Java API with the argument values since we can not
    ### pass the switches with dash(, - ) to the Java APIs. 
    if {[info exists javaArgs(-top)] && [info exists javaArgs(-file)] } {
        set ret [$b importVHDLEntity $javaArgs(-file) $javaArgs(-top) $javaArgs(-work) $javaArgs(-mapfile) $javaArgs(-excludefilelist) $javaArgs(-nowildcard)  ]
    } else {
        print_help $msg
    }
    return $ret
}

proc baya_import_mixed_hdl_filelist {args} {

    set msg    "
    Description: Imports the specified multiple Verilog and VHDL files listed in the input list file
    Options    : 
                 -filelist <list feil> , mandatory input, file where VHDL and
                                         Verilog files are listed 
                 -excludefilelist <exclude.list> , optional input to exclude 
                                                  files listed in the given file
                 -top <module to be imported> , mandatory input, top module in this file
                 -incdir <directories> , optional input, directories where include
                                         files to be searched 
                 -definedir <macros to be defined> , optional input, directives to
                                         be defined 
                 -nowildcard , optional boolean switch to disable wildcard processing in file name(s)
    Example    : baya_import_mixed_hdl_filelist -filelist infile.list -top myTop -incdir dir1+dir2  -definedir define+macro1+macro2
                "

    if { [ string compare -nocase $args "-help" ] == 0 } {
        print_help $msg
        return 
    }

    set argList [join $args " "]
    set args [ split $argList ]

    upvar #0 null nil
    set ret $nil
    set javaArgs(-incdir) ./
    set javaArgs(-definedir) $nil
    set javaArgs(-nowildcard) false
    set javaArgs(-excludefilelist) $nil


    ### Get the reference to $baya variable
    upvar #0 baya b
    ### Get the processed arguments
    array set javaArgs [processArgs  $args ]
    ### Call the Java API with the argument values since we can not
    ### pass the switches with dash(, - ) to the Java APIs. 
    ### Get the reference to $ipxactroot variable

    if {[info exists javaArgs(-filelist)]} {
        set ret [$b parseMixedHDLFileList $javaArgs(-filelist) $javaArgs(-excludefilelist) $javaArgs(-top)  $javaArgs(-incdir)  $javaArgs(-definedir) $javaArgs(-nowildcard) ]
        return $ret
    } else {
        print_help $msg
    }
    return  $ret 
}


proc baya_print_verilog {args} {

    set msg    "
    Description: Prints the current module or the one specified with the switch -module 
    Options    : 
                 -module <module name> , optional input, name of the module which 
                                         is to be printed. The current module be
                                         printed if the this switch is not provided.
                 -key <key for encrytption> , input to passs the key with which
                                        the data will get encrypted. This input
                                        is must while encryption and not 
                                        required while decrypting an encrypted
                                        file. Howver, the private is must in
                                        order to decrypt a file. 
                 -pubkey <public keyfile name> , to specify the name of the
                                         public key file
                 -keyowner <Owner of the key> , Optional input to specify 
                                         the name of the key owner. Generally,
                                         name of the company is specified here.
                 -keyname <name of the key> , Optional input to specify 
                                         the name of the key. 
                 -commented_directive  , optional boolean swtich to sepcify if the
                                       'protect' directives to be printed 
                                       as '// protect ...' . Default is
                                       false and by default it'll print the same
                                       as '`pragma protect ...' 
    Example    : baya_print_verilog -key edautils_dot_com  -pubkey publickey.dat -keyowner SiliconIndia -keyname SI_001
                "

    if { [ string compare -nocase $args "-help" ] == 0 } {
        print_help $msg
        return 
    }

    set argList [join $args " "]
    set args [ split $argList ]

    ### Get the reference to $baya variable
    upvar #0 baya b
    upvar #0 null nil
    set javaArgs(-key) $nil
    set javaArgs(-pubkey) $nil
    set javaArgs(-keyowner) $nil
    set javaArgs(-keyname) $nil
    set javaArgs(-commented_directive) false


    array set javaArgs [processArgs  $args ]
    set prevScope [baya_get_current_module]
    if {[info exists javaArgs(-module)] } {
        $b setCurrentModule $javaArgs(-module)
    }
    set ret [$b printVerilog $javaArgs(-key) $javaArgs(-pubkey) $javaArgs(-keyowner) $javaArgs(-keyname) $javaArgs(-commented_directive)  ]
    if { $prevScope != $nil } {
        $b setCurrentModule $prevScope
    }
    return $ret
}

proc baya_print_verilog_file {args} {

    set msg    "
    Description: Prints the current module or the one specified with the 
                 switch -module in the file set with baya_set_file command.
    Options    : 
                 -module <module name> , optional input, name of the module which 
                                         is to be printed. The current module be
                                         printed if the this switch is not provided.
                 -file <file name> ,     optional input to specify the file name
                 -key <key for encrytption> , input to passs the key with which
                                        the data will get encrypted. This input
                                        is must while encryption and not 
                                        required while decrypting an encrypted
                                        file. Howver, the private is must in
                                        order to decrypt a file. 
                 -pubkey <public keyfile name> , to specify the name of the
                                         public key file
                 -keyowner <Owner of the key> , Optional input to specify 
                                         the name of the key owner. Generally,
                                         name of the company is specified here.
                 -keyname <name of the key> , Optional input to specify 
                                         the name of the key. 
                 -commented_directive  , optional boolean swtich to sepcify if the
                                       'protect' directives to be printed 
                                       as '// protect ...' . Default is
                                       false and by default it'll print the same
                                       as '`pragma protect ...' 
    Example    : baya_print_verilog_file -key edautils_dot_com  -pubkey publickey.dat -keyowner SiliconIndia -keyname SI_001 -file output.v 
                "

    if { [ string compare -nocase $args "-help" ] == 0 } {
        print_help $msg
        return 
    }

    set argList [join $args " "]
    set args [ split $argList ]

    ### Get the reference to $baya variable
    upvar #0 baya b
    upvar #0 null nil
    set javaArgs(-key) $::null
    set javaArgs(-pubkey) $nil
    set javaArgs(-keyowner) $nil
    set javaArgs(-keyname) $nil
    set javaArgs(-commented_directive) false

    array set javaArgs [processArgs  $args ]
    set prevScope [baya_get_current_module]
    if {[info exists javaArgs(-module)] } {
        $b setCurrentModule $javaArgs(-module)
    }
    if {[info exists javaArgs(-file)] } {
        baya_set_file -name $javaArgs(-file)
    }
    set ret [$b printVerilogFile $javaArgs(-key) $javaArgs(-pubkey) $javaArgs(-keyowner) $javaArgs(-keyname) $javaArgs(-commented_directive) ]
    if { $prevScope != $nil } {
        $b setCurrentModule $prevScope
    }
    return $ret
}

proc baya_print_vhdl {args} {

    set msg    "
    Description: Prints in VHDL the current module or the one specified with
                 the switch -module .
    Options    : 
                 -module <module name> , optional input, name of the module which 
                                         is to be printed. The current module be 
                                         printed if the this switch is not provided.
    Example    : baya_print_vhdl -module myMod
                "

    if { [ string compare -nocase $args "-help" ] == 0 } {
        print_help $msg
        return 
    }
    set argList [join $args " "]
    set args [ split $argList ]

    ### Get the reference to $baya variable
    upvar #0 baya b
    array set javaArgs [processArgs  $args ]
    set prevScope [baya_get_current_module]
    if {[info exists javaArgs(-module)] } {
        $b setCurrentModule $javaArgs(-module)
    }
    set ret [$b printVhdl]
    if { $prevScope != $::null } {
        $b setCurrentModule $prevScope
    }
    return $ret
}

proc baya_print_vhdl_file {args} {

    set msg    "
    Description: Prints in VHDL the current module or the one specified with the 
                 switch -module in the file set with baya_set_file command.
    Options    :  
                 -module <module name> , optional input, name of the module which 
                                         is to be printed. The current module be
                                         printed if the this switch is not provided.
                 -file <file name> , optional input, to specify the file name
                                     where the given module or the currently
                                     active module needs to printed.
    Example    : baya_print_vhdl_file -module myMod \[-file foo.vhd\]
                "

    if { [ string compare -nocase $args "-help" ] == 0 } {
        print_help $msg
        return 
    }

    set argList [join $args " "]
    set args [ split $argList ]

    ### Get the reference to $baya variable
    upvar #0 baya b
    array set javaArgs [processArgs  $args ]
    set prevScope [baya_get_current_module]
    if {[info exists javaArgs(-module)] } {
        $b setCurrentModule $javaArgs(-module)
    }
    if {[info exists javaArgs(-file)] } {
        baya_set_file -name $javaArgs(-file)
    }
    set ret [$b printVhdlFile]
    if { $prevScope != $::null } {
        $b setCurrentModule $prevScope
    }
    return $ret
}

proc baya_remove_connection {args} {

    set msg    "
    Description: Removes connection between the source port or pin and the destination
                 port or pin. The source could be a input port of the top module or 
                 output pin of an instance. The destination can be an output of the
                 top module or input pin of an instance. 
    Options    : 
                 -src <port or pin name> , mandatory option to specify from where
                                           the connection starts 
                 -dest <port or pin name> , mandatory option to specify to where
                                            the connection ends 
    Example    : baya_remove_connection -src input1  -dest instance_1.in1
                 baya_remove_connection -src inst_1.out1  -dest instance_2.in1
                 baya_remove_connection -src inst_1.out1  -dest out
                "

    if { [ string compare -nocase $args "-help" ] == 0 } {
        print_help $msg
        return 
    }

    set argList [join $args " "]
    set args [ split $argList ]

    set ret ""
    ### Get the reference to $baya variable
    upvar #0 baya b
    ### Get the processed arguments
    array set javaArgs [processArgs  $args ]
    ### Call the Java API with the argument values since we can not
    ### pass the switches with dash(, - ) to the Java APIs. 
    if {[info exists javaArgs(-src)] && [info exists javaArgs(-dest)]} {
        set ret [$b removeConnection $javaArgs(-src) $javaArgs(-dest) ]
    } else {
        print_help $msg
    }
    return $ret
}

proc baya_remove_hierarchy {args} {

    set msg    "
    Description: Removes a hierarchy by filattening the the specified instance in the
                 current module. It internally pulls up the contents of the instance
                 in the current module.
    Options    : 
                 -inst <instance name> , mandatory option to provide the instance 
                 -all_levels  , optional boolean switch to specify if the hierarchy
                                of the subsequent lower levels to be removed 
    Example    : baya_remove_hierarchy -inst inst1 -all_levels
                "

    if { [ string compare -nocase $args "-help" ] == 0 } {
        print_help $msg
        return 
    }

    set argList [join $args " "]
    set args [ split $argList ]

    set ret ""
    set javaArgs(-all_levels) false
    ### Get the reference to $baya variable
    upvar #0 baya b
    ### Get the processed arguments
    array set javaArgs [processArgs  $args ]
    ### Call the Java API with the argument values since we can not
    ### pass the switches with dash(, - ) to the Java APIs. 
    if {[info exists javaArgs(-inst)]} {
        set ret [$b removeHierarchy $javaArgs(-inst) $javaArgs(-all_levels) ]
    } else {
        print_help $msg
    }
    return $ret
}

proc baya_remove_instance {args} {

    set msg    "
    Description: Removes the instance specified with the -name switch from the current module.
    Options    : 
                 -name <instance name> , the name of the instance to be removed. 
    Example    : baya_remove_instance -name inst1 
                "

    if { [ string compare -nocase $args "-help" ] == 0 } {
        print_help $msg
        return 
    }

    set argList [join $args " "]
    set args [ split $argList ]

    set ret ""
    set javaArgs(-flatten_entire_hierarcy) false
    ### Get the reference to $baya variable
    upvar #0 baya b
    ### Get the processed arguments
    array set javaArgs [processArgs  $args ]
    ### Call the Java API with the argument values since we can not
    ### pass the switches with dash(, - ) to the Java APIs. 
    if {[info exists javaArgs(-name)]} {
        set ret [$b removeInstance $javaArgs(-name) ]
    } else {
        print_help $msg
    }
    return $ret
}

proc baya_get_maturity_status {args} {

    set msg    "
    Description: Calculates te RTL maturity status w.r.t connection of the instances 
    Options    : 
                 -module <design name> , optional input to specify the  design. 
    Example    : baya_get_maturity_status -module foo 
                "

    if { [ string compare -nocase $args "-help" ] == 0 } {
        print_help $msg
        return 
    }

    set argList [join $args " "]
    set args [ split $argList ]

    set ret ""
    set javaArgs(-flatten_entire_hierarcy) false
    ### Get the reference to $baya variable
    upvar #0 baya b
    ### Get the processed arguments
    array set javaArgs [processArgs  $args ]
    set prevScope [baya_get_current_module]
    if {[info exists javaArgs(-module)] } {
        $b setCurrentModule $javaArgs(-module)
    }
    set ret [$b printMaturityStatus]
    if { $prevScope != $::null } {
        $b setCurrentModule $prevScope
    }
    return $ret
}
