// This grammar file has been taken from the below area-
// http://www.antlr3.org/grammar/1086696923011/vhdlams/vams.g
// This has been modified as per the permissing given below.
//
/*
  Antlr lexer and parser for VHDL-AMS (IEEE Std 1076.1-1999).
  
  The grammar fits to
  http://www.vhdl-ams.org/language/language.html
  with some ideas from
  vhdl.jjt [VHDL-93, VHDL-AMS grammar for JavaCC (c) 1997 Christoph Grimm].
  
  First Author: E.Fehlauer, F.Rogin
                Copyright (c) 2004 Fraunhofer-Gesellschaft.
                All rights reserved.

  Permission to use, copy, modify, and distribute this 
  software and its documentation for educational, research 
  and non-profit purposes, without fee, and without a 
  written agreement is hereby granted, provided that the 
  above copyright notice, this paragraph and the following 
  three paragraphs appear in all copies. 
  
  Permission to incorporate this software into commercial 
  products may be obtained by contacting
  
    Fraunhofer Institute for Integrated Circuits 
    Branch Lab Design Automation
    Zeunerstrae 38
    D-01069 Dresden
    Email: erhard.fehlauer@eas.iis.fhg.de
  
  This software program and documentation are copyrighted 
  by FhG IIS/EAS. The software program and documentation 
  are supplied "as is", without any accompanying services. 
  FhG IIS/EAS does not warrant that the operation of the 
  program will be uninterrupted or error-free. The end-user 
  understands that the program was developed for research 
  purposes and is advised not to rely exclusively on the 
  program for any reason. 
  
  IN NO EVENT SHALL FHG IIS/EAS BE LIABLE TO ANY PARTY FOR 
  DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL 
  DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF THE USE 
  OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF 
  FHG IIS/EAS HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH 
  DAMAGE. FHG IIS/EAS SPECIFICALLY DISCLAIMS ANY WARRANTIES, 
  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE 
  SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND 
  FHG IIS/EAS HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, 
  SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS. 

*/


grammar SortVHDL;
 

options {
  k         = 2;
  backtrack = true;
  output    = template;
  rewrite   = true;
}

@header {
    package com.eu.miscedautils.SortHDL;
    import com.eu.miscedautils.CommonUtils.LogMessages;   
    import java.util.Iterator;
    import java.util.List;
    import java.util.ArrayList;
    import java.util.LinkedList;
    import org.antlr.runtime.ANTLRFileStream;
    import org.antlr.runtime.ANTLRStringStream;
    import org.antlr.runtime.CharStream;
    import org.antlr.runtime.CommonTokenStream;
    import org.antlr.runtime.RecognitionException;
    import org.antlr.runtime.TokenSource;
    import org.antlr.runtime.TokenStream;
    import java.io.File;
    import java.io.FileInputStream;
    import java.io.IOException;
  }

@lexer::header {
    package com.eu.miscedautils.SortHDL;
    import java.util.Iterator;
    import java.util.List;
    import java.util.ArrayList;
    import java.util.LinkedList;
  }
  
@lexer::members
{
    boolean isInsideProcess = false;
    @Override public void reportError(RecognitionException re)
    {
        return;
    }
    public void displayRecognitionError(String[] tokenNames,
                                        RecognitionException e) {
        return;
    }
      @Override
  public void emitErrorMessage(String msg) 
  {
    // For the time being kept empty
  }
}

@members {
  boolean isInsideArchBody = false;
  SortHDLUnit currArch = null;        
  SortHDLUnit currEnt = null;        
  SortHDLUnit currConfig = null;        
  SortHDLUnit currPack = null;        
  SortHDLUnit currPackBody = null;    
  boolean isInsideProcess = false;
  // These libraries and use clauses will start getting populating
  // as soon as the file starts getting parsed or as soon as immediate
  // after the population of one unit. So, set these to null for
  // new population once done with one unit   
      
  ArrayList<String> libraries = null;
  ArrayList<String> useclauses = null;

  public void addLibrary(String lib)
  {
    if ( null == lib )
    {
      return;
    }
    if ( null == libraries )
    {
      libraries = new ArrayList<String>();
    }
    libraries.add(lib);
    return;
  }

  public boolean isLibrary(String lib)
  {
    if ( null == lib )
    {
      return false;
    }
    if ( null != libraries && false == libraries.isEmpty()  )
    {
      Iterator<String> itr = libraries.iterator();
      while( itr.hasNext() )
      {
        String libname = itr.next();
        if ( null != libname &&  lib.equalsIgnoreCase(libname ) )
        {
          return true;
        }
      }
    }
    return false;
  }

  public void addUseclause(String clause)
  {
    if ( null == clause )
    {
      return;
    }
    if ( null == useclauses )
    {
      useclauses = new ArrayList<String>();
    }
    useclauses.add(clause);
    return;
  }
  
  
  @Override
  public void emitErrorMessage(String msg) 
  {
    // For the time being kept empty
  }
    
  @Override 
  public String getErrorMessage(RecognitionException e, String[] tokenNames)
    {
        List stack = getRuleInvocationStack(e, this.getClass().getName());
        String msg = null;
        if ( e instanceof NoViableAltException ) 
        {
          NoViableAltException nvae = (NoViableAltException)e;
          msg = " no viable alt; token="+e.token+
          " (decision="+nvae.decisionNumber+
          " state "+nvae.stateNumber+")"+
          " decision=<<"+nvae.grammarDecisionDescription+">>";
        }
        else if ( e instanceof MismatchedTokenException ) 
        {
          MismatchedTokenException nvae = (MismatchedTokenException)e;
          msg = " MismatchedTokenException; token="+e.token+
          "";
        }
        else if ( e instanceof MismatchedTreeNodeException ) 
        {
          MismatchedTreeNodeException nvae = (MismatchedTreeNodeException)e;
          msg = " MismatchedTreeNodeException; token="+e.token+
            "";
        }
        else if ( e instanceof EarlyExitException ) 
        {
          EarlyExitException nvae = (EarlyExitException)e;
          msg = " EarlyExitException; token="+e.token+
            "";
        }
        else if ( e instanceof FailedPredicateException ) 
        {
          FailedPredicateException nvae = (FailedPredicateException)e;
          msg = " FailedPredicateException; token="+e.token+
            "";
        }
        else if ( e instanceof MismatchedRangeException ) 
        {
          MismatchedRangeException nvae = (MismatchedRangeException)e;
          msg = " MismatchedRangeException; token="+e.token+
        "";
        }
        else if ( e instanceof MismatchedSetException ) 
        {
          MismatchedSetException nvae = (MismatchedSetException)e;
          msg = " MismatchedSetException; token="+e.token+
          "";
        }
        else if ( e instanceof MismatchedNotSetException ) 
        {
          MismatchedNotSetException nvae = (MismatchedNotSetException)e;
          msg = " MismatchedNotSetException; token="+e.token+
          "";
          
        }
        else 
        {
          msg = super.getErrorMessage(e, tokenNames);
        }
        return stack+" " + msg;
    }
  
  public String getTokenErrorDisplay(Token t) 
  {
    return t.toString();
  }
  
  private List<String> errors = new LinkedList<String>();
    public void displayRecognitionError(String[] tokenNames,
                                        RecognitionException e) {
        String hdr = getErrorHeader(e);
        String msg = getErrorMessage(e, tokenNames);
        errors.add(hdr + " " + msg);
    }
    public List<String> getErrors() {
        return errors;
    }
  
}
 
 
parseVhdl
@init
{
  this.currArch = null;        
  this.currEnt = null;        
  this.currConfig = null;        
  this.currPack = null;        
  this.currPackBody = null;    
  this.libraries = null;
  this.useclauses = null;
  this.isInsideArchBody = false;
  this.isInsideProcess = false;
}
  :
  (design_file)* EOF
  ;

abstract_literal
  :
  DECIMAL_LITERAL
  | BASED_LITERAL
  ;

access_type_definition
  :
  ACCESS subtype_indication
  ;

actual_designator
  :
  expression
  | OPEN
  ;

actual_parameter_part
  :
  association_list
  ;

actual_part
  :
  (name LPAREN) => name LPAREN actual_designator RPAREN
  | actual_designator
  ;

adding_operator
  :
  PLUS
  | MINUS
  | AMPERSAND
  ;

aggregate
  :
  LPAREN element_association (COMMA element_association)* RPAREN
  ;

alias_declaration
  :
  ALIAS alias_designator (COLON alias_indication)? IS name (signature)? SEMI
  ;

alias_designator
  :
  identifier
  | CHARACTER_LITERAL
  | STRING_LITERAL
  ;

alias_indication
  :
  subtype_indication
  ;

allocator
  :
  NEW
  (
    (name APOSTROPHE) => qualified_expression
    | subtype_indication
  )
  ;

architecture_body
@init
{
  currArch = null;        
  currEnt = null;        
  currConfig = null;        
  currPack = null;        
  currPackBody = null;    
}
@after
{
  currArch = null;        
  currEnt = null;        
  currConfig = null;        
  currPack = null;        
  currPackBody = null;
  useclauses = null;
  libraries = null;      
}

  :
  ARCHITECTURE id=identifier OF nm=name 
  {
    currArch = null;        
    currEnt = null;        
    currConfig = null;        
    currPack = null;        
    currPackBody = null; 
    isInsideArchBody = false;
    String entname = $nm.tok.getText();
    SortHDLUnit primdu = SortHDLRoot.instance().getDesignUnit(entname);
    if ( null == primdu )
    {
      primdu = new SortHDLUnit( SortHDLBase.ObjType.UNDEFINED_MODULE, entname, null, $nm.tok.getLine() );
      SortHDLRoot.instance().addDesignUnit(primdu);
    }
    String archname = new String (entname + "::" + $id.tok.getText());
    SortHDLUnit du = SortHDLRoot.instance().getDesignUnit(archname);
    HDLFile file = SortHDLRoot.instance().getHDLFile();    
    if ( null == du )
    {    
      du = new SortHDLUnit( SortHDLBase.ObjType.ARCHITECTURE, archname, file, $id.tok.getLine());
      SortHDLRoot.instance().addDesignUnit(du);
    }
    else
    {
      du.setObjType(SortHDLBase.ObjType.ARCHITECTURE);
      du.setFile(file);
      du.setLine($id.tok.getLine());
    }    
    currArch = du;
    currEnt = primdu;        
    du.setEntity(primdu);  
    primdu.addArchitecture(du);
    currArch.setLibraries(libraries);
    currArch.setUseClauses( useclauses );
  } IS 
  architecture_declarative_part BEGIN {isInsideArchBody = true;}architecture_statement_part {isInsideArchBody = false;} END (ARCHITECTURE)? (identifier)? SEMI  
  ;

architecture_declarative_part
  :
  (block_declarative_item)*
  ;

architecture_statement
  :
  block_statement
  |
  (
    (label_colon)? (POSTPONED)? PROCESS
  )
    => process_statement
  |
  (
    (label_colon)? (POSTPONED)? procedure_call SEMI
  )
    => concurrent_procedure_call_statement
  |
  (
    (label_colon)? (POSTPONED)? ASSERT
  )
    => concurrent_assertion_statement
  |
  (
    (label_colon)? (POSTPONED)?
    (
      conditional_signal_assignment
      | selected_signal_assignment
    )
  )
    => concurrent_signal_assignment_statement
  | (label_colon instantiated_unit) => component_instantiation_statement
  | (label_colon generation_scheme GENERATE) => generate_statement
  | simultaneous_statement
  ;

architecture_statement_part
  :
  (architecture_statement)*
  ;

array_type_definition
  :
  (unconstrained_array_definition) => unconstrained_array_definition
  | constrained_array_definition
  ;

assertion
  :
  ASSERT condition (REPORT expression)? (SEVERITY expression)?
  ;

assertion_statement
  :
  (label_colon)? assertion SEMI
  ;

association_element
  :
  ( (formal_part ARROW) => formal_part ARROW)? actual_part
  ;

association_list
  :
  association_element (COMMA association_element)*
  ;

attribute_declaration
  :
  ATTRIBUTE label_colon name SEMI
  ;

// Need to add several tokens here, for they are both, VHDLAMS reserved words
// and attribute names.
// (25.2.2004, e.f.)

attribute_designator
  :
  identifier
  | RANGETOK
  ;

attribute_specification
  :
  ATTRIBUTE attribute_designator OF entity_specification IS expression SEMI
  ;

base_unit_declaration
  :
  identifier SEMI
  ;

binding_indication returns  [Token lib, Token ent, Token arch, Token config ]
  :
  (USE e=entity_aspect 
  {
    $lib = $e.lib; $ent = $e.ent; $arch = $e.arch; $config = $e.config;
  })? 
  (generic_map_aspect)? (port_map_aspect)?
  ;

block_configuration
//@after
//{
//    currArch = null;        
//    currEnt = null;        
//    currConfig = null;        
//    currPack = null;        
//    currPackBody = null; 
//}
  :
  FOR blk=block_specification 
  {
    assert ( null != currEnt );     
    String archname = currEnt.getName() + "::" + $blk.tok.getText();
    SortHDLUnit arch = SortHDLRoot.instance().getDesignUnit(archname);
    HDLFile file = SortHDLRoot.instance().getHDLFile();
    if ( null == arch )
    {
      arch = new SortHDLUnit( SortHDLBase.ObjType.UNDEFINED_MODULE, archname, file, $blk.tok.getLine());
      SortHDLRoot.instance().addDesignUnit(arch);
    }    
    currArch = arch;
    // This architecture got used in this configuration.
    assert( null != currConfig );
//    currArch.setEntity();
    if ( currConfig.getArchitectures().size() == 0 )
    {
      currConfig.addArchitecture(currArch);
    }
  }
    (use_clause)* 
    (configuration_item)* 
  END FOR SEMI
  ;

block_declarative_item
  :
  (subprogram_declaration) => subprogram_declaration
  | subprogram_body
  | type_declaration
  | subtype_declaration
  | constant_declaration
  | signal_declaration
  | variable_declaration
  | file_declaration
  | alias_declaration
  | component_declaration
  | attribute_declaration
  | attribute_specification
  | configuration_specification
  | disconnection_specification
  | use_clause
  | group_template_declaration
  | group_declaration
  ;

block_declarative_part
  :
  (block_declarative_item)*
  ;

block_header
  :
  (generic_clause (generic_map_aspect SEMI)?)? (port_clause (port_map_aspect SEMI)?)?
  ;

block_specification returns [Token tok] // Block/generate not supported
  :
  (identifier) => id=identifier{ $tok = $id.tok; } 
                 (LPAREN index_specification{ $tok = $id.tok; } RPAREN)?
  | nm=name { $tok = $nm.tok; }
  ;

block_statement
  :
  label_colon BLOCK (LPAREN expression RPAREN)? (IS)? block_header block_declarative_part BEGIN block_statement_part END BLOCK (identifier)? SEMI
  ;

block_statement_part
  :
  (architecture_statement)*
  ;

case_statement
  :
  (label_colon)? CASE expression IS (case_statement_alternative)+ END CASE (identifier)? SEMI
  ;

case_statement_alternative
  :
  WHEN choices ARROW sequence_of_statements
  ;

choice
  :
  (identifier) => identifier
  | (discrete_range) => discrete_range
  | simple_expression
  | OTHERS
  ;

choices
  :
  choice (BAR choice)*
  ;

component_configuration
  :
  FOR compinst=component_specification (bind=binding_indication SEMI)? 
    {
      Token ent = $bind.ent;
      Token arch = $bind.arch; 
      Token config = $bind.config;
      Token compname =  $compinst.compname;
      LinkedList<Token> instlist = $compinst.instlist;
      if ( null != instlist && false == instlist.isEmpty() )
      {
        HDLFile file = SortHDLRoot.instance().getHDLFile();
        Iterator<Token> instlistitr = instlist.iterator();
        while( instlistitr.hasNext() )
        {
          Token inst = instlistitr.next();
          if ( null != inst )
          {
            SortHDLBase.ObjType type =  SortHDLBase.ObjType.UNKNOWN;
            String mastername = null;
            String archname = null;
          
            if ( null != ent )
            {
              type =  SortHDLBase.ObjType.ENTITY;
              mastername = ent.getText();
              if ( null != arch )
              {
                archname = arch.getText();
              }
              // If we are inside a configuration declaration, we need to
              // set this entity as current entity ....              
              if ( null != currConfig )
              {
                // Set this entity as current entity
                SortHDLUnit entity = SortHDLRoot.instance().getDesignUnit(mastername);
                if ( null == entity )
                {
                  entity = new SortHDLUnit( SortHDLBase.ObjType.UNDEFINED_MODULE, mastername, file, ent.getLine());
                  SortHDLRoot.instance().addDesignUnit(entity);
                }
                currEnt = entity;
                if ( null != archname )
                {
                  String tmpArchName = mastername + "::" + archname;
                  SortHDLUnit archDef = SortHDLRoot.instance().getDesignUnit(tmpArchName);
                  if ( null == arch )
                  {
                    archDef = new SortHDLUnit( SortHDLBase.ObjType.UNDEFINED_MODULE, tmpArchName, file, -1);
                    SortHDLRoot.instance().addDesignUnit(archDef);
                  }
                  currArch = archDef;
                  //currArch.addUserunit(currConfig);
                  //currConfig.addUsedunit(currArch); 
                  currArch.setEntity(currEnt);
                  currEnt.addArchitecture(currArch);
                }
                currConfig.addUserunit(currEnt);
                currEnt.addUserunit(currConfig); 
              }
            }
            else if ( null == ent && null != config )
            {
              type =  SortHDLBase.ObjType.CONFIGURATION;
              mastername = "<UNKNOWN>" + "::" + config.getText();              
              
              SortHDLUnit configuration = SortHDLRoot.instance().getConfiguration(mastername);
              if ( null == configuration )
              {
                // This is the first pllace where we will be creating configuration 
                // only with its name... For the rest of the placces we have prepended the
                // corresponding entity name ...
                configuration = new SortHDLUnit( SortHDLBase.ObjType.UNDEFINED_CONFIGURATION, mastername, file, config.getLine());
                SortHDLRoot.instance().addDesignUnit(configuration);
              }

              if ( null != currArch )
              { 
                currArch.addUsedunit(configuration);
                configuration.addUserunit(currConfig);
              }
            }
            else
            {
              assert(false);
            }
            String libname = null;
            if ( null != $bind.lib )
            {
              libname = $bind.lib.getText();
            }
                        // If we are inside a configuration declaration, we need to
            // set this entity as current entity ....
            if ( null != libname && null != mastername )
            {
                String useclause = libname;
                String str[] = mastername.split("::");
                if ( str.length > 1 ) 
                {
                    useclause += "::" + str[1];
                }
                else
                {
                    useclause += "::" + str[0];
                }
                if ( null != currConfig )
                {
                    currConfig.addUseClause(useclause);
                }
                if ( null != currArch )
                {
                    currArch.addUseClause(useclause); 
                }
            }
            // If we are inside a configuration declaration, we need to
            // set this entity as current entity ....
            
            SortHDLBinding bind1 = new SortHDLBinding(
                                    SortHDLBase.ObjType.BINDING,
                                    inst.getText(),
                                    libname, 
                                    compname.getText(),
                                    type,
                                    mastername,
                                    file,
                                    inst.getLine()
                                    ); 
             bind1.setMasterarchname( archname );
             // This binding can only be for a given acrhitecture ...
             currArch.addBinding( bind1 );
          }
        }
      }
  }
  
  (block_configuration)? END FOR SEMI
  
  ;

//component_declaration
//  : COMPONENT identifier ( IS )?
//    ( generic_clause )?
//    ( port_clause )?
//    END COMPONENT ( identifier )? SEMI
//  ;

component_declaration
  :
  COMPONENT nm=identifier (IS)? 
  (generic_clause)? 
  (port_clause)? 
  END COMPONENT (identifier )? SEMI
  ;

component_instantiation_statement
  :
  lab=label_colon inst=instantiated_unit (generic_map_aspect)? (port_map_aspect)? SEMI
  {
    String instname = null;
    Integer line = 0;
    if ( null != $lab.tok )
    {
      instname = $lab.tok.getText();
      line = $lab.tok.getLine();
    }
    String libname = null;
    if ( null != $inst.lib )
    {
      libname = $inst.lib.getText();
    }
    String duname = null;
    if ( null != $inst.primdu )
    {
      duname = $inst.primdu.getText();
    }
    String archname = null;
    if ( null != $inst.arch )
    {
      archname = $inst.arch.getText();
    }
    SortHDLBase.ObjType insttype = $inst.type;
    HDLFile file = SortHDLRoot.instance().getHDLFile();
    SortHDLInstance instobj = new SortHDLInstance( 
                                      SortHDLBase.ObjType.INSTANCE,
                                      instname,
                                      libname, // Lib name is null
                                      duname,
                                      insttype,
                                      file,
                                      line
                                      );
    currArch.addInstance( instobj );
  }
  ;

component_specification returns [ LinkedList<Token> instlist, Token compname ]
  :
  toks=instantiation_list COLON nm=name 
  {
    $instlist = $toks.toklist;
    $compname = $nm.tok;
  }
  ;

composite_nature_definition
  :
  record_nature_definition
  ;

composite_type_definition
  :
  array_type_definition
  | record_type_definition
  ;

concurrent_assertion_statement
  :
  (label_colon)? (POSTPONED)? assertion SEMI
  ;

concurrent_procedure_call_statement
  :
  (label_colon)? (POSTPONED)? procedure_call SEMI
  ;

concurrent_signal_assignment_statement
  :
  (label_colon)? (POSTPONED)?
  (
    conditional_signal_assignment
    | selected_signal_assignment
  )
  ;

condition
  :
  expression
  ;

condition_clause
  :
  UNTIL condition
  ;

conditional_signal_assignment
  :
  target LE opts conditional_waveforms SEMI
  ;

conditional_waveforms
  :
  waveform ( (WHEN condition ELSE) => conditional_waveforms_bi)? (WHEN condition)?
  ;

conditional_waveforms_bi
  :
  WHEN condition ELSE waveform ( (WHEN condition ELSE) => conditional_waveforms_bi)?
  ;


configuration_declaration 
@init
{
  currArch = null;        
  currEnt = null;        
  currConfig = null;        
  currPack = null;        
  currPackBody = null;    
}
@after
{
  currArch = null;        
  currEnt = null;        
  currConfig = null;        
  currPack = null;        
  currPackBody = null;
  useclauses = null;
  libraries = null;     
}
  :
  CONFIGURATION id=identifier OF nm1=name IS 
  {
    currArch = null;        
    currEnt = null;        
    currConfig = null;        
    currPack = null;        
    currPackBody = null; 
  
    SortHDLUnit entity = SortHDLRoot.instance().getDesignUnit($nm1.tok.getText());
    HDLFile file = SortHDLRoot.instance().getHDLFile();
    if ( null == entity )
    {
      entity = new SortHDLUnit( SortHDLBase.ObjType.UNDEFINED_MODULE, $nm1.tok.getText(), file, $nm1.tok.getLine());
      SortHDLRoot.instance().addDesignUnit(entity);
    }    
    String confname = $nm1.tok.getText() + "::" + $id.tok.getText();
    SortHDLUnit du = SortHDLRoot.instance().getConfiguration(confname);
    if ( null == du )
    {
      du = new SortHDLUnit( SortHDLBase.ObjType.CONFIGURATION, confname , file, $id.tok.getLine());
      SortHDLRoot.instance().addDesignUnit(du);
    }
    else
    {
      assert ( SortHDLBase.ObjType.UNDEFINED_MODULE == du.getObjType() );
      du.setObjType( SortHDLBase.ObjType.CONFIGURATION );
      du.setFile( file );
      du.setLine( $id.tok.getLine() );
      du.setName( confname );
      currConfig = du;      
    }
    // This configuration is one of the used object of this
    // entity.  So, populate the user and used object
    du.setEntity(entity);  
    entity.addConfiguration(du);
    currEnt = entity;
    currConfig = du; 
    currConfig.setLibraries(libraries);
    currConfig.setUseClauses( useclauses );
  }
  configuration_declarative_part 
  block_configuration 
  END (CONFIGURATION)? (identifier)? SEMI
  ;

configuration_declarative_item
  :
  use_clause
  | attribute_specification
  | group_declaration
  ;

configuration_declarative_part
  :
  (configuration_declarative_item)*
  ;

configuration_item
  :
  block_configuration
  {
    System.err.println(LogMessages.instance().getToolprefix() + ": Warning: Block configuration is not supported yet ..." );
  }  
  | component_configuration
  ;

configuration_specification
  :
  FOR compinst=component_specification bind=binding_indication SEMI
  {
      Token ent = $bind.ent;
      Token arch = $bind.arch; 
      Token config = $bind.config;
      Token compname =  $compinst.compname;
      LinkedList<Token> instlist = $compinst.instlist;
      if ( null != instlist && false == instlist.isEmpty() )
      {
        HDLFile file = SortHDLRoot.instance().getHDLFile();
        Iterator<Token> instlistitr = instlist.iterator();
        while( instlistitr.hasNext() )
        {
          Token inst = instlistitr.next();
          if ( null != inst )
          {
            SortHDLBase.ObjType type =  SortHDLBase.ObjType.UNKNOWN;
            String mastername = null;
            String archname = null;
          
            if ( null != ent )
            {
              type =  SortHDLBase.ObjType.ENTITY;
              mastername = ent.getText();
              if ( null != arch )
              {
                archname = arch.getText();
              }
            }
            else if ( null == ent && null != config )
            {
              type =  SortHDLBase.ObjType.CONFIGURATION;
              mastername = "<UNKNOWN>" + "::" + config.getText();                            
              SortHDLUnit configuration = SortHDLRoot.instance().getConfiguration(mastername);
              if ( null == configuration )
              {
                // This is the 2nd pllace where we will be creating configuration 
                // only with its name... For the rest of the placces we have prepended the
                // corresponding entity name ...
                configuration = new SortHDLUnit( SortHDLBase.ObjType.UNDEFINED_CONFIGURATION, mastername, file, config.getLine());
                SortHDLRoot.instance().addDesignUnit(configuration);
              }
              if ( null != currConfig )
              {
                configuration.addUserunit(currConfig);
                currConfig.addUsedunit(configuration);
              }
              else if ( null != currArch )
              { 
                currArch.addUsedunit(configuration);
                configuration.addUserunit(currConfig);
              }
            }
            else
            {
              assert(false);
            }
            String libname = null;
            if ( null != $bind.lib )
            {
              libname = $bind.lib.getText();
            }
            
            SortHDLBinding bind1 = new SortHDLBinding(
                                    SortHDLBase.ObjType.BINDING,
                                    inst.getText(),
                                    libname, 
                                    compname.getText(),
                                    type,
                                    mastername,
                                    file,
                                    inst.getLine()
                                    ); 
             bind1.setMasterarchname( archname );
             currArch.addBinding( bind1 );
          }
        }
      }
  }      
  ;

constant_declaration
  :
  CONSTANT identifier_list COLON subtype_indication (VARASGN expression)? SEMI
  ;

constrained_array_definition
  :
  ARRAY index_constraint OF subtype_indication
  ;

constraint
  :
  range_constraint
  | index_constraint
  ;

context_clause
  :
  (context_item)*
  ;

context_item
  :
  library_clause
  | use_clause
  ;

delay_mechanism
  :
  TRANSPORT
  | (REJECT expression)? INERTIAL
  ;

design_file
  :
  (design_unit)* EOF
  ;

design_unit
  :
  context_clause library_unit
  ;

designator
  :
  identifier
  | STRING_LITERAL
  ;

direction
  :
  TO
  | DOWNTO
  ;

disconnection_specification
  :
  DISCONNECT guarded_signal_specification AFTER expression SEMI
  ;

discrete_range
  :
  (range) => range
  | subtype_indication
  ;

element_association
  :
  ( (choices ARROW) => choices ARROW)? expression
  ;

element_declaration
  :
  identifier_list COLON element_subtype_definition SEMI
  ;

element_subtype_definition
  :
  subtype_indication
  ;


entity_aspect returns [Token lib, Token ent, Token arch, Token config ]
  :
  ENTITY nm1=name {$lib = $nm1.lib; $ent = $nm1.tok;}  (LPAREN nm2=identifier{ $arch=$nm2.tok;} RPAREN)?
  | CONFIGURATION nm3=name {$lib = $nm3.lib;$config = $nm3.tok;}
  | OPEN { $lib = null; $ent = null; $arch = null;  $config = null; }
  ;

entity_class
  :
  ENTITY
  | ARCHITECTURE
  | CONFIGURATION
  | PROCEDURE
  | FUNCTION
  | PACKAGE
  | TYPE
  | SUBTYPE
  | CONSTANT
  | SIGNAL
  | VARIABLE
  | COMPONENT
  | LABEL
  | LITERAL
  | UNITS
  | GROUP
  | FILE
  ;

entity_class_entry
  :
  entity_class (BOX)?
  ;

entity_class_entry_list
  :
  entity_class_entry (COMMA entity_class_entry)*
  ;

entity_declaration
@init
{
  currArch = null;        
  currEnt = null;        
  currConfig = null;        
  currPack = null;        
  currPackBody = null;    
}
@after
{
  currArch = null;        
  currEnt = null;        
  currConfig = null;        
  currPack = null;        
  currPackBody = null;
  useclauses = null;
  libraries = null;      
}
  :
  ENTITY id=identifier 
  {
    currArch = null;        
    currEnt = null;        
    currConfig = null;        
    currPack = null;        
    currPackBody = null; 
  
    SortHDLUnit du = SortHDLRoot.instance().getDesignUnit($id.tok.getText());
    HDLFile file = SortHDLRoot.instance().getHDLFile();
    if ( null == du )
    {
      du = new SortHDLUnit( SortHDLBase.ObjType.ENTITY, $id.tok.getText() , file, $id.tok.getLine());
      SortHDLRoot.instance().addDesignUnit(du);
    }
    else
    {
      assert( SortHDLBase.ObjType.UNDEFINED_MODULE == du.getObjType() ) ;
      du.setObjType( SortHDLBase.ObjType.ENTITY );
      du.setFile( file );
      du.setLine( $id.tok.getLine() );
    }
    currEnt = du;
    currEnt.setLibraries(libraries);
    currEnt.setUseClauses( useclauses );
  } 
  IS entity_header entity_declarative_part 
  (BEGIN entity_statement_part)? END (ENTITY)? (identifier )? SEMI
  ;

entity_declarative_item
  :
  (subprogram_declaration) => subprogram_declaration
  | subprogram_body
  | type_declaration
  | subtype_declaration
  | constant_declaration
  | signal_declaration
  | variable_declaration
  | file_declaration
  | alias_declaration
  | attribute_declaration
  | attribute_specification
  | disconnection_specification
  | use_clause
  | group_template_declaration
  | group_declaration
  ;

entity_declarative_part
  :
  (entity_declarative_item)*
  ;

entity_designator
  :
  entity_tag (signature)?
  ;

entity_header
  :
  (generic_clause)? (port_clause)?
  ;

entity_name_list
  :
  entity_designator (COMMA entity_designator)*
  | OTHERS
  | ALL
  ;

entity_specification
  :
  entity_name_list COLON entity_class
  ;

entity_statement
  :
  (
    (label_colon)? (POSTPONED)? ASSERT
  )
    => concurrent_assertion_statement
  |
  (
    (label_colon)? (POSTPONED)? PROCESS
  )
    => process_statement
  | concurrent_procedure_call_statement
  ;

entity_statement_part
  :
  (entity_statement)*
  ;

entity_tag
  :
  identifier
  | CHARACTER_LITERAL
  | STRING_LITERAL
  ;

enumeration_literal
  :
  identifier
  | CHARACTER_LITERAL
  ;

enumeration_type_definition
  :
  LPAREN enumeration_literal (COMMA enumeration_literal)* RPAREN
  ;

exit_statement
  :
  (label_colon)? EXIT (identifier)? (WHEN condition)? SEMI
  ;

// NOTE that NAND/NOR are in (...)* now (used to be in (...)?).
// (21.1.2004, e.f.)

expression
  :
  relation (options {greedy=true;}: logical_operator relation)*
  ;

factor
  :
  primary (options {greedy=true;}: DOUBLESTAR primary)?
  | ABS primary
  | NOT primary
  ;

file_declaration
  :
  FILE identifier_list COLON subtype_indication (file_open_information)? SEMI
  ;

file_logical_name
  :
  expression
  ;

file_open_information
  :
  (OPEN expression)? IS file_logical_name
  ;

file_type_definition
  :
  FILE OF name
  ;

formal_parameter_list
  :
  interface_list
  ;

formal_part
  :
  name (LPAREN name RPAREN)?
  ;

function_call
  :
  name (LPAREN actual_parameter_part RPAREN)?
  ;

generate_statement
  :
  label_colon generation_scheme GENERATE ( (block_declarative_item)* BEGIN)? (architecture_statement)* END GENERATE (identifier)? SEMI
  ;

generation_scheme
  :
  FOR parameter_specification
  | IF condition
  ;

generic_clause
  :
  GENERIC LPAREN generic_list RPAREN SEMI
  ;

generic_list
  :
  interface_list
  ;

generic_map_aspect
  :
  GENERIC MAP LPAREN association_list RPAREN
  ;

group_constituent
  :
  name
  | CHARACTER_LITERAL
  ;

group_constituent_list
  :
  group_constituent (COMMA group_constituent)*
  ;

group_declaration
  :
  GROUP label_colon name LPAREN group_constituent_list RPAREN SEMI
  ;

group_template_declaration
  :
  GROUP identifier IS LPAREN entity_class_entry_list RPAREN SEMI
  ;

guarded_signal_specification
  :
  signal_list COLON name
  ;

//identifier
//  : BASIC_IDENTIFIER
//  | EXTENDED_IDENTIFIER
//  ;

identifier returns [Token tok]
  :
  id=BASIC_IDENTIFIER {$tok = $id;}
  | eid=EXTENDED_IDENTIFIER { $tok = $eid;}
  ;

identifier_list
  :
  identifier (COMMA identifier)*
  ;

if_statement
  :
  (label_colon)? IF condition THEN sequence_of_statements (ELSIF condition THEN sequence_of_statements)* (ELSE sequence_of_statements)? END IF (identifier)? SEMI
  ;

index_constraint
  :
  LPAREN discrete_range (COMMA discrete_range)* RPAREN
  ;

index_specification
  :
  (discrete_range) => discrete_range
  | expression
  ;

index_subtype_definition
  :
  name RANGETOK BOX
  ;

//instantiated_unit
//  : ( COMPONENT )? name
//  | ENTITY name ( LPAREN identifier RPAREN )?
//  | CONFIGURATION name
//  ;

instantiated_unit returns [Token lib, Token primdu, Token arch, SortHDLBase.ObjType type]
  :
  (COMPONENT)? nm=name 
  {
    $lib = $nm.lib;
    $primdu = $nm.tok;
    $arch = null;
    $type = SortHDLBase.ObjType.COMPONENT;
  }
  | ENTITY nm2=name 
  {
    $lib = $nm2.lib;
    $primdu = $nm2.tok;
    $arch = null;
    $type = SortHDLBase.ObjType.ENTITY;
  } (LPAREN 
        id=identifier 
        {
          $arch = $id.tok;
        }
     RPAREN)?
  | CONFIGURATION nm3=name 
  {
    $lib = $nm3.lib;
    $primdu = $nm3.tok;
    $arch = null;
    $type = SortHDLBase.ObjType.CONFIGURATION;
  }
  ;

instantiation_list returns [ LinkedList<Token> toklist]
  :
  id1=identifier 
  {
    if ( null == $toklist )
    {
      $toklist = new LinkedList<Token>();
    }
    $toklist.add($id1.tok);
  } 
    (COMMA id2=identifier)*
    {
      if ( null == $toklist )
      {
        $toklist = new LinkedList<Token>();
      }
      $toklist.add($id2.tok);
    }
  | OTHERS
    {
      if ( null == $toklist )
      {
        $toklist = new LinkedList<Token>();
      }
      $OTHERS.setText("OTHERS");
      $toklist.add($OTHERS);
    }
  | ALL
    {
      if ( null == $toklist )
      {
        $toklist = new LinkedList<Token>();
      }
      $ALL.setText("ALL");
      $toklist.add($ALL);
    }
  ;

interface_constant_declaration
  :
  (CONSTANT)? identifier_list COLON (IN)? subtype_indication (VARASGN expression)?
  ;

interface_declaration
  :
  (interface_constant_declaration) => interface_constant_declaration
  | (interface_signal_declaration) => interface_signal_declaration
  | interface_variable_declaration
  | interface_file_declaration
  ;

interface_element
  :
  interface_declaration
  ;

interface_file_declaration
  :
  FILE identifier_list COLON subtype_indication
  ;

interface_list
  :
  interface_element (SEMI interface_element)*
  ;

interface_signal_declaration
  :
  (SIGNAL)? identifier_list COLON (mode)? subtype_indication (BUS)? (VARASGN expression)?
  ;

interface_variable_declaration
  :
  (VARIABLE)? identifier_list COLON (mode)? subtype_indication (VARASGN expression)?
  ;

iteration_scheme
  :
  WHILE condition
  | FOR parameter_specification
  ;

label_colon returns [Token tok]
  :
  identifier { $tok = $identifier.tok;} COLON
  ;

library_clause
  :
  LIBRARY logical_name_list SEMI
  ;

library_unit
  :
  (
    ARCHITECTURE
    | PACKAGE BODY
  )
    => secondary_unit
  | primary_unit
  ;

literal
  :
  NULLTOK
  | BIT_STRING_LITERAL
  | (DBLQUOTE) => STRING_LITERAL
  | (enumeration_literal) => enumeration_literal
  | numeric_literal
  ;

logical_name returns [Token tok;]
  :
  identifier { $tok = $identifier.tok;}
  ;

logical_name_list
  :
  nm=logical_name 
  {
    if ( null != $nm.tok )
    {
      String libname = $nm.tok.getText();
      if ( false == libname.toLowerCase().equals( "ieee") && 
           false == libname.toLowerCase().equals( "std") )
      {
        addLibrary(libname);
      }
    } 
  } 
  (COMMA nm2=logical_name 
  {
    if ( null != $nm2.tok )
    {
      String libname = $nm2.tok.getText();          
      if ( false == libname.toLowerCase().equals( "ieee") && 
           false == libname.toLowerCase().equals( "std")  )
      {
        addLibrary( libname );
      }
    } 
  })*
  ;

logical_operator
  :
  AND
  | OR
  | NAND
  | NOR
  | XOR
  | XNOR
  ;

loop_statement
  :
  (label_colon)? (iteration_scheme)? LOOP sequence_of_statements END LOOP (identifier)? SEMI
  ;

mode
  :
  IN
  | OUT
  | INOUT
  | BUFFER
  | LINKAGE
  ;

multiplying_operator
  :
  MUL
  | DIV
  | MOD
  | REM
  ;

// was
//   name
//     : simple_name
//     | operator_symbol
//     | selected_name
//     | indexed_name
//     | slice_name
//     | attribute_name
//     ;
// changed to avoid left-recursion to name (from selected_name, indexed_name,
// slice_name, and attribute_name, respectively)
// (2.2.2004, e.f.)

name returns [Token tok, Token lib ]
  :
  (
    nm1=identifier { $tok = $nm1.tok;}
    | STRING_LITERAL {$tok = $STRING_LITERAL;}
  )
  ( options {greedy=true;}:
    (
      DOT nm2=suffix 
      {
       if ( null != $nm2.tok )
       {
         String orgName = $nm2.tok.getText();
         $lib = $tok;
         $tok = $nm2.tok;
         // Handle the case where some refers objects inside package through
         // simple name. This DU will be dependent on this those package/DUs 
         if ( isLibrary( $lib.getText() ) && isInsideArchBody && null != currArch )
         {
           // This $tok must be one of the design units. So, lets make the current
           // primary DU  dependent on this.
           String pack = $tok.getText();
           SortHDLUnit usedunit = SortHDLRoot.instance().getDesignUnit(pack);
           if ( null ==   usedunit )
           {
              HDLFile file = SortHDLRoot.instance().getHDLFile();
              usedunit = new SortHDLUnit(SortHDLBase.ObjType.UNDEFINED_MODULE, pack, file, 0);
              SortHDLRoot.instance().addDesignUnit(usedunit);
              usedunit.setTargetlib( $lib.getText() );                 
           }
           currArch.addUsedunit(usedunit);  
         }
       }
      }
      | APOSTROPHE aggregate
      | (signature)? APOSTROPHE attribute_designator
      | (LPAREN expression (COMMA expression)* RPAREN) => LPAREN expression (COMMA expression)* RPAREN
      | (LPAREN actual_parameter_part RPAREN) => LPAREN actual_parameter_part RPAREN
      | LPAREN discrete_range (COMMA discrete_range)* RPAREN
    )
  )*
  ;

next_statement
  :
  (label_colon)? NEXT (identifier)? (WHEN condition)? SEMI
  ;

numeric_literal
  :
  (abstract_literal) => abstract_literal
  | physical_literal
  ;

object_declaration
  :
  constant_declaration
  | signal_declaration
  | variable_declaration
  | file_declaration
  ;

opts
  :
  (GUARDED)? (delay_mechanism)?
  ;

//package_body
//  : PACKAGE BODY identifier IS
//    package_body_declarative_part
//    END ( PACKAGE BODY )? ( identifier )? SEMI
//  ;

package_body
@init
{
  currArch = null;        
  currEnt = null;        
  currConfig = null;        
  currPack = null;        
  currPackBody = null;    
}
@after
{
  currArch = null;        
  currEnt = null;        
  currConfig = null;        
  currPack = null;        
  currPackBody = null;
  useclauses = null;
  libraries = null;      
}
  :
  PACKAGE BODY nm=identifier IS
  {
    currArch = null;        
    currEnt = null;        
    currConfig = null;        
    currPack = null;        
    currPackBody = null;
    String bodyname = $nm.tok.getText() + "::" + $nm.tok.getText();
    SortHDLUnit du = SortHDLRoot.instance().getDesignUnit(bodyname);
    HDLFile file = SortHDLRoot.instance().getHDLFile();
    if ( null == du )
    {
      du = new SortHDLUnit( SortHDLBase.ObjType.PACKAGEBODY, bodyname , file, $nm.tok.getLine());
      SortHDLRoot.instance().addDesignUnit(du);
    }
    else
    {
      assert( SortHDLBase.ObjType.UNDEFINED_MODULE == du.getObjType() ) ;
      du.setObjType( SortHDLBase.ObjType.PACKAGEBODY );
      du.setFile( file );
      du.setLine( $nm.tok.getLine() );
    }
    // Find the package declaration and set this as dependent onto that     
    String packname = $nm.tok.getText();
    SortHDLUnit pack = SortHDLRoot.instance().getDesignUnit(packname);
    if ( null == pack )
    {
      pack = new SortHDLUnit( SortHDLBase.ObjType.UNDEFINED_MODULE, packname , file, $nm.tok.getLine());
      SortHDLRoot.instance().addDesignUnit(pack);
    }
    du.setPackage( pack );
    pack.addPackageBody( du );
    currPackBody = du;
    currPackBody.setLibraries(libraries);
    currPackBody.setUseClauses( useclauses );
  }   
  package_body_declarative_part 
  END (PACKAGE BODY)? (identifier)? SEMI
  ;

package_body_declarative_item
  :
  (subprogram_declaration) => subprogram_declaration
  | subprogram_body
  | type_declaration
  | subtype_declaration
  | constant_declaration
  | variable_declaration
  | file_declaration
  | alias_declaration
  | use_clause
  | group_template_declaration
  | group_declaration
  ;

package_body_declarative_part
  :
  (package_body_declarative_item)*
  ;

//package_declaration
//  : PACKAGE identifier IS
//    package_declarative_part
//    END ( PACKAGE )? ( identifier )? SEMI
//  ;

package_declaration
@init
{
  currArch = null;        
  currEnt = null;        
  currConfig = null;        
  currPack = null;        
  currPackBody = null;    
}
@after
{
  currArch = null;        
  currEnt = null;        
  currConfig = null;        
  currPack = null;        
  currPackBody = null;
  useclauses = null;
  libraries = null;      
}
  :
  PACKAGE nm=identifier IS
  {
    currArch = null;        
    currEnt = null;        
    currConfig = null;        
    currPack = null;        
    currPackBody = null;
    
    String packname = $nm.tok.getText();
    SortHDLUnit du = SortHDLRoot.instance().getDesignUnit(packname);
    HDLFile file = SortHDLRoot.instance().getHDLFile();
    if ( null == du )
    {
      du = new SortHDLUnit( SortHDLBase.ObjType.PACKAGE, packname , file, $nm.tok.getLine());
      SortHDLRoot.instance().addDesignUnit(du);
    }
    else
    {
      assert( SortHDLBase.ObjType.UNDEFINED_MODULE == du.getObjType() ) ;
      du.setObjType( SortHDLBase.ObjType.PACKAGE );
      du.setFile( file );
      du.setLine( $nm.tok.getLine() );
    }
    currPack = du;
    currPack.setLibraries(libraries);
    currPack.setUseClauses( useclauses );
  } 
  package_declarative_part 
  END (PACKAGE)? (nm2=identifier)? SEMI
  ;

package_declarative_item
  :
  subprogram_declaration
  | type_declaration
  | subtype_declaration
  | constant_declaration
  | signal_declaration
  | variable_declaration
  | file_declaration
  | alias_declaration
  | component_declaration
  | attribute_declaration
  | attribute_specification
  | disconnection_specification
  | use_clause
  | group_template_declaration
  | group_declaration
  ;

package_declarative_part
  :
  (package_declarative_item)*
  ;

parameter_specification
  :
  identifier IN discrete_range
  ;

physical_literal
  :
  (abstract_literal)? name
  ;

physical_type_definition
  :
  range_constraint UNITS base_unit_declaration (secondary_unit_declaration)* END UNITS (identifier)?
  ;

port_clause
  :
  PORT LPAREN port_list RPAREN SEMI
  ;

port_list
  :
  interface_list
  ;

port_map_aspect
  :
  PORT MAP LPAREN association_list RPAREN
  ;

primary
  :
  (function_call) => function_call
  | (name APOSTROPHE) => qualified_expression
  | (LPAREN expression RPAREN) => LPAREN expression RPAREN
  | literal
  | allocator
  | aggregate
  ;

primary_unit
  :
  entity_declaration
  | configuration_declaration
  | package_declaration
  ;

procedural_declarative_item
  :
  (subprogram_declaration) => subprogram_declaration
  | subprogram_body
  | type_declaration
  | subtype_declaration
  | constant_declaration
  | variable_declaration
  | alias_declaration
  | attribute_declaration
  | attribute_specification
  | use_clause
  | group_template_declaration
  | group_declaration
  ;

procedural_declarative_part
  :
  (procedural_declarative_item)*
  ;

procedural_statement_part
  :
  (sequential_statement)*
  ;

procedure_call
  :
  name (LPAREN actual_parameter_part RPAREN)?
  ;

procedure_call_statement
  :
  (label_colon)? procedure_call SEMI
  ;

process_declarative_item
  :
  (subprogram_declaration) => subprogram_declaration
  | subprogram_body
  | type_declaration
  | subtype_declaration
  | constant_declaration
  | variable_declaration
  | file_declaration
  | alias_declaration
  | attribute_declaration
  | attribute_specification
  | use_clause
  | group_template_declaration
  | group_declaration
  ;

process_declarative_part
  :
  (process_declarative_item)*
  ;

process_statement
  :
  (label_colon)? (POSTPONED)? PROCESS (LPAREN sensitivity_list RPAREN)? (IS)? 
  process_declarative_part 
  BEGIN { isInsideProcess = true;} 
  process_statement_part 
  END (POSTPONED)? PROCESS{ isInsideProcess = false;} (identifier)? SEMI
  ;

process_statement_part
  :
  (sequential_statement)*
  ;

qualified_expression
  :
  name APOSTROPHE
  (
    (aggregate) => aggregate
    | LPAREN expression RPAREN
  )
  ;


range
  :
  (simple_expression direction simple_expression) => simple_expression direction simple_expression
  | name
  ;

range_constraint
  :
  RANGETOK range
  ;

record_nature_definition
  :
  RECORD + END RECORD (identifier)?
  ;

record_type_definition
  :
  RECORD (element_declaration)+ END RECORD (identifier)?
  ;

relation
  :
  shift_expression (options {greedy=true;}: relational_operator shift_expression)?
  ;

relational_operator
  :
  EQ
  | NEQ
  | LOWERTHAN
  | LE
  | GREATERTHAN
  | GE
  ;

report_statement
  :
  (label_colon)? REPORT expression (SEVERITY expression)? SEMI
  ;

return_statement
  :
  (label_colon)? RETURN (expression)? SEMI
  ;


scalar_type_definition
  :
  (range_constraint UNITS) => physical_type_definition
  | enumeration_type_definition
  | range_constraint
  ;

secondary_unit
  :
  architecture_body
  | package_body
  ;

secondary_unit_declaration
  :
  identifier EQ physical_literal SEMI
  ;

selected_signal_assignment
  :
  WITH expression SELECT target LE opts selected_waveforms SEMI
  ;

selected_waveforms
  :
  waveform WHEN choices (COMMA waveform WHEN choices)*
  ;

sensitivity_clause
  :
  ON sensitivity_list
  ;

sensitivity_list
  :
  name (COMMA name)*
  ;

sequence_of_statements
  :
  (sequential_statement)*
  ;

sequential_statement
  :
  ( (label_colon)? WAIT) => wait_statement
  | ( (label_colon)? ASSERT) => assertion_statement
  | ( (label_colon)? REPORT) => report_statement
  | ( (label_colon)? target LE) => signal_assignment_statement
  | ( (label_colon)? target VARASGN) => variable_assignment_statement
  | ( (label_colon)? IF) => if_statement
  | ( (label_colon)? CASE) => case_statement
  |
  (
    (label_colon)? (iteration_scheme)? LOOP
  )
    => loop_statement
  | ( (label_colon)? NEXT) => next_statement
  | ( (label_colon)? EXIT) => exit_statement
  | ( (label_colon)? RETURN) => return_statement
  | ( (label_colon)? NULLTOK SEMI) => (label_colon)? NULLTOK SEMI
  | procedure_call_statement
  ;

shift_expression
  :
  simple_expression (options {greedy=true;}: shift_operator simple_expression)?
  ;

shift_operator
  :
  SLL
  | SRL
  | SLA
  | SRA
  | ROL
  | ROR
  ;

signal_assignment_statement
  :
  (label_colon)? target LE (delay_mechanism)? waveform SEMI
  ;

signal_declaration
  :
  SIGNAL identifier_list COLON subtype_indication (signal_kind)? (VARASGN expression)? SEMI
  ;

signal_kind
  :
  REGISTER
  | BUS
  ;

signal_list
  :
  name (COMMA name)*
  | OTHERS
  | ALL
  ;

signature
  :
  LBRACKET (name (COMMA name)*)? (RETURN name)? RBRACKET
  ;

// NOTE that sign is applied to first operand only (LRM does not permit
// `a op -b' - use `a op (-b)' instead).
// (3.2.2004, e.f.)

simple_expression
  :
  (
    PLUS
    | MINUS
  )?
  term (options {greedy=true;}: adding_operator term)*
  ;

simple_simultaneous_statement
  :
  (label_colon)? simple_expression ASSIGN simple_expression  SEMI
  ;

simultaneous_alternative
  :
  WHEN choices ARROW simultaneous_statement_part
  ;

simultaneous_case_statement
  :
  (label_colon)? CASE expression USE (simultaneous_alternative)+ END CASE (identifier)? SEMI
  ;

simultaneous_if_statement
  :
  (label_colon)? IF condition USE simultaneous_statement_part (ELSIF condition USE simultaneous_statement_part)* (ELSE simultaneous_statement_part)? END USE (identifier)? SEMI
  ;

simultaneous_statement
  :
  ( (label_colon)? simple_expression ASSIGN) => simple_simultaneous_statement
  | ( (label_colon)? IF condition USE) => simultaneous_if_statement
  | ( (label_colon)? CASE expression USE) => simultaneous_case_statement
  | (label_colon)? NULLTOK SEMI
  ;

simultaneous_statement_part
  :
  (simultaneous_statement)*
  ;

subprogram_body
  :
  subprogram_specification IS subprogram_declarative_part BEGIN subprogram_statement_part END (subprogram_kind)? (designator)? SEMI
  ;

subprogram_declaration
  :
  subprogram_specification SEMI
  ;

subprogram_declarative_item
  :
  (subprogram_declaration) => subprogram_declaration
  | subprogram_body
  | type_declaration
  | subtype_declaration
  | constant_declaration
  | variable_declaration
  | file_declaration
  | alias_declaration
  | attribute_declaration
  | attribute_specification
  | use_clause
  | group_template_declaration
  | group_declaration
  ;

subprogram_declarative_part
  :
  (subprogram_declarative_item)*
  ;

subprogram_kind
  :
  PROCEDURE
  | FUNCTION
  ;

subprogram_specification
  :
  PROCEDURE designator (LPAREN formal_parameter_list RPAREN)?
  |
  (
    PURE
    | IMPURE
  )?
  FUNCTION designator (LPAREN formal_parameter_list RPAREN)? RETURN name
  ;

subprogram_statement_part
  :
  (sequential_statement)*
  ;

subtype_declaration
  :
  SUBTYPE identifier IS subtype_indication SEMI
  ;

// VHDLAMS 1076.1-1999 declares first name as optional. Here, second name
// is made optional to prevent antlr nondeterminism.
// (9.2.2004, e.f.)

subtype_indication
  :
  name (name)? (constraint)?
  ;

suffix returns [Token tok;]
  :
  identifier { $tok = $identifier.tok ;}
  | CHARACTER_LITERAL
  | STRING_LITERAL
  | ALL
  ;

target
  :
  name
  | aggregate
  ;

term
  :
  factor (options {greedy=true;}: multiplying_operator factor)*
  ;

terminal_aspect
  :
  name (TO name)?
  ;


timeout_clause
  :
  FOR expression
  ;


type_declaration
  :
  TYPE identifier (IS type_definition)? SEMI
  ;

type_definition
  :
  scalar_type_definition
  | composite_type_definition
  | access_type_definition
  | file_type_definition
  ;

unconstrained_array_definition
  :
  ARRAY LPAREN index_subtype_definition (COMMA index_subtype_definition)* RPAREN OF subtype_indication
  ;

// NOTE that name is used here, so need to check if result of name is
// a seleceted_name (i.e. prefix DOT suffix).
// (30.3.2004, e.f.)
//use_clause
//  : USE name ( COMMA name )* SEMI
//  ;

use_clause
  :
  USE  nm=name 
  {
    if ( null != $nm.tok && null != $nm.lib )
    {
      String duname = $nm.tok.getText();
      String libname = $nm.lib.getText();
      
      if ( ( false == libname.toLowerCase().equals( "ieee") ) && 
           ( false == libname.toLowerCase().equals( "synopsys") ) && 
           ( false == libname.toLowerCase().equals( "std")  ) )
      {
        addUseclause( libname + "::" + duname );
      }
    } 
  }
  (COMMA nm2=name 
  {
    if ( null != $nm2.tok && null != $nm2.lib )
    {
      String duname = $nm2.tok.getText();
      String libname = $nm2.lib.getText();
      if ( ( false == libname.toLowerCase().equals( "ieee") ) && 
           ( false == libname.toLowerCase().equals( "synopsys") ) && 
           ( false == libname.toLowerCase().equals( "std")  ) )
      {
        addUseclause( libname + "::" + duname );
      }
    }   
  }
  )* SEMI 
  ;

variable_assignment_statement
  :
  (label_colon)? target VARASGN expression SEMI
  ;

variable_declaration
  :
  (SHARED)? VARIABLE identifier_list COLON subtype_indication (VARASGN expression)? SEMI
  ;

wait_statement
  :
  (label_colon)? WAIT (sensitivity_clause)? (condition_clause)? (timeout_clause)? SEMI
  ;

waveform
  :
  waveform_element (COMMA waveform_element)*
  | UNAFFECTED
  ;

waveform_element
  :
  expression (AFTER expression)?
  ;

ABS
  :
  'abs' {if (isInsideProcess) $channel=HIDDEN;}
  ;

ACCESS
  :
  'access' {if (isInsideProcess) $channel=HIDDEN;}
  ;

AFTER
  :
  'after' {if (isInsideProcess) $channel=HIDDEN;}
  ;

ALIAS
  :
  'alias' {if (isInsideProcess) $channel=HIDDEN;}
  ;

ALL
  :
  'all' {if (isInsideProcess) $channel=HIDDEN;}
  ;

AND
  :
  'and' {if (isInsideProcess) $channel=HIDDEN;}
  ;

ARCHITECTURE
  :
  'architecture' {if (isInsideProcess) $channel=HIDDEN;}
  ;

ARRAY
  :
  'array' {if (isInsideProcess) $channel=HIDDEN;}
  ;

ASSERT
  :
  'assert' {if (isInsideProcess) $channel=HIDDEN;}
  ;

ATTRIBUTE
  :
  'attribute' {if (isInsideProcess) $channel=HIDDEN;}
  ;

BEGIN
  :
  'begin' {if (isInsideProcess) $channel=HIDDEN;}
  ;

BLOCK
  :
  'block' {if (isInsideProcess) $channel=HIDDEN;}
  ;

BODY
  :
  'body' {if (isInsideProcess) $channel=HIDDEN;}
  ;

BUFFER
  :
  'buffer' {if (isInsideProcess) $channel=HIDDEN;}
  ;

BUS
  :
  'bus' {if (isInsideProcess) $channel=HIDDEN;}
  ;

CASE
  :
  'case' {if (isInsideProcess) $channel=HIDDEN;}
  ;

COMPONENT
  :
  'component' {if (isInsideProcess) $channel=HIDDEN;}
  ;

CONFIGURATION
  :
  'configuration' {if (isInsideProcess) $channel=HIDDEN;}
  ;

CONSTANT
  :
  'constant' {if (isInsideProcess) $channel=HIDDEN;}
  ;

DISCONNECT
  :
  'disconnect' {if (isInsideProcess) $channel=HIDDEN;}
  ;

DOWNTO
  :
  'downto' {if (isInsideProcess) $channel=HIDDEN;}
  ;

ELSE
  :
  'else' {if (isInsideProcess) $channel=HIDDEN;}
  ;

ELSIF
  :
  'elsif' {if (isInsideProcess) $channel=HIDDEN;}
  ;

END
  :
  'end' {if (isInsideProcess) $channel=HIDDEN;}
  ;

ENTITY
  :
  'entity' {if (isInsideProcess) $channel=HIDDEN;}
  ;

EXIT
  :
  'exit' {if (isInsideProcess) $channel=HIDDEN;}
  ;

FILE
  :
  'file' {if (isInsideProcess) $channel=HIDDEN;}
  ;

FOR
  :
  'for' {if (isInsideProcess) $channel=HIDDEN;}
  ;

FUNCTION
  :
  'function' {if (isInsideProcess) $channel=HIDDEN;}
  ;

GENERATE
  :
  'generate' {if (isInsideProcess) $channel=HIDDEN;}
  ;

GENERIC
  :
  'generic' {if (isInsideProcess) $channel=HIDDEN;}
  ;

GROUP
  :
  'group' {if (isInsideProcess) $channel=HIDDEN;}
  ;

GUARDED
  :
  'guarded' {if (isInsideProcess) $channel=HIDDEN;}
  ;

IF
  :
  'if' {if (isInsideProcess) $channel=HIDDEN;}
  ;

IMPURE
  :
  'impure' {if (isInsideProcess) $channel=HIDDEN;}
  ;

IN
  :
  'in' {if (isInsideProcess) $channel=HIDDEN;}
  ;

INERTIAL
  :
  'inertial' {if (isInsideProcess) $channel=HIDDEN;}
  ;

INOUT
  :
  'inout' {if (isInsideProcess) $channel=HIDDEN;}
  ;

IS
  :
  'is' {if (isInsideProcess) $channel=HIDDEN;}
  ;

LABEL
  :
  'label' {if (isInsideProcess) $channel=HIDDEN;}
  ;

LIBRARY
  :
  'library' {if (isInsideProcess) $channel=HIDDEN;}
  ;

LINKAGE
  :
  'linkage' {if (isInsideProcess) $channel=HIDDEN;}
  ;

LITERAL
  :
  'literal' {if (isInsideProcess) $channel=HIDDEN;}
  ;

LOOP
  :
  'loop' {if (isInsideProcess) $channel=HIDDEN;}
  ;

MAP
  :
  'map' {if (isInsideProcess) $channel=HIDDEN;}
  ;

MOD
  :
  'mod' {if (isInsideProcess) $channel=HIDDEN;}
  ;

NAND
  :
  'nand' {if (isInsideProcess) $channel=HIDDEN;}
  ;

NEW
  :
  'new' {if (isInsideProcess) $channel=HIDDEN;}
  ;

NEXT
  :
  'next' {if (isInsideProcess) $channel=HIDDEN;}
  ;

NOR
  :
  'nor' {if (isInsideProcess) $channel=HIDDEN;}
  ;

NOT
  :
  'not' {if (isInsideProcess) $channel=HIDDEN;}
  ;

NULLTOK
  :
  'null' {if (isInsideProcess) $channel=HIDDEN;}
  ;
//  NULL : 'null';

OF
  :
  'of' {if (isInsideProcess) $channel=HIDDEN;}
  ;

ON
  :
  'on' {if (isInsideProcess) $channel=HIDDEN;}
  ;

OPEN
  :
  'open' {if (isInsideProcess) $channel=HIDDEN;}
  ;

OR
  :
  'or' {if (isInsideProcess) $channel=HIDDEN;}
  ;

OTHERS
  :
  'others' {if (isInsideProcess) $channel=HIDDEN;}
  ;

OUT
  :
  'out' {if (isInsideProcess) $channel=HIDDEN;}
  ;

PACKAGE
  :
  'package' {if (isInsideProcess) $channel=HIDDEN;}
  ;

PORT
  :
  'port' {if (isInsideProcess) $channel=HIDDEN;}
  ;

POSTPONED
  :
  'postponed' {if (isInsideProcess) $channel=HIDDEN;}
  ;

PROCEDURE
  :
  'procedure' {if (isInsideProcess) $channel=HIDDEN;}
  ;

PROCESS
  :
  'process' 
  {
    isInsideProcess = !isInsideProcess; 
    if (false == isInsideProcess)
    { 
      $channel=HIDDEN; 
    }
  }
  ;

PURE
  :
  'pure' {if (isInsideProcess) $channel=HIDDEN;}
  ;

RANGETOK
  :
  'range' {if (isInsideProcess) $channel=HIDDEN;}
  ;

RECORD
  :
  'record' {if (isInsideProcess) $channel=HIDDEN;}
  ;

REGISTER
  :
  'register' {if (isInsideProcess) $channel=HIDDEN;}
  ;

REJECT
  :
  'reject' {if (isInsideProcess) $channel=HIDDEN;}
  ;

REM
  :
  'rem' {if (isInsideProcess) $channel=HIDDEN;}
  ;

REPORT
  :
  'report' {if (isInsideProcess) $channel=HIDDEN;}
  ;

RETURN
  :
  'return' {if (isInsideProcess) $channel=HIDDEN;}
  ;

ROL
  :
  'rol' {if (isInsideProcess) $channel=HIDDEN;}
  ;

ROR
  :
  'ror' {if (isInsideProcess) $channel=HIDDEN;}
  ;

SELECT
  :
  'select' {if (isInsideProcess) $channel=HIDDEN;}
  ;

SEVERITY
  :
  'severity' {if (isInsideProcess) $channel=HIDDEN;}
  ;

SIGNAL
  :
  'signal' {if (isInsideProcess) $channel=HIDDEN;}
  ;

SHARED
  :
  'shared' {if (isInsideProcess) $channel=HIDDEN;}
  ;

SLA
  :
  'sla' {if (isInsideProcess) $channel=HIDDEN;}
  ;

SLL
  :
  'sll' {if (isInsideProcess) $channel=HIDDEN;}
  ;

SRA
  :
  'sra' {if (isInsideProcess) $channel=HIDDEN;}
  ;

SRL
  :
  'srl' {if (isInsideProcess) $channel=HIDDEN;}
  ;

SUBTYPE
  :
  'subtype' {if (isInsideProcess) $channel=HIDDEN;}
  ;

THEN
  :
  'then' {if (isInsideProcess) $channel=HIDDEN;}
  ;

TO
  :
  'to' {if (isInsideProcess) $channel=HIDDEN;}
  ;

TRANSPORT
  :
  'transport' {if (isInsideProcess) $channel=HIDDEN;}
  ;

TYPE
  :
  'type' {if (isInsideProcess) $channel=HIDDEN;}
  ;

UNAFFECTED
  :
  'unaffected' {if (isInsideProcess) $channel=HIDDEN;}
  ;

UNITS
  :
  'units' {if (isInsideProcess) $channel=HIDDEN;}
  ;

UNTIL
  :
  'until' {if (isInsideProcess) $channel=HIDDEN;}
  ;

USE
  :
  'use' {if (isInsideProcess) $channel=HIDDEN;}
  ;

VARIABLE
  :
  'variable' {if (isInsideProcess) $channel=HIDDEN;}
  ;

WAIT
  :
  'wait' {if (isInsideProcess) $channel=HIDDEN;}
  ;

WHEN
  :
  'when' {if (isInsideProcess) $channel=HIDDEN;}
  ;

WHILE
  :
  'while' {if (isInsideProcess) $channel=HIDDEN;}
  ;

WITH
  :
  'with' {if (isInsideProcess) $channel=HIDDEN;}
  ;

XNOR
  :
  'xnor' {if (isInsideProcess) $channel=HIDDEN;}
  ;

XOR
  :
  'xor' {if (isInsideProcess) $channel=HIDDEN;}
  ;

TAB
  :
  ('\t')+ { $channel = HIDDEN;}
  ;

SPACE
  :
  (' ')+ { $channel = HIDDEN;}
  ;

NEWLINE
  :
  '\n' { $channel = HIDDEN;}
  ;

COMMENT
  :
  '--' (~'\n')* { $channel = HIDDEN;}
  ;

DOUBLESTAR
  :
  '**' {if (isInsideProcess) $channel=HIDDEN;}
  ;

ASSIGN
  :
  '==' {if (isInsideProcess) $channel=HIDDEN;}
  ;

LE
  :
  '<=' {if (isInsideProcess) $channel=HIDDEN;}
  ;

GE
  :
  '>=' {if (isInsideProcess) $channel=HIDDEN;}
  ;

ARROW
  :
  '=>' {if (isInsideProcess) $channel=HIDDEN;}
  ;

NEQ
  :
  '/=' {if (isInsideProcess) $channel=HIDDEN;}
  ;

VARASGN
  :
  ':=' {if (isInsideProcess) $channel=HIDDEN;}
  ;

BOX
  :
  '<>' {if (isInsideProcess) $channel=HIDDEN;}
  ;

DBLQUOTE
  :
  '\"' {if (isInsideProcess) $channel=HIDDEN;}
  ;

SEMI
  :
  ';' {if (isInsideProcess) $channel=HIDDEN;}
  ;

COMMA
  :
  ',' {if (isInsideProcess) $channel=HIDDEN;}
  ;

AMPERSAND
  :
  '&' {if (isInsideProcess) $channel=HIDDEN;}
  ;

APOSTROPHE
  :
  '\''
  (
    (
      (
        'a'..'z'
        | '0'..'9'
        | '&'
        | '\''
        | '('
        | ')'
        | '+'
        | ','
        | '-'
        | '.'
        | '/'
        | ':'
        | ';'
        | '<'
        | '='
        | '>'
        | '|'
        | ' '
        | OTHER_SPECIAL_CHARACTER
        | '\\'
        | '#'
        | '['
        | ']'
        | '_'
        | '*'
        | '"'
      )
      '\''
    )
      =>
    (
      'a'..'z'
      | '0'..'9'
      | '&'
      | '\''
      | '('
      | ')'
      | '+'
      | ','
      | '-'
      | '.'
      | '/'
      | ':'
      | ';'
      | '<'
      | '='
      | '>'
      | '|'
      | ' '
      | OTHER_SPECIAL_CHARACTER
      | '\\'
      | '#'
      | '['
      | ']'
      | '_'
      | '*'
      | '"'
    )
    '\'' { $type = CHARACTER_LITERAL;}
  )? {if (isInsideProcess) $channel=HIDDEN;}
  ;



LPAREN
  :
  '(' {if (isInsideProcess) $channel=HIDDEN;}
  ;

RPAREN
  :
  ')' {if (isInsideProcess) $channel=HIDDEN;}
  ;

LBRACKET
  :
  '[' {if (isInsideProcess) $channel=HIDDEN;}
  ;

RBRACKET
  :
  ']' {if (isInsideProcess) $channel=HIDDEN;}
  ;

COLON
  :
  ':' {if (isInsideProcess) $channel=HIDDEN;}
  ;

MUL
  :
  '*' {if (isInsideProcess) $channel=HIDDEN;}
  ;

DIV
  :
  '/' {if (isInsideProcess) $channel=HIDDEN;}
  ;

PLUS
  :
  '+' {if (isInsideProcess) $channel=HIDDEN;}
  ;

MINUS
  :
  '-' {if (isInsideProcess) $channel=HIDDEN;}
  ;

LOWERTHAN
  :
  '<' {if (isInsideProcess) $channel=HIDDEN;}
  ;

GREATERTHAN
  :
  '>' {if (isInsideProcess) $channel=HIDDEN;}
  ;

EQ
  :
  '=' {if (isInsideProcess) $channel=HIDDEN;}
  ;

BAR
  :
  '|' {if (isInsideProcess) $channel=HIDDEN;}
  ;

DOT
  :
  '.' {if (isInsideProcess) $channel=HIDDEN;}
  ;

BACKSLASH
  :
  '\\' {if (isInsideProcess) $channel=HIDDEN;}
  ;

BASIC_IDENTIFIER
  :
  'a'..'z'
  (
    'a'..'z'
    | '0'..'9'
    | '_'
  )* {if (isInsideProcess) $channel=HIDDEN;}
  ;

EXTENDED_IDENTIFIER
  :
  '\\'
  (
    'a'..'z'
    | '0'..'9'
    | '&'
    | '\''
    | '('
    | ')'
    | '+'
    | ','
    | '-'
    | '.'
    | '/'
    | ':'
    | ';'
    | '<'
    | '='
    | '>'
    | '|'
    | ' '
    | OTHER_SPECIAL_CHARACTER
    | '\\'
    | '#'
    | '['
    | ']'
    | '_'
  )+
  '\\' {if (isInsideProcess) $channel=HIDDEN;}
  ;

BIT_STRING_LITERAL
  :
  (
    'b'
    | 'o'
    | 'x'
  )
  '\"' BASED_INTEGER '\"'
  ;

//STRING_LITERAL
//  : '\"' ( 'a'..'z' | '0'..'9' | '&' | '\'' | '(' | ')'
//    | '+' | ',' | '-' | '.' | '/' | ':' | ';' | '<' | '=' | '>' | '|'
//    | ' ' | OTHER_SPECIAL_CHARACTER | '\\' | "\"\"" | '*'
//    | '#' | '[' | ']' | '_' )* '\"'
//  ;

STRING_LITERAL
  :
  '"' ( ~('\"' ))* '"'
  ;

// NOTE that BASED_LITERAL is implicitely defined by referencing in the
// parser grammar.
// (2.2.2004, e.f.)

DECIMAL_LITERAL
  :
  INTEGER
  (
    (
      ('.' INTEGER)? (EXPONENT)?
    )
    |
    (
      '#' BASED_INTEGER ('.' BASED_INTEGER)? '#' (EXPONENT)?
    )
    { $type = BASED_LITERAL;}
  ) {if (isInsideProcess) $channel=HIDDEN;}
  ;

CR
  :
  '\r' { $channel = HIDDEN;}
  ;

fragment
OTHER_SPECIAL_CHARACTER
  :
  '!'
  | '$'
  | '%'
  | '@'
  | '?'
  | '^'
  | '`'
  | '{'
  | '}'
  | '~'
  | '' 
  ;

//fragment PREFEF_ATTIB:  //Added by KLG
//  'event'
//  |'range'
//  |'stable'
//  |'left'
//  |'right'
//  | 'reverse_range'
//  ;

fragment
INTEGER
  :
  '0'..'9'
  (
    '_'
    | '0'..'9'
  )*
  ;

fragment
EXPONENT
  :
  'e'
  (
    '+'
    | '-'
  )?
  INTEGER
  ;

fragment
BASED_INTEGER
  :
  (
    '0'..'9'
    | 'a'..'f'
  )
  (
    '_'
    | '0'..'9'
    | 'a'..'f'
  )*
  ;
