// This grammar file has been copied from the below area
// http://svparser.googlecode.com/svn-history/r2/trunk/trivial1.g
// https://github.com/BIOFAB/ClothoBiofabEdition/blob/master/ClothoDevelopment/Cello/ANTLRVerilog/VerilogA.g
// 
// The above grammar has been modified to meet my requirement.
// Kanai L Ghosh
//
grammar VerilogPP;


options 
{
    language = Java;
    k=2;
}

@header 
{
    package com.eu.miscedautils.verilogparser;
    import com.eu.miscedautils.CommonUtils.LogMessages;
    import com.eu.miscedautils.verilogparser.FVPCompilerDirectiveCond.Directive;
    import com.eu.miscedautils.verilogparser.*;     
    import java.util.Iterator;
    import java.util.List;
    import java.util.ArrayList;
    import java.util.LinkedList;

    import org.antlr.runtime.ANTLRFileStream;
    import org.antlr.runtime.ANTLRStringStream;
    import org.antlr.runtime.CharStream;
    import org.antlr.runtime.CommonTokenStream;
    import org.antlr.runtime.RecognitionException;
    import org.antlr.runtime.TokenSource;
    import org.antlr.runtime.TokenStream;
    import org.antlr.runtime.CommonTokenStream;
    import org.antlr.runtime.RecognitionException;
    import org.antlr.runtime.TokenRewriteStream;
    import org.antlr.runtime.TokenStream;
    import java.io.File;
    import java.io.FileInputStream;
    import java.io.File;
    import java.io.FileNotFoundException;
    import java.io.FileOutputStream;
    import java.io.IOException;
    import java.io.PrintStream;
    import java.io.IOException;
}

@members
{
    String fragStr = null; // This varaible has been defined in order to support testcase like the verilogparser/ifdef_macro_3
    String macrovalueStr = "";
    boolean isInsideDefine = false;
    String macroID = null;
    boolean debugMode = false;
    LinkedList<String> incpaths = null;
    boolean isTickBeginKeyword = false;
    public void setDebugMode ( boolean val )
    {
        this.debugMode = val;
    }
    
    public boolean isDebugMode ( )
    {
        return this.debugMode;
    }
        
    private boolean skipInputs()
    {
        return false == FVPDescription.instance().readInputs();
    }
    
    @Override
    public void displayRecognitionError(String[] tokenNames,
                                        RecognitionException e) 
    {       
        String hdr = getErrorHeader(e);
        String msg = getErrorMessage(e, tokenNames);
        String errorMsg = FVPDescription.instance().getFile()  + ": " + hdr + " : " + msg ;
                       
        if ( isDebugMode() && ( e instanceof NoViableAltException ) )
        {                                 
            List stack = getRuleInvocationStack(e, this.getClass().getName());
            NoViableAltException nvae = (NoViableAltException)e;
            errorMsg =  stack + " " + FVPDescription.instance().getFile() + ": no viable alt; token=" + e.token +
                      " (decision=" + nvae.decisionNumber +
                      " state " + nvae.stateNumber + ")" +
                      " decision=<<" + nvae.grammarDecisionDescription + ">>";
        }
        
        LogMessages.instance().print(1811, LogMessages.MSG_SEVERITY.ERROR, errorMsg );
    }
}

@init 
{
    LinkedList<String> incpaths = null;
}
       
@lexer::header 
{
    package com.eu.miscedautils.verilogparser;
    import com.eu.miscedautils.verilogparser.FVPCompilerDirectiveCond.Directive;
    import com.eu.miscedautils.CommonUtils.LogMessages;
    import com.eu.miscedautils.verilogparser.*;   
    import org.antlr.runtime.*;
    import org.antlr.runtime.CommonTokenStream;
    import org.antlr.runtime.RecognitionException;
    import org.antlr.runtime.TokenRewriteStream;
    import org.antlr.runtime.TokenStream;
    import java.io.FileNotFoundException;
    import java.io.FileOutputStream;
    import java.io.IOException;
    import java.io.PrintStream;
    
    import java.util.Iterator;
    import java.util.List;
    import java.util.ArrayList;
    import java.util.LinkedList;
    import java.io.File;
    import java.io.FileInputStream;
    import java.io.File;
    import java.io.FileNotFoundException;
    import java.io.FileOutputStream;
    import java.io.IOException;
    import java.io.PrintStream;
    import java.io.IOException;
}

@lexer::members
{
    String fragStr = null; // This varaible has been defined in order to support testcase like the verilogparser/ifdef_macro_3
    private LinkedList<String> incpaths = null;         
    Stack<myBool> directiveConds = new Stack<myBool>();
    private boolean debugMode = false;
    String macrovalueStr = "";
    boolean isDollar = false;
    boolean isInsideDefine = false;
    boolean isTickIfDef = false;
    boolean isTickIfNDef = false;
    boolean isTickElsIf = false;
    boolean isTickElse = false;
    boolean isTickEndif = false;
    boolean isTickInclude = false; 
    boolean isTick = false;
    boolean isTickUndef = false;
    boolean isTickBeginKeyword = false;

    @Override
    public void displayRecognitionError(String[] tokenNames,
                                        RecognitionException e) 
    {       
        String hdr = getErrorHeader(e);
        String msg = getErrorMessage(e, tokenNames);
        String errorMsg = FVPDescription.instance().getFile()  + ": " + hdr + " : " + msg ;
                       
        if ( isDebugMode() && ( e instanceof NoViableAltException ) )
        {                                 
            List stack = getRuleInvocationStack(e, this.getClass().getName());
            NoViableAltException nvae = (NoViableAltException)e;
            errorMsg =  stack + " " + FVPDescription.instance().getFile() + ": no viable alt; token=" + e.token +
                      " (decision=" + nvae.decisionNumber +
                      " state " + nvae.stateNumber + ")" +
                      " decision=<<" + nvae.grammarDecisionDescription + ">>";
        }
        
        LogMessages.instance().print(1811, LogMessages.MSG_SEVERITY.ERROR, errorMsg );
    }
    
    public class myBool
    {
        public boolean value;
        public myBool( boolean v)
        {
            this.value = v;
        }
    };
    
    public String truncateLeadingBlanks(String str)
    {
        if ( null == str )
        {
            return null;
        }
        String retval = new String("");
        Integer len = str.length();
        Integer index = 0;
        boolean isLeadeingSpece = true;
        while( index < len )
        {
            char c = str.charAt(index);
            index++;
            if ( ( ' ' == c || '\t' == c ) && ( true == isLeadeingSpece ) )
            {
                continue;
            }
            isLeadeingSpece = false;
            retval += c;
        }
        return retval;
    }
    
    public void setIncludeDirs( LinkedList<String> dirs )
    {
        this.incpaths = dirs;
    }
        
    public LinkedList<String> getIncludeDirs( )
    {
        return this.incpaths;
    }
    
    public void setDebugMode ( boolean val )
    {
        this.debugMode = val;
    }
    
    public boolean isDebugMode( )
    {
        return this.debugMode;
    }
    
    // This function is required in order to remove all the compiler
    // directives during the second pass.
    // During the first pass it cleans all the internal directives nested
    // compiler directives. We clean all the directives during the second pass
    //
    public boolean isRemoveDirectives( )
    {
        return ( FVPDescription.instance().getPreprocessorLevel() == 2 );
    }
    
    void pushSLastBlockCond()
    {
        boolean cond = FVPDescription.instance().readInputs();
        myBool val = new myBool(cond);
        directiveConds.push( val );
    }
    
    void popSLastBlockCond()
    {
        myBool popval = directiveConds.pop();
    }
    
    // Statring value of the directive cond, its true at the starting point
    // 
    boolean prevCond()
    {
        boolean retval = true;
        if ( false == directiveConds.isEmpty() )
        {
            retval = directiveConds.lastElement().value;
        }
        return retval;
    }
    
     
    String macroID = null;
    private boolean skipInputs()
    {
        return false == FVPDescription.instance().readInputs();
    }
    
    private int getChannel()
    {
        return skipInputs() ? HIDDEN : DEFAULT_TOKEN_CHANNEL; 
    }
    
    private int getActiveChannel()
    {
        return ( isInsideDefine | isTickIfDef | isTickIfNDef | isTickElsIf | isTickUndef | isTickInclude | isDollar ) ? 
                DEFAULT_TOKEN_CHANNEL : HIDDEN ;
    }
    
    private void addMacroString(String str)
    {
        String replacedStr = str;
        if ( isInsideDefine )
        {
            macrovalueStr += str ;
            replacedStr = "";
        }
        else if ( skipInputs() )
        {
            replacedStr = "";
        }
        setText( replacedStr );
    }
    
    void proccessIncludeFile( String fName, LinkedList<String> incpaths ) throws RecognitionException
    {
        CharStream istream = null;
//        fName = fName.substring(1,fName.length()-1);
        String workingdir = System.getProperty("user.dir");
        FVPDescription desc = FVPDescription.instance();
        // First check if this file exists in the run directory
        String fullPathFile = workingdir + java.io.File.separatorChar + fName;
        // Found inconsistancies with the File.exists() and other APIs.
        // So trying workarounds
        String actualFile = null;
        try
        {
            istream = new ANTLRFileStream(fullPathFile);
        }
        catch (IOException e) 
        {
            ; // No action ...
        }
        if ( null != incpaths )
        {
            Iterator<String> itr = incpaths.iterator();
            while( null == istream && itr.hasNext() )
            {
                String path = (String)itr.next();
                if ( null == path || path.isEmpty() || path.length() < 1)
                {
                    continue;
                }
                if ( '\\' != java.io.File.separatorChar )
                {
	                if ( path.charAt(0)  !=  java.io.File.separatorChar  )
	                {
	                    path = workingdir + java.io.File.separatorChar + path;
	                }            
	                fullPathFile = path + java.io.File.separatorChar + fName ;
                }
                else
                {
                    try 
                    {
                        File f = new File( path );
                        fullPathFile = f.getCanonicalPath() + java.io.File.separatorChar + fName ;
                    }
                    catch (IOException e) 
                    {
                        // TODO Auto-generated catch block
                        e.printStackTrace();
                    }
                
                }
                try 
                {
                    istream = new ANTLRFileStream(fullPathFile);
                    actualFile = fullPathFile;
                }
                catch (IOException e) 
                {
                    ; // No action ...
                }        
            }
        }
        else // This file is expected in the run directory, +indir+ is not given
        {
            try 
            {
                istream = new ANTLRFileStream(fName);
                actualFile = fName;
            } 
            catch (IOException e) 
            {
                ; // No action ...
            }              
        }      
        if ( null == istream )
        {
            LogMessages.instance().print(1812, LogMessages.MSG_SEVERITY.ERROR, "Problem to open include file " + 
                                fName + " in read mode ... ");
            return;
        }                 
        String prevFile = FVPDescription.instance().getFile();
        FVPDescription.instance().setFile(fName);
                
        VerilogPPLexer pplexer = new VerilogPPLexer( istream);
        pplexer.setIncludeDirs( incpaths );
        TokenRewriteStream ppTokenStream = new TokenRewriteStream(pplexer);
        VerilogPPParser pparser = new VerilogPPParser(ppTokenStream);
        String tmpoutfile = null;
        // Create temporary file.
        File tempfile = null;
        if ( isDebugMode() )
        {
            tmpoutfile = "tmp_" + fName;
        }
        else
        {
            try 
            {
                
                String currDir = System.getProperty("user.dir");
                tempfile = File.createTempFile("edautils_temp_", ".v", new File(currDir));
                tempfile.deleteOnExit();
            } 
            catch (IOException e2) 
            {
                // TODO Auto-generated catch block
                e2.printStackTrace();
                return;
            }
            // Delete temp file when program exits.
            tempfile.deleteOnExit();
            // Write to temp fil
            tmpoutfile = tempfile.toString();
        }
        FVPDescription.instance().getRoot().addPreprocessedIncludeFile( fName, tmpoutfile );
        FVPBase.ObjType type = FVPDescription.instance().getScope().getObjType();
        switch (type) {
        case MODULE:
            pparser.module_items();
            break;
        case TASK_OR_FUNCTION_DECL:
            pparser.all_statement_or_nulls();
            break;
        case SEQ_STATEMENT:
        {    
            FVPSeqStmtBase scope = (FVPSeqStmtBase)FVPDescription.instance().getScope();         
            switch ( scope.getStatementType() )
            {
            case SEQ_IF_STMT:            
                pparser.conditional_statement();
                break;        
            case SEQ_CASE_ITEM:
                pparser.case_statement();
                break;
            case SEQ_LOOP_STMT:
                pparser.loop_statement();
                break;
            case ALWAYS_BLOCK:
                pparser.always_statement();
                break;
            case INITIAL_BLOCK:
                pparser.initial_statement();
                break;
            default:
            }
            break;
        }    
        
        case ROOT:
            pparser.preprocessRTL(incpaths);
            break;
        case UNKNOWN:
            LogMessages.instance().print(1813, LogMessages.MSG_SEVERITY.PROGRAMMING_ERROR,
            "Trying to read include file from wrong scope ...");
            break;
        }
        
        FileOutputStream fout = null;
        try 
        {
            fout = new FileOutputStream(tmpoutfile);
        } 
        catch (FileNotFoundException e1) 
        {
            // TODO Auto-generated catch block
            e1.printStackTrace();
        }
        PrintStream p = new PrintStream(fout);
        p.println(ppTokenStream.toString());
        p.close();
        LogMessages.instance().print(1814, LogMessages.MSG_SEVERITY.NOTE,
            "Finished preprocessing of the included file '" + fName + "' ...");
        FVPDescription.instance().setFile(prevFile);
        return;
    }
}
  

preprocessRTL [ LinkedList<String> incpaths ] 
    @init 
    {
        LogMessages.instance().print(1815, LogMessages.MSG_SEVERITY.NOTE, 
                                      "Preprocessing input file '" +
                                      FVPDescription.instance().getFile() + "' ...");        
        this.incpaths = incpaths;
    }
    :
    ( description )*
    EOF
    ;


description: 
    module 
    |
    udp
    |
    directive
    ;

module :
    ( 
        KW_MODULE 
        | 
        KW_MACROMODULE
    ) 
    name_of_module 
    ( list_of_ports )? SEMI
    ( module_item )*
    KW_ENDMODULE 
    ;

list_of_ports :
    LPAREN port ( COMMA port )* RPAREN
    ;

port:
    ( port_expression )? 
    |
    DOT name_of_port LPAREN (port_expression)? RPAREN
    ;

port_expression: 
    port_reference 
    |
    LCURLY p1=port_reference ( COMMA p2=port_reference )* RCURLY
    ;

// TBD: Only simple way of port declaration is considered
//      Need to include the other ways of port decls

port_reference :
    ( name_of_variable LBRACK expression COLON ) =>
            name_of_variable LBRACK expression COLON expression RBRACK 
    |
    ( name_of_variable LBRACK ) =>
            name_of_variable LBRACK expression RBRACK 
    |
    name_of_variable 
    ;

module_items: 
    (module_item)* EOF
    ;
    
module_item :
    // ambiguity between net_declaration and continuous_assign,
    // but parser gets it right: keyword chosen over IDENTIFIER.
    p1=parameter_declaration
    |
    input_declaration
    |
    output_declaration
    |
    inout_declaration
    |
    net_declaration 
    | 
    reg_declaration
    | 
    time_declaration 
    | 
    integer_declaration 
    |
    real_declaration 
    |
    event_declaration 
    |
    gate_declaration 
    |
    instantiation  
    |
    parameter_override 
    |
    continuous_assign 
    | 
    specify_block 
    |
    initial_statement 
    |
    always_statement 
    |
    task 
    |
    function 
    |
    directive
    ;

instantiation:
    (module_instantiation) => module_instantiation
    |
    udp_instantiation
    ;

//----------------------------------------------------------------------------
// UDP specs
//----------------------------------------------------------------------------

udp 
    :
    KW_PRIMITIVE  name_of_UDP
    LPAREN  name_of_variable ( COMMA name_of_variable )* RPAREN SEMI
    (udp_declaration)+
    (udp_initial_statement)?
    table_definition
    KW_ENDPRIMITIVE  
    ;
 
udp_port_list :
    udp_name_of_port ( COMMA udp_name_of_port )*
    ;


udp_declaration :
    input_declaration
    |
    output_declaration
    |
    reg_declaration
    ;

udp_initial_statement :
// Baya        KW_INITIAL output_terminal_name ASSIGN init_val SEMI
    KW_INITIAL output_terminal_name ASSIGN NUMBER SEMI
//    {
//    output_terminal_name returns [String id, Integer line]
//    }
    
    ;

   // Use a semantic predicate to determine whether a matched NUMBER
   // is a valid special value in the given context.
   // This kludge avoids having the special values in the Literals table,
   // thus avoiding a lexical conflict.
   
// Baya : Below is commented by Baya otherwise we get error on
// continuous assignment like assign lhs = 1'b1 or assign lhs = 1'b0

//init_val :
//        '1\'b0' |
//        '1\'b1' |
//        '1\'bx' |
//    n=NUMBER
//    { $n.text=="0" || $n.text=="1"}?
//    ;

table_definition :
    TABLE  table_entries ENDTABLE 
    ;

   // Don't try to parse table entries; just collect them.
   // There are ambiguities between edge_symbol and level_symbol,
   // and textbook Verilog examples don't seem to follow rules
   // completely. For example,
   //    '0    00    :    0;'
   // doesn't match grammar because of '00', but is frequently used.

table_entries :
    //(sequential_entry) => (sequential_entry)+ |
    //(combinational_entry)+
    (( ~(SEMI | ENDTABLE) )+ SEMI)*
    ;

task:
    KW_TASK  name_of_task SEMI
    (tf_declaration)*
    st=statement_or_null
    KW_ENDTASK
    ;

function
    :
    (
    KW_FUNCTION
    (range_or_type 
    )? 
    name_of_function SEMI
    ) // This set of bracked added by Baya
    (tf_declaration)+
    st=statement
    KW_ENDFUNCTION
    ;

range_or_type 
    :
    range
    |
    KW_INTEGER
    |
    KW_REAL
    ;

tf_declaration :
    p1=parameter_declaration
    |
    input_declaration
    |
    output_declaration
    |
    inout_declaration
    |
    reg_declaration  
    |
    time_declaration 
    |
    integer_declaration 
    |
    real_declaration 
    |
    event_declaration 
    ;


//----------------------------------------------------------------------------
// Declarations
//----------------------------------------------------------------------------

parameter_declaration:
    KW_PARAMETER (range)?  list_of_param_assignments SEMI 
    ;

list_of_param_assignments:
    p1=param_assignment ( COMMA p2=param_assignment )*
    ;

param_assignment:
    identifier ASSIGN expression
    ;


input_declaration :
    KW_INPUT  ( range )? 
    list_of_variables
    SEMI
    ;

output_declaration :
    KW_OUTPUT ( range )?  list_of_variables SEMI
    ;

inout_declaration :
    KW_INOUT ( range )?  list_of_variables SEMI
    ;

net_declaration 
    :
    (net_type (range)? IDENTIFIER ASSIGN) => net_declaration_with_assign
    |
    net_type (range)?  list_of_variables SEMI
    ;
    
net_declaration_with_assign
    :    
    n2=net_type (drive_strength)? 
    (expandrange)? 
    (delay)? list_of_assignments SEMI
    ;

net_type :
    KW_WIRE
    | 
    KW_TRI 
    |
    KW_TRI1 
    |
    KW_SUPPLY0  
    |
    KW_WAND
    |
    KW_TRIAND 
    |
    KW_TRI0 
    |
    KW_SUPPLY1  
    |
    KW_WOR 
    |
    KW_TRIOR 
    |
    KW_TRIREG 
    ;

expandrange :
    KW_SCALARED range
    |
    KW_VECTORED range 
    |
    range
    ;

reg_declaration  
    :
    KW_REG (range)? 
    l1=list_of_register_variables SEMI
    ;

time_declaration :
    KW_TIME list_of_register_variables SEMI
    ;

integer_declaration  
    :
    KW_INTEGER l1=list_of_register_variables SEMI
    ;

real_declaration
    :
    KW_REAL  l1=list_of_variables SEMI
    ;

event_declaration :
    KW_EVENT  name_of_event ( COMMA name_of_event )* SEMI
    ;

continuous_assign:    
    KW_ASSIGN (drive_strength)? (delay)? (l1=list_of_assignments) SEMI
//    |
//    net_type (drive_strength)? (expandrange)? (delay)?  list_of_assignments SEMI
    ;

parameter_override 
    :
    KW_DEFPARAM l1=list_of_param_assignments SEMI
    ;

list_of_variables 
    :
    id1=name_of_variable
    ( COMMA 
        id2=name_of_variable
    )*
    |
    def1=DEFINE
    ( COMMA def2=DEFINE
    )*
    ;

list_of_assigned_variables 
    :
    (v1=name_of_variable
    ( 
        (ASSIGN e1=expression) 
    )?
    )
    ( COMMA
        ( 
            v2=name_of_variable
            ( 
                ASSIGN e2=expression 
            )?
        ) 
    )*
    ;

list_of_register_variables 
    :
    r1=register_variable
    ( COMMA 
        r2=register_variable 
    )*
    |
    def1=DEFINE
    ( COMMA def2=DEFINE
    )*
    
    ;

register_variable 
    :
    name_of_register 
    |
    name_of_memory LBRACK e1=expression COLON e2=expression RBRACK
    ;

charge_strength :
    LPAREN KW_SMALL RPAREN 
    |
    LPAREN KW_MEDIUM RPAREN 
    |
    LPAREN KW_LARGE  RPAREN
    ;

drive_strength :
    LPAREN strength0 COMMA strength1 RPAREN 
    |
    LPAREN strength1 COMMA strength0 RPAREN
    ;

strength0 :
    KW_SUPPLY0 
    |
    KW_STRONG0 
    |
    KW_PULL0 
    |
    KW_WEAK0 
    |
    KW_HIGHZ0
    ;

strength1 :
    KW_SUPPLY1 
    |
    KW_STROG1  
    |
    KW_PULL1 
    |
    KW_WEAK1 
    |
    KW_HIGHZ1
    ;


range
    :
    (LBRACK expression COLON) => LBRACK lt= expression
    COLON 
    rt=expression
    RBRACK 
    |
    LBRACK bit=expression RBRACK
    ;

list_of_assignments
    :
    asgn1=assignment
    ( 
        COMMA asgn2=assignment 
    )*
    ;

//----------------------------------------------------------------------------
// Primitive Instances
//----------------------------------------------------------------------------

gate_declaration
    :
    gate_type (drive_strength)? (delay)?  
    inst1=gate_instance 
    ( COMMA inst2=gate_instance 
    )* SEMI
    ;

gate_type :
    KW_AND  
    |
    KW_NAND 
    |
    KW_OR   
    |
    KW_NOR 
    |
    KW_XOR 
    |
    KW_XNOR 
    |
    KW_BUF 
    |
    KW_BUFIF0 
    |
    KW_BUFIF1 
    |
    KW_NOT 
    |
    KW_NOTIF0 
    |
    KW_NOTIF1 
    |
    KW_PULLDOWN 
    |
    KW_PULLUP 
    |
    KW_NMOS 
    |
    KW_RNMOS  
    |
    KW_PMOS  
    |
    KW_RPMOS  
    |
    KW_CMOS  
    |
    KW_RCMOS  
    |
    KW_TRAN 
    |
    KW_RTRAN  
    |
    KW_TRANIF0  
    |
    KW_RTRANIF0  
    |
    KW_TRANIF1  
    |
    KW_RTRANIF1
    ;

delay 
    :
    POUND NUMBER 
    | 
    POUND identifier 
    |
    POUND LPAREN mintypmax_expression
    ( 
        COMMA mintypmax_expression
        ( 
            COMMA mintypmax_expression 
        )?
    )?
    RPAREN
    ;

gate_instance
    :
    (name_of_gate_instance )?  
    LPAREN t1=terminal 
    ( COMMA t2=terminal )* RPAREN
    ;

udp_instance :
    (
        name_of_UDP_instance
    )?  
    LPAREN 
    t1=terminal
    ( COMMA 
        t2=terminal 
    )* RPAREN
    ;

terminal 
    :
    expression
    // | IDENTIFIER
    ;
        
//----------------------------------------------------------------------------
// Module Instantiations
//----------------------------------------------------------------------------

module_instantiation
    :
    name_of_module ( p=parameter_value_assignment)?
    module_instance 
    ( COMMA inst2=module_instance )* SEMI        
    ;

parameter_value_assignment 
    :
    POUND LPAREN 
    e1=expression
    ( 
        COMMA e2=expression
    )* 
    RPAREN
    ;

module_instance
    :
    name_of_instance LPAREN list_of_module_connections RPAREN
    ;

list_of_module_connections
    :
    mod_prt_conn1=module_port_connection
    ( 
        COMMA mod_prt_conn2=module_port_connection
    )*
    |
    nmd_prt_conn1=named_port_connection 
    ( 
        COMMA nmd_prt_conn2=named_port_connection 
    )*
    ;

module_port_connection 
    :
    expression 
     //| NULL
    ;

    // expression below isn't optional according to Palnitkar, but
    // several examples generated by Cadence use this syntax.
named_port_connection
    :
    DOT IDENTIFIER LPAREN (expression )? RPAREN 
    ;

//----------------------------------------------------------------------------
// Behavioral Statements
//----------------------------------------------------------------------------

initial_statement:
    KW_INITIAL  
    st=statement
    ;
    

always_statement:
    KW_ALWAYS
    st=statement
    ;

all_statement_or_nulls:
    (statement_or_null)*
    ;
    
statement_or_null:
    (statement) => st1=statement
    |
    SEMI
    ;

all_statements:
    (statement)* 
    ;
    
statement
    :
    (lvalue ASSIGN) => blocking_assignment SEMI 
    |
    (lvalue LE) => non_blocking_assignment SEMI 
    |
    conditional_statement 
    |
    case_statement
    |
    loop_statement
    |
    ptc=procedural_timing_control_statement 
    |
    wait=wait_statement 
    |
    event_trigger 
    |
    seq_block 
    |
    par_block 
    |
    task_enable
    |
    system_task_enable 
    |
    disable_statement 
    |
    procedural_continuous_assignment
    ;

assignment 
    :
    lvalue ASSIGN expression 
    ;

blocking_assignment
    :
    lvalue ASSIGN ( delay_or_event_control)?  expression
    ;
 
non_blocking_assignment  
    :
    lvalue LE ( delay_or_event_control )?  expression
    ;

// KW_ELSE clause is inherently ambiguous; ANTLR gets it right,
// so suppress warning.
conditional_statement 
    :
    KW_IF LPAREN expression
    RPAREN st1=statement_or_null
    ( : KW_ELSE 
         st2=statement_or_null
    )?
    ;

case_statement 
    :
    case_keyword LPAREN expression RPAREN 
    (
        case_item
    )+ 
    'endcase' 
    ;

case_keyword
    :
    KW_CASE
    | 
    KW_CASEZ
    | 
    KW_CASEX
    ;

case_item
    :
    e1=expression
    ( COMMA e2=expression
    )* 
    COLON  
    st1=statement_or_null
    |
    KW_DEFAULT (COLON)? 
    st2=statement_or_null
    ;

loop_statement 
    :
    KW_FOREVER 
    st1=statement
    |
    KW_REPEAT LPAREN e2=expression RPAREN 
    st2=statement
    |
    KW_WHILE  LPAREN e3=expression RPAREN
    st3=statement
    |
    KW_FOR  LPAREN asgn1=assignment SEMI e4=expression SEMI asgn2=assignment RPAREN
    st4=statement
    ;

procedural_timing_control_statement :
    edg=delay_or_event_control st=statement_or_null
    ;

wait_statement
    :
    KW_WAIT LPAREN expression RPAREN statement_or_null
    ;

event_trigger :
    TRIGGER name_of_event SEMI
    ;

disable_statement :
    KW_DISABLE  IDENTIFIER SEMI
    ;

seq_block 
    :
    KW_BEGIN 
    ( 
        COLON name_of_block (decl=block_declaration)*
    )?
    (statement
    )*
    KW_END 
    ;

par_block :
    KW_FORK 
    ( COLON name_of_block (block_declaration)* )?
    (statement)*
    KW_JOIN 
    ;

block_declaration:
    parameter_declaration |
    reg_declaration |
    integer_declaration |
    real_declaration |
    time_declaration |
    event_declaration 
    ;

task_enable
    :
    name_of_task 
    ( LPAREN e1=expression 
    (COMMA 
        (
            e2=expression
        )?
    )* RPAREN )?
    SEMI
    ;

system_task_enable :
    SYSTEM_TASK_NAME ( LPAREN expression (COMMA (expression)?)* RPAREN )?
    SEMI
    ;

procedural_continuous_assignment :
    KW_ASSIGN assignment SEMI |
    KW_DEASSIGN  lvalue SEMI |
    KW_FORCE  assignment SEMI |
    KW_RELEASE  lvalue SEMI
    ;

delay_or_event_control
    :
    delay_control
    |
    event_control
    ;

//----------------------------------------------------------------------------
// Specify Section
//----------------------------------------------------------------------------

specify_block :
    KW_SPECIFY  (specify_item)* KW_ENDSPECIFY
    ;

specify_item :
    spec_param_declaration |
    (path_declaration) => path_declaration |
    system_timing_check
    | sdpd
    ;

spec_param_declaration :
    KW_SPECPARAM  list_of_specparam_assignments SEMI
    ;

list_of_specparam_assignments :
    specparam_assignment ( COMMA specparam_assignment )*
    ;

specparam_assignment :
    identifier ASSIGN expression
    ;

path_declaration :
   (simple_path_declaration) => simple_path_declaration SEMI 
   |
   (level_sensitive_path_declaration) => level_sensitive_path_declaration SEMI
   |
   edge_sensitive_path_declaration SEMI
   ;

simple_path_declaration :
    (parallel_path_description) => parallel_path_description ASSIGN path_delay_value 
    |
    full_path_descriptor ASSIGN path_delay_value
    ;

parallel_path_description :
    LPAREN specify_terminal_descriptor PPATH specify_terminal_descriptor RPAREN
    ;

full_path_descriptor :
    LPAREN list_of_path_terminals  FPATH list_of_path_terminals RPAREN
    ;

list_of_path_terminals :
    specify_terminal_descriptor ( COMMA specify_terminal_descriptor )*
    ;

specify_terminal_descriptor :
    (identifier LBRACK expression COLON) => identifier LBRACK expression COLON expression RBRACK 
    |
    (identifier LBRACK) => identifier LBRACK expression RBRACK 
    |
    identifier
    ;

path_delay_value 
    :
    (path_delay_expression) => path_delay_expression 
    |
    LPAREN list_of_path_delay_expressions RPAREN
    ;

list_of_path_delay_expressions 
    :
    path_delay_expression COMMA path_delay_expression
    ( 
        COMMA path_delay_expression
        ( 
            COMMA path_delay_expression COMMA path_delay_expression 
            COMMA path_delay_expression 
        )? 
    )?
    ; 

path_delay_expression 
    :
    mintypmax_expression
    ;

system_timing_check 
    :
    KW_SETUP LPAREN timing_check_event COMMA timing_check_event COMMA
     timing_check_limit ( COMMA notify_register )? RPAREN SEMI 
    |
    KW_HOLD LPAREN timing_check_event COMMA timing_check_event COMMA
     timing_check_limit ( COMMA notify_register )? RPAREN SEMI 
    |
    KW_PERIOD  LPAREN controlled_timing_check_event COMMA
     timing_check_limit ( COMMA notify_register )? RPAREN SEMI 
    |
    KW_WIDTH LPAREN controlled_timing_check_event COMMA
     timing_check_limit ( COMMA expression COMMA notify_register )? RPAREN SEMI
    |
    KW_SKEW LPAREN timing_check_event COMMA timing_check_event COMMA 
     timing_check_limit ( COMMA notify_register )? RPAREN SEMI 
    |
    KW_RECOVERY LPAREN controlled_timing_check_event COMMA 
     timing_check_event COMMA timing_check_limit
     ( COMMA notify_register )? RPAREN SEMI 
    |
    KW_SETUPHOLD LPAREN timing_check_event COMMA timing_check_event COMMA
     timing_check_limit COMMA timing_check_limit
     ( COMMA notify_register )? RPAREN SEMI
    ;

timing_check_event :
    (timing_check_event_control)? specify_terminal_descriptor
    ( KW_TRIPPLE_AND timing_check_condition )?
    ;

controlled_timing_check_event :
    timing_check_event_control specify_terminal_descriptor
    ( KW_TRIPPLE_AND timing_check_condition )?
    ;

timing_check_event_control :
    KW_POSEDGE  |
    KW_NEGEDGE  |
    edge_control_specifier
    ;

edge_control_specifier :
    KW_EDGE  LBRACK edge_descriptor ( COMMA edge_descriptor )* RBRACK
    ;

   // Use semantic predicates to determine whether a matched
   // NUMBER or IDENTIFIER is a valid special value in the given context.
   // This kludge avoids having the special values in the Literals table,
   // thus avoiding a lexical conflict.

edge_descriptor :
    CONST_0_X  
    |
    CONST_1_X 
    |
    n=NUMBER
    { 
        $n.text.equals("01") || $n.text.equals("10")
    }?
    |
    i=IDENTIFIER
    { 
        $i.text.equals("x1") || $i.text.equals("x0")
    }?
    ;
    
timing_check_condition :
    scalar_timing_check_condition
    ; 

scalar_timing_check_condition :
    expression
    ;

timing_check_limit :
    expression
    ;

notify_register :
    name_of_register
    ;

level_sensitive_path_declaration :
    (parallel_level_sensitive_path_description) => 
         parallel_level_sensitive_path_description ASSIGN path_delay_value SEMI
    |
    full_level_sensitive_path_description ASSIGN path_delay_value SEMI
    ;

parallel_level_sensitive_path_description :
    KW_IF LPAREN expression RPAREN
    LPAREN specify_terminal_descriptor (polarity_operator)?
    PPATH specify_terminal_descriptor RPAREN
    ;

full_level_sensitive_path_description :
    KW_IF  LPAREN expression RPAREN
    LPAREN list_of_path_terminals (polarity_operator)?
    FPATH list_of_path_terminals RPAREN
    ;

polarity_operator :
    PLUS 
    |
    MINUS
    ;

edge_sensitive_path_declaration 
    :
    ( KW_IF  LPAREN expression RPAREN )?
        LPAREN (edge_identifier)? specify_terminal_descriptor
        ( PPATH | FPATH )
        LPAREN ( (list_of_path_terminals) => list_of_path_terminals 
    |
    specify_terminal_descriptor )
    (polarity_operator)? COLON data_source_expression
    RPAREN
    RPAREN
    ASSIGN path_delay_value SEMI
    ;

data_source_expression 
    :
    expression
    ;

edge_identifier :
    KW_POSEDGE  
    |
    KW_NEGEDGE 
    ;

sdpd :
    KW_IF  LPAREN expression RPAREN
    simple_path_declaration
    SEMI
    ;

//----------------------------------------------------------------------------
// Expressions
//----------------------------------------------------------------------------

lvalue 
    :
    (id1=identifier range) // Baya => identifier range
    |
    id2=identifier
    |
    concatenation
    |
    def1=DEFINE
    ;

concatenation 
    :
    (LCURLY expression LCURLY) => 
        LCURLY e1=expression 
        LCURLY e2=expression
    ( 
        COMMA e3=expression
    )* RCURLY RCURLY
    |
    LCURLY e4=expression 
    ( COMMA e5=expression
     )* RCURLY
    ;

mintypmax_expression
    :
    e1=expression
    ( 
        COLON e2=expression COLON e3=expression
    )?
    ;

exp11 
    :
    STRING
    |
    NUMBER 
    |
    (function_call) => fc=function_call
    |
    lvalue 
    ;

exp10 
    :
    exp11
    | 
    LPAREN expression RPAREN
    ;

exp9 
    :
    exp10 
    |
    unary_operator e9=exp9
    ;

exp8 
    :
    e91=exp9
    ( 
        binary_operator
        e92=exp9 
    )*
    ;

exp7 
    :
    exp8
    ( 
        QUESTION 
        e71=exp7
        COLON 
        e72=exp7 
        
    )?
    ;

exp0 
    :
    exp7
    ;

expression
    :
    exp0
    ;

function_call
    :
    name_of_function
    LPAREN exprs=expression_list RPAREN 
    |
    SYSTEM_TASK_NAME ( LPAREN expression_list RPAREN )?
    ;

expression_list 
    :
    e1=expression
    ( 
        COMMA e2=expression
    )*
    ;

unary_operator  
    :
    PLUS
    |
    MINUS
    |
    LNOT
    |
    BNOT
    |
    BAND 
    |
    RNAND 
    |
    BOR 
    |
    RNOR 
    |
    BXOR 
    |
    BXNOR 
    ;

binary_operator 
    :
    PLUS 
    |
    MINUS 
    |
    STAR 
    |
    DIV 
    |
    MOD 
    |
    EQUAL 
    |
    NOT_EQ 
    |
    EQ_CASE 
    |
    NOT_EQ_CASE 
    |
    LAND 
    |
    LOR 
    |
    LT_ 
    |
    LE 
    |
    GT 
    |
    GE 
    |
    BAND 
    |
    BOR 
    |
    BXOR 
    |
    BXNOR 
    |
    SR 
    |
    SSR
    |
    SL 
    |
    SSL
    ;


//----------------------------------------------------------------------------
// Identifiers
//----------------------------------------------------------------------------

name_of_module:
    local_identifier 
    ;

name_of_port:
    local_identifier 
    ;

name_of_variable:
    local_identifier 
    ;

name_of_UDP:
    local_identifier 
    ;

name_of_UDP_instance:
    local_identifier 
    ;

name_of_event:
    local_identifier 
    ;

name_of_task:
    local_identifier 
    ;

real_identifier:
    identifier 
    ;

name_of_memory:
    local_identifier 
    ;

net_identifier:
    identifier 
    ;

name_of_function:
    local_identifier 
    ;

specparam_identifier:
    identifier 
    ;

udp_name_of_port:
    identifier 
    ;

name_of_register :
    local_identifier
    ;

name_of_gate_instance  :
    local_identifier 
    ;

name_of_instance :
    local_identifier 
    ;

name_of_block :
    local_identifier 
    ;

output_terminal_name :
    local_identifier 
    ;



//----------------------------------------------------------------------------
// General
//----------------------------------------------------------------------------

identifier :
    identifier_path 
    ;

identifier_path :
    id1=local_identifier 
    ( 
        DOT id2=local_identifier
    )* 
    ;

local_identifier :
    IDENTIFIER 
    |
    ESCAPED_IDENTIFIER 
    ;

delay_control :
    POUND NUMBER
    |
    POUND id=identifier
    |
    POUND LPAREN mintypmax_expression RPAREN
    ;

event_control :
    AT identifier 
    |
    AT LPAREN event_expression RPAREN
    ;

event_expression :
    e1=sub_event_expression
    ( KW_OR  e2=sub_event_expression
    )*
    ;

sub_event_expression :
    expression
    |
    KW_POSEDGE  expression 
    |
    KW_NEGEDGE  expression
    ;

//----------------------------------------------------------------------------
// Compiler directives
//----------------------------------------------------------------------------

directive : 
    { !skipInputs() }? =>  def1=define_directive
    | { skipInputs() }? => define_directive
    |
    include_directive 
    |
    timescale_directive 
    |
    undef_directive 
    |
    resetall_directive 
    |
    celldefine_directive
    |
    endcelldefine_directive
    |
    conditional_compilation_directive
    |
    line_compiler_directive
    ;

//
// The rule for #define in C is 
// "#define" WS RAW_IDENTIFIER
//       (
//            ( '('  (WS)? RAW_IDENTIFIER (WS)? ( COMMA (WS)? RAW_IDENTIFIER (WS)? )* ')'
//       | 
//       ' '|'\t'|'\f'
//       )
//      ( options{greedy=true;}: ' '|'\t'|'\f' )*
//            defineText:MACRO_TEXT {args.set(0,defineText.getText());}
//      )? '\n' 
//  
//   
//    '`define' IDENTIFIER  ( options {greedy=false;} : (IDENTIFIER|NUMBER))* '~\r'? ~'\n' 
//    '`define' IDENTIFIER  ( options {greedy=false;} : .{System.out.println("YYY");})* '~\r'? ~'\n' 
//    '`define'{isTickDefine = true;} IDENTIFIER  (c=~('\n'){rawval += $c.text; })+  '\n'

define_directive returns [String id, String value]
    @init
    {
        macrovalueStr = "";
    }
    @after
    {
        macrovalueStr = "";
        isInsideDefine = false;
    }
    : 
    TICK_DEFINE id1=IDENTIFIER (
                                      KW_MODULE
                                    | KW_MACROMODULE
                                    | KW_ENDMODULE
                                    | KW_PRIMITIVE
                                    | KW_ENDPRIMITIVE
                                    | KW_INPUT
                                    | KW_OUTPUT
                                    | KW_INOUT
                                    | KW_INTEGER
                                    | KW_REAL
                                    | KW_EVENT
                                    | KW_SCALARED
                                    | KW_VECTORED
                                    | KW_DEFPARAM
                                    | KW_PARAMETER
                                    | TABLE
                                    | ENDTABLE
                                    | KW_TASK
                                    | KW_ENDTASK
                                    | KW_FUNCTION
                                    | KW_ENDFUNCTION
                                    | KW_INITIAL
                                    | KW_ALWAYS
                                    | KW_BEGIN
                                    | KW_END
                                    | KW_IF
                                    | KW_ELSE
                                    | KW_CASE
                                    | KW_CASEZ
                                    | KW_CASEX
                                    | KW_ENDCASE
                                    | KW_DEFAULT
                                    | KW_FOREVER
                                    | KW_REPEAT
                                    | KW_WHILE
                                    | KW_FOR
                                    | KW_FORK
                                    | KW_JOIN
                                    | KW_ASSIGN
                                    | KW_TRIREG
                                    | KW_DEASSIGN
                                    | KW_FORCE
                                    | KW_RELEASE
                                    | KW_SPECIFY
                                    | KW_ENDSPECIFY
                                    | KW_SPECPARAM
                                    | KW_SETUP
                                    | KW_HOLD
                                    | KW_PERIOD
                                    | KW_WIDTH
                                    | KW_SKEW
                                    | KW_RECOVERY
                                    | KW_SETUPHOLD
                                    | KW_TRIPPLE_AND
                                    | KW_POSEDGE
                                    | KW_NEGEDGE
                                    | KW_EDGE
                                    | KW_SMALL
                                    | KW_MEDIUM
                                    | KW_LARGE
                                    | KW_WAIT
                                    | KW_DISABLE
                                    | KW_AND
                                    | KW_NAND
                                    | KW_OR
                                    | KW_NOR
                                    | KW_XOR
                                    | KW_XNOR
                                    | KW_BUF
                                    | KW_BUFIF0
                                    | KW_BUFIF1
                                    | KW_NOT
                                    | KW_NOT_IF0
                                    | KW_NOTIF1
                                    | KW_PULLDOWN
                                    | KW_PULLUP
                                    | KW_NMOS
                                    | KW_RMOS
                                    | KW_PMOS
                                    | KW_RPMOS
                                    | KW_CMOS
                                    | KW_RCMOS
                                    | KW_TRAN
                                    | KW_RTRAN
                                    | KW_TRANIF0
                                    | KW_RTRANIF0
                                    | KW_TRANIF1
                                    | KW_RTRANIF1
                                    | KW_WIRE
                                    | KW_REG
                                    | KW_TIME
                                    | KW_TRI
                                    | KW_TRI1
                                    | KW_SUPPLY0
                                    | KW_STRONG0
                                    | KW_PULL0
                                    | KW_WEAK0
                                    | KW_HIGHZ0
                                    | KW_HIGHZ1
                                    | KW_STROG1
                                    | KW_PULL1
                                    | KW_WEAK1
                                    | KW_WAND
                                    | KW_TRIAND
                                    | KW_TRI0
                                    | KW_SUPPLY1
                                    | KW_WOR
                                    | KW_TRIOR
                                    | CONST_0_X
                                    | CONST_1_X
                                    | AT
                                    | COLON
                                    | COMMA
                                    | DOT
                                    | ASSIGN
                                    | MINUS
                                    | LBRACK
                                    | RBRACK
                                    | LCURLY
                                    | RCURLY
                                    | LPAREN
                                    | RPAREN
                                    | POUND
                                    | QUESTION
                                    | SEMI
                                    | PLUS
                                    | LNOT
                                    | BNOT
                                    | BAND
                                    | RNAND
                                    | BOR
                                    | RNOR
                                    | BXOR
                                    | BXNOR
                                    | STAR
                                    | DIV
                                    | MOD
                                    | EQUAL
                                    | NOT_EQ
                                    | NOT_EQ_CASE
                                    | EQ_CASE
                                    | LAND
                                    | LOR
                                    | LT_
                                    | LE
                                    | GT
                                    | GE
                                    | SR
                                    | SSR
                                    | SL
                                    | SSL
                                    | TRIGGER
                                    | PPATH
                                    | FPATH
                                    | IDENTIFIER
                                    | ESCAPED_IDENTIFIER
                                    | SYSTEM_TASK_NAME
                                    | DOLLAR
                                    | STRING
                                    | NUMBER
                                )* 
    {
        $id = $id1.text;
        $value = macrovalueStr;
        isInsideDefine = false;
    }
    ;

include_directive returns[Integer line, FVPIncludeDirective incdir ]
    :
    INCLUDE f=STRING
    {
        if ( false == skipInputs() && null != $f.text && false == $f.text.isEmpty() )
        {
            $line = $f.line;
            String fName =new String($f.text);
            fName = fName.substring(1,fName.length()-1);
            $incdir = new FVPIncludeDirective( $line, fName );
            FVPDescription desc = FVPDescription.instance();
            FVPDescription.instance().addDeclaration ( $incdir );
            fName = null;
        }
    }
    ;
    
//    TIMESCALE  NUMBER ('ps' |'ns' | 'fs' | 'us' | 'ms' | 's') DIV 
//    NUMBER ('ps' | 'ns' | 'fs' | 'us' | 'ms' | 's') 
    
timescale_directive
    @init
    {
        FVPTimeScale.TimeUnit numUnit = FVPTimeScale.TimeUnit.UNKNOWN;
        FVPTimeScale.TimeUnit denoUnit = FVPTimeScale.TimeUnit.UNKNOWN;
    }
    :
    TIMESCALE  num1=NUMBER (KW_PS |KW_NS KW_FS ) 
    DIV     
    num2=NUMBER (KW_PS |KW_NS KW_FS )
    ;
        
undef_directive: 
    TICK_UNDEF IDENTIFIER
    ;
    
resetall_directive
    :
    '`resetall'
    {
        FVPDescription.instance().getRoot().resetAllDefines();
    }
    ;    

celldefine_directive
    : 
    '`celldefine'
    ;

endcelldefine_directive
    : 
    '`endcelldefine'
    ;

conditional_compilation_directive 
    :
    TICK_IFDEF  IDENTIFIER 
    |
    TICK_IFNDEF  IDENTIFIER 
    |
    TICK_ELSIF IDENTIFIER
    |
    TICK_ELSE  
    |    
    TICK_ENDIF
    ;
    
            
line_compiler_directive:
    LINE NUMBER STRING
    ;    

udp_body :
    (combinational_body)=> combinational_body 
    |
    sequential_body
    ;        

combinational_body :
    TABLE combinational_entry ( combinational_entry )* ENDTABLE 
    ;
        
combinational_entry :
    level_input_list COLON output_symbol SEMI
    ;

sequential_entry :
    seq_input_list COLON state COLON next_state SEMI
    ;
    
sequential_body :
    ( udp_initial_statement )? TABLE 
    sequential_entry ( sequential_entry )* 
    ENDTABLE
    ;    
        
//udp_initial_statement :
//        KW_INITIAL output_terminal_name ASSIGN init_val SEMI
//        ;
        
seq_input_list :
    (level_input_list) => level_input_list 
    |
    edge_input_list
    ;

level_input_list :
    (level_symbol)+
    ;

edge_input_list :
    (options{greedy = true;}:(level_symbol))* 
    edge_indicator (options{greedy = true;}:(level_symbol))*
    ;

edge_indicator :
    LPAREN level_symbol level_symbol RPAREN
    |
    edge_symbol
    ;

state :
    level_symbol
    ;

next_state :
    output_symbol 
    |
    MINUS
    ;

   // Next 3 rules use semantic predicates to determine whether a matched
   // NUMBER or IDENTIFIER is a valid special value in the given context.
   // This kludge avoids having the special values in the Literals table,
   // thus avoiding a lexical conflict.

output_symbol:
    n=NUMBER
    { $n.text.equals("0") || $n.text.equals("1")}?
    |
    i=IDENTIFIER
    { 
        $i.text.equals("x") || $i.text.equals("X")
    }?
    ;

level_symbol:
    QUESTION
    |
    n=NUMBER
    { 
        $n.text.equals("0") || $n.text.equals("1")
    }?
    |
    i=IDENTIFIER
    { 
        $i.text.equals("x") || $i.text.equals("X") ||
        $i.text.equals("b") || $i.text.equals("B") 
    }?
    ;

edge_symbol
    :
    STAR  
      | 
    i=IDENTIFIER
    {
        $i.text.equals("r") || $i.text.equals("R") ||
        $i.text.equals("f") || $i.text.equals("F") ||
        $i.text.equals("p") || $i.text.equals("P") ||
        $i.text.equals("n") || $i.text.equals("N") 
    }?
    ;

udp_instantiation 
    
    :
    name_of_UDP
    (drive_strength)? (delay)?  
    inst1=udp_instance
    ( 
        COMMA inst2=udp_instance
        
    )* SEMI
    ;

    
// Keywords


KW_MODULE:
    'module'
    {
        addMacroString(getText()); $channel = getActiveChannel();
    }
    ;

KW_MACROMODULE:
    'macromodule'
    {
        addMacroString(getText()); $channel = getActiveChannel();
    }
    ;

KW_ENDMODULE:
    'endmodule'
    {
        addMacroString(getText()); $channel = getActiveChannel();
    }
    ;
KW_PRIMITIVE:
    'primitive' 
    {
        addMacroString(getText()); $channel = getActiveChannel(); 
    }
    ;
KW_ENDPRIMITIVE:
    'endprimitive' 
    {
        addMacroString(getText()); $channel = getActiveChannel(); 
    }
    ;
    
KW_INPUT:
    'input'
    {
        addMacroString(getText()); $channel = getActiveChannel();
    }
    ;

KW_OUTPUT:
    'output'
    {
        addMacroString(getText()); $channel = getActiveChannel();
    }
    ;
    
KW_INOUT:
    'inout'
    {
        addMacroString(getText()); $channel = getActiveChannel();
    }
    ;
    
KW_INTEGER: 
   'integer' {addMacroString(getText()); $channel = getActiveChannel(); };
    
KW_REAL:
    'real' {addMacroString(getText()); $channel = getActiveChannel(); };
    
KW_EVENT: 'event' {addMacroString(getText()); $channel = getActiveChannel(); };

KW_SCALARED: 'scalared' {addMacroString(getText()); $channel = getActiveChannel(); };

KW_VECTORED: 'vectored' {addMacroString(getText()); $channel = getActiveChannel(); };

KW_DEFPARAM: 'defparam' {addMacroString(getText()); $channel = getActiveChannel(); };

KW_PARAMETER:
    'parameter' {addMacroString(getText()); $channel = getActiveChannel(); }
    ;
    
TABLE:
    'table'
    {
        addMacroString(getText()); $channel = getActiveChannel(); 
    }
    ;
    
    
 ENDTABLE:
    'endtable'
    {
        addMacroString(getText()); $channel = getActiveChannel(); 
    }
    ;

KW_TASK:
    'task'
    {
        addMacroString(getText()); $channel = getActiveChannel(); 
    }
    ;
KW_ENDTASK:
    'endtask'
    {
        addMacroString(getText()); $channel = getActiveChannel(); 
    }
    ;
    
KW_FUNCTION:
    'function'
    {
        addMacroString(getText()); $channel = getActiveChannel(); 
    }
    ;
    
KW_ENDFUNCTION:
    'endfunction'
    {
        addMacroString(getText()); $channel = getActiveChannel(); 
    }
    ;
    
KW_INITIAL:
    'initial' 
    {
        addMacroString(getText()); $channel = getActiveChannel(); 
    }
    ;

KW_ALWAYS:
    'always' 
    {
        addMacroString(getText()); $channel = getActiveChannel(); 
    }
    ;

KW_BEGIN:
    'begin' 
    {
        addMacroString(getText()); $channel = getActiveChannel(); 
    }
    ;
KW_END:
    'end' 
    {
        addMacroString(getText()); $channel = getActiveChannel(); 
    }
    ;

KW_IF:
    'if' 
    {
        addMacroString(getText()); $channel = getActiveChannel(); 
    }
    ;
KW_ELSE:
    'else' 
    {
        addMacroString(getText()); $channel = getActiveChannel(); 
    }
    ;

KW_CASE: 'case' {addMacroString(getText()); $channel = getActiveChannel(); };
KW_CASEZ: 'casez'  {addMacroString(getText()); $channel = getActiveChannel(); };
KW_CASEX: 'casex' {addMacroString(getText()); $channel = getActiveChannel(); };
KW_ENDCASE: 'endcase' {addMacroString(getText()); $channel = getActiveChannel(); };
KW_DEFAULT: 'default' {addMacroString(getText()); $channel = getActiveChannel(); };
KW_FOREVER: 'forever''forever' {addMacroString(getText()); $channel = getActiveChannel(); };
KW_REPEAT: 'repeat' {addMacroString(getText()); $channel = getActiveChannel(); };
KW_WHILE: 'while' {addMacroString(getText()); $channel = getActiveChannel(); };
KW_FOR: 'for' {addMacroString(getText()); $channel = getActiveChannel(); };

KW_FORK: 'fork' {addMacroString(getText()); $channel = getActiveChannel(); };
KW_JOIN: 'join' {addMacroString(getText()); $channel = getActiveChannel(); };
KW_ASSIGN: 'assign' {addMacroString(getText()); $channel = getActiveChannel(); };
KW_TRIREG:'trireg' {addMacroString(getText()); $channel = getActiveChannel(); };

KW_DEASSIGN: 'deassign' {addMacroString(getText()); $channel = getActiveChannel(); };
KW_FORCE: 'force' {addMacroString(getText()); $channel = getActiveChannel(); }; 
KW_RELEASE: 'release' {addMacroString(getText()); $channel = getActiveChannel(); };
KW_SPECIFY: 'specify' {addMacroString(getText()); $channel = getActiveChannel(); };
KW_ENDSPECIFY: 'endspecify' {addMacroString(getText()); $channel = getActiveChannel(); };
KW_SPECPARAM: 'specparam' {addMacroString(getText()); $channel = getActiveChannel(); }; 
KW_SETUP: '$setup' {addMacroString(getText()); $channel = getActiveChannel(); };
KW_HOLD: '$hold' {addMacroString(getText()); $channel = getActiveChannel(); };
KW_PERIOD: '$period' {addMacroString(getText()); $channel = getActiveChannel(); };
KW_WIDTH: '$width' {addMacroString(getText()); $channel = getActiveChannel(); };
KW_SKEW: '$skew' {addMacroString(getText()); $channel = getActiveChannel(); };
KW_RECOVERY: '$recovery' {addMacroString(getText()); $channel = getActiveChannel(); };
KW_SETUPHOLD: '$setuphold' {addMacroString(getText()); $channel = getActiveChannel(); };
KW_TRIPPLE_AND: '&&&' {addMacroString(getText()); $channel = getActiveChannel(); } ;
KW_POSEDGE: 'posedge' {addMacroString(getText()); $channel = getActiveChannel(); } ;
KW_NEGEDGE: 'negedge' {addMacroString(getText()); $channel = getActiveChannel(); };
KW_EDGE: 'edge' {addMacroString(getText()); $channel = getActiveChannel(); };



KW_SMALL: 'small' {addMacroString(getText()); $channel = getActiveChannel(); };
KW_MEDIUM: 'medium' {addMacroString(getText()); $channel = getActiveChannel(); };

KW_LARGE: 'large' {addMacroString(getText()); $channel = getActiveChannel(); };
KW_WAIT: 'wait' {addMacroString(getText()); $channel = getActiveChannel(); };
KW_DISABLE: 'disable' {addMacroString(getText()); $channel = getActiveChannel(); };

KW_AND:
    'and' {addMacroString(getText()); $channel = getActiveChannel(); }; 
KW_NAND:
    'nand' {addMacroString(getText()); $channel = getActiveChannel(); }; 
KW_OR:
    'or' {addMacroString(getText()); $channel = getActiveChannel(); }; 
KW_NOR:
    'nor' {addMacroString(getText()); $channel = getActiveChannel(); }; 
KW_XOR:
    'xor' {addMacroString(getText()); $channel = getActiveChannel(); };
KW_XNOR:
    'xnor' {addMacroString(getText()); $channel = getActiveChannel(); }; 
KW_BUF:
    'buf' {addMacroString(getText()); $channel = getActiveChannel(); }; 
KW_BUFIF0:
    'bufif0' {addMacroString(getText()); $channel = getActiveChannel(); }; 
KW_BUFIF1:
    'bufif1' {addMacroString(getText()); $channel = getActiveChannel(); }; 
KW_NOT:
    'not' {addMacroString(getText()); $channel = getActiveChannel(); };
KW_NOT_IF0:
    'notif0' {addMacroString(getText()); $channel = getActiveChannel(); }; 
KW_NOTIF1:
    'notif1' {addMacroString(getText()); $channel = getActiveChannel(); };
KW_PULLDOWN:
    'pulldown' {addMacroString(getText()); $channel = getActiveChannel(); }; 
KW_PULLUP:
    'pullup' {addMacroString(getText()); $channel = getActiveChannel(); }; 
KW_NMOS:
    'nmos' {addMacroString(getText()); $channel = getActiveChannel(); };
KW_RMOS:
    'rnmos' {addMacroString(getText()); $channel = getActiveChannel(); }; 
KW_PMOS:
    'pmos' {addMacroString(getText()); $channel = getActiveChannel(); }; 
KW_RPMOS:
    'rpmos' {addMacroString(getText()); $channel = getActiveChannel(); }; 
KW_CMOS:
    'cmos' {addMacroString(getText()); $channel = getActiveChannel(); }; 
KW_RCMOS:
    'rcmos' {addMacroString(getText()); $channel = getActiveChannel(); }; 
KW_TRAN:
    'tran' {addMacroString(getText()); $channel = getActiveChannel(); };
KW_RTRAN:
    'rtran' {addMacroString(getText()); $channel = getActiveChannel(); };
KW_TRANIF0:
    'tranif0' {addMacroString(getText()); $channel = getActiveChannel(); };
KW_RTRANIF0:
    'rtranif0' {addMacroString(getText()); $channel = getActiveChannel(); }; 
KW_TRANIF1:
    'tranif1' {addMacroString(getText()); $channel = getActiveChannel(); }; 
KW_RTRANIF1:
    'rtranif1' {addMacroString(getText()); $channel = getActiveChannel(); }
    ;
KW_WIRE:    
    'wire' {addMacroString(getText()); $channel = getActiveChannel(); };

KW_REG:    
    'reg' {addMacroString(getText()); $channel = getActiveChannel(); };
    
KW_TIME:    
    'time' {addMacroString(getText()); $channel = getActiveChannel(); };
    
KW_TRI:    
    'tri' {addMacroString(getText()); $channel = getActiveChannel(); }
    ;
KW_TRI1: 
    'tri1' {addMacroString(getText()); $channel = getActiveChannel(); } 
    ;
KW_SUPPLY0:    
    'supply0' {addMacroString(getText()); $channel = getActiveChannel(); } 
    ;
KW_STRONG0:    
    'strong0' {addMacroString(getText()); $channel = getActiveChannel(); } 
    ;
    
KW_PULL0:    
    'pull0' {addMacroString(getText()); $channel = getActiveChannel(); } 
    ;
    
KW_WEAK0:    
    'weak0' {addMacroString(getText()); $channel = getActiveChannel(); } 
    ;
    
KW_HIGHZ0:    
    'highz0' {addMacroString(getText()); $channel = getActiveChannel(); }
    ;
    
KW_HIGHZ1:    
    'highz1' {addMacroString(getText()); $channel = getActiveChannel(); }
    ;

KW_STROG1:
    'strong1' {addMacroString(getText()); $channel = getActiveChannel(); } 
    ;
    
KW_PULL1:    
    'pull1' {addMacroString(getText()); $channel = getActiveChannel(); } 
    ;
    
KW_WEAK1:    
    'weak1' {addMacroString(getText()); $channel = getActiveChannel(); } 
    ;
        
KW_WAND:    
    'wand' {addMacroString(getText()); $channel = getActiveChannel(); }
    ;
    
KW_TRIAND:    
    'triand' {addMacroString(getText()); $channel = getActiveChannel(); } 
    ;
    
KW_TRI0:    
   'tri0' {addMacroString(getText()); $channel = getActiveChannel(); }
    ;
    
KW_SUPPLY1:    
    'supply1' {addMacroString(getText()); $channel = getActiveChannel(); } 
    ;
   
KW_WOR:    
    'wor' {addMacroString(getText()); $channel = getActiveChannel(); }
    ;    
    
KW_TRIOR: 
    'trior' {addMacroString(getText()); $channel = getActiveChannel(); } 
    ;
fragment KW_PS: 'ps';
fragment KW_NS: 'ns';
fragment KW_FS: 'fs';
 
TICK_DEFINE: 
    '`define' 
    {
        if ( false == skipInputs() ) 
        {
            isInsideDefine = true; 
        }
        else
        {
            $channel = HIDDEN;
            isInsideDefine = false; 
            setText("");
        }
        if( isRemoveDirectives() )
        {
            setText("");
        }
    }
    ;

TICK_IFDEF: 
    '`ifdef' 
    {
        isTickIfDef = true;
        if ( isRemoveDirectives() )
        {
            setText("");
        } 
    }
    ;
    
TICK_IFNDEF:    
    '`ifndef'
    {
        isTickIfNDef = true;
        if ( isRemoveDirectives() )
        {
            setText("");
        } 
    }
    ;
    
TICK_ELSIF:
    '`elsif'
    {
        isTickElsIf = true;
        if ( isRemoveDirectives() )
        {
            setText("");
        } 
    }
    ;
    
TICK_ELSE:    
    '`else'
    {
        FVPCompilerDirectiveCond dirnode = new FVPCompilerDirectiveCond( getLine(), null, Directive.ELSE );
        if ( prevCond() && false == FVPDescription.instance().isOneblockalreadyvisited() )
        {
            dirnode.setCondTrue(true);
        }
        FVPDescription.instance().pushCompilerDirective( dirnode );
        if ( isRemoveDirectives() )
        {
            setText("");
        } 
    }
    ;
    
TICK_ENDIF:        
    '`endif'
    {
        FVPDescription.instance().processEndifCompilerDirective();
        popSLastBlockCond();
        isTickIfDef = false;
        isTickIfNDef = false;
        isTickElsIf = false;
        isTickElse = false;
        if ( isRemoveDirectives() )
        {
            setText("");
        } 
    }
    ;
        
TICK_UNDEF:    
    '`undef'
    {
        if ( skipInputs() )
        {
            $channel = HIDDEN;
            isTickUndef = false;
            setText("");
        }
        else
        {
            isTickUndef = true;
        }
        if ( isRemoveDirectives() )
        {
            setText("");
        } 
    }
    ;
    
CONST_0_X: '0x' {addMacroString(getText()); $channel = getActiveChannel(); }; 
    
CONST_1_X:  '1x' {addMacroString(getText()); $channel = getActiveChannel(); };

            
//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
// The Verilog scanner
//----------------------------------------------------------------------------


// Operators
AT        
    : 
    '@' {addMacroString(getText()); $channel = getActiveChannel(); }
    ;

COLON        
    : 
    ':' {addMacroString(getText()); $channel = getActiveChannel(); }
    ;

COMMA        
    : 
    ',' {addMacroString(getText()); $channel = getActiveChannel(); }
    ;

DOT        
    : 
    '.' {addMacroString(getText()); $channel = getActiveChannel(); }
    ;

ASSIGN        
    : 
    '='  {addMacroString(getText()); $channel = getActiveChannel(); }
    ;

MINUS        
    : 
    '-'  {addMacroString(getText()); $channel = getActiveChannel(); }
    ;

LBRACK        
    : 
    '['  {addMacroString(getText()); $channel = getActiveChannel(); }
    ;

RBRACK        
    : 
    ']' {addMacroString(getText()); $channel = getActiveChannel(); }
    ;

LCURLY        
    : 
    '{' {addMacroString(getText()); $channel = getActiveChannel(); }
    ;

RCURLY        
    : 
    '}'  {addMacroString(getText()); $channel = getActiveChannel(); }
    ;

LPAREN        
    : 
    '('  {addMacroString(getText()); $channel = getActiveChannel(); }
    ;

RPAREN        
    : 
    ')'  {addMacroString(getText()); $channel = getActiveChannel(); }
    ;

POUND        
    : 
    '#'   {addMacroString(getText()); $channel = getActiveChannel(); }
    ;

QUESTION    
    : 
    '?'  {addMacroString(getText()); $channel = getActiveChannel(); }
    ;

SEMI        
    : 
    ';'  {addMacroString(getText()); $channel = getActiveChannel(); }
    ;

PLUS        
    : 
    '+'   {addMacroString(getText()); $channel = getActiveChannel(); } 
    ;

LNOT        
    : 
    '!' {addMacroString(getText()); $channel = getActiveChannel(); }
    ;

BNOT        
    : 
    '~'  {addMacroString(getText()); $channel = getActiveChannel(); }  
    ;

BAND        
    : 
    '&' {addMacroString(getText()); $channel = getActiveChannel(); } 
    ;

RNAND       
    : 
    '~&' {addMacroString(getText()); $channel = getActiveChannel(); }
    ;

BOR         
    : 
    '|' {addMacroString(getText()); $channel = getActiveChannel(); }
    ;

RNOR        
    : 
    '~|' {addMacroString(getText()); $channel = getActiveChannel(); }
    ;

BXOR        
    : 
    '^' {addMacroString(getText()); $channel = getActiveChannel(); }
    ;

BXNOR       
    : 
    '~^' {addMacroString(getText()); $channel = getActiveChannel(); } 
    | '^~'  {addMacroString(getText()); $channel = getActiveChannel(); }
    ;

STAR        
    : 
    '*' {addMacroString(getText()); $channel = getActiveChannel(); }
    ;

DIV         
    : 
    '/' {addMacroString(getText()); $channel = getActiveChannel(); }
    ;

MOD         
    : 
    '%' {addMacroString(getText()); $channel = getActiveChannel(); }
    ;

EQUAL       
    : '==' {addMacroString(getText()); $channel = getActiveChannel(); }
    ;

NOT_EQ      
    : 
    '!=' {addMacroString(getText()); $channel = getActiveChannel(); }
    ;

NOT_EQ_CASE 
    : 
    '!==' {addMacroString(getText()); $channel = getActiveChannel(); }
    ;

EQ_CASE     
    : 
    '===' {addMacroString(getText()); $channel = getActiveChannel(); }
    ;

LAND        
    : 
    '&&'  {addMacroString(getText()); $channel = getActiveChannel(); }
    ;

LOR         
    : 
    '||' {addMacroString(getText()); $channel = getActiveChannel(); }
    ;

LT_         
    : 
    '<' {addMacroString(getText()); $channel = getActiveChannel(); }  
    ;

LE          
    : 
    '<=' {addMacroString(getText()); $channel = getActiveChannel(); }
    ;

GT          
    : 
    '>' {addMacroString(getText()); $channel = getActiveChannel(); }
    ;

GE          
    : 
    '>=' {addMacroString(getText()); $channel = getActiveChannel(); }
    ;

SR          
    : 
    '>>' {addMacroString(getText()); $channel = getActiveChannel(); }
    ;

SL          
    : 
    '<<' {addMacroString(getText()); $channel = getActiveChannel(); }   
    ;


SSR          
    : 
    '>>>' {addMacroString(getText()); $channel = getActiveChannel(); } 
    ;

SSL          
    : 
    '<<<' {addMacroString(getText()); $channel = getActiveChannel(); } 
    ;
    
TRIGGER     
    : 
    '->' {addMacroString(getText()); $channel = getActiveChannel(); }
    ;

PPATH       
    : 
    '=>' {addMacroString(getText()); $channel = getActiveChannel(); }   
    ;

FPATH       
    : 
    '*>' {addMacroString(getText()); $channel = getActiveChannel(); }
    ;
    
        

    // an identifier.  Note that testLiterals is set to true!  This means
    // that after we match the rule, we look in the Literals table to see
    // if it's a literal or really an identifer.

IDENTIFIER 
    :
    ('a'..'z'|'A'..'Z'|'_') ('a'..'z'|'A'..'Z'|'_'|DOLLAR|'0'..'9')*
    {
        String strVal = getText();      
        Integer lineNo = getLine();
        if ( isInsideDefine )
        {       
            if ( null == macroID )
            {
                macroID = strVal;
            }
            else
            {
                String macroval = strVal;
                if ( isTick )
                {
                    String tmpStr = strVal.substring(1,strVal.length());
                    macroval = FVPDescription.instance().getRoot().getMacrolValue(tmpStr);
                    if ( null == macroval )
                    {
                        macroval = "";
                    }
                    isTick = false;
                }
                addMacroString(macroval); 
            }
        }
        else if ( isTickIfDef )
        {
            pushSLastBlockCond();
            FVPCompilerDirectiveCond dirnode = new FVPCompilerDirectiveCond( lineNo, strVal, Directive.IFDEF );
            dirnode.setIdentifier( strVal );
            if ( FVPDescription.instance().getRoot().isDefined( strVal ) && prevCond() )
            {
                dirnode.setCondTrue(true);
                dirnode.setOneblockalreadyvisited(true);
            }
            FVPDescription.instance().pushCompilerDirective( dirnode );
        }
        else if ( isTickIfNDef )
        {
            pushSLastBlockCond();
            FVPCompilerDirectiveCond dirnode = new FVPCompilerDirectiveCond( lineNo, strVal, Directive.IFNDEF );
            dirnode.setIdentifier( strVal );
            if ( false == FVPDescription.instance().getRoot().isDefined( strVal ) && prevCond() )
            {
                dirnode.setCondTrue(true);
                dirnode.setOneblockalreadyvisited(true);
            }
            FVPDescription.instance().pushCompilerDirective( dirnode );
        } 
        else if ( isTickElsIf )
        {
            FVPCompilerDirectiveCond dirnode = new FVPCompilerDirectiveCond(lineNo, strVal, Directive.ELSIF );
            dirnode.setIdentifier( strVal );
            if ( prevCond() )
            {
                if ( true == FVPDescription.instance().getRoot().isDefined( strVal ) )
                {
                    dirnode.setCondTrue(true);
                    dirnode.setOneblockalreadyvisited(true);
                }
                else if ( true == FVPDescription.instance().isOneblockalreadyvisited() )
                {
                    dirnode.setOneblockalreadyvisited(true);
                }
            }
            
            FVPDescription.instance().pushCompilerDirective( dirnode );  
        }
        if ( isTick )
        {
            String tmpStr = strVal.substring(1,strVal.length());
            String macroval = FVPDescription.instance().getRoot().getMacrolValue(tmpStr);
            if ( null == macroval )
            {
                macroval = "";
            }
            String trimmedStr = truncateLeadingBlanks(macroval);
            setText(macroval);
            isTick = false;
        }
        else if ( isTickUndef ) 
        {
            FVPDescription.instance().getRoot().removeDefine( strVal );
        }
        $channel = getActiveChannel();
        if ( skipInputs() && false == isTickElsIf && false ==  isTickIfDef && false == isTickIfNDef && false == isTick)
        {           
            setText("");
        }
        else if ( isRemoveDirectives() &&  // We'll revove macros at the second pass
                  ( isTickIfDef || isTickIfNDef || isTickElsIf || isInsideDefine || isTickUndef ) )
        {
            setText("");
        }
        isTickIfDef = false;
        isTickIfNDef = false;
        isTickElsIf = false;
        isTickElse = false;
        isTickEndif = false;
        isTickUndef = false;
        isTick = false;
        isDollar = false;
    }
    ;

ESCAPED_IDENTIFIER :
    '\\' (~ '\u0020' )+ ('\u0020'|'\t'|'\n')
    {
        if ( isInsideDefine && null == macroID )
        {
            macroID = getText();
        }
        else
        {
             addMacroString(getText());
             $channel = getActiveChannel(); 
        }
    }
    ;
        
    // string literals
STRING :
    '"' (~('"'))* '"'  
    {
        if ( isTickBeginKeyword )
        {
          $channel = HIDDEN;
          setText("");
          isTickBeginKeyword = false;
        }
        else
        {
            String str = getText();
            addMacroString( str ); 
            $channel = getActiveChannel(); 
            if ( isTickInclude && false == skipInputs() )
            {
                String fName = str.substring(1,str.length()-1); // Removed quotes
                proccessIncludeFile( fName, incpaths );
            }
            isTickInclude = false;
        } 
    }
    ;
    
LINE        
    : 
    '`line' 
    {addMacroString(getText()); $channel = getActiveChannel(); }
    ;

INCLUDE     
    : '`include' 
    { 
        isTickInclude = true; 
        addMacroString(getText()); 
        $channel = getActiveChannel(); 
    }
    ;

TIMESCALE   
    : 
    '`timescale' 
    {addMacroString(getText()); $channel = getActiveChannel(); }
    ;

TICK_BEGIN_KEYWORD   
    : 
    '`begin_keywords' 
    { $channel=HIDDEN; setText("");isTickBeginKeyword=true; }
    ;

//TICK_BEGIN_KEYWORD   
//    : 
//    '`end_keyword' 
//    {isTickEndKeyword=true; }
//    ;

    // 'compiler' define/macro.
DEFINE
      //options {testLiterals=true;}
    :
    id1=TICK id2=IDENTIFIER
    {
        $channel=HIDDEN;
        isTick = false;
        if ( FVPDescription.instance().getPreprocessorLevel() == 1 )
        {
            String tmpStr = $id2.text;
            String macroval = FVPDescription.instance().getRoot().getMacrolValue(tmpStr);
            if ( null == macroval )
            {
                macroval = "";
            }
            if ( null != fragStr && null != macroval && false == fragStr.equals( macroval ) )
            {
                macroval = new String(fragStr + macroval);
            }
            setText(macroval);
        }
        fragStr = null;
    } 
    ;

SYSTEM_TASK_NAME :
    DOLLAR
    {
        if ( isInsideDefine )
        {
            setText(""); 
        }
        $channel=HIDDEN;
    } 
    IDENTIFIER 
    ;
     
 fragment TICK : '`' {isTick = true; };
    
 fragment DOLLAR: 
    '$'   
    {
        isDollar = true;
        //$channel = HIDDEN;
    }
    ;  
      
   // a dummy rule to force vocabulary to be all characters (except special
   //   ones that ANTLR uses internally (0 to 2)
fragment
VOCAB :
    '\u0003'..'\u007F'
    ;
//    
//NUMBER :
//    ( (SIZE (WS)*)? BASE (WS)* SIZED_DIGIT ) => SIZED_NUMBER
//    | UNSIZED_NUMBER
//    ;

    
NUMBER :
    ( (SIZE (WS)*)? BASE (WS)* SIZED_DIGIT ) => SIZED_NUMBER
    {
        addMacroString(getText()); 
        $channel = getActiveChannel(); 
    }
    |
    ( (SIZE (WS)*)? BASE (WS)* (BIT)+ ) => SIZED_NUMBER
    {
        addMacroString(getText()); 
        $channel = getActiveChannel(); 
    }
    | UNSIZED_NUMBER
    {
        addMacroString(getText()); 
        $channel = getActiveChannel(); 
    }
    ;

fragment
BIT
  :
  (
    'x'
    | 'z'
    | 'X'
    | 'Z'
    | '?'
    | '_'
  )
  ;


//// Baya: 1'b1 and 1'b0 does not match. thi is a bug a of today     
//fragment
//SIZED_NUMBER :
//    (SIZE (WS)*)? BASE (WS)* SIZED_DIGIT 
//    (SIZED_DIGIT | '_' )*
//    ;

// Removing the WS, no space is supported as of today, otherwise #10 out1 = does not work
fragment
SIZED_NUMBER
@init
{
//   fragStr = null;
     String str = null;
} 
:
    (SIZE
    { 
        str = getText();
        if ( null != str && isInsideDefine )
        {
            fragStr = truncateLeadingBlanks(str);
//          fragStr = str;
        }
    }
    )? 
    BASE
    { 
        str = getText();
        if ( null != str && isInsideDefine )
        {
            fragStr = truncateLeadingBlanks(str);
//          fragStr = str;
        }
    }
    (
    SIZED_DIGIT
    { 
        str = getText();
        if ( null != str && isInsideDefine )
        {
            fragStr = truncateLeadingBlanks(str);
//          fragStr = str;
        }
    }   
    | 
    BIT
    { 
        str = getText();
        if ( null != str && isInsideDefine )
        {
            fragStr = truncateLeadingBlanks(str);
//          fragStr = str;
        }
    } 
    )+
    (
    SIZED_DIGIT
    { 
        str = getText();
        if ( null != str && isInsideDefine )
        {
            fragStr = truncateLeadingBlanks(str);
//          fragStr = str;
        }
    }
    | '_'
    { 
        str = "_";
        if ( null != str && isInsideDefine )
        {
            fragStr = truncateLeadingBlanks(str);
//          fragStr = str;
        }
    } 
    )*
//    {
//      fragStr = null;
//    }
    ;


fragment
SIZE
@init
{
//   fragStr = null;
     String str = null;
} 
:
    (
    DIGIT
    { 
        str = getText();
        if ( null != str && isInsideDefine )
        {
            fragStr = truncateLeadingBlanks(str);
//          fragStr = str;
        }
    }
    )+
    ;

fragment
BASE 
@init
{
//   fragStr = null;
     String str = null;
}
:
    '\'' ('s'|'S')? ( 'd' | 'D' | 'h' | 'H' | 'o' | 'O' | 'b' | 'B' )
    { 
        str = getText();
        if ( null != str && isInsideDefine )
        {
            fragStr = truncateLeadingBlanks(str);
//          fragStr = str;
        }
    }
    ;

fragment
SIZED_DIGIT 
@init
{
//   fragStr = null;
     String str = null;
}
:
    DIGIT
    { 
        str = getText();
        if ( null != str && isInsideDefine )
        {
            fragStr = truncateLeadingBlanks(str);
//          fragStr = str;
        }
    }
    | 
    HEXDIGIT
    { 
        str = getText();
        if ( null != str && isInsideDefine )
        {
            fragStr = truncateLeadingBlanks(str);
//          fragStr = str;
        }
    }
    ;

fragment
UNSIZED_NUMBER 
@init
{
//   fragStr = null;
     String str = null;
}
:
    DIGIT 
    { 
        str = getText();
        if ( null != str && isInsideDefine )
        {
            fragStr = truncateLeadingBlanks(str);
        }
    }
    ( 
    DIGIT
    { 
        str = getText();
        if ( null != str && isInsideDefine )
        {
            fragStr = truncateLeadingBlanks(str);
        }
    }
    | 
    '_'
    { 
        str = getText();
        if ( null != str && isInsideDefine )
        {
            fragStr = truncateLeadingBlanks(str);
        }
    } 
    )*
    ;

fragment
DIGIT 
:
        ('0'..'9')
        | 
        DEFINE
        ;

fragment
HEXDIGIT 
@init
{
//   fragStr = null;
     String str = null;
}
:
        ('A'..'F'|'a'..'f')
        { 
            str = getText();
            if ( null != str && isInsideDefine )
            {
                fragStr = truncateLeadingBlanks(str);
            }
        }
        ;
        
fragment SPACE_OR_TAB
    :    (
            ' '
        |    '\t'
        )
    ;

 
WS
    :
    (SPACE_OR_TAB
    {$channel = HIDDEN; if ( isInsideDefine ) macrovalueStr += " "; /*getActiveChannel(); */}
    )+ 
    ;
 
// The folling rule for newline has been added by Baya as it was not in the orignal grammar file
NEW_LINE
    :
    (('\r''\n') | ('\n'))+
    { 
        $channel = HIDDEN;
        if ( isInsideDefine &&  false == FVPDescription.instance().getRoot().isDefined( macroID )) 
        {
            FVPIdentifier id = new FVPIdentifier(getLine(), macroID );
            // Trim the initial blank spaces
            String trimmedStr = truncateLeadingBlanks(macrovalueStr);
            FVPExprBase val = new FVPExprString( getLine(), trimmedStr);
            FVPDefine def = new FVPDefine( getLine(), id, val);
            FVPDescription.instance().getRoot().addDefine(def);
//          System.out.println("Adding macro : " +  macroID + " value is " + macrovalueStr);
            FVPDescription.instance().getRoot().addMacrolValue(  new String(macroID) , new String(trimmedStr) );
        }
        isInsideDefine = false; 
        isTickUndef = false;
        macrovalueStr = "";
        fragStr = null;
        macroID = null;
    }
    ;
    
PRAGMA_DIRECTIVE
    : 
    '`pragma'  ( options {greedy=false;} : . )*  '\r'? '\n' 
    {
        $channel=HIDDEN;
        if ( isInsideDefine ) 
        {
            setText("");
        }
    }
    ;
        
ML_COMMENT
    :   '/*' ( options {greedy=false;} : . )* '*/' 
    {
        $channel=HIDDEN;
        if ( isInsideDefine ) 
        {
            setText("");
        }
    }
    ;
  
SL_COMMENT
    : '//'  ( options {greedy=false;} : . )*  '\r'? '\n' 
    {
        $channel=HIDDEN;
        
        if ( isInsideDefine ) 
        {
            setText("\n");
        }
        
        if ( isInsideDefine &&  false == FVPDescription.instance().getRoot().isDefined( macroID )) 
        {
            FVPIdentifier id = new FVPIdentifier(getLine(), macroID );
            // Trim the initial blank spaces
            String trimmedStr = truncateLeadingBlanks(macrovalueStr);
            FVPExprBase val = new FVPExprString( getLine(), trimmedStr);
            FVPDefine def = new FVPDefine( getLine(), id, val);
            FVPDescription.instance().getRoot().addDefine(def);
            FVPDescription.instance().getRoot().addMacrolValue(  new String(macroID) , new String(trimmedStr) );
        }
        isInsideDefine = false; 
        macrovalueStr = "";
        macroID = null;
    }
    ;
    