// This grammar file has been copied from the below area
// http://svparser.googlecode.com/svn-history/r2/trunk/trivial1.g
// https://github.com/BIOFAB/ClothoBiofabEdition/blob/master/ClothoDevelopment/Cello/ANTLRVerilog/VerilogA.g
// 
// The above grammar has been modified to meet my requirement.
// Kanai L Ghosh
//

grammar Verilog;


options 
{
    language = Java;
    k=2;
    output=templte;
    rewrite=true;
}
  
@header 
{
    package com.eu.miscedautils.verilogparser;
    import com.eu.miscedautils.CommonUtils.LogMessages;
    import com.eu.miscedautils.verilogparser.FVPCompilerDirectiveCond.Directive;
    import com.eu.miscedautils.verilogparser.*;     
    import java.util.Iterator;
    import java.util.List;
    import java.util.ArrayList;
    import java.util.LinkedList;

    import org.antlr.runtime.ANTLRFileStream;
    import org.antlr.runtime.ANTLRStringStream;
    import org.antlr.runtime.CharStream;
    import org.antlr.runtime.CommonTokenStream;
    import org.antlr.runtime.RecognitionException;
    import org.antlr.runtime.TokenSource;
    import org.antlr.runtime.TokenStream;
    import java.io.File;
    import java.io.FileInputStream;
    import java.io.IOException;
}
 
@members
{
    public Stack<FVPExprBase> expressionstack = null;
    LinkedList<String> incpaths = null;
    
    @Override
    public void displayRecognitionError(String[] tokenNames,
                                        RecognitionException e) {
        String hdr = getErrorHeader(e);
        String msg = getErrorMessage(e, tokenNames);
        LogMessages.instance().print(1816,
                                     LogMessages.MSG_SEVERITY.ERROR, 
                                     hdr + " : " + msg );
    }
    
    public void processPortList( 
                      LinkedList<String> ids,
                      Integer line,
                      FVPRange range,
                      FVPPort.Direction dir,
                      boolean isSigned,
                      FVPPort.ObjClass portClass
                      )
    {
        LinkedList<String> id_strs = ids;
        // Search in the symbol table and set the direction of the port
        Iterator itr = id_strs.iterator(); 
        while(itr.hasNext()) 
        {
            String symbol = (String)itr.next();
            // Cleanup this String object from the list
            itr.remove(); 
            FVPBase val = FVPDescription.instance().getSymbol( symbol );
            FVPBase mod_or_tf = FVPDescription.instance().getScope();
            if ( null != val  && mod_or_tf == val.getScope() )
            {
                LogMessages.instance().print(1832,  LogMessages.MSG_SEVERITY.ERROR,
                               "Port name '"+ symbol + "' is already found as object " +
                                val.getObjType().toString() +
                               " at " + FVPDescription.instance().getFile() + 
                               ":" + val.getLine() );
            }
            
            val =  new FVPIdentifier( line, symbol );
            FVPNet net = new FVPNet ( line, (FVPIdentifier)val, 
                                      FVPNet.NetType.IMPLICIT_PORT_WIRE, null );
            net.setSigned( isSigned );                          
//            FVPDescription.instance().addDeclaration( net );
            if ( null == net.getRange() )
            {
              net.setRange(range);
            }
            FVPPort port = new FVPPort ( net, dir, line );
            port.setRange(net.getRange());
            port.setSigned( isSigned );
            FVPBase decl = null;
            
            if ( null != portClass )
            {
                
                port.setPortClass( portClass );
                switch ( portClass )
                {
                    case REG:
                    {
                      port.setPortClass( FVPPort.ObjClass.REG );
                      FVPRegister reg = new FVPRegister ( net.getLine(), net.getIdentifier(), range, null );
                      reg.setSigned( isSigned );
                      decl = reg;
                      break;
                    }
                    case INTEGER:
                    {
                      port.setPortClass( FVPPort.ObjClass.INTEGER );
                      FVPInteger reg = new FVPInteger ( net.getLine(), net.getIdentifier(), null );
                      decl = reg;
                      break;
                    }
                    case REAL:
                    {
                      port.setPortClass( FVPPort.ObjClass.REAL );
                      FVPReal reg = new FVPReal ( net.getLine(), net.getIdentifier(), null );
                      decl = reg;
                      break;
                    }
                }
            }
            FVPBase mod = FVPDescription.instance().getCurrentscope();
                switch( mod.getObjType() )
                {
                   case MODULE:
                       if ( null != decl )
                       {    
                           ((FVPModule)mod).addDeclaration ( decl );
                       }
                       ((FVPModule)mod).addPort( port );
                       ((FVPModule)mod).addPortId( net.getIdentifier() );
                       break;
                  case UDP:
                       if ( null != decl )
                       {    
                           ((FVPUDP)mod).addDeclaration ( decl );
                       }
                       ((FVPUDP)mod).addPort( port );
                       ((FVPUDP)mod).addPortId( net.getIdentifier() );
                       break;
                   case TASK_OR_FUNCTION_DECL:
                       ((FVPTaskOrFunctionDecl)mod).addPort( port );
                       if ( null != decl )
                       {    
                            ((FVPTaskOrFunctionDecl)mod).addDeclaration ( decl );
                       }
                       break;
                   default:
                         LogMessages.instance().print(0,
                                    LogMessages.MSG_SEVERITY.ERROR,
                                    "Unsupported scope type " + mod.getObjType().toString() + " found for declaration in " + mod.getFile() + 
                                    ":" + mod.getLine() );
                }
            FVPDescription.instance().insertSymbol( symbol, port );
        }
    }
    public void portDeclWithAssignments( 
                                Integer line,
                                LinkedList<FVPNet> nets,
                                LinkedList<FVPLvalue> lvals, 
                                boolean isSigned,
                                FVPRange range,
                                FVPPort.ObjClass portClass,
                                LinkedList<FVPSeqAssignment> assignments
                              )
    {
        FVPDescription desc = FVPDescription.instance();
        FVPBase mod_or_tf = FVPDescription.instance().getScope();
        if ( null != nets )
        {
          Iterator itr = nets.iterator();
          Iterator<FVPLvalue>  itr2 = lvals.iterator(); 
           
          while(itr.hasNext()) 
          {
              FVPNet net = (FVPNet)itr.next();
              FVPLvalue lv = itr2.next();
              
              if ( null == net )
              {
                  continue;
              }
              String symbol = net.getName();
              net.clearDummy();
              net.getIdentifier().clearDummy();
              net.setSigned( isSigned );
              ((FVPLvalue)lv).setActualObj( net );                                  
              if ( null == net.getRange() )
              {
                net.setRange(range);
              }
              FVPPort port = new FVPPort ( net, FVPPort.Direction.OUTPUT, line );
              port.setRange(range);
              port.setSigned( isSigned );
              FVPBase decl = null;
            
              if ( null != portClass )
              { 
                port.setPortClass( portClass );
                switch ( portClass )
                {
                    case REG:
                    {
                      port.setPortClass( FVPPort.ObjClass.REG );
                      FVPRegister reg = new FVPRegister ( net.getLine(), net.getIdentifier(), range, null );
                      reg.setSigned( isSigned );
                      decl = reg;
                      break;
                    }
                    case INTEGER:
                    {
                      port.setPortClass( FVPPort.ObjClass.INTEGER );
                      FVPInteger reg = new FVPInteger ( net.getLine(), net.getIdentifier(), null );
                      decl = reg;
                      break;
                    }
                    case REAL:
                    {
                      port.setPortClass( FVPPort.ObjClass.REAL );
                      FVPReal reg = new FVPReal ( net.getLine(), net.getIdentifier(), null );
                      decl = reg;
                      break;
                    }
                }
              }
              FVPBase mod = FVPDescription.instance().getCurrentscope();
              switch( mod.getObjType() )
              {
                case MODULE:
                    if ( null != decl )
                    {    
                        ((FVPModule)mod).addDeclaration ( decl );
                    }
                    ((FVPModule)mod).addPort( port );
                    ((FVPModule)mod).addPortId( net.getIdentifier() );
                    break;
                case UDP:
                    if ( null != decl )
                    {    
                        ((FVPUDP)mod).addDeclaration ( decl );
                    }
                    ((FVPUDP)mod).addPort( port );
                    ((FVPUDP)mod).addPortId( net.getIdentifier() );
                    break;
                case TASK_OR_FUNCTION_DECL:
                    ((FVPTaskOrFunctionDecl)mod).addPort( port );
                    if ( null != decl )
                    {    
                        ((FVPTaskOrFunctionDecl)mod).addDeclaration ( decl );
                    }
                    break;
                default:
                    LogMessages.instance().print(0,
                                    LogMessages.MSG_SEVERITY.ERROR,
                                    "Unsupported scope type " + mod.getObjType().toString() + " found for declaration in " + mod.getFile() + 
                                    ":" + mod.getLine() );
              }  
              FVPDescription.instance().insertSymbol( symbol, port );
          } // End of while loop
        }
        if ( null !=assignments && false == assignments.isEmpty() )
        {
            Iterator<FVPSeqAssignment> itr1 = assignments.iterator(); 
            while(itr1.hasNext()) 
            {
                FVPSeqAssignment assgn = itr1.next();
                if ( null == assgn )
                {
                    continue;
                }
                FVPExprBase rval = assgn.getRvalue();
                FVPBase lval = assgn.getLvalue();
                FVPBase actualObj = ((FVPLvalue)lval).getActualObj();
                if ( null == actualObj )
                {
                    actualObj = lval;
                }
                FVPPort port = null; //.setInitialValue( rval );
	            switch( mod_or_tf.getObjType() )
	            {
	                case MODULE:
	                    ((FVPModule)mod_or_tf).removeDeclaration ( actualObj );
	                    ((FVPModule)mod_or_tf).removeStatement( assgn );
	                    FVPRegister reg1 = ((FVPModule)mod_or_tf).getRegister( ((FVPNet)actualObj).getName() );
	                    if ( null != reg1 )
	                    {
	                       reg1.setInitialValue( rval );
	                    }
	                    else
	                    {
	                       port = ((FVPModule)mod_or_tf).getPort( ((FVPNet)actualObj).getName() );
	                       if ( null != port )
	                       {
	                           port.setInitialValue( rval );
	                       }
	                       else
	                       {
	                           ((FVPNet)actualObj).setInitialValue( rval );
	                       }
	                    }
	                    break;
	                case UDP:
	                    ((FVPUDP)mod_or_tf).removeDeclaration ( actualObj );
	                    ((FVPUDP)mod_or_tf).removeStatement( assgn );
	                    port = ((FVPUDP)mod_or_tf).getPort( ((FVPNet)actualObj).getName() );
                        if ( null != port )
                        {
                           port.setInitialValue( rval );
                        }
                        else
                        {
                           ((FVPNet)actualObj).setInitialValue( rval );
                        }
	                    break;
	                case TASK_OR_FUNCTION_DECL:
                        ((FVPTaskOrFunctionDecl)mod_or_tf).removeDeclaration ( actualObj );
	                    ((FVPTaskOrFunctionDecl)mod_or_tf).removeStatement( assgn );
	                    port = ((FVPTaskOrFunctionDecl)mod_or_tf).getPort( ((FVPNet)actualObj).getName() );
                        if ( null != port )
                        {
                           port.setInitialValue( rval );
                        }
                        else
                        {
                           ((FVPNet)actualObj).setInitialValue( rval );
                        }
	                    break;
	            }  
            }
        }
    }
}

@init 
{
    LinkedList<String> incpaths = null;
}
    
@lexer::header 
{
    package com.eu.miscedautils.verilogparser;
    import com.eu.miscedautils.verilogparser.FVPCompilerDirectiveCond.Directive;  
    import com.eu.miscedautils.verilogparser.*;   
    import org.antlr.runtime.*;
}
 
  
@lexer::init 
{
    LinkedList<String> incpaths = null;
}

parseRTL [ LinkedList<String> incpaths ] returns [ FVPRoot root] 
    @init 
    {
        LogMessages.instance().print(1817,
                                     LogMessages.MSG_SEVERITY.NOTE,
                                     "Parsing input file '" +
                                     FVPDescription.instance().getFile() + "' ...");        
        this.incpaths = incpaths;
        this.expressionstack = new Stack<FVPExprBase>();
        FVPDescription.instance().initComment(-1);
    }
    :
    ( description 
        {
            $root = (FVPRoot)$description.desc.getRoot();
        } 
    )*
    EOF
    ;

description returns [ FVPDescription desc ]:
    module 
    {
        $desc = FVPDescription.instance();
    }
    |
    udp
    {
        $desc = FVPDescription.instance();
    } 
    |
    directive
    { 
        $desc = FVPDescription.instance();
    }
    ;

module returns [ FVPModule mod, Integer line]
    @init
    {
        FVPComment comment = null;
    }
   :
    ( 
        kwmod=KW_MODULE
        {
            if ( true == FVPDescription.instance().isStoreComment() )
            {
                comment = FVPDescription.instance().getComment( $kwmod.getLine() );
            }
        }
        | 
        'macromodule'
    ) 
    name_of_module 
    {
        FVPDescription desc = FVPDescription.instance();
        String modname = $name_of_module.id;
        FVPBase sym = desc.getRoot().findModuleOrUDP( modname );
    
        if ( null != sym )
        {          
            if ( FVPBase.ObjType.MODULE == sym.getObjType() )  
            {
                if ( sym.isDummy() )
                {
                    ((FVPModule)sym).clearDummy();
                }
                else
                {
                    LogMessages.instance().print(1818,
                                       LogMessages.MSG_SEVERITY.WARNING,
                                       "Overwriting the module '" + modname +
                                       "' found at " + desc.instance().getFile() + ":" + 
                                       $name_of_module.line + 
                                       " which was is already defined at " +
                                       sym.getFile() + ":" + 
                                       sym.getLine() + " ...");
                }
                ((FVPModule)sym).setName(  $name_of_module.id );
                ((FVPModule)sym).setLine( $name_of_module.line );
                ((FVPModule)sym).resetAll();
                $mod = ((FVPModule)sym);
                    
                desc.insertSymbol( $name_of_module.id, $mod);
                desc.pushScope($mod);
            }
            else
            { 
                LogMessages.instance().print(1818,
                                   LogMessages.MSG_SEVERITY.ERROR,
                                   "Symbol '" + modname +
                                   "' found at " + desc.instance().getFile() + ":" + 
                                   $name_of_module.line + 
                                   " is already found at " +
                                   sym.getFile() + ":" + 
                                   sym.getLine() + " as object type " + sym.getObjType().toString() + " ...");
            }
        }
        else
        {
            $mod = new FVPModule( $name_of_module.id, $name_of_module.line );
            FVPDescription.instance().addDeclaration ( $mod );
            desc.insertSymbol( $name_of_module.id, $mod);
            desc.pushScope($mod);
        }
    }
(  
  (
    ( 
        list_of_ports
    )? SEMI
  )  
  |
  (
    (
    POUND LPAREN  
    (
        p1=parameter_declaration_no_semi
        {
            FVPDescription desc = FVPDescription.instance();
            if ( null != $p1.params )
            {
                Iterator itr = $p1.params.iterator(); 
                while(itr.hasNext()) 
                {
                    FVPParamDecl param = (FVPParamDecl)itr.next();
                    if ( null != $p1.range )
                    {
                        param.setRange($p1.range);
                    }
                }
            } 
        }
        |
        lp1=localparam_declaration_no_semi
        {
            FVPDescription desc = FVPDescription.instance();
            if ( null != $lp1.params )
            {
                Iterator itr = $lp1.params.iterator(); 
                while(itr.hasNext()) 
                {
                    FVPLocalParam param = (FVPLocalParam)itr.next();
                    if ( null != $lp1.range )
                    {
                        param.setRange($lp1.range);
                    }
                }
            }
        }
    )
    (   COMMA
        p2=parameter_declaration_no_semi
        {
            FVPDescription desc = FVPDescription.instance();
            if ( null != $p2.params )
            {
                Iterator itr = $p2.params.iterator(); 
                while(itr.hasNext()) 
                {
                    FVPParamDecl param = (FVPParamDecl)itr.next();
                    if ( null != $p2.range )
                    {
                        param.setRange($p2.range);
                    }
                }
            } 
        }
        |
        lp2=localparam_declaration_no_semi
        {
            FVPDescription desc = FVPDescription.instance();
            if ( null != $lp2.params )
            {
                Iterator itr = $lp2.params.iterator(); 
                while(itr.hasNext()) 
                {
                    FVPLocalParam param = (FVPLocalParam)itr.next();
                    if ( null != $lp2.range )
                    {
                        param.setRange($lp2.range);
                    }
                }
            }
        }
    )*
    RPAREN
    )?
    (
        ( LPAREN  
        ( port_declaration_no_semi
        )
        ( COMMA
            port_declaration_no_semi
        )*
        RPAREN
        )
        |
        list_of_ports
    )
    ? SEMI  
  )
)  
    ( module_item )*
    KW_ENDMODULE 
    {
        FVPDescription.instance().popScope();
        if ( true == FVPDescription.instance().isStoreComment() )
        {
            $mod.setComment( comment ) ;
        }
    }
    ;

function returns [Integer line, FVPTaskOrFunctionDecl func]
    @init
    {
        FVPRange rng = null;
        boolean isReal = false;
        boolean isInteger = false;
        boolean isRealTime = false;
        boolean isSigned = false;
        boolean isAutomatic = false;
        boolean isReg = false;
    } 
    :
    (
    KW_FUNCTION 
    ('automatic' { isAutomatic = true; })?
    (KW_REG { isReg = true; })?
    ('signed' { isSigned = true; })? 
    (range_or_type 
    {
        rng = $range_or_type.range;
        isReal = $range_or_type.isReal;
        isInteger = $range_or_type.isInteger;
        isRealTime = $range_or_type.isRealTime;
    }
    )? 
    name_of_function
    ) // This set of bracked added by Baya
    {
        $line = $name_of_function.line;
        FVPDescription desc = FVPDescription.instance();
        String funcname = $name_of_function.id;
        FVPBase sym = (FVPBase)desc.getSymbol( funcname );
        if ( null != sym )
        {   
            if ( sym.isDummy() && FVPBase.ObjType.TASK_OR_FUNCTION_DECL == sym.getObjType() )
            {
                $func = (FVPTaskOrFunctionDecl)sym;
                $func.setLine( $line );
                sym.clearDummy();
                $func.setRange( rng );
                $func.setIntegerRet( isInteger );
                $func.setRealRet( isReal );
                $func.setRealTimeRet( isRealTime );
                $func.setSigned( isSigned );
                $func.setReg( isReg );
                $func.setAutomatic( isAutomatic );
                desc.pushScope($func);
            }
            else
            {
                LogMessages.instance().print( 1830,
                                   LogMessages.MSG_SEVERITY.ERROR, 
                                   "Symbol '" + funcname +
                                   "' found at " + desc.getFile() + ":" + 
                                   $line + 
                                   " is already found at " +
                                   sym.getFile() + ":" + 
                                   sym.getLine() + " ...");
            }                           
        }
        else
        {
            $func = new FVPTaskOrFunctionDecl ( $line, funcname );
            desc.addDeclaration($func);
            $func.setRange( rng );
            $func.setIntegerRet( isInteger );
            $func.setRealRet( isReal );
            $func.setRealTimeRet( isRealTime );
            $func.setSigned( isSigned );
            $func.setReg( isReg );
            $func.setAutomatic( isAutomatic );
            desc.insertSymbol( funcname, $func);            
            desc.pushScope($func);
        }    
    }
    ( LPAREN  
        ( port_declaration_no_semi
        )
        ( COMMA
            port_declaration_no_semi
        )*
        RPAREN
    )? SEMI  
    (tf_declaration)*
    st=all_statement_or_nulls
    'endfunction'
    {
        FVPDescription.instance().popScope();
    }
    ;


task returns [ Integer line, FVPTaskOrFunctionDecl task]
    @init
    {
        boolean isAutomatic = false;
    } 
    :
    KW_TASK ('automatic' { isAutomatic = true; })? name_of_task
    {
        $line = $name_of_task.line;
        FVPDescription desc = FVPDescription.instance();
        String taskname = $name_of_task.id;
        FVPBase sym = (FVPBase)desc.getSymbol( taskname );
        if ( null != sym )
        {   
            if ( sym.isDummy() && FVPBase.ObjType.TASK_OR_FUNCTION_DECL == sym.getObjType() )
            {
                $task = (FVPTaskOrFunctionDecl)sym;
                $task.setLine( $line );
                $task.setTask(true);
                $task.setAutomatic( isAutomatic );
                desc.pushScope($task);
                sym.clearDummy();
            }
            else  if ( desc.getScope() == sym.getScope() )
            {
                LogMessages.instance().print(1829, LogMessages.MSG_SEVERITY.ERROR, 
                                  "Symbol '" + taskname +
                                   "' found at " + desc.getFile() + ":" + 
                                   $line + 
                                   " is already found at " +
                                   sym.getFile() + ":" + 
                                   sym.getLine() + " as object " + sym.getObjType().toString() + " ...");
            }
        }
        else
        {
            $task = new FVPTaskOrFunctionDecl ( $line, taskname );
            $task.setTask(true);
            desc.insertSymbol( taskname, $task);
            desc.addDeclaration($task);
            $task.setAutomatic( isAutomatic );
            desc.pushScope($task);
        }    
    }
    ( LPAREN  
        ( port_declaration_no_semi
        )
        ( COMMA
            port_declaration_no_semi
        )*
        RPAREN
    )
    ? SEMI  
    (tf_declaration)*
    all_statements
    'endtask'
    {
        FVPDescription.instance().popScope();
    }
    ;


port_declaration_no_semi
    :
    input_declaration_no_semi
    | 
    output_declaration_no_semi 
    | 
    inout_declaration_no_semi
    ;
        
list_of_ports returns [ LinkedList<FVPIdentifier> plist] :
    LPAREN 
    p1=port
    {
        $plist = new LinkedList<FVPIdentifier>(); 
        $plist.add($p1.p);
    } 
    ( COMMA p2=port
    {
        if ( null != plist )
        {
            $plist.add($p2.p);
        }
    } 
    )* 
    RPAREN
    ;

port returns [FVPIdentifier p, Integer line]:
    ( 
        port_expression
        {
            $p = $port_expression.p;
            $line = $port_expression.line;
        } 
    )? 
    |
    DOT name_of_port LPAREN (port_expression)? RPAREN
    {
        LogMessages.instance().print(1819,
                           LogMessages.MSG_SEVERITY.ERROR,
                           "Port declaration found at " + 
                           FVPDescription.instance().getFile() + ":" + $DOT.getLine() + 
                           "as .PortName(Expression) is not supported yet ...");
    }
    ;

port_expression returns [FVPIdentifier p, Integer line]:
    port_reference 
    {
        $p = $port_reference.p ;
        $line = $port_reference.line;
    } 
    |
    LCURLY p1=port_reference ( COMMA p2=port_reference )* RCURLY
    {
        LogMessages.instance().print(1820,
                           LogMessages.MSG_SEVERITY.ERROR,
                           "Unsupported  port declaration found " +
                           "around line :" + $p1.line );
    }
    ;

// TBD: Only simple way of port declaration is considered
//      Need to include the other ways of port decls

port_reference returns [FVPIdentifier p, Integer line]:
    ( name_of_variable LBRACK expression COLON ) =>
            name_of_variable LBRACK expression COLON expression RBRACK 
    {
        LogMessages.instance().print(1821,
                                     LogMessages.MSG_SEVERITY.ERROR,
                                     "Port reference as PortName[num:num] found at " +
                           FVPDescription.instance().getFile() + ":" + $name_of_variable.line + 
                           " is not supported yet ...");
    }
    |
    ( name_of_variable LBRACK ) =>
            name_of_variable LBRACK expression RBRACK 
    {
        LogMessages.instance().print(1822,
                           LogMessages.MSG_SEVERITY.ERROR,
                           "Port reference as PortName[num] found at " +
                           FVPDescription.instance().getFile() + ":" + $name_of_variable.line + 
                           " is not supported yet ...");
    }
    |
    name_of_variable 
    {
        FVPDescription desc = FVPDescription.instance();
        FVPBase sym = (FVPBase)desc.getSymbol( $name_of_variable.id);
        if ( null != sym && desc.getScope() == sym.getScope() )
        {
            LogMessages.instance().print(1823,
                               LogMessages.MSG_SEVERITY.ERROR,
                               "Port name '"+ $name_of_variable.id + 
                               "' is already defined in " +
                               ((FVPBase)sym).getFile() +
                               ":" + ((FVPBase)sym).getLine() );
        }
        FVPIdentifier prt = new FVPIdentifier( $name_of_variable.line,$name_of_variable.id );
        $p = prt;
    }
    ;

module_items: 
    (module_item)* EOF
    ;
    
module_item :
    // ambiguity between net_declaration and continuous_assign,
    // but parser gets it right: keyword chosen over IDENTIFIER.
    p1=parameter_declaration
    {
        FVPDescription desc = FVPDescription.instance();
        if ( null != $p1.params )
        {
            Iterator itr = $p1.params.iterator(); 
            while(itr.hasNext()) 
            {
                FVPParamDecl param = (FVPParamDecl)itr.next();
                if ( null != $p1.range )
                {
                    param.setRange($p1.range);
                }
            }
        } 
    }
    |
    lp1=localparam_declaration
    {
        FVPDescription desc = FVPDescription.instance();
        if ( null != $lp1.params )
        {
            Iterator itr = $lp1.params.iterator(); 
            while(itr.hasNext()) 
            {
                FVPLocalParam param = (FVPLocalParam)itr.next();
                if ( null != $lp1.range )
                {
                    param.setRange($lp1.range);
                }
            }
        } 
    }
    |
    input_declaration
    |
    output_declaration
    |
    inout_declaration
    |
    net_declaration 
    | 
    reg_declaration
    | 
    time_declaration 
    | 
    integer_declaration 
    |
    real_declaration 
    |
    event_declaration 
    |
    gate_declaration 
    |
    instantiation  
    |
    parameter_override 
    |
    continuous_assign 
    | 
    specify_block 
    |
    initial_statement 
    |
    always_statement 
    |
    task 
    |
    function 
    |
    directive
    |
    module_or_generate_item 
    ;

instantiation:
    (module_instantiation) => module_instantiation
    |
    udp_instantiation
    ;

module_or_generate_item:
  module_or_generate_item_declaration
  | 
  loop_generate_construct
  | 
  conditional_generate_construct
  |
  generate_region
    ;

module_or_generate_item_declaration
  : 
  genvar_declaration
    ;

generate_region returns [FVPGenerateBase generate]
  : 
  g='generate' 
  (module_or_generate_item )* 
  'endgenerate'
  ;

genvar_declaration returns [Integer line,  LinkedList<FVPGenVar> genvars]
  :
  g='genvar' 
  l1=list_of_variables
    {
        LinkedList<String> id_strs = $l1.ids;
        $line = $l1.line;
        // Search in the symbol table and set the direction of the port
        Iterator itr = id_strs.iterator(); 
        while(itr.hasNext()) 
        {
            String symbol = (String)itr.next();
            // Cleanup this String object from the list
            itr.remove(); 
            FVPDescription desc = FVPDescription.instance();  
            FVPBase val = desc.getSymbol( symbol );
            if ( null != val && desc.getScope() == val.getScope() )
            {                
                LogMessages.instance().print(1824, 
                                      LogMessages.MSG_SEVERITY.ERROR, 
                                      "Genvar symbol '" + symbol +
                                      "' is already defined in file " +
                                      val.getFile() + ":" + val.getLine() );
            }
            else
            {
                FVPIdentifier var = new FVPIdentifier ( $line, symbol );
                FVPGenVar genvar = new FVPGenVar ( $line, var );
                FVPDescription.instance().addDeclaration( genvar );                
                FVPDescription.instance().insertSymbol( symbol, genvar );
                if ( null == $genvars )
                {
                    $genvars = new LinkedList<FVPGenVar>();
                }
                $genvars.add(genvar);
            }
        }
    }    
    SEMI
    ;


loop_generate_construct returns [FVPGenerateBase forgen] 
  : 
  f='for' LPAREN 
      genvar_initialization SEMI 
      expression SEMI 
      genvar_iteration 
      RPAREN 
  {
    LinkedList<FVPBase> stmts = new  LinkedList<FVPBase>();
    $forgen = new  FVPForGenerate( $f.getLine(), stmts, $expression.expr );
    ((FVPForGenerate)$forgen).setInitial( $genvar_initialization.assgn );
    ((FVPForGenerate)$forgen).setSteps( $genvar_iteration.assgn );
    FVPDescription.instance().pushScope( $forgen );
  }
  generate_block_or_null
  {
    FVPDescription.instance().popScope();
    FVPDescription.instance().addStatement( $forgen );
  }
  ;

list_of_genvar_identifiers 
  :
  genvar_identifier (COMMA genvar_identifier)*
  ;

genvar_initialization returns [FVPSeqAssignment assgn]
  :
  assignment
  {
    $assgn = $assignment.assgn;
  }
  ;


genvar_iteration returns [FVPSeqAssignment assgn]
  :
  assignment
  {
    $assgn = $assignment.assgn;
  }
  ;

genvar_primary 
  : 
  (constant_primary)=> constant_primary
  | 
  genvar_identifier
  ;

conditional_generate_construct 
  :
  if_generate_construct
  |
  case_generate_construct
  ;

if_generate_construct returns [ FVPIfGenerate ifgen ]
  :
  i='if' LPAREN e=expression RPAREN
  {
    LinkedList<FVPBase> stmts = new  LinkedList<FVPBase>();
    $ifgen = new  FVPIfGenerate( $i.getLine(), e.expr, stmts);
    FVPDescription.instance().pushScope( $ifgen );
  }
    generate_block_or_null
  (
    : els='else'
    {
      FVPElseGenerate elsegen = new FVPElseGenerate( $els.getLine() );
      if ( null != $ifgen )
      {
	      $ifgen.setElseBlock( elsegen );
	      FVPDescription.instance().pushScope( elsegen );
      }
    }
     generate_block_or_null
     {
      FVPDescription.instance().popScope();
     }
  )?
  {
    FVPDescription.instance().popScope();
    FVPDescription.instance().addStatement( $ifgen );
  }
  ;

case_generate_construct
  :
  c='case' LPAREN expression RPAREN
  {
    LogMessages.instance().print(1825, 
                               LogMessages.MSG_SEVERITY.ERROR, 
                               "Unsupported construct 'generate statement' found " +
                               " at " + FVPDescription.instance().getFile() +
                               ":" + $c.getLine() );
  }
    case_generate_item 
    (case_generate_item )* 
  'endcase'
  ;

case_generate_item :
  expression (COMMA expression)* COLON generate_block_or_null
    |
  'default' (COLON)? generate_block_or_null
  ;

    
generate_block_or_null returns [ Integer line, FVPIdentifier id ]
  :
  generate_block
  {
    $line = $generate_block.line;
    $id = $generate_block.id;
  }
  |
  SEMI
  ;
  
generate_block returns [ Integer line, FVPIdentifier id ]
  :
  conc_stmts_in_generate
  |
  'begin' 
  (COLON 
    generate_block_identifier
    {
      $line = $generate_block_identifier.line;
      $id = $generate_block_identifier.id;
      ((FVPGenerateBase)FVPDescription.instance().getScope()).setLabel( $id.getName() );
    }
  )? 
    (conc_stmts_in_generate)* 
  'end'
;

conc_stmts_in_generate:
  net_declaration 
    | 
    reg_declaration
    | 
    time_declaration 
    | 
    integer_declaration 
    |
    real_declaration 
    |
    event_declaration 
    |
    gate_declaration 
    |
    instantiation  
    |
    parameter_override 
    |
    continuous_assign 
    | 
    specify_block 
    |
    initial_statement 
    |
    always_statement 
    |
    task 
    |
    function 
    |
    module_or_generate_item
  ;
  
generate_block_identifier  returns [ Integer line, FVPIdentifier id ]
  :
  identifier
  {
    $line = $identifier.line;
    $id = new FVPIdentifier( $line, $identifier.id );
  }
  ;

genvar_identifier returns [ Integer line, FVPIdentifier id ]
  :
  identifier
  {
    $line = $identifier.line;
    $id = new FVPIdentifier( $line, $identifier.id );
  }
  ;

constant_primary  returns [ Integer line, FVPExprBase expr ]
  :
  primary
  {
    $line = $primary.line;
    $expr = $primary.expr;
  }
  ;

primary  returns [ Integer line, FVPExprBase expr ]
  : 
  expression
  {
    $line = $expression.line;
    $expr = $expression.expr;
  }
  ; 

attribute_instance returns [ Integer line, FVPAttributeInstance inst ]
  :
  LPAREN STAR 
    a1=attr_spec
    {
      $line = $a1.line;
      $inst = new FVPAttributeInstance(  $line, $a1.spec );
    } 
    (
      COMMA a2=attr_spec
      {
        $inst.addAttribSpec( $a2.spec );
      }
    )* 
  STAR RPAREN
  ;

attr_spec returns [ Integer line, FVPAttributeSpec spec ]
  :
  attr_name
  {
    $line = $attr_name.line;
    $spec = new FVPAttributeSpec( $line, $attr_name.id );
  } 
  (
    ASSIGN expression
    {
      $spec.setValue( $expression.expr );
    }
  )?
  ;

attr_name returns [ Integer line, FVPIdentifier id ]
  :
  identifier
  {
    $line = $identifier.line;
    $id = new FVPIdentifier(  $identifier.line, $identifier.id );
  }
    ;


//----------------------------------------------------------------------------
// UDP specs
//----------------------------------------------------------------------------

udp returns [FVPUDP udp]
    @init
    {
         FVPDescription desc = FVPDescription.instance();
    }
    :
    'primitive'  name_of_UDP
    {
        $udp = new FVPUDP( $name_of_UDP.id , $name_of_UDP.line );
        FVPDescription.instance().addDeclaration ( $udp );
        desc.insertSymbol( $udp.getName(), $udp);
        desc.pushScope($udp);
    }   
    LPAREN  
    n1=name_of_variable 
    {
        FVPBase sym = (FVPBase)desc.getSymbol( $n1.id);
        if ( null != sym && desc.getScope() == sym.getScope() )
        {
            LogMessages.instance().print(1826, 
                               LogMessages.MSG_SEVERITY.ERROR, 
                               "Port name '"+ $n1.id + 
                               "' is already defined in " +
                               ((FVPBase)sym).getFile() +
                               ":" + ((FVPBase)sym).getLine() );
        }
    }
    ( 
        COMMA n2=name_of_variable
        {
            FVPBase sym = (FVPBase)desc.getSymbol( $n2.id);
            if ( null != sym && desc.getScope() == sym.getScope() )
            {
                LogMessages.instance().print(1827, 
                               LogMessages.MSG_SEVERITY.ERROR,
                               "Port name '"+ $n2.id + 
                               "' is already defined in " +
                               ((FVPBase)sym).getFile() +
                               ":" + ((FVPBase)sym).getLine() );
            }
        } 
    )* RPAREN SEMI
    (udp_declaration)+
    (udp_initial_statement)?
    table_definition
    'endprimitive'  
    {
        desc.popScope();
    }
    ;
 
udp_port_list :
    udp_name_of_port ( COMMA udp_name_of_port )*
    ;


udp_declaration :
    input_declaration
    |
    output_declaration
    |
    reg_declaration
    ;

udp_initial_statement :
// Baya        'initial' output_terminal_name ASSIGN init_val SEMI
    KW_INITIAL output_terminal_name ASSIGN NUMBER SEMI
    ;

   // Use a semantic predicate to determine whether a matched NUMBER
   // is a valid special value in the given context.
   // This kludge avoids having the special values in the Literals table,
   // thus avoiding a lexical conflict.
   
// Baya : Below is commented by Baya otherwise we get error on
// continuous assignment like assign lhs = 1'b1 or assign lhs = 1'b0

//init_val :
//        '1\'b0' |
//        '1\'b1' |
//        '1\'bx' |
//    n=NUMBER
//    { $n.text=="0" || $n.text=="1"}?
//    ;

table_definition :
    'table' table_entries 'endtable'
    {
        LogMessages.instance().print(1828, 
                                      LogMessages.MSG_SEVERITY.WARNING,
                                      "table definition is not supported, ignoring it ...");
    }
    ;

   // Don't try to parse table entries; just collect them.
   // There are ambiguities between edge_symbol and level_symbol,
   // and textbook Verilog examples don't seem to follow rules
   // completely. For example,
   //    '0    00    :    0;'
   // doesn't match grammar because of '00', but is frequently used.

table_entries :
    //(sequential_entry) => (sequential_entry)+ |
    //(combinational_entry)+
    (( ~(SEMI | 'endtable') )+ SEMI)*
    ;

range_or_type returns [FVPRange range, boolean isInteger, boolean isReal, boolean isRealTime]
    :
    range
    {
        $range = $range.range;
    } 
    |
    KW_INTEGER 
    {
        $isInteger = true;
    }
    |
    KW_REAL
    {
        $isReal = true;
    }
    |
    KW_REALTIME
    {
        $isRealTime = true;
    }
    ;

tf_declaration :
    p1=parameter_declaration
    {
        FVPDescription desc = FVPDescription.instance();
        FVPTaskOrFunctionDecl tf = (FVPTaskOrFunctionDecl)desc.getCurrentscope();
        if ( null != $p1.params )
        {
            Iterator itr = $p1.params.iterator(); 
            while(itr.hasNext()) 
            {
                FVPParamDecl param = (FVPParamDecl)itr.next();
                if ( null != $p1.range )
                {
                    param.setRange($p1.range);
                }
                tf.addParamDecl(param);
            }
        } 
    }
    |
    input_declaration
    |
    output_declaration
    |
    inout_declaration 
    |
    reg_declaration  
    |
    time_declaration 
    |
    integer_declaration 
    |
    real_declaration 
    |
    event_declaration 
    ;


//----------------------------------------------------------------------------
// Declarations
//----------------------------------------------------------------------------

parameter_declaration returns [Integer line, 
                               LinkedList<FVPParamDecl> params,
                               FVPRange range 
                               ]:
    parameter_declaration_no_semi SEMI
    {
        $line = $parameter_declaration_no_semi.line;
        $params = $parameter_declaration_no_semi.params;
        $range = $parameter_declaration_no_semi.range;
    }
    ;

parameter_declaration_no_semi returns [Integer line, 
                               LinkedList<FVPParamDecl> params,
                               FVPRange range 
                               ]
    @init
    {
      boolean isInteger = false;
      boolean isReal = false;
      boolean isSigned = false;
      FVPComment comment = null;
    }                               
    :
    l1=KW_PARAMETER
    {
        if ( true == FVPDescription.instance().isStoreComment() )
        {
            comment = FVPDescription.instance().getComment( $l1.getLine() );
        }
    }
       (
       KW_INTEGER { isInteger = true;}
       |
       'signed' { isSigned = true;}
       |
       KW_REAL { isReal = true;}
       )?
       (r1=range {$range = $r1.range;} )? 
       la1=list_of_param_assignments
    {
        $line = $l1.line;
        $params = $la1.params;
      Iterator<FVPParamDecl> itr = $params.iterator();
      while ( itr.hasNext() )
      { 
        FVPParamDecl param = itr.next();
        if ( null == param )
        {
          continue;
        }
        if ( isInteger )
        {
          param.setType( FVPParamDecl.ParamType.INTEGER );
        }
        if ( isReal )
        {
          param.setType( FVPParamDecl.ParamType.REAL );
        }
        if ( isSigned )
        {
          param.setType( FVPParamDecl.ParamType.SIGNED );
        }
        FVPDescription.instance().addDeclaration( param );
        FVPDescription.instance().insertSymbol(param.getName(), param );
        if ( true == FVPDescription.instance().isStoreComment() )
        {
            param.setComment( comment );
        }
      }
    }
    ;


localparam_declaration returns [Integer line, 
                               LinkedList<FVPLocalParam> params,
                               FVPRange range 
                               ]:
    localparam_declaration_no_semi SEMI
    {
        $line = $localparam_declaration_no_semi.line;
        $params = $localparam_declaration_no_semi.params;
        $range = $localparam_declaration_no_semi.range;
    }
    ;

localparam_declaration_no_semi returns [Integer line, 
                               LinkedList<FVPLocalParam> params,
                               FVPRange range 
                               ]:
    l1=KW_LOCALPARAM (r1=range {$range = $r1.range;} )? 
       list_of_localparam_assignments
    {
        $line = $l1.line;
        $params = $list_of_localparam_assignments.params;
        Iterator<FVPLocalParam> itr = $params.iterator();
        while ( itr.hasNext() )
        { 
            FVPLocalParam param = itr.next();
            FVPDescription.instance().addDeclaration( param );
            FVPDescription.instance().insertSymbol(param.getName(), param );
        }
    }
    ;

list_of_param_assignments returns [Integer line, 
                                   LinkedList<FVPParamDecl> params
                                  ]:
    p1=param_assignment
    {
        $line = $p1.line;
        if (null == $params )
        {
            $params = new LinkedList<FVPParamDecl>();
        }
        $params.add($p1.paramdecl);
    }
    ( COMMA 
        p2=param_assignment 
        {
            $params.add($p2.paramdecl);
        }
    )*
    ;
    
list_of_localparam_assignments returns [Integer line, 
                                   LinkedList<FVPLocalParam> params
                                  ]:
    p1=localparam_assignment
    {
        $line = $p1.line;
        if (null == $params )
        {
            $params = new LinkedList<FVPLocalParam>();
        }
        $params.add($p1.localparam);
    }
    ( COMMA 
        p2=localparam_assignment 
        {
            $params.add($p2.localparam);
        }
    )*
    ;

param_assignment returns [ Integer line, FVPParamDecl paramdecl ]:
    identifier ASSIGN expression
   {
        FVPDescription desc = FVPDescription.instance();
        FVPBase sym = desc.getSymbol($identifier.id);
        if (null != sym && desc.getScope() == sym.getScope() )
        {
            LogMessages.instance().print(1831, LogMessages.MSG_SEVERITY.ERROR, "Symbol '"+ $identifier.id + "' at '" + FVPDescription.instance().getFile() + "':" + $identifier.line + "' is already defined at " +
                           sym.getFile() + ":" + sym.getLine() );
        }
        $line = $identifier.line;
        FVPIdentifier id = new FVPIdentifier($line, $identifier.id );
        $paramdecl = new FVPParamDecl( $line, id, $expression.expr );
        FVPDescription.instance().insertSymbol($identifier.id, $paramdecl );
    }
    ;

localparam_assignment returns [ Integer line, FVPLocalParam localparam ]:
    identifier ASSIGN expression
   {
        FVPDescription desc = FVPDescription.instance();
        FVPBase sym = desc.getSymbol($identifier.id);
        if (null != sym && desc.getScope() == sym.getScope() )
        {
            LogMessages.instance().print(1831, LogMessages.MSG_SEVERITY.ERROR, "Symbol '"+ $identifier.id + "' at '" + FVPDescription.instance().getFile() + "':" + $identifier.line + "' is already defined at " +
                           sym.getFile() + ":" + sym.getLine() );
        }
        $line = $identifier.line;
        FVPIdentifier id = new FVPIdentifier($line, $identifier.id );
        $localparam = new FVPLocalParam( $line, id, $expression.expr );
        FVPDescription.instance().insertSymbol($identifier.id, $localparam );
    }
    ;


input_declaration_no_semi returns [ Integer line ]
    @init
    {
        FVPPort.ObjClass portClass = FVPPort.ObjClass.UNKNOWN;
        boolean isSigned= false;
    }
  :
    KW_INPUT
    (
        (
          KW_WIRE
          {
            portClass = FVPPort.ObjClass.WIRE;
          }
        )
        |
        (
          KW_REG
          {
            portClass = FVPPort.ObjClass.REG;
          }
        )
        |
        (
          KW_INTEGER
          {
            portClass = FVPPort.ObjClass.INTEGER;
          }
        )
        |
        (
          KW_REAL
          {
            portClass = FVPPort.ObjClass.REAL;
          }
        )
     )?
    (
      'signed'
      {
        isSigned = true;
      }
    )? 
    (
        range
    )? 
    list_of_variables
    {
        $line = $list_of_variables.line;
        processPortList( $list_of_variables.ids, 
                         $line, 
                         $range.range, 
                         FVPPort.Direction.INPUT, 
                         isSigned, portClass ); 
    }
    ;

output_declaration_no_semi returns [Integer line]
    @init
    {
        FVPPort.ObjClass portClass = FVPPort.ObjClass.UNKNOWN;
        boolean isSigned= false;
    }
    :
    (KW_OUTPUT (KW_REG)? ('signed')? (range)? IDENTIFIER ASSIGN) => output_declaration_with_assign
    |
    KW_OUTPUT 
    (
        (
          'wire'
          {
            portClass = FVPPort.ObjClass.WIRE;
          }
        )
        |
        (
          KW_REG
          {
            portClass = FVPPort.ObjClass.REG;
          }
        )
        |
        (
          KW_INTEGER
          {
            portClass = FVPPort.ObjClass.INTEGER;
          }
        )
        |
        (
          KW_REAL
          {
            portClass = FVPPort.ObjClass.REAL;
          }
        )
     )?    
    (
      'signed'
      {
        isSigned = true;
      }
    )?
    (
        range
    )? 
    list_of_variables
    {
        $line = $list_of_variables.line;
        processPortList( $list_of_variables.ids, 
                         $line, 
                         $range.range, 
                         FVPPort.Direction.OUTPUT, 
                         isSigned, 
                         portClass ); 
    }  
    ;

output_declaration_with_assign returns [ Integer line ]
    @init
    {
        FVPPort.ObjClass portClass = FVPPort.ObjClass.UNKNOWN;
        boolean isSigned= false;
    }
  :
    o1=KW_OUTPUT 
    (
        (
          KW_WIRE
          {
            portClass = FVPPort.ObjClass.WIRE;
          }
        )
        |
        (
          'reg'
          {
            portClass = FVPPort.ObjClass.REG;
          }
        )
        |
        (
          KW_INTEGER
          {
            portClass = FVPPort.ObjClass.INTEGER;
          }
        )
        |
        (
          KW_REAL
          {
            portClass = FVPPort.ObjClass.REAL;
          }
        )
     )?    
    (
      'signed'
      {
        isSigned = true;
      }
    )?
    (
        range
    )?
    (
        list_of_variables
        {
            $line = $list_of_variables.line;
            processPortList( $list_of_variables.ids, 
                             $line, 
                             $range.range, 
                             FVPPort.Direction.OUTPUT, 
                             isSigned, 
                             portClass ); 
        }   
        COMMA
    )*
    l2=list_of_assignments
    {
        $line = $o1.getLine();
        LinkedList<FVPNet> nets = $l2.nets;
        LinkedList<FVPLvalue> lvals = $l2.lvals;
        portDeclWithAssignments( $line, 
                                 nets, 
                                 lvals, 
                                 isSigned, 
                                 $range.range, 
                                 portClass, 
                                 $l2.assignments );
    }
    ;

inout_declaration_no_semi returns [Integer line]
  @init
    {
        FVPPort.ObjClass portClass = FVPPort.ObjClass.UNKNOWN;
        boolean isSigned= false;
    }
  :
    KW_INOUT 
     (
        (
          KW_WIRE
          {
            portClass = FVPPort.ObjClass.WIRE;
          }
        )
        |
        (
          KW_REG
          {
            portClass = FVPPort.ObjClass.REG;
          }
        )
        |
        (
          KW_INTEGER
          {
            portClass = FVPPort.ObjClass.INTEGER;
          }
        )
        |
        (
          KW_REAL
          {
            portClass = FVPPort.ObjClass.REAL;
          }
        )
     )?
    (
      'signed'
      {
        isSigned = true;
      }
    )?
    (
        range
    )? 
    list_of_variables
    {
        $line = $list_of_variables.line;
        processPortList( $list_of_variables.ids, 
                             $line, 
                             $range.range, 
                             FVPPort.Direction.INOUT, 
                             isSigned,
                             portClass );
    }
    ;



input_declaration returns [ Integer line ]
    :
      input_declaration_no_semi SEMI
      {
        $line = $input_declaration_no_semi.line;
      }
      ;


output_declaration returns [Integer line]
  : 
  output_declaration_no_semi SEMI
  {
    $line = $output_declaration_no_semi.line;
  }
    ;

inout_declaration returns [Integer line]
    :
    inout_declaration_no_semi SEMI
    {
        $line = $inout_declaration_no_semi.line;
    }
    ;

net_declaration returns [Integer line, LinkedList<FVPNet> nets,FVPRange r]
    @init
    {
        FVPRange rng = null;
        boolean isSigned = false;
        LinkedList<FVPRange> dimensions = null;
        LinkedList<FVPIdentifier> ids = null;
    }
    :
    (net_type ('signed')? (range)? IDENTIFIER ASSIGN) => net_declaration_with_assign
    |
    n1=net_type ('signed' { isSigned = true; })? (rng1=range{$r = $rng1.range; rng = $rng1.range; })? 
    vlist=list_of_net_variables
    {
        dimensions = $vlist.ranges;
        $line = $vlist.line;
        ids = $vlist.fvpids;;
        // Search in the symbol table and set the direction of the port
        if ( null != ids && false == ids.isEmpty() )
        {
	        Iterator<FVPIdentifier> itr = ids.iterator(); 
	        while(itr.hasNext()) 
	        {
	            FVPIdentifier symId = itr.next();
	            if ( null == symId )
	            {
	                continue;
	            }
	            String symbol = symId.getName();
	            // Cleanup this String object from the list
	            itr.remove(); 
	            FVPDescription desc = FVPDescription.instance();  
	            FVPBase val = desc.getSymbol( symbol );
	            if ( null != val  && desc.getScope() == val.getScope())
	            {                
	                if ( ( ( FVPBase.ObjType.PORT == val.getObjType() ) && 
	                         ( FVPNet.NetType.IMPLICIT_PORT_WIRE == 
	                         ((FVPPort)val).getNet().getType() ) ) ||
	                         desc.getScope() != val.getScope() )
	                {
	                    ((FVPPort)val).setPortClass( FVPPort.ObjClass.WIRE );
	                    ((FVPPort)val).getNet().setRange(rng);
	                    ((FVPPort)val).getNet().setDimensions( dimensions );
	                    ((FVPPort)val).getNet().setSigned( isSigned );
	                }
	            }
	            else
	            {
	                FVPIdentifier netname = symId;
	                FVPNet net = new FVPNet ( $line, netname, 
	                                              $n1.nettype, dimensions );
	                FVPDescription.instance().addDeclaration( net );
	                net.setRange( rng );
	                net.setSigned( isSigned );
	                FVPDescription.instance().insertSymbol( symbol, net );
	                if ( null == $nets )
	                {
	                    $nets = new LinkedList<FVPNet>();
	                }
	                $nets.add(net);
	            }
	        }
        }
    }
    (   COMMA 
        l2=list_of_assignments
        {
            LinkedList<FVPNet> nets = $l2.nets;
            LinkedList<FVPLvalue> lvals = $l2.lvals;
            
            $line = $l2.line;
            FVPDescription desc = FVPDescription.instance();
            if ( null != nets )
            {
              Iterator itr = nets.iterator();
              Iterator<FVPLvalue>  itr2 = lvals.iterator(); 
               
              while(itr.hasNext()) 
              {
                  FVPNet net = (FVPNet)itr.next();
                  FVPLvalue lv = itr2.next();
                  
                  if ( null == net )
                  {
                      continue;
                  }
                  String symbol = net.getName();
                  net.clearDummy();
                  net.getIdentifier().clearDummy();
                  net.setType($n1.nettype);
                  net.setRange(rng);
                  net.setDimensions( dimensions );
                  net.setSigned( isSigned );
                  if ( null == $nets )
                  {
                      $nets = new LinkedList<FVPNet>();
                  }
                  $nets.add(net);
                  ((FVPLvalue)lv).setActualObj( net );
                  FVPDescription.instance().addSymbol( symbol, net );
              }
            }
            if ( null != $l2.assignments && false == $l2.assignments.isEmpty() )
            {
                Iterator<FVPSeqAssignment> itr1 = $l2.assignments.iterator(); 
                while(itr1.hasNext()) 
                {
                    FVPSeqAssignment assgn = itr1.next();
                    if ( null != assgn )
                    {
                       assgn.setType( FVPAssignment.AssignmentType.CONTINUOUS );
                    }
                }
            }
        }
    )*
    SEMI
    ;

    
reg_declaration_with_assign returns [Integer line]
    @init
    {
        FVPRange rng = null;
        boolean isSigned = false;
    }
    :    
    rg=KW_REG 
    ('signed' { isSigned = true; })? 
    (range{  rng = $range.range; })? 
    l2=list_of_assignments
    {
      LinkedList<FVPNet> nets = $l2.nets;
      LinkedList<FVPLvalue> lvals = $l2.lvals;
      
      $line = $rg.getLine(); 
      FVPDescription desc = FVPDescription.instance();
      FVPModule mod = (FVPModule)desc.getCurrentscope();
      
      if ( null != nets )
      {
          Iterator<FVPNet>  itr = nets.iterator(); 
          Iterator<FVPLvalue>  itr2 = lvals.iterator(); 
          while(itr.hasNext()) 
          {
              FVPNet net = itr.next();
              FVPLvalue lv = itr2.next();
              if ( null == net )
              {
                  continue;
              }
              String symbol = net.getName();
              if ( null == $line )
              {
                $line = net.getLine();
              }
              mod.removeDeclaration(net);
              net.clearDummy();
              net.getIdentifier().clearDummy();
              FVPRegister reg = new FVPRegister ( $line , net.getIdentifier(), rng, null );
              mod.addDeclaration ( reg );
              reg.setSigned( isSigned );
//              FVPBase val = (FVPBase)FVPDescription.instance().getSymbol( symbol );
              ((FVPLvalue)lv).setActualObj( reg );
              FVPDescription.instance().addSymbol( symbol, reg );
          }
        }
        if ( null != $l2.assignments && false == $l2.assignments.isEmpty() )
        {
          // Now create an initial block and put these assignments as part of that
            FVPInitialBlock initialBlk = new FVPInitialBlock( $line );
            Iterator<FVPSeqAssignment> itr1 = $l2.assignments.iterator(); 
            while(itr1.hasNext()) 
            {
                FVPSeqAssignment assgn = itr1.next();
                if ( null == assgn )
                {
                    continue;
                }
                assgn.setType( FVPAssignment.AssignmentType.BLOCKING );
                initialBlk.addSeqStmt( assgn );
                assgn.setScope( initialBlk );
                mod.removeStatement( assgn ); // It was added as cont assign 
            }
            mod.addStatement( initialBlk );
        }
    }
    SEMI
    ;

//real_declaration_with_assign returns [Integer line]
//    @init
//    {
//        boolean isRealTime = false;
//    }
//    :    
//    (rl1=KW_REAL | rl2=KW_REALTIME { isRealTime = true; } ) 
//    l2=list_of_assignments
//    {
//      LinkedList<FVPNet> nets = $l2.nets;
//      LinkedList<FVPLvalue> lvals = $l2.lvals;
//      
//      $line = ( null != $rl1 ) ? $rl1.getLine() : ( ($rl2 != null) ? $rl2.getLine() : -1 ); 
//      FVPDescription desc = FVPDescription.instance();
//      if ( null != nets )
//        {
//          Iterator<FVPNet>  itr = nets.iterator(); 
//          Iterator<FVPLvalue>  itr2 = lvals.iterator(); 
//          while(itr.hasNext()) 
//          {
//              FVPNet net = itr.next();
//              FVPLvalue lv = itr2.next();
//              if ( null == net )
//              {
//                  continue;
//              }
//              String symbol = net.getName();
//              if ( null == $line )
//              {
//                $line = net.getLine();
//              }
//              FVPDescription.instance().removeDeclaration(net);
//              net.clearDummy();
//              net.getIdentifier().clearDummy();
//              FVPReal reg = new FVPReal ( $line , net.getIdentifier(), null);
//              FVPDescription.instance().addDeclaration ( reg );
//              ((FVPLvalue)lv).setActualObj( reg );
//              FVPDescription.instance().addSymbol( symbol, reg );
//              reg.setRealTime( isRealTime );
//          }
//        }
//        if ( null != $l2.assignments && false == $l2.assignments.isEmpty() && FVPDescription.instance().getCurrentscope().getObjType() == FVPBase.ObjType.MODULE )
//        {
//          // Now create an initial block and put these assignments as part of that
//            FVPModule mod = (FVPModule)FVPDescription.instance().getCurrentscope();
//            FVPInitialBlock initialBlk = new FVPInitialBlock( $line );
//            Iterator<FVPSeqAssignment> itr1 = $l2.assignments.iterator(); 
//            while(itr1.hasNext()) 
//            {
//                FVPSeqAssignment assgn = itr1.next();
//                if ( null == assgn )
//                {
//                    continue;
//                }
//                assgn.setType( FVPAssignment.AssignmentType.BLOCKING );
//                initialBlk.addSeqStmt( assgn );
//                assgn.setScope( initialBlk );
//                mod.removeStatement( assgn ); // It was added as cont assign 
//            }
//            mod.addStatement( initialBlk );
//        }
//    }
//    SEMI
//    ;

integer_declaration_with_assign returns [Integer line]
    :    
    rg=KW_INTEGER l2=list_of_assignments
    {
      LinkedList<FVPNet> nets = $l2.nets;
      LinkedList<FVPLvalue> lvals = $l2.lvals;
      
      $line = $rg.getLine(); 
      FVPDescription desc = FVPDescription.instance();
      if ( null != nets )
        {
          Iterator<FVPNet>  itr = nets.iterator(); 
          Iterator<FVPLvalue>  itr2 = lvals.iterator(); 
          while(itr.hasNext()) 
          {
              FVPNet net = itr.next();
              FVPLvalue lv = itr2.next();
              if ( null == net )
              {
                  continue;
              }
              String symbol = net.getName();
              if ( null == $line )
              {
                $line = net.getLine();
              }
              FVPDescription.instance().removeDeclaration(net);
              net.clearDummy();
              net.getIdentifier().clearDummy();
              FVPInteger reg = new FVPInteger ( $line , net.getIdentifier(), null);
              FVPDescription.instance().addDeclaration ( reg );
              ((FVPLvalue)lv).setActualObj( reg );
              FVPDescription.instance().addSymbol( symbol, reg );
          }
        }
        if ( null != $l2.assignments && false == $l2.assignments.isEmpty() && FVPDescription.instance().getCurrentscope().getObjType() == FVPBase.ObjType.MODULE )
        {
          // Now create an initial block and put these assignments as part of that
            FVPModule mod = (FVPModule)FVPDescription.instance().getCurrentscope();
            FVPInitialBlock initialBlk = new FVPInitialBlock( $line );
            Iterator<FVPSeqAssignment> itr1 = $l2.assignments.iterator(); 
            while(itr1.hasNext()) 
            {
                FVPSeqAssignment assgn = itr1.next();
                if ( null == assgn )
                {
                    continue;
                }
                assgn.setType( FVPAssignment.AssignmentType.BLOCKING );
                initialBlk.addSeqStmt( assgn );
                assgn.setScope( initialBlk );
                mod.removeStatement( assgn ); // It was added as cont assign 
            }
            mod.addStatement( initialBlk );
        }
    }
    SEMI
    ;
    
net_declaration_with_assign returns [Integer line, LinkedList<FVPNet> nets,FVPRange r]
    @init
    {
        FVPRange rng = null;
        boolean isSigned = false;
    }
    :    
    n2=net_type (drive_strength)? 
    ('signed' { isSigned = true; })? (expandrange{$r = $expandrange.range; rng = $expandrange.range; })? 
    (delay)? 
    l2=list_of_assignments
    {
        LinkedList<FVPNet> nets = $l2.nets;
        LinkedList<FVPLvalue> lvals = $l2.lvals;
        
        $line = $l2.line;
        FVPDescription desc = FVPDescription.instance();
        if ( null != nets )
        {
          Iterator itr = nets.iterator();
          Iterator<FVPLvalue>  itr2 = lvals.iterator(); 
           
          while(itr.hasNext()) 
          {
              FVPNet net = (FVPNet)itr.next();
              FVPLvalue lv = itr2.next();
              
              if ( null == net )
              {
                  continue;
              }
              String symbol = net.getName();
              net.clearDummy();
              net.getIdentifier().clearDummy();
              net.setType($n2.nettype);
              net.setRange(rng);
              net.setSigned( isSigned );
              if ( null == $nets )
              {
                  $nets = new LinkedList<FVPNet>();
              }
              $nets.add(net);
              ((FVPLvalue)lv).setActualObj( net );
              FVPDescription.instance().addSymbol( symbol, net );
          }
        }
        if ( null != $l2.assignments && false == $l2.assignments.isEmpty() )
        {
            Iterator<FVPSeqAssignment> itr1 = $l2.assignments.iterator(); 
            while(itr1.hasNext()) 
            {
                FVPSeqAssignment assgn = itr1.next();
                if ( null == assgn )
                {
                    continue;
                }
                FVPExprBase rval = assgn.getRvalue();
                FVPBase lval = assgn.getLvalue();
                FVPBase actualObj = ((FVPLvalue)lval).getActualObj();
                if ( null == actualObj )
                {
                    actualObj = lval;
                }
                switch (actualObj.getObjType())
                {
                    case NET:
	                    ((FVPNet)actualObj).setInitialValue( rval );
	                    break;
                    case PORT:
	                    ((FVPPort)actualObj).setInitialValue( rval );
	                    break;
                    case REGISTER:
	                    ((FVPRegister)actualObj).setInitialValue( rval );
	                    break;
                }
                FVPBase mod_or_tf = FVPDescription.instance().getScope();                
                switch( mod_or_tf.getObjType() )
                {
                    case MODULE:
                       ((FVPModule)mod_or_tf).removeStatement( assgn );
                       break;
                    case UDP:
                        ((FVPUDP)mod_or_tf).removeStatement( assgn );
                        break;
                    case TASK_OR_FUNCTION_DECL:
                        ((FVPTaskOrFunctionDecl)mod_or_tf).removeStatement( assgn );
                        break;
                }  
            }
        }
    }
    SEMI
    ;

net_type returns [ FVPNet.NetType nettype ]:
    t=KW_WIRE { $nettype = FVPNet.NetType.WIRE; }
    | 
    'tri' { $nettype = FVPNet.NetType.TRI; }
    |
    'tri1' { $nettype = FVPNet.NetType.TRI1; }
    |
    'supply0' {$nettype = FVPNet.NetType.SUPPLY0; }
    |
    'wand' { $nettype = FVPNet.NetType.WAND; }
    |
    'triand' { $nettype = FVPNet.NetType.TRIAND; }
    |
    'tri0' {  $nettype = FVPNet.NetType.TRI0; }
    |
    'supply1' { $nettype = FVPNet.NetType.SUPPLY1; }
    |
    'wor' { $nettype = FVPNet.NetType.WOR; }
    |
    'trior' { $nettype = FVPNet.NetType.TRIOR; }
    |
    KW_TRIREG { $nettype = FVPNet.NetType.TRIREG; }
    ;

expandrange returns [FVPRange range]:
    'scalared' range
    {
        LogMessages.instance().print(1842,
                  LogMessages.MSG_SEVERITY.ERROR,
                  "Unsuported expanded range : 'scalared' range" );
    } 
    |
    'vectored' range 
    {
        LogMessages.instance().print(1843,
                  LogMessages.MSG_SEVERITY.ERROR,
                  "Unsuported expanded range : 'vectored' range" );
    }
    |
    r1=range
    {
        $range = $r1.range;
    }
    ;

reg_declaration returns [Integer line, 
                         LinkedList<FVPRegister> regs,
                         LinkedList<FVPMemory> mems, FVPRange r1] 
    @init
    {
        FVPRange rng = null;
        boolean isSigned = false;
    }
    :
    (KW_REG ('signed')? (range)? IDENTIFIER ASSIGN) => reg_declaration_with_assign
    |
    rg=KW_REG ('signed' { isSigned = true; })? (range{ $r1 = $range.range; rng = $range.range; })? 
    l1=list_of_register_variables
    {
        $line = $l1.line;
        Iterator itr = $l1.fvpids.iterator();
        $regs = null;
        $mems = null;
        while(itr.hasNext()) 
        {
            FVPIdentifier id = (FVPIdentifier)itr.next(); 
            FVPDescription desc = FVPDescription.instance();
            FVPBase sym = (FVPBase)desc.getSymbol( id.getName() );
            FVPBase mod_or_tf = desc.getScope();
            if ( null != sym  && desc.getScope() == sym.getScope())
            {
                if ( ( FVPBase.ObjType.PORT == sym.getObjType() ) && 
                     ( FVPNet.NetType.IMPLICIT_PORT_WIRE == 
                       ((FVPPort)sym).getNet().getType() ) )
                {
                    ((FVPPort)sym).setPortClass( FVPPort.ObjClass.REG );
                }
                else if ( desc.getScope() == sym.getScope() )
                {    
                    LogMessages.instance().print(1844,
                               LogMessages.MSG_SEVERITY.ERROR,
                               "Symbol '" + id.getName() + "' at '" + id.getFile() + ":" + id.getLine() +
                               "' is already defined in file " +
                                sym.getFile() + ":" + sym.getLine() );
                }
            }
            if ( null != $l1.range && ( null == $l1.ranges || true == $l1.ranges.isEmpty() ) )
            {
                FVPMemory mem = new FVPMemory($line, 
                                                 id,
                                                 $l1.range, 
                                                 $r1 );
                FVPDescription.instance().addDeclaration( mem );
                if ( null == $mems )
                {
                    $mems = new  LinkedList<FVPMemory>();
                }        
                $mems.add(mem);
                desc.insertSymbol( id.getName(), mem );
            }
            else
            {
                if ( null != $l1.ranges )
                {
                    $l1.ranges.add( $l1.range );
                }
                FVPRegister reg = new FVPRegister( $line, 
                                                      id, 
                                                   $r1,
                                                   $l1.ranges );
                FVPDescription.instance().addDeclaration( reg );
                FVPDescription.instance().insertSymbol(id.getName(), reg);
                
                if ( null == $regs )
                {
                    $regs = new  LinkedList<FVPRegister>();
                }
                reg.setSigned( isSigned );
                $regs.add(reg);
            }
        } 
    }
        (   COMMA
        l2=list_of_assignments
        {
          LinkedList<FVPNet> nets = $l2.nets;
          LinkedList<FVPLvalue> lvals = $l2.lvals;
          
          $line = $rg.getLine(); 
          FVPDescription desc = FVPDescription.instance();
          FVPModule mod = (FVPModule)desc.getCurrentscope();
          if ( null != nets )
            {
              Iterator<FVPNet>  itr = nets.iterator(); 
              Iterator<FVPLvalue>  itr2 = lvals.iterator(); 
              while(itr.hasNext()) 
              {
                  FVPNet net = itr.next();
                  FVPLvalue lv = itr2.next();
                  if ( null == net )
                  {
                      continue;
                  }
                  String symbol = net.getName();
                  if ( null == $line )
                  {
                    $line = net.getLine();
                  }
                  mod.removeDeclaration(net);
                  net.clearDummy();
                  net.getIdentifier().clearDummy();
                  FVPRegister reg = new FVPRegister ( $line , net.getIdentifier(), rng, null );
                  mod.addDeclaration ( reg );
                  reg.setSigned( isSigned );
//                  FVPBase val = (FVPBase)FVPDescription.instance().getSymbol( symbol );
                  ((FVPLvalue)lv).setActualObj( reg );
                  FVPDescription.instance().addSymbol( symbol, reg );
              }
            }
            if ( null != $l2.assignments && false == $l2.assignments.isEmpty() )
            {
              // Now create an initial block and put these assignments as part of that
                FVPInitialBlock initialBlk = new FVPInitialBlock( $line );
                Iterator<FVPSeqAssignment> itr1 = $l2.assignments.iterator(); 
                while(itr1.hasNext()) 
                {
                    FVPSeqAssignment assgn = itr1.next();
                    if ( null == assgn )
                    {
                       continue;
                    }
                    assgn.setType( FVPAssignment.AssignmentType.BLOCKING );
                    initialBlk.addSeqStmt( assgn );
                    assgn.setScope( initialBlk );
                    mod.removeStatement( assgn ); // It was added as cont assign 
                }
                mod.addStatement( initialBlk );
            }
        }
    )?
    SEMI
    ;

time_declaration :
    'time' list_of_register_variables SEMI
    {
        LogMessages.instance().print(1845,
                       LogMessages.MSG_SEVERITY.WARNING,
                       "Time type variables/objects found at " +
                       FVPDescription.instance().getFile() + 
                       ":" + $SEMI + " is not supported yet ...");
    }
    ;

integer_declaration returns [Integer line, 
                     LinkedList<FVPInteger> variables]  
    :
    (KW_INTEGER IDENTIFIER ASSIGN) => integer_declaration_with_assign
    |
    KW_INTEGER l1=list_of_register_variables SEMI
    {
        $line = $l1.line;
        LinkedList<FVPRange> dimensions = null;
        if ( null != $l1.range )
        {
            dimensions = new LinkedList<FVPRange>();
            dimensions.add( $l1.range );
        }
        if ( null != $l1.ranges && false == $l1.ranges.isEmpty() )
        {
            if ( null == dimensions )
            {
                dimensions = new LinkedList<FVPRange>();
            }
            Iterator<FVPRange> itr = $l1.ranges.iterator();
            while ( itr.hasNext() )
            {
                FVPRange rng = itr.next();
                if ( null == rng )
                {
                    continue;
                }
                dimensions.add( rng );
            }
        }
        $variables = new LinkedList<FVPInteger>();
        Iterator itr = $l1.fvpids.iterator();
        while(itr.hasNext()) 
        {
            FVPIdentifier id = (FVPIdentifier)itr.next(); 
            FVPDescription desc = FVPDescription.instance();
            FVPBase sym = (FVPBase)desc.getSymbol( id.getName() );
            if ( null != sym )
            {
                if ( ( FVPBase.ObjType.PORT == sym.getObjType() ) && 
                     ( FVPNet.NetType.IMPLICIT_PORT_WIRE == 
                       ((FVPPort)sym).getNet().getType() ) )
                {
                    ((FVPPort)sym).setPortClass( FVPPort.ObjClass.REG );
                }
                else if ( desc.getScope() == sym.getScope() )
                {    
                    LogMessages.instance().print(1846,
                               LogMessages.MSG_SEVERITY.ERROR,
                               "Symbol '" + id.getName() + "' at '" + id.getFile() + ":" + id.getLine() +
                               "' is already defined in file " +
                                sym.getFile() + ":" + sym.getLine() );
                }
            }
            
            FVPInteger variable = new FVPInteger( $line, id, dimensions );
            desc.insertSymbol( id.getName(), variable );
            FVPDescription.instance().addDeclaration( variable );
            $variables.add( variable );
        } 
    }
    ;


real_declaration returns [LinkedList<FVPReal> variables, Integer line]
  @init
  {
    boolean isRealTime = false;
    LinkedList<FVPRange> dimensions = null;
    LinkedList<FVPIdentifier> ids = null;
    
  } 
  :
//    (KW_REAL IDENTIFIER ASSIGN) => real_declaration_with_assign
//    |
//    (KW_REALTIME IDENTIFIER ASSIGN) => real_declaration_with_assign
//    |
    (KW_REAL | KW_REALTIME { isRealTime = true; } ) 
    vlist=list_of_net_variables SEMI
    {
        dimensions = $vlist.ranges;
        $line = $vlist.line;
        ids = $vlist.fvpids;
    
        $variables = new LinkedList<FVPReal>();
        if ( null != ids && false == ids.isEmpty() )
        {
	        Iterator<FVPIdentifier> itr = ids.iterator();
	        while(itr.hasNext()) 
	        {
	            FVPIdentifier id = itr.next();  
	            FVPDescription desc = FVPDescription.instance();
	            FVPBase sym = (FVPBase)desc.getSymbol( id.getName() );
	            if ( null != sym )
	            {
	                if ( ( FVPBase.ObjType.PORT == sym.getObjType() ) && 
	                     ( FVPNet.NetType.IMPLICIT_PORT_WIRE == 
	                       ((FVPPort)sym).getNet().getType() ) )
	                {
	                    ((FVPPort)sym).setPortClass( FVPPort.ObjClass.REG );
	                }
	                else  if ( desc.getScope() == sym.getScope() )
	                {    
	                    LogMessages.instance().print(1847,
	                               LogMessages.MSG_SEVERITY.ERROR,
	                               "Symbol '" + id.getName() + "' at '" + id.getFile() + ":" + id.getLine() +
	                               "' is already defined in file " +
	                                sym.getFile() + ":" + sym.getLine() );
	                }
	            }
	            
	            FVPReal variable = new FVPReal( $line, id, dimensions );
	            variable.setRealTime( isRealTime );
	            desc.insertSymbol( id.getName(), variable );
	            FVPDescription.instance().addDeclaration( variable );
	            $variables.add( variable );
	        }
        } 
    }
    ;

event_declaration :
    'event' name_of_event ( COMMA name_of_event )* SEMI
    {
        LogMessages.instance().print(1848,
                       LogMessages.MSG_SEVERITY.WARNING,
                       "Event type variables/objects found at " +
                       FVPDescription.instance().getFile() + 
                       ":" + $SEMI.getLine() + " is not supported yet ...");
    }
    ;

continuous_assign returns [ Integer line,
                            LinkedList<FVPSeqAssignment> assignments ]
    @init
    {
      FVPDelayControl del = null;
    }
    :   
                             
    KW_ASSIGN (drive_strength)? (delay { del = $delay.expr;})? (l1=list_of_assignments) SEMI
    {
        if ( false == $l1.assignments.isEmpty() )
        {
            FVPDescription desc = FVPDescription.instance();
            Iterator<FVPSeqAssignment> itr = $l1.assignments.iterator(); 
            while(itr.hasNext()) 
            {
                FVPSeqAssignment assgn = itr.next();
                assgn.setType( FVPAssignment.AssignmentType.CONTINUOUS );
                assgn.setDelayOrControlAfterAssgnOp( del );
            }
            $assignments = $l1.assignments;
        }
    } 
    ;

parameter_override returns [LinkedList<FVPDefParam> defparams, Integer line] 
  :
    'defparam' l1=list_of_param_assignments SEMI
    {
      $line = $l1.line;
      $defparams = new LinkedList<FVPDefParam>();
      // We need to create the def params
      Iterator<FVPParamDecl> itr = $l1.params.iterator();
      while ( itr.hasNext() )
      { 
        FVPParamDecl param = itr.next();
        if ( null == param )
        {
          continue;
        }
        FVPDefParam defparam = new FVPDefParam( $l1.line, param.getName(), param.getValue() );
        FVPDescription.instance().addDeclaration( defparam );
        FVPDescription.instance().insertSymbol(defparam.getName(), defparam );
      }
    }
    ;

list_of_variables returns [ LinkedList<String> ids, Integer line]
    :
    id1=name_of_variable
    {
        if ( null == $ids )
        {
            $ids = new LinkedList<String>();
        }
        $ids.add( $id1.id );
        $line = $id1.line;
    }
    ( COMMA 
        id2=name_of_variable
        {
            if ( null == $ids )
            {
                $ids = new LinkedList<String>();
            }
            $ids.add( $id2.id );
            $line = $id2.line;
        }
    )*
    |
    def1=DEFINE
    {
        $line = $def1.line;
        StringBuffer buf = new StringBuffer($def1.text);
        StringBuffer modstr = buf.deleteCharAt(0);
        String str = modstr.toString();
        FVPDescription desc = FVPDescription.instance();
        FVPBase sym = (FVPBase)desc.getSymbol(str);
        if ( null == sym )
        {
            LogMessages.instance().print(1849,
                                LogMessages.MSG_SEVERITY.ERROR,
                                "Undefined symbol `"+ str + 
                                " found in file " + desc.getFile() +
                                ":" + $line );
        } 
        if ( null != sym && FVPBase.ObjType.DEFINE_DIRECTIVE != sym.getObjType() )
        {
            LogMessages.instance().print(1850,
                                LogMessages.MSG_SEVERITY.ERROR,
                                "The symbol `"+ str + " in file " + 
                                desc.getFile() + ":" + sym.getLine() + 
                                " is not a macro ..."  );
        }
        FVPDefine def = (FVPDefine)sym;
    
        if ( null == $ids )
        {
            $ids = new LinkedList<String>();
        }
        
        if ( FVPExprBase.ExprType.LVALUE == def.getValue().getType() )
        {
            FVPBase actual = ((FVPLvalue)((FVPExprLvalue)def.getValue()).getLvalue()).getActualObj();
            FVPBase.ObjType type =  actual.getObjType();
            switch ( type)
            {
                case IDENTIFIER:
                    actual.clearDummy();
                    $ids.add(((FVPIdentifier)actual).getName());
                    break;
                default:
                    LogMessages.instance().print(1851,
                                LogMessages.MSG_SEVERITY.PROGRAMMING_ERROR,
                                "Undefined object type found as directive in the identifier list "+
                                type.name() + " in file " + 
                                desc.getFile() + ":" + $line + 
                                " ..."  );
                    break;
            }
        }       
    }
    ( COMMA def2=DEFINE
    {
        $line = $def2.line;
        StringBuffer buf = new StringBuffer($def2.text);
        StringBuffer modstr = buf.deleteCharAt(0);
        String str = modstr.toString();
        FVPDescription desc = FVPDescription.instance();
        FVPBase sym = (FVPBase)desc.getSymbol(str);
        if ( null == sym )
        {
            LogMessages.instance().print(1852,
                                LogMessages.MSG_SEVERITY.ERROR,
                                "Undefined symbol `"+ str + 
                                " found in file " + desc.getFile() +
                                ":" + $line );
        } 
        if ( null != sym && FVPBase.ObjType.DEFINE_DIRECTIVE != sym.getObjType() )
        {
            LogMessages.instance().print(1853,
                                LogMessages.MSG_SEVERITY.ERROR,
                                "The symbol `"+ str + " in file " + 
                                desc.getFile() + ":" + sym.getLine() + 
                                " is not a macro ..."  );
        }
        FVPDefine def = (FVPDefine)sym;
    
          if ( null == $ids )
        {
            $ids = new LinkedList<String>();
        }
        
        if ( FVPExprBase.ExprType.LVALUE == def.getValue().getType() )
        {
            FVPBase actual = ((FVPLvalue)((FVPExprLvalue)def.getValue()).getLvalue()).getActualObj();
            FVPBase.ObjType type =  actual.getObjType();
            switch ( type)
            {
                case IDENTIFIER:
                    $ids.add(((FVPIdentifier)actual).getName());
                    break;
                default:
                    LogMessages.instance().print(1854,
                                LogMessages.MSG_SEVERITY.PROGRAMMING_ERROR,
                                "Undefined object type found as directive in the identifier list "+
                                type.name() + " in file " + 
                                desc.getFile() + ":" + $line + 
                                " ..."  );
                    break;
            }
        }       
    }
    )*
    ;

list_of_assigned_variables returns [Integer line, 
                LinkedList<FVPIdentifier> fvpids,
                FVPIdentifier fvpid1,
                FVPIdentifier fvpid2 ]
    :
    (v1=name_of_variable
    {
        $line = $v1.line;
        $fvpid1 = new FVPIdentifier($line, $v1.id);
    }
    ( 
        (ASSIGN e1=expression) 
        {
            $fvpid1.setDefaultValue($e1.expr);
        }
    )?
    )
    {
        if ( null == $fvpids )
        {
            $fvpids = new LinkedList<FVPIdentifier>();
        }
        $fvpids.add($fvpid1);
    }
    ( COMMA
        ( 
            v2=name_of_variable
            {
                $line = $v2.line;
                $fvpid2 = new FVPIdentifier($line, $v2.id);
            } 
            ( 
                ASSIGN e2=expression 
                {
                    $fvpid2.setDefaultValue($e2.expr);
                }
            )?
        ) 
        {
            if ( null == $fvpids )
            {
                $fvpids = new LinkedList<FVPIdentifier>();
            }
            $fvpids.add($fvpid2);
        }
    )*
    ;

list_of_register_variables returns [Integer line, 
                LinkedList<FVPIdentifier> fvpids,
                FVPRange range,
                LinkedList<FVPRange> ranges ]
    :
    r1=register_variable
    {
        $line = $r1.line;
        if ( null == $fvpids )
        {
            $fvpids = new LinkedList<FVPIdentifier>();
        }
        $fvpids.add( $r1.fvpid );
        $range = $r1.range;
        $ranges = $r1.ranges;
    } 
    ( COMMA 
        r2=register_variable 
        {
            $line = $r2.line;
            if ( null == $fvpids )
            {
                $fvpids = new LinkedList<FVPIdentifier>();
            }
            $fvpids.add( $r2.fvpid );
            $range = $r2.range;
            $ranges = $r2.ranges;
        }
    )*
    |
    def1=DEFINE
    {
        $line = $def1.line;
        StringBuffer buf = new StringBuffer($def1.text);
        String str = buf.toString();
        if ( false == str.isEmpty() &&  str.charAt(0) == '`' )
        {
          StringBuffer modstr = buf.deleteCharAt(0);
          str = modstr.toString();
        }
        FVPDescription desc = FVPDescription.instance();
        FVPBase sym = (FVPBase)desc.getSymbol(str);
        if ( null == sym )
        {
            LogMessages.instance().print(1855,
                                LogMessages.MSG_SEVERITY.ERROR,
                                "Undefined symbol `"+ str + 
                                " found in file " + desc.getFile() +
                                ":" + $line );
        } 
        if ( null != sym && FVPBase.ObjType.DEFINE_DIRECTIVE != sym.getObjType() )
        {
            LogMessages.instance().print(1856,
                                LogMessages.MSG_SEVERITY.ERROR,
                                "The symbol `"+ str + " in file " + 
                                desc.getFile() + ":" + sym.getLine() + 
                                " is not a macro ..."  );
        }
        FVPDefine def = (FVPDefine)sym;
        if ( null == $fvpids )
        {
            $fvpids = new LinkedList<FVPIdentifier>();
        }
        if ( FVPExprBase.ExprType.LVALUE == def.getValue().getType() )
        {
            FVPBase actual = ((FVPLvalue)((FVPExprLvalue)def.getValue()).getLvalue()).getActualObj();
            FVPBase.ObjType type =  actual.getObjType();
            switch ( type)
            {
                case IDENTIFIER:
                    actual.clearDummy();
                    $fvpids.add((FVPIdentifier)actual);
                    break;
                case PORT:
                {
                    actual.clearDummy();
                    FVPIdentifier id = ((FVPPort)actual).getNet().getIdentifier();
                    $fvpids.add(id);
                    break;
                }
                default:
                    LogMessages.instance().print(1857,
                                LogMessages.MSG_SEVERITY.PROGRAMMING_ERROR,
                                "Undefined object type found as directive in the register list "+
                                type.name() + " in file " + 
                                desc.getFile() + ":" + $line + 
                                " ..."  );
                    break;
            }
        }
        else
        {
            LogMessages.instance().print(1858,
                                LogMessages.MSG_SEVERITY.PROGRAMMING_ERROR,
                                "Undefined expression type found as directive in the register list "+
                                def.getValue().getType().name() + " in file " + 
                                desc.getFile() + ":" + $line + 
                                " ..."  );

        }       
    }
    ( COMMA def2=DEFINE
    {
        $line = $def2.line;
        StringBuffer buf = new StringBuffer($def2.text);
        StringBuffer modstr = buf.deleteCharAt(0);
        String str = modstr.toString();
        FVPDescription desc = FVPDescription.instance();
        FVPBase sym = (FVPBase)desc.getSymbol(str);
        if ( null == sym )
        {
            LogMessages.instance().print(1859,
                                LogMessages.MSG_SEVERITY.ERROR,
                                "Undefined symbol `"+ str + 
                                " found in file " + desc.getFile() +
                                ":" + $line );
        } 
        if ( null != sym && FVPBase.ObjType.DEFINE_DIRECTIVE != sym.getObjType() )
        {
            LogMessages.instance().print(1860,
                                LogMessages.MSG_SEVERITY.ERROR,
                                "The symbol `"+ str + " in file " + 
                                desc.getFile() + ":" + sym.getLine() + 
                                " is not a macro ..."  );
        }
        FVPDefine def = (FVPDefine)sym;
        if ( null == $fvpids )
        {
            $fvpids = new LinkedList<FVPIdentifier>();
        }
        if ( FVPExprBase.ExprType.LVALUE == def.getValue().getType() )
        {
            FVPBase actual = ((FVPLvalue)((FVPExprLvalue)def.getValue()).getLvalue()).getActualObj();
            FVPBase.ObjType type =  actual.getObjType();
            switch ( type)
            {
                case IDENTIFIER:
                    actual.clearDummy();
                    $fvpids.add((FVPIdentifier)actual);
                    break;
                case PORT:
                {
                    actual.clearDummy();
                    FVPIdentifier id = ((FVPPort)actual).getNet().getIdentifier();
                    $fvpids.add(id);
                    break;
                }
                default:
                    LogMessages.instance().print(1861,
                                LogMessages.MSG_SEVERITY.PROGRAMMING_ERROR,
                                "Undefined object type found as directive in the register list "+
                                type.name() + " in file " + 
                                desc.getFile() + ":" + $line + 
                                " ..."  );
                    break;
            }
        }
        else
        {
            LogMessages.instance().print(1862,
                                LogMessages.MSG_SEVERITY.PROGRAMMING_ERROR,
                                "Undefined expression type found as directive in the register list "+
                                def.getValue().getType().name() + " in file " + 
                                desc.getFile() + ":" + $line + 
                                " ..."  );

        }               
    }    
    )*
    ;

register_variable returns [Integer line, FVPIdentifier fvpid, FVPRange range, LinkedList<FVPRange> ranges] 
    :
    name_of_register 
    {
        $line = $name_of_register.line;
        $fvpid = new FVPIdentifier( $name_of_register.line,
                                    $name_of_register.id );
        $range = null;
    }
    |
    name_of_memory LBRACK e1=expression COLON e2=expression RBRACK
    {
        $line = $name_of_memory.line;
        $fvpid = new FVPIdentifier( $name_of_memory.line,
                                    $name_of_memory.id );
        $range = new FVPRange( $e1.expr, $e2.expr, $line );
    }
    ( LBRACK e3=expression COLON e4=expression RBRACK
    {
        if ( null == $ranges )
        {
             $ranges = new LinkedList<FVPRange>();
        }
        $ranges.add( $range );
        $range = new FVPRange( $e3.expr, $e4.expr, $line );
    } )*
    ;


list_of_net_variables returns [Integer line, 
                LinkedList<FVPIdentifier> fvpids,
                LinkedList<FVPRange> ranges ]
    :
    r1=net_variable
    {
        $line = $r1.line;
        if ( null == $fvpids )
        {
            $fvpids = new LinkedList<FVPIdentifier>();
        }
        $fvpids.add( $r1.fvpid );
        $ranges = $r1.ranges;
    } 
    ( COMMA 
        r2=net_variable 
        {
            $line = $r2.line;
            if ( null == $fvpids )
            {
                $fvpids = new LinkedList<FVPIdentifier>();
            }
            $fvpids.add( $r2.fvpid );
            $ranges = $r2.ranges;
        }
    )*
    |
    def1=DEFINE
    {
        $line = $def1.line;
        StringBuffer buf = new StringBuffer($def1.text);
        String str = buf.toString();
        if ( false == str.isEmpty() &&  str.charAt(0) == '`' )
        {
          StringBuffer modstr = buf.deleteCharAt(0);
          str = modstr.toString();
        }
        FVPDescription desc = FVPDescription.instance();
        FVPBase sym = (FVPBase)desc.getSymbol(str);
        if ( null == sym )
        {
            LogMessages.instance().print(1855,
                                LogMessages.MSG_SEVERITY.ERROR,
                                "Undefined symbol `"+ str + 
                                " found in file " + desc.getFile() +
                                ":" + $line );
        } 
        if ( null != sym && FVPBase.ObjType.DEFINE_DIRECTIVE != sym.getObjType() )
        {
            LogMessages.instance().print(1856,
                                LogMessages.MSG_SEVERITY.ERROR,
                                "The symbol `"+ str + " in file " + 
                                desc.getFile() + ":" + sym.getLine() + 
                                " is not a macro ..."  );
        }
        FVPDefine def = (FVPDefine)sym;
        if ( null == $fvpids )
        {
            $fvpids = new LinkedList<FVPIdentifier>();
        }
        if ( FVPExprBase.ExprType.LVALUE == def.getValue().getType() )
        {
            FVPBase actual = ((FVPLvalue)((FVPExprLvalue)def.getValue()).getLvalue()).getActualObj();
            FVPBase.ObjType type =  actual.getObjType();
            switch ( type)
            {
                case IDENTIFIER:
                    actual.clearDummy();
                    $fvpids.add((FVPIdentifier)actual);
                    break;
                case PORT:
                {
                    actual.clearDummy();
                    FVPIdentifier id = ((FVPPort)actual).getNet().getIdentifier();
                    $fvpids.add(id);
                    break;
                }
                default:
                    LogMessages.instance().print(1857,
                                LogMessages.MSG_SEVERITY.PROGRAMMING_ERROR,
                                "Undefined object type found as directive in the register list "+
                                type.name() + " in file " + 
                                desc.getFile() + ":" + $line + 
                                " ..."  );
                    break;
            }
        }
        else
        {
            LogMessages.instance().print(1858,
                                LogMessages.MSG_SEVERITY.PROGRAMMING_ERROR,
                                "Undefined expression type found as directive in the register list "+
                                def.getValue().getType().name() + " in file " + 
                                desc.getFile() + ":" + $line + 
                                " ..."  );

        }       
    }
    ( COMMA def2=DEFINE
    {
        $line = $def2.line;
        StringBuffer buf = new StringBuffer($def2.text);
        StringBuffer modstr = buf.deleteCharAt(0);
        String str = modstr.toString();
        FVPDescription desc = FVPDescription.instance();
        FVPBase sym = (FVPBase)desc.getSymbol(str);
        if ( null == sym )
        {
            LogMessages.instance().print(1859,
                                LogMessages.MSG_SEVERITY.ERROR,
                                "Undefined symbol `"+ str + 
                                " found in file " + desc.getFile() +
                                ":" + $line );
        } 
        if ( null != sym && FVPBase.ObjType.DEFINE_DIRECTIVE != sym.getObjType() )
        {
            LogMessages.instance().print(1860,
                                LogMessages.MSG_SEVERITY.ERROR,
                                "The symbol `"+ str + " in file " + 
                                desc.getFile() + ":" + sym.getLine() + 
                                " is not a macro ..."  );
        }
        FVPDefine def = (FVPDefine)sym;
        if ( null == $fvpids )
        {
            $fvpids = new LinkedList<FVPIdentifier>();
        }
        if ( FVPExprBase.ExprType.LVALUE == def.getValue().getType() )
        {
            FVPBase actual = ((FVPLvalue)((FVPExprLvalue)def.getValue()).getLvalue()).getActualObj();
            FVPBase.ObjType type =  actual.getObjType();
            switch ( type)
            {
                case IDENTIFIER:
                    actual.clearDummy();
                    $fvpids.add((FVPIdentifier)actual);
                    break;
                case PORT:
                {
                    actual.clearDummy();
                    FVPIdentifier id = ((FVPPort)actual).getNet().getIdentifier();
                    $fvpids.add(id);
                    break;
                }
                default:
                    LogMessages.instance().print(1861,
                                LogMessages.MSG_SEVERITY.PROGRAMMING_ERROR,
                                "Undefined object type found as directive in the register list "+
                                type.name() + " in file " + 
                                desc.getFile() + ":" + $line + 
                                " ..."  );
                    break;
            }
        }
        else
        {
            LogMessages.instance().print(1862,
                                LogMessages.MSG_SEVERITY.PROGRAMMING_ERROR,
                                "Undefined expression type found as directive in the register list "+
                                def.getValue().getType().name() + " in file " + 
                                desc.getFile() + ":" + $line + 
                                " ..."  );

        }               
    }    
    )*
    ;
    
net_variable returns [Integer line, FVPIdentifier fvpid, LinkedList<FVPRange> ranges] 
    :
    name_of_net
    {
        $line = $name_of_net.line;
        $fvpid = new FVPIdentifier( $name_of_net.line,
                                    $name_of_net.id );
    }
    ( LBRACK e3=expression COLON e4=expression RBRACK
    {
        if ( null == $ranges )
        {
             $ranges = new LinkedList<FVPRange>();
        }
        FVPRange r = new FVPRange( $e3.expr, $e4.expr, $line );      
        $ranges.add( r );
    } )*
    ;
    
charge_strength :
    LPAREN 'small'  RPAREN 
    |
    LPAREN 'medium' RPAREN 
    |
    LPAREN 'large'  RPAREN
    ;

drive_strength :
    LPAREN strength0 COMMA strength1 RPAREN 
    {
        LogMessages.instance().print(1863,
                       LogMessages.MSG_SEVERITY.WARNING,
                       "drive_strength found at " +
                       FVPDescription.instance().getFile() + 
                       ":" + $RPAREN + " is not supported yet ...");
    }
    |
    LPAREN strength1 COMMA strength0 RPAREN
    {
        LogMessages.instance().print(1864,
                       LogMessages.MSG_SEVERITY.WARNING,
                       "drive_strength found at " +
                       FVPDescription.instance().getFile() + 
                       ":" + $RPAREN + " is not supported yet ...");
    }
    ;

strength0 :
    'supply0' 
    |
    'strong0' 
    |
    'pull0' 
    |
    'weak0' 
    |
    'highz0'
    ;

strength1 :
    'supply1' 
    |
    'strong1' 
    |
    'pull1' 
    |
    'weak1' 
    |
    'highz1'
    ;


range returns [
                FVPRange range, FVPExprBase left, 
                FVPExprBase right,Integer line
                ]
    @init
    {
       FVPRange.ConsecutiveType consec = FVPRange.ConsecutiveType.NONE;
    }
    :
    (LBRACK expression (PLUS|MINUS)? COLON) => 
    LBRACK lt= expression
    {
        $left=$lt.expr;
        $line = $lt.line;
    } 
    (
        PLUS {consec = FVPRange.ConsecutiveType.PLUS;}
        |
        MINUS {consec = FVPRange.ConsecutiveType.MINUS;}
    )?
    COLON 
    rt=expression
    {
        $right=$rt.expr;
    } 
    RBRACK 
    {
        $range = new FVPRange( $lt.expr, $rt.expr, $lt.line);
        $range.setConsecutiveType ( consec );
    }
    |
    LBRACK bit=expression RBRACK
    {
        $left = $bit.expr;
        $right = null;
        $line = $bit.line;
    }
    ;

    
list_of_assignments returns [
                             Integer line, 
                             LinkedList<FVPSeqAssignment> assignments,
                             LinkedList<FVPNet> nets,
                             LinkedList<FVPLvalue> lvals  
                             ]
    :
    asgn1=assignment
    {
        if ( null == $assignments )
        {
            $assignments = new LinkedList<FVPSeqAssignment>();
        }
        $assignments.add($asgn1.assgn);
        if ( null != $asgn1.dummyNet )
        {
            if ( null == $nets )
            {
                $nets = new LinkedList<FVPNet>();
                $lvals = new LinkedList<FVPLvalue>();
            }            
            FVPIdentifier id = (FVPIdentifier)$asgn1.dummyNet;
            if ( null == FVPDescription.instance().getSymbol(id.getName() ) )
            {
                id.clearDummy();
                FVPNet net = new FVPNet( $asgn1.line, id, FVPNet.NetType.UNKNOWN, null );
                FVPDescription.instance().addDeclaration( net );
                $nets.add(net);
                $lvals.add(  $asgn1.dummyLval );
                if ( null != $asgn1.dummyLval )
                {
                    $asgn1.dummyLval.setActualObj( net );
                }
            }
        }
        FVPDescription.instance().addStatement( $asgn1.assgn );
    } 
    ( 
        COMMA asgn2=assignment 
        {
            if ( null == $assignments )
            {
                $assignments = new LinkedList<FVPSeqAssignment>();
            }
            $assignments.add($asgn2.assgn);
            if ( null != $asgn2.dummyNet )
            {
                if ( null == $nets )
                {
                    $nets = new LinkedList<FVPNet>();
                    $lvals = new LinkedList<FVPLvalue>();
                }
                FVPIdentifier id = (FVPIdentifier)$asgn2.dummyNet;
                if ( null == FVPDescription.instance().getSymbol(id.getName() ) )
                {
                    id.clearDummy();
                    FVPNet net = new FVPNet( $asgn2.line, id, FVPNet.NetType.UNKNOWN, null );
                    FVPDescription.instance().addDeclaration( net );
                    $nets.add(net);
                    $lvals.add(  $asgn2.dummyLval );
                    if ( null != $asgn2.dummyLval )
                    {
                        $asgn2.dummyLval.setActualObj( net );
                    }
                }        
            }
            FVPDescription.instance().addStatement( $asgn2.assgn );
        }
    )*
    ;

//----------------------------------------------------------------------------
// Primitive Instances
//----------------------------------------------------------------------------

gate_declaration returns [LinkedList<FVPGateInstance> gateinsts]
    :
    gate_type (drive_strength)? (delay)?  
    inst1=gate_instance 
    {
        $inst1.inst.setGateType( $gate_type.type);
        $gateinsts = new LinkedList<FVPGateInstance>();
        $gateinsts.add( $inst1.inst ); 
    }
    ( COMMA inst2=gate_instance 
    {
        $inst2.inst.setGateType( $gate_type.type);
        if ( null != $gateinsts )
        {
            $gateinsts.add( $inst2.inst );
        } 
    }
    )* SEMI
    ;

gate_type returns [FVPGateInstance.GateType type]:
    'and' { $type = FVPGateInstance.GateType.FVP_PRIM_AND; }
    |
    'nand' { $type = FVPGateInstance.GateType.FVP_PRIM_NAND; } 
    |
    'or'   { $type = FVPGateInstance.GateType.FVP_PRIM_OR; }
    |
    'nor' { $type = FVPGateInstance.GateType.FVP_PRIM_NOR; }
    |
    'xor' { $type = FVPGateInstance.GateType.FVP_PRIM_XOR; }
    |
    'xnor' { $type = FVPGateInstance.GateType.FVP_PRIM_XNOR; }
    |
    'buf' { $type = FVPGateInstance.GateType.FVP_PRIM_BUF; }
    |
    'bufif0' { $type = FVPGateInstance.GateType.FVP_PRIM_BUFIF0; }
    |
    'bufif1' { $type = FVPGateInstance.GateType.FVP_PRIM_BUFIF1; }
    |
    'not' { $type = FVPGateInstance.GateType.FVP_PRIM_NOT; }
    |
    'notif0' { $type = FVPGateInstance.GateType.FVP_PRIM_NOTIF0; }
    |
    'notif1' { $type = FVPGateInstance.GateType.FVP_PRIM_NOTIF1; }
    |
    'pulldown' { $type = FVPGateInstance.GateType.FVP_PRIM_PULLDOWN; }
    |
    'pullup' { $type = FVPGateInstance.GateType.FVP_PRIM_PULLUP; }
    |
    'nmos' { $type = FVPGateInstance.GateType.FVP_PRIM_NMOS; }
    |
    'rnmos' { $type = FVPGateInstance.GateType.FVP_PRIM_RNMOS; }
    |
    'pmos' { $type = FVPGateInstance.GateType.FVP_PRIM_PMOS; }
    |
    'rpmos' { $type = FVPGateInstance.GateType.FVP_PRIM_RPMOS; }
    |
    'cmos' { $type = FVPGateInstance.GateType.FVP_PRIM_CMOS; }
    |
    'rcmos' { $type = FVPGateInstance.GateType.FVP_PRIM_RCMOS; }
    |
    'tran' { $type = FVPGateInstance.GateType.FVP_PRIM_TRAN; }
    |
    'rtran' { $type = FVPGateInstance.GateType.FVP_PRIM_RTRAN; }
    |
    'tranif0' { $type = FVPGateInstance.GateType.FVP_PRIM_TRANIF0; }
    |
    'rtranif0' { $type = FVPGateInstance.GateType.FVP_PRIM_RTRANIF0; }
    |
    'tranif1' { $type = FVPGateInstance.GateType.FVP_PRIM_TRANIF1; }
    |
    'rtranif1'{ $type = FVPGateInstance.GateType.FVP_PRIM_RTRANIF1; }
    ;

delay returns [FVPDelayControl expr]
    :
    POUND NUMBER 
    {
        FVPExprInteger val = new FVPExprInteger( $NUMBER.getLine(), Integer.parseInt( $NUMBER.text ) );
        $expr = new FVPDelayControl( $POUND.getLine(), val );
    }
    | 
    POUND identifier 
    {
        LogMessages.instance().print(1866,
                       LogMessages.MSG_SEVERITY.WARNING,
                       "Ignoring delay found at " +
                       FVPDescription.instance().getFile() + 
                       ":" + $POUND.getLine() + "...");
    }
    |
    POUND LPAREN mintypmax_expression
    (
        COMMA mintypmax_expression
        ( 
            COMMA mintypmax_expression 
        )?
    )?
    RPAREN
    {
        LogMessages.instance().print(1867,
                       LogMessages.MSG_SEVERITY.WARNING,
                       "Ignoring delay found at " +
                       FVPDescription.instance().getFile() + 
                       ":" + $POUND.getLine() + "...");
    }
    ;

gate_instance returns [ FVPGateInstance inst, FVPRange range] 
    @init
    {
        String name = null;
        LinkedList<FVPExprBase> terminals = new LinkedList<FVPExprBase>();
    }
    :
    (name_of_gate_instance { name = $name_of_gate_instance.id; } )?
    ( r1=range {$range = $r1.range;} )?  
    LPAREN t1=terminal { terminals.add( $t1.expr ); }
    ( COMMA t2=terminal { terminals.add( $t2.expr ); } )* RPAREN
    {
        $inst = new FVPGateInstance( $t1.line , name );
        $inst.setRange( $range );
        FVPDescription.instance().addStatement( $inst );
        $inst.setTerminals( terminals );
    }
    ;

udp_instance returns [FVPIdentifier id, 
                      LinkedList<FVPExprBase> terminals,
                      FVPRange range]:
    (
        name_of_UDP_instance
        {
            $id = new FVPIdentifier( $name_of_UDP_instance.line, $name_of_UDP_instance.id );
        }
    )?
    ( r1=range {$range = $r1.range;} )?
    LPAREN 
    t1=terminal
    {
        $terminals = new LinkedList<FVPExprBase>();
        $terminals.add( $t1.expr );
    } 
    ( COMMA 
        t2=terminal 
        {
            $terminals.add( $t2.expr );
        }
    )* RPAREN
    ;

terminal returns [FVPExprBase expr, Integer line]
    :
    expression
    {
        $expr = $expression.expr;
        $line = $expression.line;
    }
    // | IDENTIFIER
    ;
        
//----------------------------------------------------------------------------
// Module Instantiations
//----------------------------------------------------------------------------

module_instantiation returns [Integer line, FVPBase inst]
    @init
    {
        LinkedList<FVPParamMap> parammaps = null;
        boolean isDummyMod = false;  
    }
    :
    name_of_module 
    (
        p=list_of_parameter_value_assignment
        {
          $line = $p.line;
          LinkedList<FVPExprBase> actuals = ( null !=   $p.posParamConn ) ?
                                                        $p.posParamConn : 
                                                        $p.namedParamConn ;
          LinkedList<String> formalParams = $p.formalParams;                
          
          Iterator actualItr = actuals.iterator();
                                                        
          FVPDescription desc = FVPDescription.instance();
          
          
          FVPBase sym = desc.getRoot().findModuleOrUDP( $name_of_module.id );
          
          if ( null == sym )
          {
              FVPBase currScp = desc.popScope(); 
              FVPModule mod = new FVPModule( $name_of_module.id, $name_of_module.line );
              FVPDescription.instance().addDeclaration( mod );      
              isDummyMod = true;
              mod.setDummy();
              desc.insertSymbol( $name_of_module.id, mod);
              desc.pushScope(mod);
              LinkedList<FVPParamDecl> frmlParamList = new LinkedList<FVPParamDecl>();
              
              if ( ( null != actuals ) && ( false == actuals.isEmpty() ) )
              {
                  if ( null != formalParams && ( false == formalParams.isEmpty() ) )
                  {
                      Iterator itr = formalParams.iterator();
                      String name = null;
                      while(itr.hasNext()) 
                      {
                          name = (String)itr.next();
                          // Cleanup this String object from the list
                          itr.remove();
                          FVPIdentifier paramid = new FVPIdentifier ( 0, name );
                          
                          FVPParamDecl param = new FVPParamDecl ( 0, paramid, null );
                          FVPDescription.instance().addDeclaration( param );
                          param.setDummy(); // blackbox
                          frmlParamList.add(param);
                          desc.insertSymbol( name, param );
                      }
                  }
                  else
                  {
                      Integer index = 0;
                      Iterator itr = actuals.iterator();
                      while(itr.hasNext()) 
                      {
                          itr.next();
                          index++;
                          String name = $name_of_module.id + "_dummy_param_" + index;
                          FVPParamDecl param =  mod.findParameter( name );
                          if ( null == param )
                          {  
	                          FVPIdentifier paramid = new FVPIdentifier ( 0, name );
	                          param = new FVPParamDecl ( 0, paramid, null );                        
	                          FVPDescription.instance().addDeclaration( param );
	                          param.setDummy(); //blackbox
	                          frmlParamList.add(param);
	                          desc.insertSymbol( name, param );
                          }
                      }
                  }
              }
              mod.setParameters(frmlParamList);
              mod.setDummy();
              desc.popScope(); 
              desc.pushScope(currScp);
              sym = mod;       
          }
          
          List<FVPParamDecl> formals = null; 
          parammaps = new LinkedList<FVPParamMap>();
          
          switch( sym.getObjType() )
          {
              case MODULE:
                  formals = ((FVPModule)sym).getParamDecls();              
                  break;
              case UDP:
                  formals = null;
                  break;
              default:
                  LogMessages.instance().print(1, LogMessages.MSG_SEVERITY.ERROR, 
                                                  "Expecting Module or UDP but found object of type '" + 
                                                  sym.getObjType().name() + "' ...");
                  break;
          }
          if ( null != formalParams )
          {
            formals = new LinkedList<FVPParamDecl>();
          
            Iterator<String> fmlItr = formalParams.iterator();
            while ( fmlItr.hasNext() )
            {
                String pname = fmlItr.next();
                if ( null == pname || pname.isEmpty() )
                {
                    continue;
                }
                FVPParamDecl prm = ((FVPModule)sym).findParameter( pname );
                if ( null != prm )
                {
                    formals.add( prm );
                }
            }
           }
          
          Iterator<FVPParamDecl> formalItr = null;
          if ( null != formals )
          {
           formalItr = formals.iterator();
          }
          while( null != actualItr && actualItr.hasNext() ) 
          {
              FVPParamDecl formal = null;
              if ( null != formalItr && formalItr.hasNext() )
              {
                 formal = formalItr.next();
              }
              FVPExprBase actual = (FVPExprBase)actualItr.next();
              if ( null != formal && null != actual )
              {
               FVPParamMap pmap = new FVPParamMap( $line, formal, actual);
               parammaps.add ( pmap );
              }
          }        
        }
    )?
    inst1=module_instance 
    {
        $line = $inst1.line; 
        
        LinkedList<FVPExprBase> actualParams = ( null != $inst1.posParamConn ) ?
                                                         $inst1.posParamConn : 
                                                         $inst1.namedParamConn ;
       LinkedList<String> formalParams = $inst1.formalParams;                   
                                                          
       LinkedList<FVPExprBase> actuals = ( null !=   $inst1.posPortConn ) ?
                                                      $inst1.posPortConn : 
                                                      $inst1.namedPortConn ;
        
        
       LinkedList<String> formalPorts = $inst1.formalPorts;                
        
       Iterator actualItr = actuals.iterator();
                                                      
       FVPDescription desc = FVPDescription.instance();
        
        
        // To get the definition of the module, we need to search from the 
        // top in the symbol table
        FVPBase sym = desc.getRoot().findModuleOrUDP( $name_of_module.id );
        
        if ( null == sym || true == isDummyMod )
        {
            // Note down the current scope and come out of it
            // Create a new dummy module and then revert
            // back the scope.
            // This is very dangerous to play with the 
            // scope and hence be extremely careful while doing this.
            
            FVPBase currScp = desc.popScope();
            FVPModule mod = null;
            if ( null != sym )
            {
              mod = (FVPModule)sym;
            }
            else
            {
              mod = new FVPModule( $name_of_module.id, $name_of_module.line );
              FVPDescription.instance().addDeclaration( mod );      
            }      
            mod.setDummy();
            desc.insertSymbol( $name_of_module.id, mod);
            desc.pushScope(mod);
            
              
              if ( ( null != actualParams ) && ( false == actualParams.isEmpty() ) )
              {
                  parammaps = new LinkedList<FVPParamMap>();
                  // Required for the 2001 style param maps
                  LinkedList<FVPParamDecl> frmlParamList = new LinkedList<FVPParamDecl>();
                  if ( null != formalParams && ( false == formalParams.isEmpty() ) )
                  {
                      Iterator itr = formalParams.iterator();
                      String name = null;
                      while(itr.hasNext()) 
                      {
                          name = (String)itr.next();
                          // Cleanup this String object from the list
                          itr.remove();
                          FVPIdentifier paramid = new FVPIdentifier ( 0, name );
                          
                          FVPParamDecl param = new FVPParamDecl ( 0, paramid, null );
                          param.setDummy(); // blackbox
                          frmlParamList.add(param);
                          desc.insertSymbol( name, param );
                      }
                  }
                  else
                  {
                      Integer index = 0;
                      Iterator itr = actualParams.iterator();
                      while(itr.hasNext()) 
                      {
                          itr.next();
                          index++;
                          String name = $name_of_module.id + "_dummy_param_" + index;
                          FVPParamDecl param =  mod.findParameter( name );
                          if ( null == param )
                          {
	                          FVPIdentifier paramid = new FVPIdentifier ( 0, name );
	                          param = new FVPParamDecl ( 0, paramid, null );                        
	                          param.setDummy(); //blackbox
	                          frmlParamList.add(param);
	                          desc.insertSymbol( name, param );
                          }
                      }
                  }
                 mod.setParameters(frmlParamList);            
                List<FVPParamDecl> formals = frmlParamList;
                Iterator<FVPParamDecl> formalItr = null;
                if ( null != formals )
                {
                    formalItr = formals.iterator();
                }
                
                Iterator actualParamItr = actualParams.iterator();
                while( null != actualParamItr && actualParamItr.hasNext() ) 
                {
                    FVPParamDecl formal = null;
                    if ( null != formalItr && formalItr.hasNext() )
                    {
                        formal = formalItr.next();
                    }
                    FVPExprBase actual = (FVPExprBase)actualParamItr.next();
                    if ( null != formal && null != actual )
                    {
                        FVPParamMap pmap = new FVPParamMap( $line, formal, actual);
                        parammaps.add ( pmap );
                    }
                }        
            }
            
            // End of param processing  
            
            LinkedList<FVPPort> frmlPortList = new LinkedList<FVPPort>();
            LinkedList<FVPIdentifier> frmlPortIDList = 
                                          new LinkedList<FVPIdentifier>();
            if ( ( null != actuals ) && ( false == actuals.isEmpty() ) )
            {
                if ( null != formalPorts && ( false == formalPorts.isEmpty() ) )
                {
                    Iterator itr = formalPorts.iterator();
                    String name = null;
                    while(itr.hasNext()) 
                    {
                        name = (String)itr.next();
                        // Cleanup this String object from the list
                        itr.remove();
                        FVPIdentifier netname = new FVPIdentifier ( 0, name );
                        frmlPortIDList.add(netname);
                        FVPNet net = new FVPNet ( 0, netname, 
                                          FVPNet.NetType.IMPLICIT_PORT_WIRE, null );
                        FVPDescription.instance().addDeclaration( net );
                        // TODO: TBD: 
                        // Get the range of the actual expression and set that
                        // range for this dummy net
                        FVPPort port = new FVPPort ( net, 
                                                 FVPPort.Direction.INOUT, 0 );
                        port.setDummy(); // blackbox
                        frmlPortList.add(port);
                        desc.insertSymbol( name, port );
                    }
                }
                else
                {
                    Integer index = 0;
                    Iterator itr = actuals.iterator();
                    while(itr.hasNext()) 
                    {
                        itr.next();
                        index++;
                        String name = $name_of_module.id + "_dummy_port_" + index;
                        FVPIdentifier netname = new FVPIdentifier ( 0, name );
                        netname.setDummy();
                        frmlPortIDList.add(netname);
                        FVPNet net = new FVPNet ( 0, netname, 
                                         FVPNet.NetType.IMPLICIT_PORT_WIRE, null );
                        FVPDescription.instance().addDeclaration( net );
                        // TODO: TBD: 
                        // Get the range of the actual expression and set that
                        // range for this dummy net
                        FVPPort port = new FVPPort ( net, FVPPort.Direction.INOUT, 0 );
                        port.setDummy(); //blackbox
                        frmlPortList.add(port);
                        desc.insertSymbol( name, port );
                    }
                }
            }
            mod.setPorts(frmlPortList);
            mod.setPortIds(frmlPortIDList);
            mod.setDummy();
            desc.popScope(); 
            desc.pushScope(currScp);
            sym = mod;       
        }
        
        List<FVPPort> formals = null; //mod.getPorts();
        LinkedList<FVPPortMap> portmaps = new LinkedList<FVPPortMap>();
        
        switch( sym.getObjType() )
        {
            case MODULE:
                formals = ((FVPModule)sym).getPorts();
                $inst = new FVPModuleInstance($line, $inst1.id );
                ((FVPModuleInstance)$inst).setRange( $inst1.range );
                ((FVPModuleInstance)$inst).setMaster( (FVPModule)sym );
                ((FVPModuleInstance)$inst).setPortMaps( portmaps );               
                break;
            case UDP:
                $inst = new FVPUDPInstance($line, $inst1.id );
                ((FVPUDPInstance)$inst).setRange( $inst1.range );
                ((FVPUDPInstance)$inst).setMaster( (FVPUDP)sym );
                ((FVPUDPInstance)$inst).setPortMaps( portmaps );
                formals = ((FVPUDP)sym).getPorts();
                break;
            default:
                assert(false);
        }
        FVPDescription.instance().addStatement( $inst );
        Iterator itr = ( ( null != formalPorts ) && ( false == formalPorts.isEmpty() ) ) ? formalPorts.iterator() : null;
        Iterator formalItr = ( null == formals ) ? null : formals.iterator();
        while(actualItr.hasNext()) 
        {
            FVPPort formal = null;
            FVPExprBase actual = (FVPExprBase)actualItr.next();
            if ( ( null != formalPorts ) && ( false == formalPorts.isEmpty() ) )
            {
                String fpname = (String)itr.next();
                if ( FVPBase.ObjType.MODULE == sym.getObjType() )
                { 
                    formal = ((FVPModule)sym).getPort(fpname);
                }
                else
                {
                    formal = ((FVPUDP)sym).getPort(fpname);
                }
            }     
            else if ( formalItr.hasNext() )
            {
                formal = (FVPPort)formalItr.next(); 
            }
            if ( null == formal )
            {
              
            }
            FVPPortMap pmap = new FVPPortMap( $line, formal, actual);
            portmaps.add ( pmap );
        }
        if ( null != parammaps )
        {
            Iterator mapItr = parammaps.iterator();
            while(mapItr.hasNext()) 
            {
                FVPParamMap map = (FVPParamMap)mapItr.next();
                ((FVPModuleInstance)$inst).addParamMap( map );
            }
        }
    } 
    ( 
    COMMA inst2=module_instance 
    {
        $line = $inst2.line; 
        
        LinkedList<FVPExprBase> actualParams = ( null != $inst2.posParamConn ) ?
                                                         $inst2.posParamConn : 
                                                         $inst2.namedParamConn ;
        LinkedList<String> formalParams = $inst2.formalParams; 
        
        LinkedList<FVPExprBase> actuals = ( null !=   $inst2.posPortConn ) ?
                                                      $inst2.posPortConn : 
                                                      $inst2.namedPortConn ;
        LinkedList<String> formalPorts = $inst2.formalPorts;                
        
        Iterator actualItr = actuals.iterator();
                                                      
        FVPDescription desc = FVPDescription.instance();
        
        // To get the definition of the module, we need to search from the 
        // top in the symbol table
        FVPBase sym = desc.getRoot().findModuleOrUDP( $name_of_module.id );
        
        if ( null == sym )
        {
            // Note down the current scope and come out of it
            // Create a new dummy module and then revert
            // back the scope.
            // This is very dangerous to play with the 
            // scope and hence be extremely careful while doing this.
            
            FVPBase currScp = desc.popScope(); 
            FVPModule mod = new FVPModule( $name_of_module.id, $name_of_module.line );
            FVPDescription.instance().addDeclaration( mod );
            mod.setDummy();
            desc.insertSymbol( $name_of_module.id, mod);
            desc.pushScope(mod);
            
            // 2001 style param map processing
            if ( ( null != actualParams ) && ( false == actualParams.isEmpty() ) )
            {
                  parammaps = new LinkedList<FVPParamMap>();
                  // Required for the 2001 style param maps
                  LinkedList<FVPParamDecl> frmlParamList = new LinkedList<FVPParamDecl>();
                  if ( null != formalParams && ( false == formalParams.isEmpty() ) )
                  {
                      Iterator itr = formalParams.iterator();
                      String name = null;
                      while(itr.hasNext()) 
                      {
                          name = (String)itr.next();
                          // Cleanup this String object from the list
                          itr.remove();
                          FVPIdentifier paramid = new FVPIdentifier ( 0, name );
                          
                          FVPParamDecl param = new FVPParamDecl ( 0, paramid, null );
                          param.setDummy(); // blackbox
                          frmlParamList.add(param);
                          desc.insertSymbol( name, param );
                      }
                  }
                  else
                  {
                      Integer index = 0;
                      Iterator itr = actualParams.iterator();
                      while(itr.hasNext()) 
                      {
                          itr.next();
                          index++;
                          String name = $name_of_module.id + "_dummy_param_" + index;
                          FVPParamDecl param =  mod.findParameter( name );
                          if ( null == param )
                          {
	                          FVPIdentifier paramid = new FVPIdentifier ( 0, name );
	                          param = new FVPParamDecl ( 0, paramid, null );                        
	                          param.setDummy(); //blackbox
	                          frmlParamList.add(param);
	                          desc.insertSymbol( name, param );
                          }
                      }
                  }
                 mod.setParameters(frmlParamList);            
                List<FVPParamDecl> formals = frmlParamList;
                Iterator<FVPParamDecl> formalItr = null;
                if ( null != formals )
                {
                    formalItr = formals.iterator();
                }
                
                              
                Iterator actualParamItr = actualParams.iterator();
                while( null != actualParamItr && actualParamItr.hasNext() ) 
                {
                    FVPParamDecl formal = null;
                    if ( null != formalItr && formalItr.hasNext() )
                    {
                        formal = formalItr.next();
                    }
                    FVPExprBase actual = (FVPExprBase)actualParamItr.next();
                    if ( null != formal && null != actual )
                    {
                        FVPParamMap pmap = new FVPParamMap( $line, formal, actual);
                        parammaps.add ( pmap );
                    }
                }        
            }
            // End of param processing  
            
            
            LinkedList<FVPPort> frmlPortList = new LinkedList<FVPPort>();
            LinkedList<FVPIdentifier> frmlPortIDList = 
                                          new LinkedList<FVPIdentifier>();
            if ( ( null != actuals ) && ( false == actuals.isEmpty() ) )
            {
                if ( null != formalPorts && ( false == formalPorts.isEmpty() ) )
                {
                    Iterator itr = formalPorts.iterator();
                    String name = null;
                    while(itr.hasNext()) 
                    {
                        name = (String)itr.next();
                        // Cleanup this String object from the list
                        itr.remove();
                        FVPIdentifier netname = new FVPIdentifier ( 0, name );
                        frmlPortIDList.add(netname);
                        FVPNet net = new FVPNet ( 0, netname, 
                                          FVPNet.NetType.IMPLICIT_PORT_WIRE, null );
                        FVPDescription.instance().addDeclaration( net );
                        // TODO: TBD: 
                        // Get the range of the actual expression and set that
                        // range for this dummy net
                        FVPPort port = new FVPPort ( net, 
                                                 FVPPort.Direction.INOUT, 0 );
                        port.setDummy(); // blackbox
                        frmlPortList.add(port);
                        desc.insertSymbol( name, port );
                    }
                }
                else
                {
                    Integer index = 0;
                    Iterator itr = actuals.iterator();
                    while(itr.hasNext()) 
                    {
                        itr.next();
                        index++;
                        String name = $name_of_module.id + "_dummy_port_" + index;
                        FVPIdentifier netname = new FVPIdentifier ( 0, name );
                        netname.setDummy();
                        frmlPortIDList.add(netname);
                        FVPNet net = new FVPNet ( 0, netname, 
                                         FVPNet.NetType.IMPLICIT_PORT_WIRE, null );
                        FVPDescription.instance().addDeclaration( net );
                        // TODO: TBD: 
                        // Get the range of the actual expression and set that
                        // range for this dummy net
                        FVPPort port = new FVPPort ( net, FVPPort.Direction.INOUT, 0 );
                        port.setDummy(); //blackbox
                        frmlPortList.add(port);
                        desc.insertSymbol( name, port );
                    }
                }
            }
            mod.setPorts(frmlPortList);
            mod.setPortIds(frmlPortIDList);
            mod.setDummy();
            desc.popScope(); 
            desc.pushScope(currScp);
            sym = mod;       
        }
        
        List<FVPPort> formals = null; //mod.getPorts();
        LinkedList<FVPPortMap> portmaps = new LinkedList<FVPPortMap>();
        
        switch( sym.getObjType() )
        {
            case MODULE:
                formals = ((FVPModule)sym).getPorts();
                $inst = new FVPModuleInstance($line, $inst2.id );
                ((FVPModuleInstance)$inst).setRange( $inst2.range );
                ((FVPModuleInstance)$inst).setMaster( (FVPModule)sym );
                ((FVPModuleInstance)$inst).setPortMaps( portmaps );               
                break;
            case UDP:
                $inst = new FVPUDPInstance($line, $inst2.id );
                ((FVPUDPInstance)$inst).setRange( $inst2.range );
                ((FVPUDPInstance)$inst).setMaster( (FVPUDP)sym );
                ((FVPUDPInstance)$inst).setPortMaps( portmaps );
                formals = ((FVPUDP)sym).getPorts();
                break;
            default:
                assert(false);
        }
        FVPDescription.instance().addStatement( $inst );
        Iterator itr = ( ( null != formalPorts ) && ( false == formalPorts.isEmpty() ) ) ? formalPorts.iterator() : null ;
        Iterator formalItr = ( null == formals ) ? null : formals.iterator();
        while(actualItr.hasNext()) 
        {
            FVPPort formal = null;
            FVPExprBase actual = (FVPExprBase)actualItr.next();
            if ( ( null != formalPorts ) && ( false == formalPorts.isEmpty() ) )
            {
                String fpname = (String)itr.next();
                if ( FVPBase.ObjType.MODULE == sym.getObjType() )
                { 
                    formal = ((FVPModule)sym).getPort(fpname);
                }
                else
                {
                    formal = ((FVPUDP)sym).getPort(fpname);
                }
            }     
            else
            {
                formal = (FVPPort)formalItr.next(); 
            }
            assert ( ( null != formal ) && ( null != actual ) );
            FVPPortMap pmap = new FVPPortMap( $line, formal, actual);
            portmaps.add ( pmap );
        }
        if ( null != parammaps )
        {
            Iterator mapItr = parammaps.iterator();
            while(mapItr.hasNext()) 
            {
                FVPParamMap map = (FVPParamMap)mapItr.next();
                ((FVPModuleInstance)$inst).addParamMap( map );
            }
        }
    } 
    )* SEMI        
    ;

list_of_parameter_value_assignment returns [ Integer line, 
                         LinkedList<FVPExprBase> posParamConn, 
                         LinkedList<FVPExprBase> namedParamConn,
                         LinkedList<String> formalParams]
    :
    POUND LPAREN 
    l1=parameter_value_assignment
    {
        $line = $l1.line;
        $posParamConn = $l1.posParamConn;
        $namedParamConn = $l1.namedParamConn;
        $formalParams = $l1.formalParams;
    }
    RPAREN
    ;

module_instance returns [Integer line, 
                         FVPRange range,
                         LinkedList<FVPExprBase> posPortConn, 
                         LinkedList<FVPExprBase> namedPortConn,
                         LinkedList<String> formalPorts,
                         String id,
                         LinkedList<FVPExprBase> posParamConn, 
                         LinkedList<FVPExprBase> namedParamConn,
                         LinkedList<String> formalParams] 
    :
    name_of_instance ( r1=range {$range = $r1.range;} )? 
    (
        p=list_of_parameter_value_assignment
        {
            $posParamConn = $p.posParamConn;
            $namedParamConn = $p.namedParamConn;
            $formalParams = $p.formalParams;
        }
    )?
    LPAREN list_of_module_connections RPAREN
    {
        $line = $name_of_instance.line;
        $id = $name_of_instance.id;
        
        $posPortConn = $list_of_module_connections.posPortConn;
        $namedPortConn = $list_of_module_connections.namedPortConn;
        $formalPorts = $list_of_module_connections.formalPorts;
    }
    ;

list_of_module_connections returns [Integer line, 
                                    LinkedList<FVPExprBase> posPortConn, 
                                    LinkedList<FVPExprBase> namedPortConn,
                                    LinkedList<String> formalPorts]
    :
    mod_prt_conn1=module_port_connection
    {
        $line = $mod_prt_conn1.line;
        $posPortConn = new LinkedList<FVPExprBase>();
        $posPortConn.add($mod_prt_conn1.actual);
    } 
    ( 
        COMMA mod_prt_conn2=module_port_connection
        {
            $posPortConn.add($mod_prt_conn2.actual);
        } 
    )*
    |
    nmd_prt_conn1=named_port_connection 
    {
        $line = $nmd_prt_conn1.line;
        $namedPortConn = new LinkedList<FVPExprBase>();
        $namedPortConn.add($nmd_prt_conn1.actual);
        $formalPorts = new LinkedList<String>();
        $formalPorts.add($nmd_prt_conn1.formal);
    }
    ( 
        COMMA nmd_prt_conn2=named_port_connection 
        {
            $namedPortConn.add($nmd_prt_conn2.actual);
            $formalPorts.add($nmd_prt_conn2.formal);
        }
    )*
    ;

parameter_value_assignment returns [Integer line, 
                                    LinkedList<FVPExprBase> posParamConn, 
                                    LinkedList<FVPExprBase> namedParamConn,
                                    LinkedList<String> formalParams]
    :
    mod_prm_conn1=module_param_connection
    {
        $line = $mod_prm_conn1.line;
        $posParamConn = new LinkedList<FVPExprBase>();
        $posParamConn.add($mod_prm_conn1.actual);
    } 
    ( 
        COMMA mod_prm_conn2=module_param_connection
        {
            $posParamConn.add($mod_prm_conn2.actual);
        } 
    )*
    |
    nmd_prm_conn1=named_param_connection 
    {
        $line = $nmd_prm_conn1.line;
        $namedParamConn = new LinkedList<FVPExprBase>();
        $namedParamConn.add($nmd_prm_conn1.actual);
        $formalParams = new LinkedList<String>();
        $formalParams.add($nmd_prm_conn1.formal);
    }
    ( 
        COMMA nmd_prm_conn2=named_param_connection 
        {
            $namedParamConn.add($nmd_prm_conn2.actual);
            $formalParams.add($nmd_prm_conn2.formal);
        }
    )*
    ;

module_port_connection returns [ Integer line, FVPExprBase actual ]
    :
    expression 
    {
        $line = $expression.line;
        $actual = $expression.expr;
    }
     //| NULL
    ;

    // expression below isn't optional according to Palnitkar, but
    // several examples generated by Cadence use this syntax.
named_port_connection returns [ 
                                Integer line, 
                                String formal, 
                                FVPExprBase actual 
                              ]
    :
    DOT local_identifier LPAREN (expression {$actual = $expression.expr; })? RPAREN 
    {
        $line = $local_identifier.line ;
        $formal =  $local_identifier.id ;
    }       
    ;

module_param_connection returns [ Integer line, FVPExprBase actual ]
    :
    expression 
    {
        $line = $expression.line;
        $actual = $expression.expr;
    }
     //| NULL
    ;

    // expression below isn't optional according to Palnitkar, but
    // several examples generated by Cadence use this syntax.
named_param_connection returns [ 
                                Integer line, 
                                String formal, 
                                FVPExprBase actual 
                              ]
    :
    DOT IDENTIFIER LPAREN (expression {$actual = $expression.expr; })? RPAREN 
    {
        $line = $IDENTIFIER.line ;
        $formal =  $IDENTIFIER.text ;
    }       
    ;
    
//----------------------------------------------------------------------------
// Behavioral Statements
//----------------------------------------------------------------------------

initial_statement returns [FVPInitialBlock init] :
    KW_INITIAL 
    {
        $init = new FVPInitialBlock( $KW_INITIAL.getLine() );
        FVPDescription desc = FVPDescription.instance();        
//        FVPModule mod = (FVPModule)desc.getCurrentscope();
        FVPDescription.instance().addStatement( $init );
        desc.pushScope($init);
        if ( true == FVPDescription.instance().isStoreComment() )
        {
            FVPComment comment = FVPDescription.instance().getComment( $KW_INITIAL.getLine() );
            $init.setComment( comment );
        }
    } 
    st=statement
    {
        if ( null != $init )
        {
            $init.setLine( $st.line );
	        if ( null != $st.blkname )
	        {
	          $init.setLabel($st.blkname);
            }
        }
        FVPDescription.instance().popScope();
    }
    ;
    

always_statement returns [FVPAlwaysBlock always]
    :
    KW_ALWAYS
    {
        $always = new FVPAlwaysBlock( $KW_ALWAYS.getLine() );
        FVPDescription desc = FVPDescription.instance();        
        FVPDescription.instance().addStatement($always);
        desc.pushScope($always);
        if ( true == FVPDescription.instance().isStoreComment() )
        {
            FVPComment comment = FVPDescription.instance().getComment( $KW_ALWAYS.getLine() );
            $always.setComment( comment );
        }
    } 
    st=statement
    {
//        $always.setLine( $st.line );
        if ( null != $always )
        {
	        if ( null != $st.blkname )
	        {
	          $always.setLabel($st.blkname);
	        }
	        $always.setSensitivityList($st.edges);
        } 
        FVPDescription.instance().popScope();
    }
    ;
    
all_statement_or_nulls:
    all_statements
    |
    SEMI
    {
        LogMessages.instance().print(1868,
                       LogMessages.MSG_SEVERITY.WARNING, 
                       "Ignoring NULL statement found at " +
                       FVPDescription.instance().getFile() + 
                       ":" + $SEMI.getLine() + "...");
    }
    ;    
    
statement_or_null returns [LinkedList<FVPExprBase> edges,
                           LinkedList<FVPBase> stmts, 
                           Integer  line]:
    (statement) => st1=statement
    {
        $edges = $st1.edges;
        $stmts = $st1.stmts;
        $line = $st1.line;
    }
    |
    SEMI
    {
        LogMessages.instance().print(1868,
                       LogMessages.MSG_SEVERITY.WARNING, 
                       "Ignoring NULL statement found at " +
                       FVPDescription.instance().getFile() + 
                       ":" + $SEMI.getLine() + "...");
    }
    ;

all_statements:
    (statement)* 
    ;
    
statement returns [ LinkedList<FVPExprBase> edges,
                    LinkedList<FVPBase> stmts, 
                    Integer line,
                    String blkname
                  ]
    :
    (lvalue ASSIGN) => blocking_assignment SEMI 
    {

        $line = $blocking_assignment.line;
        $stmts = new LinkedList<FVPBase>();
        $stmts.add( $blocking_assignment.stmt );
    }
    |
    (lvalue '<=') => non_blocking_assignment SEMI 
    {
        $line = $non_blocking_assignment.line;
        $stmts = new LinkedList<FVPBase>();
        $stmts.add( $non_blocking_assignment.stmt );
    }
    |
    conditional_statement 
    {
        $line = $conditional_statement.line;
        $stmts = new LinkedList<FVPBase>();
        $stmts.add( $conditional_statement.stmt );
    }
    |
    case_statement
    {
        $line = $case_statement.line;
        $stmts = new LinkedList<FVPBase>();
        $stmts.add( $case_statement.stmt );
    }
    |
    loop_statement
    {
        $stmts = $loop_statement.stmts;
        $line = $loop_statement.line;
    } 
    |
    ptc=procedural_timing_control_statement 
    {
        $edges = $ptc.edges;
        $line = $ptc.line;
        $stmts = $ptc.stmts;
    }
    |
    wait=wait_statement 
    {
        $stmts = $wait.stmts;
        $line = $wait.line;
    }
    |
    event_trigger 
    {
        LogMessages.instance().print(1869,
                       LogMessages.MSG_SEVERITY.WARNING, 
                       "Ignoring event_trigger statement found in file " +
                       FVPDescription.instance().getFile()  );
    }
    |
    seq_block 
    {
        $stmts = $seq_block.stmts;
        $line = $seq_block.line;
        $blkname = $seq_block.blkname;
    }
    |
    par_block 
    |
    task_enable
    {
        $line = $task_enable.line;
        $stmts = new LinkedList<FVPBase>();
        $stmts.add( $task_enable.stmt );
    } 
    |
    system_task_enable 
    {
        $line = $system_task_enable.line;
        $stmts = new LinkedList<FVPBase>();
        $stmts.add( $system_task_enable.stmt );
//        LogMessages.instance().print(1870,
//                       LogMessages.MSG_SEVERITY.WARNING, 
//                       "Ignoring system task statement found in file " +
//                       FVPDescription.instance().getFile()  );
    }
    |
    disable_statement 
    {
        LogMessages.instance().print(1871,
                       LogMessages.MSG_SEVERITY.WARNING, 
                       "Ignoring disable_statement statement found in file " +
                       FVPDescription.instance().getFile()  );
    }
    |
    pasgn=procedural_continuous_assignment
    {
        FVPDescription.instance().addStatement( $pasgn.stmt );
    }
    ;

assignment returns [ Integer line, FVPSeqAssignment assgn, FVPBase dummyNet, FVPLvalue dummyLval ] 
    :
    lvalue ASSIGN expression 
    {
        $line = $lvalue.line;     
        $assgn = new FVPSeqAssignment($line, $lvalue.lval, 
                                   $expression.expr, 
                                   FVPAssignment.AssignmentType.UNKNOWN);
        $dummyNet = $lvalue.dummyNet;
        $dummyLval = $lvalue.dummyLval;
    }
    ;

blocking_assignment returns [FVPSeqAssignment stmt, Integer line]
    @init
    {
        FVPDelayControl del = null;
    }
    :
    lvalue ASSIGN ( delay_or_event_control{ del = $delay_or_event_control.delay_ctrl;  } )?  expression
    {
        //$line = $lvalue.line;
        $line = (null != $lvalue.line) ? $lvalue.line : $expression.line; 
        $stmt = new FVPSeqAssignment( $lvalue.line, $lvalue.lval, $expression.expr, 
                                       FVPAssignment.AssignmentType.BLOCKING );
         if ( null != del )
         {
             ((FVPSeqAssignment)retval.stmt).setDelayOrControlAfterAssgnOp(del);
            del.setDelayType(FVPDelayControl.DelayType.INTRA_ASSIGNMENT);  
         }
         FVPDescription.instance().addStatement( $stmt );
    }
    ;
 
non_blocking_assignment  returns [FVPSeqAssignment stmt, Integer line] 
    @init 
    {
        FVPDelayControl del = null;
    }
    :
    lvalue '<=' ( delay_or_event_control{ del = $delay_or_event_control.delay_ctrl;} )?  expression
    {
        //$line = $lvalue.line;
        $line = (null != $lvalue.line) ? $lvalue.line : $expression.line; 
        $stmt = new FVPSeqAssignment( $lvalue.line, $lvalue.lval, $expression.expr, 
                                       FVPAssignment.AssignmentType.NONBLOCKING );
         if ( null != del )
         {
             ((FVPSeqAssignment)retval.stmt).setDelayOrControlAfterAssgnOp(del);
            del.setDelayType(FVPDelayControl.DelayType.INTRA_ASSIGNMENT);  
         }
         FVPDescription.instance().addStatement( $stmt );
    }
    ;

// 'else' clause is inherently ambiguous; ANTLR gets it right,
// so suppress warning.
conditional_statement returns [FVPBase stmt, Integer line]
    @init
    {
        FVPBase if_stmt = null;
        FVPBase else_stmt = null;
    }
    @after
    {
        $stmt = if_stmt;
    }
    :
    'if' LPAREN expression
    {
        $line = $expression.line;
        if_stmt = new FVPSeqIfStmt( $expression.line, $expression.expr, null);
        FVPDescription.instance().addStatement( if_stmt );
        FVPDescription.instance().pushScope( if_stmt );
    }
    RPAREN st1=statement_or_null
    {
        FVPDescription.instance().popScope();
    } 
    ( : 'else'
        {
            else_stmt = new FVPSeqElseStmt( $line);
            FVPDescription.instance().pushScope( else_stmt );
        }
         st2=statement_or_null
         {
            if ( null != if_stmt )
            {
             ((FVPSeqIfStmt)if_stmt).setElseBlock(else_stmt);
            }
            FVPDescription.instance().popScope( );
         }
    )?
    ;

case_statement returns [Integer line, FVPBase stmt]
    @init
    {
        LinkedList<FVPSeqCaseItem> items = new LinkedList<FVPSeqCaseItem>();
    } 
    :
    case_keyword LPAREN expression RPAREN 
    (
        case_item
        {
            items.add( $case_item.item);
        }
    )+ 
    'endcase'
    {
        $line = $expression.line;
        $stmt = new FVPSeqCaseStatement( 
                                $expression.line, 
                                $case_keyword.type,
                                items,
                                $expression.expr);
        FVPDescription.instance().addStatement( $stmt );
    }
    ;

case_keyword returns [FVPSeqCaseStatement.CaseType type]
    :
    'case'
    {
        $type = FVPSeqCaseStatement.CaseType.CASE;
    } 
    | 
    'casez' 
    {
        $type = FVPSeqCaseStatement.CaseType.CASEZ;
    }
    | 
    'casex'
    {
        $type = FVPSeqCaseStatement.CaseType.CASEX;
    }
    ;

case_item returns [Integer line, FVPSeqCaseItem item]
    @init
    {
        FVPSeqCaseItem it = null;
        LinkedList<FVPExprBase> alters = new LinkedList<FVPExprBase>();
    }
    @after
    {
        $item = it;
    }
    :
    e1=expression
    {
        $line = $e1.line;
        alters.add( $e1.expr );
    }
    ( COMMA e2=expression
    {
        alters.add( $e2.expr );
    } 
    )* 
    COLON  
    {
        it = new FVPSeqCaseItem( $line, 
                                 alters,
                                 null);
        FVPDescription.instance().pushScope( it );
    }
    st1=statement_or_null
    {
        FVPDescription.instance().popScope();
    }
    |
    'default' (COLON)? 
    {
        $line = -1;
        it = new FVPSeqCaseItem( $line, 
                                 null,
                                 null);
        it.setDefaultAlternative(true);
        FVPDescription.instance().pushScope( it );
    }
    st2=statement_or_null
    {
        if ( null != it )
        {
            it.setLine( $line );
        }
        FVPDescription.instance().popScope();
    }
    ;

loop_statement returns [ LinkedList<FVPBase> stmts, 
                         Integer line ] 
    @init
    {
        FVPSeqLoopStatment loop = null;    
    }
    @after
    {
        $stmts = new LinkedList<FVPBase>();
        if ( null != loop )
        {
            $stmts.add( loop );
        }
    }
    :
    'forever' 
    {
        loop = new FVPSeqLoopStatment( 
                        -1, 
                        FVPSeqLoopStatment.LoopType.FOREVER,
                        null,
                        null);
        FVPDescription.instance().addStatement ( loop );
        FVPDescription.instance().pushScope( loop );
    } 
    st1=statement
    {
        if ( null != loop )
        {
            loop.setLine($st1.line);
        }
        FVPDescription.instance().popScope();
    }
    |
    'repeat' LPAREN e2=expression RPAREN 
    {
        loop = new FVPSeqLoopStatment( 
                        -1, 
                        FVPSeqLoopStatment.LoopType.REPEAT,
                        null,
                        $e2.expr);
        FVPDescription.instance().addStatement ( loop );
        FVPDescription.instance().pushScope( loop );
    }
    st2=statement
    {
    if ( null != loop )
        {
            loop.setLine($st2.line);
        }
        FVPDescription.instance().popScope();
    } 
    |
    'while' LPAREN e3=expression RPAREN
    {
        loop = new FVPSeqLoopStatment( 
                        -1, 
                        FVPSeqLoopStatment.LoopType.WHILE,
                        null,
                        $e3.expr);
        FVPDescription.instance().addStatement( loop );
        FVPDescription.instance().pushScope( loop );
    }
    st3=statement
    {
        if ( null != loop )
        {
            loop.setLine($st3.line);
        }
        FVPDescription.instance().popScope();
    } 
    |
    'for' LPAREN asgn1=assignment SEMI e4=expression SEMI asgn2=assignment RPAREN
    {
        loop = new FVPSeqLoopStatment( 
                        -1, 
                        FVPSeqLoopStatment.LoopType.FOR,
                        null,
                        $e4.expr);
        if ( null != $asgn1.assgn )
        {
          ((FVPSeqAssignment)$asgn1.assgn).setType(FVPAssignment.AssignmentType.BLOCKING);
        }          
        loop.setInitial( $asgn1.assgn );
        if ( null != $asgn2.assgn )
        {
          ((FVPSeqAssignment)$asgn2.assgn).setType(FVPAssignment.AssignmentType.BLOCKING);
        } 
        loop.setSteps( $asgn2.assgn );
        FVPDescription.instance().addStatement( loop );
        FVPDescription.instance().pushScope( loop );
    }
    st4=statement
    {
        if ( null != loop )
        {
            loop.setLine($st4.line);
        }
        FVPDescription.instance().popScope();
    }
    ;

procedural_timing_control_statement returns [LinkedList<FVPExprBase> edges, LinkedList<FVPBase> stmts, Integer line]:
    edg=delay_or_event_control st=statement_or_null
    {
        $edges = $edg.edges;
        $line = (null != $edg.line) ? $edg.line : $st.line; 
        $stmts = $st.stmts;
        if ( null != $edg.delay_ctrl && null != $stmts )
        {
            $edg.delay_ctrl.setDelayType( FVPDelayControl.DelayType.INTER_ASSIGNMENT );
            Iterator<FVPBase> itr = $stmts.iterator();
            while (itr.hasNext()) 
            {
                FVPSeqStmtBase stmt = (FVPSeqStmtBase)itr.next();
                if ( FVPSeqStmtBase.StmtType.SEQ_ASSIGNMENT == stmt.getStatementType() )
                {
                    ((FVPSeqAssignment)stmt).setDelayOrControlBeforeAssgnOp( $edg.delay_ctrl );
                }
                else if ( FVPSeqStmtBase.StmtType.SYSTEM_TASK_CALL == stmt.getStatementType() )
                {
                    ((FVPSystemTaskCall)stmt).setDelayOrControlBeforeAssgnOp( $edg.delay_ctrl );
                }
                
                else
                {
                    LogMessages.instance().print(-1, LogMessages.MSG_SEVERITY.WARNING,
                            "Ignoring delay at " + stmt.getFile() + ":" + stmt.getLine() );
                }
            }
        }
        
    }
    ;

wait_statement returns [LinkedList<FVPBase> stmts, Integer line] 
    :
    'wait' LPAREN expression RPAREN statement_or_null
    {
        $line = $statement_or_null.line;
        FVPSeqWaitStatement stmt = new FVPSeqWaitStatement ( $line, $expression.expr, $statement_or_null.stmts);
        FVPDescription.instance().addStatement ( stmt );
        $stmts = new LinkedList<FVPBase>();
        $stmts.add( stmt );
    }
    ;

event_trigger :
    e='->' name_of_event SEMI
    {
        LogMessages.instance().print(1873,
                       LogMessages.MSG_SEVERITY.WARNING,
                       "Ignoring event_trigger statement found in file " +
                       FVPDescription.instance().getFile()  + ":" + $e.getLine()  );
    }
    ;

disable_statement :
    'disable' IDENTIFIER SEMI
    ;

seq_block returns [LinkedList<FVPBase> stmts, 
                   String blkname, 
                   LinkedList<FVPBase> decllist, 
                   Integer line]
    @init
    {
        LinkedList<FVPBase> all_stmts = null;
    }
    @after
    {
        $stmts = all_stmts;
    }
    :
    'begin'
    ( 
        COLON name_of_block (decl=block_declaration)*
        {
            $blkname = $name_of_block.text;
            FVPBase scope = FVPDescription.instance().getScope();
            
            if ( FVPBase.ObjType.SEQ_STATEMENT == scope.getObjType() ) 
            {
              ((FVPSeqStmtBase)scope).setLabel( $blkname );
            }     
        }
    )?
    (statement
    {
        if ( null != $statement.stmts )
        {
            if ( null == all_stmts )
            {
                all_stmts = new LinkedList<FVPBase>();
            }
            Iterator<FVPBase> itr = $statement.stmts.iterator();
            while (itr.hasNext()) 
            {
                FVPBase stmt = itr.next();
                if (null != stmt)
                {
                     all_stmts.add( stmt );                
                }
            }
        }
        $line = $statement.line;
        if ( null != $statement.blkname )
        {
          $blkname = $statement.blkname;
        }
    }
    )*
    'end'
    ;

par_block :
    f1='fork'
    ( COLON name_of_block (block_declaration)* )?
    (statement)*
    'join'
    {
        LogMessages.instance().print(1874,
                       LogMessages.MSG_SEVERITY.WARNING, 
                       "Ignoring fork-join statement found in file " +
                       FVPDescription.instance().getFile()  + ":" + $f1.getLine()  );
    }
    ;

block_declaration:
    parameter_declaration |
    localparam_declaration |
    reg_declaration |
    integer_declaration |
    real_declaration |
    time_declaration |
    event_declaration 
    ;

task_enable returns [FVPBase stmt, 
                     Integer  line]
    @init
    {
        LinkedList<FVPExprBase> args = null;
    }
    :
    name_of_task 
    ( LPAREN e1=expression 
    {
        args = new LinkedList<FVPExprBase>();
        args.add($e1.expr);
    }    
    (COMMA 
        (
            e2=expression
            {
                if ( null != args )
                {
                    args.add($e2.expr);
                }
            }
        )?
    )* RPAREN )?
    SEMI
    {
        String taskname = $name_of_task.id;
        $line = $name_of_task.line;
        FVPDescription desc = FVPDescription.instance();
        FVPBase sym = (FVPBase)desc.getSymbol( taskname );
        if ( null == sym ) // Probbably it is defined later ...
        {
            sym = new FVPTaskOrFunctionDecl(-1, taskname );
            ((FVPTaskOrFunctionDecl)sym).setDummy();
            FVPDescription.instance().insertSymbol( taskname, sym );
        }
        $stmt = new FVPSeqTaskCall($line, (FVPTaskOrFunctionDecl)sym, args);
        FVPDescription.instance().addStatement ( $stmt );
    }
    ;

system_task_enable returns [FVPSystemTaskCall stmt, 
                     Integer  line]
    @init
    {
        LinkedList<FVPExprBase> args = null;
    } 
    :
    SYSTEM_TASK_NAME 
    {
        $line = $SYSTEM_TASK_NAME.getLine();
        $stmt = new FVPSystemTaskCall( $SYSTEM_TASK_NAME.getLine(), $SYSTEM_TASK_NAME.getText(), null);
        FVPDescription.instance().addStatement ( $stmt );
        FVPBase scp = $stmt.getScope();
        while ( null != scp && FVPBase.ObjType.MODULE != scp.getObjType() )
        {
          scp = scp.getScope();
        }
        if ( null != scp && FVPBase.ObjType.MODULE == scp.getObjType() )
        {
          ((FVPModule)scp).addSystemTask( $stmt );
        }
    }
    ( 
    LPAREN 
        e1=expression 
        {
            if ( null == args )
            {
                args = new LinkedList<FVPExprBase>();
                $stmt.setArguments( args );
            }
            args.add( $e1.expr );
        }
        (COMMA 
            (
                e2=expression
                {
                    if ( null == args )
                    {
                        args = new LinkedList<FVPExprBase>();
                        $stmt.setArguments( args );
                    }
                    args.add( $e2.expr );
                }
             )?
        )*
    RPAREN )?
    SEMI
    ;

procedural_continuous_assignment returns [FVPBase stmt, 
                     Integer  line]
    :
    KW_ASSIGN a1=assignment SEMI
    {
        $line =  $KW_ASSIGN.line;
        FVPSeqAssignment asmnt = (FVPSeqAssignment)$a1.assgn;
        $stmt = new FVPProceduralContinuousAssignment( $line, asmnt.getLvalue(), asmnt.getRvalue(), FVPProceduralContinuousAssignment.ProcContAssgnType.ASSIGN );
    } 
    |
    das='deassign' lv1=lvalue SEMI
    {
        $line =  $das.line;
        $stmt = new FVPProceduralContinuousAssignment( $line, $lv1.lval, null, FVPProceduralContinuousAssignment.ProcContAssgnType.DEASSIGN );
    } 
    |
    frc='force' a2=assignment SEMI
    {
        $line =  $frc.line;
        FVPSeqAssignment asmnt = (FVPSeqAssignment)$a2.assgn;
        $stmt = new FVPProceduralContinuousAssignment( $line, asmnt.getLvalue(), asmnt.getRvalue(), FVPProceduralContinuousAssignment.ProcContAssgnType.FORCE );
    }
    |
    rel='release' lv2=lvalue SEMI
    {
        $line =  $rel.line;
        $stmt = new FVPProceduralContinuousAssignment( $line, $lv2.lval, null, FVPProceduralContinuousAssignment.ProcContAssgnType.RELEASE );
    }
    ;

delay_or_event_control returns [LinkedList<FVPExprBase> edges, 
                                FVPDelayControl delay_ctrl, Integer line]
    :
    delay_control
    {
        $delay_ctrl = $delay_control.delay_ctrl;
        $line = $delay_control.line;
    } 
    |
    event_control
    {
        $edges = $event_control.edges;
        $line = $event_control.line;
    }
    ;

//----------------------------------------------------------------------------
// Specify Section
//----------------------------------------------------------------------------

specify_block :
    'specify' (specify_item)* 'endspecify'
    {
        LogMessages.instance().print(1877,
                       LogMessages.MSG_SEVERITY.WARNING, 
                       "Ignoring specify_block statement found in file " +
                       FVPDescription.instance().getFile() );
    }
    ;

specify_item :
    spec_param_declaration |
    (path_declaration) => path_declaration |
    system_timing_check
    | sdpd
    ;

spec_param_declaration :
    'specparam' list_of_specparam_assignments SEMI
    ;

list_of_specparam_assignments :
    specparam_assignment ( COMMA specparam_assignment )*
    ;

specparam_assignment :
    identifier ASSIGN expression
    ;

path_declaration :
   (simple_path_declaration) => simple_path_declaration SEMI 
   |
   (level_sensitive_path_declaration) => level_sensitive_path_declaration SEMI
   |
   edge_sensitive_path_declaration SEMI
   ;

simple_path_declaration :
    (parallel_path_description) => parallel_path_description ASSIGN path_delay_value 
    |
    full_path_descriptor ASSIGN path_delay_value
    ;

parallel_path_description :
    LPAREN specify_terminal_descriptor '=>' specify_terminal_descriptor RPAREN
    ;

full_path_descriptor :
    LPAREN list_of_path_terminals  '*>' list_of_path_terminals RPAREN
    ;

list_of_path_terminals :
    specify_terminal_descriptor ( COMMA specify_terminal_descriptor )*
    ;

specify_terminal_descriptor :
    (identifier LBRACK expression COLON) => identifier LBRACK expression COLON expression RBRACK 
    |
    (identifier LBRACK) => identifier LBRACK expression RBRACK 
    |
    identifier
    ;

path_delay_value 
    :
    (path_delay_expression) => path_delay_expression 
    |
    LPAREN list_of_path_delay_expressions RPAREN
    ;

list_of_path_delay_expressions 
    :
    path_delay_expression COMMA path_delay_expression
    ( 
        COMMA path_delay_expression
        ( 
            COMMA path_delay_expression COMMA path_delay_expression 
            COMMA path_delay_expression 
        )? 
    )?
    ; 

path_delay_expression 
    :
    mintypmax_expression
    ;

system_timing_check 
    :
    '$setup' LPAREN timing_check_event COMMA timing_check_event COMMA
     timing_check_limit ( COMMA notify_register )? RPAREN SEMI 
    |
    '$hold' LPAREN timing_check_event COMMA timing_check_event COMMA
     timing_check_limit ( COMMA notify_register )? RPAREN SEMI 
    |
    '$period' LPAREN controlled_timing_check_event COMMA
     timing_check_limit ( COMMA notify_register )? RPAREN SEMI 
    |
    '$width' LPAREN controlled_timing_check_event COMMA
     timing_check_limit ( COMMA expression COMMA notify_register )? RPAREN SEMI
    |
    '$skew' LPAREN timing_check_event COMMA timing_check_event COMMA 
     timing_check_limit ( COMMA notify_register )? RPAREN SEMI 
    |
    '$recovery' LPAREN controlled_timing_check_event COMMA 
     timing_check_event COMMA timing_check_limit
     ( COMMA notify_register )? RPAREN SEMI 
    |
    '$setuphold' LPAREN timing_check_event COMMA timing_check_event COMMA
     timing_check_limit COMMA timing_check_limit
     ( COMMA notify_register )? RPAREN SEMI
    ;

timing_check_event :
    (timing_check_event_control)? specify_terminal_descriptor
    ( '&&&' timing_check_condition )?
    ;

controlled_timing_check_event :
    timing_check_event_control specify_terminal_descriptor
    ( '&&&' timing_check_condition )?
    ;

timing_check_event_control :
    'posedge' |
    'negedge' |
    edge_control_specifier
    ;

edge_control_specifier :
    'edge' LBRACK edge_descriptor ( COMMA edge_descriptor )* RBRACK
    ;

   // Use semantic predicates to determine whether a matched
   // NUMBER or IDENTIFIER is a valid special value in the given context.
   // This kludge avoids having the special values in the Literals table,
   // thus avoiding a lexical conflict.

edge_descriptor :
    '0x' 
    |
    '1x'
    |
    n=NUMBER
    { 
        $n.text.equals("01") || $n.text.equals("10")
    }?
    |
    i=IDENTIFIER
    { 
        $i.text.equals("x1") || $i.text.equals("x0")
    }?
    ;
    
timing_check_condition :
    scalar_timing_check_condition
    ; 

scalar_timing_check_condition :
    expression
    ;

timing_check_limit :
    expression
    ;

notify_register :
    name_of_register
    ;

level_sensitive_path_declaration :
    (parallel_level_sensitive_path_description) => 
         parallel_level_sensitive_path_description ASSIGN path_delay_value SEMI
    |
    full_level_sensitive_path_description ASSIGN path_delay_value SEMI
    ;

parallel_level_sensitive_path_description :
    'if' LPAREN expression RPAREN
    LPAREN specify_terminal_descriptor (polarity_operator)?
    '=>' specify_terminal_descriptor RPAREN
    ;

full_level_sensitive_path_description :
    'if' LPAREN expression RPAREN
    LPAREN list_of_path_terminals (polarity_operator)?
    '*>' list_of_path_terminals RPAREN
    ;

polarity_operator :
    PLUS 
    |
    MINUS
    ;

edge_sensitive_path_declaration 
    :
    ( 'if' LPAREN expression RPAREN )?
        LPAREN (edge_identifier)? specify_terminal_descriptor
        ( '=>' | '*>' )
        LPAREN ( (list_of_path_terminals) => list_of_path_terminals 
    |
    specify_terminal_descriptor )
    (polarity_operator)? COLON data_source_expression
    RPAREN
    RPAREN
    ASSIGN path_delay_value SEMI
    ;

data_source_expression 
    :
    expression
    ;

edge_identifier :
    'posedge' 
    |
    'negedge'
    ;

sdpd :
    'if' LPAREN expression RPAREN
    simple_path_declaration
    SEMI
    ;

//----------------------------------------------------------------------------
// Expressions
//----------------------------------------------------------------------------

lvalue returns [Integer line, 
                FVPBase lval, 
                FVPBase dummyNet,
                FVPLvalue dummyLval ]
    @init
    {
        LinkedList<FVPExprBase> indexes = null;
        FVPRange slice = null;
    } 
    :
    (id1=identifier 
    r1=range
    {
        if ( null != $r1.range )
        {
            slice = $r1.range;
        }
        else if ( null != $r1.left )
        {
            if ( null == indexes )
            {
                indexes = new LinkedList<FVPExprBase>();
            }
            indexes.add( $r1.left );
        }
    } 
        (
            r2=range
            {
                if ( null == indexes )
                {
                    indexes = new LinkedList<FVPExprBase>();
                }
                if ( null != $r2.range )
                {
                    slice = $r2.range;
                }
                else if ( null != $r2.left )
                {
                    indexes.add(  $r2.left );
                }
            }
        )*
    ) // Baya => identifier range
    {
        $line = $id1.line;
        FVPBase ident = null;
        // I am expecting that this ID is already there
        // in the current symbol table
        FVPBase val = (FVPBase)FVPDescription.instance().getSymbol( $id1.id );
        FVPBase net = null ; // net, reg or mem
        FVPLvalue lv = null;
        if ( null == val ) // blackbox
        {
            val = new FVPIdentifier( $line, $id1.id );
            $dummyNet = val;
        }
        lv = new FVPLvalue( $line, val );
        $dummyLval = (FVPLvalue)lv;
        FVPExprLvalue lvexpr = new FVPExprLvalue( $line, lv );
        if ( null != slice )
        { 
            FVPLvalPartSel lval_psel = new FVPLvalPartSel( $line, lvexpr, 
                                                           slice, indexes );
            $lval = (FVPBase)lval_psel;
        }
        else if ( null != indexes && false == indexes.isEmpty() )
        {
            FVPLvalBitSel lval_bsel = new FVPLvalBitSel( $line, lvexpr, 
                                                    indexes );
            $lval = (FVPBase)lval_bsel;
        }
        else
        {
            $lval = lv;
        }
    } 
    |
    id2=identifier
    {
        $line = $id2.line;
        FVPBase val = (FVPBase)FVPDescription.instance().getSymbol( $id2.id );
        FVPBase net = null ; // net, reg or mem
        if ( null == val || FVPBase.ObjType.MODULE == val.getObjType() ) //blackbox
        {
            val = new FVPIdentifier( $line, $id2.id );
            $dummyNet = val;
        }
        $lval = new FVPLvalue( $line, val );
        $dummyLval = (FVPLvalue)$lval;
    } 
    |
    concatenation
    {
        $line = $concatenation.line;
        $lval = $concatenation.expr;
    }
    |
    def1=DEFINE
    {
        $line = $def1.line;
        StringBuffer buf = new StringBuffer($def1.text);
        StringBuffer modstr = buf.deleteCharAt(0);
        String str = modstr.toString();
        FVPDescription desc = FVPDescription.instance();
        FVPBase sym = (FVPBase)desc.getSymbol(str);
        if ( null == sym )
        {
            LogMessages.instance().print(1878,
                                LogMessages.MSG_SEVERITY.ERROR,
                                "Undefined symbol `"+ str + 
                                " found in file " + desc.getFile() +
                                ":" + $line );
        } 
        if ( null != sym && FVPBase.ObjType.DEFINE_DIRECTIVE != sym.getObjType() )
        {
            LogMessages.instance().print(1879,
                                LogMessages.MSG_SEVERITY.ERROR,
                                "The symbol `"+ str + " in file " + 
                                desc.getFile() + ":" + sym.getLine() + 
                                " is not a macro ..."  );
        }
        FVPDefine def = (FVPDefine)sym;
        if ( null == def.getValue() )
        {
            return null;
        }
        if ( FVPExprBase.ExprType.LVALUE == def.getValue().getType() )
        {
            $lval = ((FVPExprLvalue)def.getValue()).getLvalue();
        }
        else //if ( FVPExprBase.ExprType.CONSTINTEGER == def.getValue().getType() )
        {
            $lval = def.getValue();
        }
    }    
    ;
 
concatenation returns [Integer line, FVPExprBase expr]
    :
    (LCURLY expression LCURLY) => 
        LCURLY e1=expression 
        LCURLY e2=expression
    {
        $line = $e1.line;
        $expr = new FVPExprConcat( $line, $e2.expr, $e1.expr );       
    } 
    ( 
        COMMA e3=expression
        {
            ((FVPExprConcat)$expr).addOperand($e3.expr);
        }
    )* RCURLY RCURLY
    |
    LCURLY e4=expression 
    {
        $line = $e4.line;
        $expr = new FVPExprConcat( $line, $e4.expr, null );
    }
    ( COMMA e5=expression
        {
            ((FVPExprConcat)$expr).addOperand($e5.expr);
        }
     )* RCURLY
    ;

mintypmax_expression returns [Integer line, FVPExprBase mintypmax]
    :
    e1=expression
    {
        $line = $e1.line;
        $mintypmax = new FVPExprDelayMinTypMax( $line, $e1.expr);
    } 
    ( 
        COLON e2=expression COLON e3=expression
        {
            ((FVPExprDelayMinTypMax)$mintypmax).setTyp($e2.expr);
            ((FVPExprDelayMinTypMax)$mintypmax).setMax($e3.expr);
        } 
    )?
    ;

// term
exp11 returns [Integer line, FVPExprBase expr]
    :
    STRING 
    {
        $line = $STRING.line;
        $expr = new FVPExprString( $line, $STRING.text );
    }
    |
    NUMBER 
    {
        $line = $NUMBER.line;
        if ( -1 < $NUMBER.text.indexOf("'") ) // check for bitstring e.g. 2'b10 
        {
            $expr = new FVPExprString($line, $NUMBER.text );
        }
        else if ( -1 < $NUMBER.text.indexOf(".") ) // check for bitstring e.g. 2'b10 
        {
            $expr = new FVPExprReal($line, $NUMBER.text );
        }
        else
        {
            Integer intval = 0;
            try
            {
                intval = Integer.parseInt($NUMBER.text);
                $expr = new FVPExprInteger( $line, intval );
            }
            catch(NumberFormatException e)
            {
                LogMessages.instance().print(1880,
                                    LogMessages.MSG_SEVERITY.ERROR, 
                                    "Unsupported expression " +
                                    $NUMBER.text + " in file " + 
                                    FVPDescription.instance().getFile() + 
                                    ":" + $line ); 
            }              
        }
    }
    |
    (function_call) => fc=function_call
    {
        $line = $function_call.line;
        $expr = $fc.expr;
    } 
    |
    lvalue 
    {
        $line = $lvalue.line;
        if ( null != $lvalue.lval )
        {
            $expr = new FVPExprLvalue( $line, $lvalue.lval );
        }
        else
        {
            LogMessages.instance().print(1881,
                   LogMessages.MSG_SEVERITY.ERROR,
                   "Programming error: Unsupported Lvalue found in " +
                   $line );
        }
    }
    ;


exp10 returns [Integer line, FVPExprBase expr]
    :
    exp11
    {
        $line = $exp11.line;
        $expr = $exp11.expr;
    } 
    | 
    LPAREN expression RPAREN
    {
        $line = $expression.line;
        $expr = $expression.expr;
    }
    ;

// Unary
exp9 returns [ Integer line, FVPExprBase expr ]
    :
    exp10 
    {
        $line = $exp10.line;
        $expr = $exp10.expr;
    }
    |
    unary_operator e9=exp9
    {
        $line = $e9.line;
        FVPExprBase e = new FVPExprUnary( $line, 
                                          $e9.expr,
                                          $unary_operator.optype
                                          ); 
        $expr = e;
    }
    ;

expr_binary_power  returns [Integer line, FVPExprBase expr ] 
    :
    e91=exp9
    {
        $line = $e91.line;
        $expr = $e91.expr;
    }
    (
        binary_operator_power 
        e92=exp9 
        {
          $expr = new FVPExprBinary( $e92.line, 
                                       $expr,$e92.expr,
                                       $binary_operator_power.optype );
        } 
    )*
    ;

expr_binary_star_div_mod  returns [Integer line, FVPExprBase expr ] 
    :
    e91=expr_binary_power
    {
        $line = $e91.line;
        $expr = $e91.expr;
    }
    (
       binary_operator_star_div_mod 
        e92=expr_binary_power 
        {
          $expr = new FVPExprBinary( $e92.line, 
                                       $expr,$e92.expr,
                                       $binary_operator_star_div_mod.optype  );
        } 
    )*
    ;


expr_binary_plus_minus  returns [Integer line, FVPExprBase expr ] 
    :
    e91=expr_binary_star_div_mod
    {
        $line = $e91.line;
        $expr = $e91.expr;
    }
    (
       binary_operator_plus_minus 
        e92=expr_binary_star_div_mod 
        {
          $expr = new FVPExprBinary( $e92.line, 
                                       $expr,$e92.expr,
                                       $binary_operator_plus_minus.optype  );
        } 
    )*
    ;

expr_binary_shift  returns [Integer line, FVPExprBase expr ] 
    :
    e91=expr_binary_plus_minus
    {
        $line = $e91.line;
        $expr = $e91.expr;
    }
    (
       binary_operator_shift 
        e92=expr_binary_plus_minus 
        {
          $expr = new FVPExprBinary( $e92.line, 
                                       $expr,$e92.expr,
                                       $binary_operator_shift.optype  );
        } 
    )*
    ;

expr_binary_relational  returns [Integer line, FVPExprBase expr ] 
    :
    e91=expr_binary_shift
    {
        $line = $e91.line;
        $expr = $e91.expr;
    }
    (
       binary_operator_relational 
        e92=expr_binary_shift 
        {
          $expr = new FVPExprBinary( $e92.line, 
                                       $expr,$e92.expr,
                                       $binary_operator_relational.optype  );
        } 
    )*
    ;

expr_binary_equality  returns [Integer line, FVPExprBase expr ] 
    :
    e91=expr_binary_relational
    {
        $line = $e91.line;
        $expr = $e91.expr;
    }
    (
       binary_operator_equality 
        e92=expr_binary_relational 
        {
          $expr = new FVPExprBinary( $e92.line, 
                                       $expr,$e92.expr,
                                       $binary_operator_equality.optype  );
        } 
    )*
    ;


expr_binary_band  returns [Integer line, FVPExprBase expr ] 
    :
    e91=expr_binary_equality
    {
        $line = $e91.line;
        $expr = $e91.expr;
    }
    (
       binary_operator_band 
        e92=expr_binary_equality 
        {
          $expr = new FVPExprBinary( $e92.line, 
                                       $expr,$e92.expr,
                                       $binary_operator_band.optype  );
        } 
    )*
    ;


expr_binary_bxor  returns [Integer line, FVPExprBase expr ] 
    :
    e91=expr_binary_band
    {
        $line = $e91.line;
        $expr = $e91.expr;
    }
    (
       binary_operator_bxor 
        e92=expr_binary_band 
        {
          $expr = new FVPExprBinary( $e92.line, 
                                       $expr,$e92.expr,
                                       $binary_operator_bxor.optype  );
        } 
    )*
    ;


expr_binary_bor  returns [Integer line, FVPExprBase expr ] 
    :
    e91=expr_binary_bxor
    {
        $line = $e91.line;
        $expr = $e91.expr;
    }
    (
       binary_operator_bor 
        e92=expr_binary_bxor 
        {
          $expr = new FVPExprBinary( $e92.line, 
                                       $expr,$e92.expr,
                                       $binary_operator_bor.optype  );
        } 
    )*
    ;


expr_binary_land  returns [Integer line, FVPExprBase expr ] 
    :
    e91=expr_binary_bor
    {
        $line = $e91.line;
        $expr = $e91.expr;
    }
    (
       binary_operator_land 
        e92=expr_binary_bor 
        {
          $expr = new FVPExprBinary( $e92.line, 
                                       $expr,$e92.expr,
                                       $binary_operator_land.optype  );
        } 
    )*
    ;


expr_binary_lor_lnand_lnor  returns [Integer line, FVPExprBase expr ] 
    :
    e91=expr_binary_land
    {
        $line = $e91.line;
        $expr = $e91.expr;
    }
    (
        (
           binary_operator_lor 
            e92=expr_binary_land 
            {
              $expr = new FVPExprBinary( $e92.line, 
                                           $expr,$e92.expr,
                                           $binary_operator_lor.optype  );
            } 
        )
        |
        (
         binary_operator_rnand 
          e92=expr_binary_land 
          {
            $expr = new FVPExprBinary( $e92.line, 
                                         $expr,$e92.expr,
                                         $binary_operator_rnand.optype  );
          } 
      )
      |
      (
         binary_operator_rnor 
          e2=expr_binary_land 
          {
            $expr = new FVPExprBinary( $e2.line, 
                                         $expr,$e2.expr,
                                         $binary_operator_rnor.optype  );
          } 
      )
    )*
    ;


exp8 returns [Integer line, FVPExprBase expr ] 
    :
    e=expr_binary_lor_lnand_lnor
    {
        $line = $e.line;
        $expr = $e.expr;
    }
    ;


exp7 returns [ Integer line, FVPExprBase expr ]
    @init   
    {
        FVPExprBase cond = null; 
        FVPExprBase op1 = null; 
        FVPExprBase op2 = null;
    }
    :
    exp8 
    {
        $line = $exp8.line;
        $expr = $exp8.expr;
        cond =  $exp8.expr;
    }
    ( 
        QUESTION 
        e71=exp7
        {
            op1 =  $e71.expr;
        } 
        COLON 
        e72=exp7 
        {
            op2 =  $e72.expr;
        }
        
    )?
    {
        if ( null != cond && null != op1 && null != op2 )
        {
            $expr = new FVPExprTernary($exp8.line, cond,  op1,  op2);
        }
    }
    ;

exp0 returns [Integer line, FVPExprBase expr]
    :
    exp7
    {
        $line = $exp7.line;
        $expr = $exp7.expr;
    }
    ;

expression returns [Integer line, FVPExprBase expr]
    :
    exp0
    {
        $expr = $exp0.expr;
        $line = $exp0.line;
    }
    ;

function_call returns [Integer line, FVPExprBase expr]
    @init
    {
        LinkedList<FVPExprBase> args = null;
    }
    :
    name_of_function
    {
        $line = $name_of_function.line;
    } 
    ( LPAREN (exprs=expression_list { args = $exprs.exprs; } )? RPAREN )
    {
        String funcname = $name_of_function.id;
        FVPDescription desc = FVPDescription.instance();
        FVPBase sym = (FVPBase)desc.getSymbol( funcname );
        if ( null == sym )
        {
            sym = new FVPTaskOrFunctionDecl(-1, funcname );
            ((FVPTaskOrFunctionDecl)sym).setDummy();
            desc.insertSymbol( funcname, sym );
        }
        $expr = new FVPExprFuncionCall($line, (FVPTaskOrFunctionDecl)sym, args);
    }
    |
    ( SYSTEM_TASK_NAME LPAREN exprs=expression_list { args = exprs.exprs; } RPAREN )
    {
        $line = $SYSTEM_TASK_NAME.getLine();
        $expr = new FVPExprSystemFuncionCall($SYSTEM_TASK_NAME.getLine(), $SYSTEM_TASK_NAME.getText(), null );
    }
    |
    SYSTEM_TASK_NAME // Only $time and $realtime are of type expression
    {
        $line = $SYSTEM_TASK_NAME.getLine();
        $expr = new FVPExprSystemFuncionCall($SYSTEM_TASK_NAME.getLine(), $SYSTEM_TASK_NAME.getText(), null );
    }
    ;

expression_list returns [Integer line, LinkedList<FVPExprBase> exprs]
    :
    e1=expression
    {
        $line = $e1.line;
        $exprs = new LinkedList<FVPExprBase>();
        $exprs.add($e1.expr);
    } 
    ( 
        COMMA e2=expression
        {
            $exprs.add($e2.expr);
        }
    )*
    ;

unary_operator returns [Integer line, FVPExprUnary.OpType optype] 
    :
    PLUS
    {
        $line = $PLUS.line; $optype = FVPExprUnary.OpType.PLUS;
    }
    |
    MINUS
    {
        $line = $MINUS.line;$optype = FVPExprUnary.OpType.MINUS;
    }  
    |
    LNOT
    {
        $line = $LNOT.line;$optype = FVPExprUnary.OpType.LNOT;
    }
    |
    BNOT
    { 
        $line = $BNOT.line;$optype = FVPExprUnary.OpType.BNOT; 
    }
    |
    BAND 
    {
        $line = $BAND.line;$optype = FVPExprUnary.OpType.BAND; 
    }
    |
    BOR 
    { 
        $line = $BOR.line;$optype = FVPExprUnary.OpType.BOR; 
    }
    |
    RNAND 
    { 
        $line = $RNAND.line;$optype = FVPExprUnary.OpType.RNAND; 
    }
    |
    RNOR 
    { 
        $line = $RNOR.line;$optype = FVPExprUnary.OpType.RNOR; 
    }
    |
    BXOR 
    { 
        $line = $BXOR.line;$optype = FVPExprUnary.OpType.BXOR; 
    }
    |
    BXNOR 
    { 
        $line = $BXNOR.line;$optype = FVPExprUnary.OpType.BXNOR; 
    }
    ;

binary_operator_power returns [Integer line, FVPExprBinary.OpType optype ]
    :
    POWER 
    { 
        $line = $POWER.line; $optype = FVPExprBinary.OpType.POWER;  
    }       
    ;


binary_operator_star_div_mod returns [Integer line, FVPExprBinary.OpType optype ]
    :
    STAR 
    { 
        $line = $STAR.line; $optype = FVPExprBinary.OpType.STAR;  
    }       
    |
    DIV 
    { 
        $line = $DIV.line; $optype = FVPExprBinary.OpType.DIV; 
    }
    |
    MOD 
    { 
        $line = $MOD.line; $optype = FVPExprBinary.OpType.MOD; 
    }
    ;


binary_operator_plus_minus returns [Integer line, FVPExprBinary.OpType optype ]
    :
    PLUS 
    { 
        $line = $PLUS.line; $optype = FVPExprBinary.OpType.PLUS; 
    }        
    |
    MINUS 
    { 
        $line = $MINUS.line; $optype = FVPExprBinary.OpType.MINUS;  
    }       
    ;

binary_operator_shift returns [Integer line, FVPExprBinary.OpType optype ]
    :
    op1='>>' 
    { 
        $line = $op1.line; $optype = FVPExprBinary.OpType.SR; 
    }
    |
    op2='>>>' 
    { 
        $line = $op2.line; $optype = FVPExprBinary.OpType.SSR; 
    }
    |
    op3='<<' 
    { 
        $line = $op3.line; $optype = FVPExprBinary.OpType.SL; 
    }
    |
    op4='<<<' 
    { 
        $line = $op4.line; $optype = FVPExprBinary.OpType.SSL; 
    }
    ;

   
binary_operator_relational returns [Integer line, FVPExprBinary.OpType optype ]
    :
    op5='<' 
    { 
        $line = $op5.line; $optype = FVPExprBinary.OpType.LT; 
    }
    |
    op6='<='
    { 
        $line = $op6.line; $optype = FVPExprBinary.OpType.LE; 
    }
    |
    op7='>' 
    { 
        $line = $op7.line; $optype = FVPExprBinary.OpType.GT; 
    }
    |
    op8='>='
    { 
        $line = $op8.line; $optype = FVPExprBinary.OpType.GE; 
    }
    ;
   
binary_operator_equality returns [Integer line, FVPExprBinary.OpType optype ]
    :
    EQUAL 
    { 
        $line = $EQUAL.line; $optype = FVPExprBinary.OpType.EQUAL; 
    }
    |
    NOT_EQ 
    { 
        $line = $NOT_EQ.line; $optype = FVPExprBinary.OpType.NOT_EQ; 
    }
    |
    EQ_CASE 
    { 
        $line = $EQ_CASE.line; $optype = FVPExprBinary.OpType.EQ_CASE; 
    }
    |
    NOT_EQ_CASE 
    { 
        $line = $NOT_EQ_CASE.line; $optype = FVPExprBinary.OpType.NOT_EQ_CASE; 
    } 
    ;

  
      
binary_operator_band returns [Integer line, FVPExprBinary.OpType optype ]
    :
    BAND 
    { 
        $line = $BAND.line; $optype = FVPExprBinary.OpType.BAND; 
    }
    ;
    
     
binary_operator_bxor returns [Integer line, FVPExprBinary.OpType optype ]
    :
    BXOR 
    { 
        $line = $BXOR.line; $optype = FVPExprBinary.OpType.BXOR; 
    }
    |
    BXNOR 
    { 
        $line = $BXNOR.line; $optype = FVPExprBinary.OpType.BXNOR; 
    }
    ;
    
    
binary_operator_bor returns [Integer line, FVPExprBinary.OpType optype ]
    :
    BOR 
    { 
        $line = $BOR.line; $optype = FVPExprBinary.OpType.BOR; 
    }
    ;

    
binary_operator_land returns [Integer line, FVPExprBinary.OpType optype ]
    :
    LAND 
    { 
        $line = $LAND.line; $optype = FVPExprBinary.OpType.LAND; 
    }
    ;

    
binary_operator_lor returns [Integer line, FVPExprBinary.OpType optype ]
    :
    LOR 
    { 
        $line = $LOR.line; $optype = FVPExprBinary.OpType.LOR; 
    }
    ;            
  
binary_operator_rnand returns [Integer line, FVPExprBinary.OpType optype ]
    :
    RNAND 
    { 
        $line = $RNAND.line; $optype = FVPExprBinary.OpType.RNAND; 
    }
    ;            

binary_operator_rnor returns [Integer line, FVPExprBinary.OpType optype ]
    :
    RNOR 
    { 
        $line = $RNOR.line; $optype = FVPExprBinary.OpType.RNOR; 
    }
    ;            


KW_MODULE
    :
    mod='module'
    {
        if ( true == FVPDescription.instance().isStoreComment() )
        {
            FVPComment comment = FVPDescription.instance().getComment();
//          String tok_id = getText() + "___" + getLine();
//          FVPDescription.instance().setComment(tok_id, comment);
//          FVPDescription.instance().clearComment();
            FVPDescription.instance().setComment(getLine(), comment);
            FVPDescription.instance().clearComment();
        }
    }
    ;
    
KW_ENDMODULE:
    'endmodule'
    {
        if ( true == FVPDescription.instance().isStoreComment() )
        {
            FVPDescription.instance().clearComment();
            FVPDescription.instance().initComment(-1);
        }
    }
    ;

KW_PARAMETER:
    'parameter'
    {
        if ( true == FVPDescription.instance().isStoreComment() )
        {
            FVPComment comment = new FVPComment("");
            comment.setLine(getLine());
            FVPDescription.instance().setCurrentComment( comment );
            FVPDescription.instance().setComment(getLine(), comment);
            
            comment = new FVPComment("");
            comment.setLine( getLine() + 1 );
            FVPDescription.instance().setNonObjectComment( comment );
        }
    }
    ;

KW_LOCALPARAM:
    'localparam'
    {
        if ( true == FVPDescription.instance().isStoreComment() )
        {
            FVPComment comment = new FVPComment("");
            comment.setLine(getLine());
            FVPDescription.instance().setCurrentComment( comment );
            FVPDescription.instance().setComment(getLine(), comment);
            
            comment = new FVPComment("");
            comment.setLine( getLine() + 1 );
            FVPDescription.instance().setNonObjectComment( comment );            
        }
    }
    ;

KW_ALWAYS:
    'always'
    {
        if ( true == FVPDescription.instance().isStoreComment() )
        {
            FVPComment comment = FVPDescription.instance().getComment( getLine() );
            if ( null == comment )
            {
                comment = new FVPComment("");
            }
            comment.setLine(getLine());
            FVPDescription.instance().setCurrentComment( comment );
            FVPDescription.instance().setComment(getLine(), comment);
        }
    }
    ;


KW_INITIAL:
    'initial'
    {
        if ( true == FVPDescription.instance().isStoreComment() )
        {
            FVPComment comment = FVPDescription.instance().getNonObjectComment( );
            if ( null == comment )
            {
                comment = new FVPComment("");
            }
            comment.setLine(getLine());
            FVPDescription.instance().setCurrentComment( comment );
            FVPDescription.instance().setComment(getLine(), comment);
        }
    }
    ;
    
KW_REG:
    'reg'
    {
        if ( true == FVPDescription.instance().isStoreComment() )
        {
            FVPComment comment = new FVPComment("");
            comment.setLine(getLine());
            FVPDescription.instance().setCurrentComment( comment );
            FVPDescription.instance().setComment(getLine(), comment);
                        
            comment = new FVPComment("");
            comment.setLine( getLine() + 1 );
            FVPDescription.instance().setNonObjectComment( comment );
        }
    }
    ;

    
KW_INTEGER:
    'integer'
    {
        if ( true == FVPDescription.instance().isStoreComment() )
        {
            FVPComment comment = new FVPComment("");
            comment.setLine(getLine());
            FVPDescription.instance().setCurrentComment( comment );
            FVPDescription.instance().setComment(getLine(), comment);
                        
            comment = new FVPComment("");
            comment.setLine( getLine() + 1 );
            FVPDescription.instance().setNonObjectComment( comment );
        }
    }
    ;

KW_REAL:
    'real'
    {
        if ( true == FVPDescription.instance().isStoreComment() )
        {
            FVPComment comment = new FVPComment("");
            comment.setLine(getLine());
            FVPDescription.instance().setCurrentComment( comment );
            FVPDescription.instance().setComment(getLine(), comment);
                        
            comment = new FVPComment("");
            comment.setLine( getLine() + 1 );
            FVPDescription.instance().setNonObjectComment( comment );
        }
    }
    ;


KW_REALTIME:
    'realtime'
    {
        if ( true == FVPDescription.instance().isStoreComment() )
        {
            FVPComment comment = new FVPComment("");
            comment.setLine(getLine());
            FVPDescription.instance().setCurrentComment( comment );
            FVPDescription.instance().setComment(getLine(), comment);
                        
            comment = new FVPComment("");
            comment.setLine( getLine() + 1 );
            FVPDescription.instance().setNonObjectComment( comment );
        }
    }
    ;
                  
KW_INPUT:
    'input'
    {
        if ( true == FVPDescription.instance().isStoreComment() )
        {
            FVPComment comment = new FVPComment("");
            comment.setLine(getLine());
            FVPDescription.instance().setCurrentComment( comment );
            FVPDescription.instance().setComment(getLine(), comment);
                        
            comment = new FVPComment("");
            comment.setLine( getLine() + 1 );
            FVPDescription.instance().setNonObjectComment( comment );
        }
    }
    ;
    
KW_OUTPUT:
    'output'
    {
        if ( true == FVPDescription.instance().isStoreComment() )
        {
            FVPComment comment = new FVPComment("");
            comment.setLine(getLine());
            FVPDescription.instance().setCurrentComment( comment );
            FVPDescription.instance().setComment(getLine(), comment);
                        
            comment = new FVPComment("");
            comment.setLine( getLine() + 1 );
            FVPDescription.instance().setNonObjectComment( comment );
        }
    }
    ;

KW_INOUT:
    'inout'
    {
        if ( true == FVPDescription.instance().isStoreComment() )
        {
            FVPComment comment = new FVPComment("");
            comment.setLine(getLine());
            FVPDescription.instance().setCurrentComment( comment );
            FVPDescription.instance().setComment(getLine(), comment);
                        
            comment = new FVPComment("");
            comment.setLine( getLine() + 1 );
            FVPDescription.instance().setNonObjectComment( comment );
        }
    }
    ;
    
KW_WIRE:
    'wire'
    {
        if ( true == FVPDescription.instance().isStoreComment() )
        {
            FVPComment comment = new FVPComment("");
            comment.setLine(getLine());
            FVPDescription.instance().setCurrentComment( comment );
            FVPDescription.instance().setComment(getLine(), comment);
                        
            comment = new FVPComment("");
            comment.setLine( getLine() + 1 );
            FVPDescription.instance().setNonObjectComment( comment );
        }
    }
    ;    
    
KW_FUNCTION:
    'function'
    {
        if ( true == FVPDescription.instance().isStoreComment() )
        {
            FVPComment comment = new FVPComment("");
            comment.setLine(getLine());
            FVPDescription.instance().setCurrentComment( comment );
            FVPDescription.instance().setComment(getLine(), comment);
                        
            comment = new FVPComment("");
            comment.setLine( getLine() + 1 );
            FVPDescription.instance().setNonObjectComment( comment );
        }
    }
    ;    

KW_TASK:
    'task'
    {
        if ( true == FVPDescription.instance().isStoreComment() )
        {
            FVPComment comment = new FVPComment("");
            comment.setLine(getLine());
            FVPDescription.instance().setCurrentComment( comment );
            FVPDescription.instance().setComment(getLine(), comment);
                        
            comment = new FVPComment("");
            comment.setLine( getLine() + 1 );
            FVPDescription.instance().setNonObjectComment( comment );
        }
    }
    ;    
    
KW_ASSIGN:
    'assign'
    {
        if ( true == FVPDescription.instance().isStoreComment() )
        {
            FVPComment comment = new FVPComment("");
            comment.setLine(getLine());
            FVPDescription.instance().setCurrentComment( comment );
            FVPDescription.instance().setComment(getLine(), comment);
                        
            comment = new FVPComment("");
            comment.setLine( getLine() + 1 );
            FVPDescription.instance().setNonObjectComment( comment );
        }
    }
    ;

KW_TRIREG:
    'trireg'
    {
        if ( true == FVPDescription.instance().isStoreComment() )
        {
            FVPComment comment = new FVPComment("");
            comment.setLine(getLine());
            FVPDescription.instance().setCurrentComment( comment );
            FVPDescription.instance().setComment(getLine(), comment);
        }
    }
    ;

//----------------------------------------------------------------------------
// Identifiers
//----------------------------------------------------------------------------

name_of_module returns [String id, Integer line] :
    local_identifier 
    {
        $id   = $local_identifier.id; 
        $line = $local_identifier.line;
    }
    ;

name_of_port returns [String id, Integer line] :
    local_identifier 
    {
        $id = $local_identifier.id; 
        $line = $local_identifier.line;
    }
    ;

name_of_variable returns [String id, Integer line] :
    local_identifier 
    {
        $id = $local_identifier.id;
        $line = $local_identifier.line;
    }
    ;

name_of_UDP returns [String id, Integer line] :
    local_identifier 
    {
        $id = $local_identifier.id;
        $line = $local_identifier.line;
    }
    ;

name_of_UDP_instance returns [String id, Integer line] :
    local_identifier 
    {
        $id = $local_identifier.id;
        $line = $local_identifier.line;
    }
    ;

name_of_event returns [String id, Integer line] :
    local_identifier 
    {
        $id = $local_identifier.id;
        $line = $local_identifier.line;
    }
    ;

name_of_task returns [String id, Integer line] :
    local_identifier 
    {
        $id = $local_identifier.id;
        $line = $local_identifier.line;
    }
    ;

real_identifier returns [String id, Integer line] :
    identifier 
    {
        $id = $identifier.id;
        $line = $identifier.line; 
    }
    ;

name_of_memory returns [String id, Integer line] :
    local_identifier 
    {
        $id = $local_identifier.id;
        $line = $local_identifier.line;
    }
    ;

net_identifier returns [String id, Integer line] :
    identifier 
    {
        $id = $identifier.id;
        $line = $identifier.line;
    }
    ;

name_of_function returns [String id, Integer line] :
    local_identifier 
    {
        $id = $local_identifier.id;
        $line = $local_identifier.line;
    }
    ;

specparam_identifier returns [String id, Integer line] :
    identifier 
    {
        $id = $identifier.id;
        $line = $identifier.line;
    }
    ;

udp_name_of_port returns [String id, Integer line] :
    identifier 
    {
        $id = $identifier.id;
        $line = $identifier.line;
    }
    ;

name_of_register returns [String id, Integer line] :
    local_identifier
    {
        $id = $local_identifier.id;
        $line = $local_identifier.line;
    }
    ;

name_of_net returns [String id, Integer line] :
    local_identifier
    {
        $id = $local_identifier.id;
        $line = $local_identifier.line;
    }
    ;

name_of_gate_instance returns [String id, Integer line] :
    local_identifier 
    {
        $id = $local_identifier.id;
        $line = $local_identifier.line;
    }
    ;

name_of_instance returns [String id, Integer line] :
    local_identifier 
    {
        $id = $local_identifier.id;
        $line = $local_identifier.line;
    }
    ;

name_of_block returns [String id, Integer line] :
    local_identifier 
    {
        $id = $local_identifier.id;
        $line = $local_identifier.line;
    }
    ;

output_terminal_name returns [String id, Integer line] :
    local_identifier 
    {
        $id = $local_identifier.id;
        $line = $local_identifier.line;
    }
    ;



//----------------------------------------------------------------------------
// General
//----------------------------------------------------------------------------

identifier returns [String id, Integer line ] :
    identifier_path 
    {
        $id = $identifier_path.text;
        $line = $identifier_path.line;
    }
    ;

identifier_path returns [String id, Integer line ]:
    id1=local_identifier 
    {
        $line = $id1.line; 
        $id = $id1.text; 
    }
    | (IDENTIFIER DOT )=> HIERIDENTIFIER
    ;

// The below HIERIDENTIFIER is added by Baya to support hier reference
HIERIDENTIFIER:
    IDENTIFIER DOT IDENTIFIER
    {
        $type = IDENTIFIER;
    }
    (
        DOT IDENTIFIER
        {
            $type = IDENTIFIER;
        }
    )*
    ;
    
local_identifier returns [String id, Integer line ]:
    IDENTIFIER 
    {
        $id = $IDENTIFIER.text;
        $line = $IDENTIFIER.line;
    }
    |
    ESCAPED_IDENTIFIER 
    {
        $id = $ESCAPED_IDENTIFIER.text;
        $line = $ESCAPED_IDENTIFIER.line;
    }
    ;

delay_control returns [FVPDelayControl delay_ctrl, Integer line]:
    POUND NUMBER
    {
        $line = $NUMBER.line;
        FVPExprBase expr = null;
        if ( -1 < $NUMBER.text.indexOf("'") ) // check for bitstring e.g. 2'b10 
        {
            expr = new FVPExprString($line, $NUMBER.text );
        }
        else if ( -1 < $NUMBER.text.indexOf(".") ) // check for bitstring e.g. 2'b10 
        {
            expr = new FVPExprReal($line, $NUMBER.text );
        }
        else
        {
            Integer intval = 0;
            try
            {
                intval = Integer.parseInt($NUMBER.text);
                expr = new FVPExprInteger( $line, intval );
            }
            catch(NumberFormatException e)
            {
                LogMessages.instance().print(1884,
                                    LogMessages.MSG_SEVERITY.ERROR, 
                                    "Unsupported expression " +
                                    $NUMBER.text + " in file " + 
                                    FVPDescription.instance().getFile() + 
                                    ":" + $line ); 
            }              
        }
        $delay_ctrl = new FVPDelayControl($line, expr);
    }  
    |
    POUND id=identifier
    {
        $line = $id.line;
        FVPBase val = (FVPBase)FVPDescription.instance().getSymbol( $id.id );
        FVPBase net = null ; // net, reg or mem
        FVPLvalue lval = null;
        if ( null == val || FVPBase.ObjType.MODULE == val.getObjType() ) //blackbox
        {
            val = new FVPIdentifier( $line, $id.id );
            val.setDummy(); // blackbox
            lval = new FVPLvalue( $line, val );
        }
        else if ( FVPBase.ObjType.LVALUE != val.getObjType() )
        {
            lval = new FVPLvalue( $line, val );
        }
        else
        {
            lval = (FVPLvalue)val;
        }
        FVPExprLvalue exprlval = new FVPExprLvalue( $line, lval );
        $delay_ctrl = new FVPDelayControl($line, exprlval);
    } 
    |
    POUND LPAREN mintypmax_expression RPAREN
    {
        $line = $mintypmax_expression.line;
        $delay_ctrl = new FVPDelayControl($line, $mintypmax_expression.mintypmax);
    }
    ;

event_control returns [ LinkedList<FVPExprBase> edges, Integer line]:
    AT identifier 
    {
        LogMessages.instance().print(1885,
                               LogMessages.MSG_SEVERITY.WARNING,
                               "Identifier as control is not supported yet found at " + 
                               FVPDescription.instance().getFile() + ":" + 
                               $AT.getLine() + " ... ");
    }
    |
    AT LPAREN e1=event_expression RPAREN
    {
        $edges = $e1.edges;
        $line = $e1.line;
    }
    |
    AT e2=event_expression
    {
        $edges = $e2.edges;
        $line = $e2.line;
    }
    ;

event_expression returns [ LinkedList<FVPExprBase> edges, Integer line]:
    e1=sub_event_expression
    {
        $edges = new LinkedList<FVPExprBase>();
        $edges.add($e1.expr);
        $line = $e1.line;
    }
    ( 
        ('or' | ',' )
        e2=sub_event_expression
        {
            $edges.add($e2.expr);
        } 
    )*
    ;

sub_event_expression returns [ FVPExprBase expr, Integer line]:
    expression
    {
        $expr = new FVPExprEdge($expression.line, FVPExprEdge.EdgeType.NOEDGE, $expression.expr);
        $line = $expression.line;
    } 
    |
    'posedge' expression 
    {
        $expr = new FVPExprEdge($expression.line, FVPExprEdge.EdgeType.POSEDGE, $expression.expr);
        $line = $expression.line;
    }
    |
    'negedge' expression
    {
        $expr = new FVPExprEdge($expression.line, FVPExprEdge.EdgeType.NEGEDGE, $expression.expr);
        $line = $expression.line;
    }
    |
    STAR
    {
      $expr = new FVPExprStar($STAR.line);
      $line = $STAR.line;
    }
    ;

//----------------------------------------------------------------------------
// Compiler directives
//----------------------------------------------------------------------------

directive returns [FVPDefine define_dir]: 
    define_directive
    |
    include_directive 
    |
    timescale_directive 
    |
    undef_directive 
    |
    resetall_directive 
    |
    celldefine_directive
  |
  endcelldefine_directive
  |
  protect_directive
  |
  endprotect_directive
    |
    conditional_compilation_directive
    |
    line_compiler_directive
    ;



undef_directive
    : 
    '`undef' id1=IDENTIFIER  
    {
      FVPDescription.instance().getRoot().removeDefine($id1.text); 
    }
    ;
  
define_directive returns [Integer line, String id, String value]
    : 
    '`define' id1=IDENTIFIER  
    {
      $id = $id1.text;
    }
    ( 
      id2=IDENTIFIER
      {
        $value =  FVPDescription.instance().getRoot().getMacrolValue($id2.text);
      }
    )*
    ;
    

include_directive returns [Integer line, FVPIncludeDirective incdir]
    :
    INCLUDE f=STRING
    {
        $line = $f.line;
        CharStream istream = null;
        String tmpStr =  new String($f.text);
        tmpStr = tmpStr.substring(1,tmpStr.length()-1);
        String preprocessedFile = FVPDescription.instance().getRoot().getPreprocessedIncludeFile( tmpStr );
        
        String fName = null;
        if ( null != preprocessedFile ) 
        {
         fName = preprocessedFile;
        }
        else
        {
          fName = tmpStr;
        }
        String workingdir = System.getProperty("user.dir");
        FVPDescription desc = FVPDescription.instance();
        // First check if this file exists in the run directory
        String fullPathFile = workingdir + java.io.File.separatorChar + fName;
        // Found inconsistancies with the File.exists() and othe APIs.
        // So trying workarounds
        //File incfile = new File(fullPathFile);
        try
        {
          istream = new ANTLRFileStream(fName);
        }
        catch (IOException e) 
        {
          ; // No action ...
        } 
        if ( null != incpaths )
        {
            Iterator<String> itr = incpaths.iterator();
            while( null == istream && itr.hasNext() )
            {
                String path = (String)itr.next();
                if ( null == path || path.isEmpty() || path.length() < 1)
                {
                    continue;
                }
                if ( path.charAt(0)  !=  java.io.File.separatorChar  )
                {
                    path = workingdir + java.io.File.separatorChar + path;
                }            
                fullPathFile = path + java.io.File.separatorChar + fName ;
                try 
                {
                    istream = new ANTLRFileStream(fullPathFile);
                }
                catch (IOException e) 
                {
                    ; // No action ...
                }        
            }
        }
        else // This file is expected in the run directory, +indir+ is not given
        {
            try 
            {
                istream = new ANTLRFileStream(fName);
            } 
            catch (IOException e) 
            {
                ; // No action ...
            }              
        }      
        if ( null == istream )
        {
            LogMessages.instance().print(1886,
                                LogMessages.MSG_SEVERITY.ERROR,
                                "Problem to open include file " + 
                                fName + " in read mode ... ");
            return null;
        }                 
        String prevFile = FVPDescription.instance().getFile();
        
        FVPDescription.instance().setFile(tmpStr);
        VerilogLexer lexer = new VerilogLexer(istream);
        TokenStream tokenStream = new CommonTokenStream(lexer);
        VerilogParser parser = new VerilogParser(tokenStream );
        FVPBase.ObjType type = FVPDescription.instance().getScope().getObjType();
        
        LogMessages.instance().print(1887,
                                     LogMessages.MSG_SEVERITY.NOTE, 
                                     "Parsing included file '" +
                                     FVPDescription.instance().getFile() + "' from file " + prevFile + " ...");
        switch (type) {
        case MODULE:
            parser.module_items();
            break;
        case TASK_OR_FUNCTION_DECL:
            parser.all_statement_or_nulls();
            break;
        case SEQ_STATEMENT:
        {    
            FVPSeqStmtBase scope = (FVPSeqStmtBase)FVPDescription.instance().getScope();         
            switch ( scope.getStatementType() )
            {
            case SEQ_IF_STMT:            
                parser.conditional_statement();
                break;        
            case SEQ_CASE_ITEM:
                parser.case_statement();
                break;
            case SEQ_LOOP_STMT:
                parser.loop_statement();
                break;
            case ALWAYS_BLOCK:
                parser.always_statement();
                break;
            case INITIAL_BLOCK:
                parser.initial_statement();
                break;
            default:
            }
            break;
        }    
        
        case ROOT:
            parser.parseRTL(incpaths);
            break;
        case UNKNOWN:
            LogMessages.instance().print(1888,
                                         LogMessages.MSG_SEVERITY.PROGRAMMING_ERROR,
                                         "Trying to read include file from wrog scope ...");
            break;
        }
        LogMessages.instance().print(1889,
                                     LogMessages.MSG_SEVERITY.NOTE, 
                                     "Finished parsing included file '" +
                                     FVPDescription.instance().getFile() + "' ...");
        //FVPRoot root = parser.parseRTL(incpaths);
        // Reset the file
        FVPDescription.instance().setFile(prevFile);
        fName = null;
        istream = null;
        lexer = null;
        tokenStream = null;
        parser = null;
    }
    ;
    
timescale_directive returns[Integer line, FVPTimeScale ts]
  @init
  {
      FVPTimeScale.TimeUnit numUnit = FVPTimeScale.TimeUnit.UNKNOWN;
        FVPTimeScale.TimeUnit denoUnit = FVPTimeScale.TimeUnit.UNKNOWN;
  }
    :
    TIMESCALE  num1=NUMBER id1=IDENTIFIER DIV  num2=NUMBER id2=IDENTIFIER 
    {
        if ( null != $id1.text )
        {
	        if ( $id1.text.equals( "ps" ) )
	        {
	           numUnit =FVPTimeScale.TimeUnit.PS;
            }
            else if ( $id1.text.equals( "ns" ) )
            {
               numUnit =FVPTimeScale.TimeUnit.NS;
            }
            else if ( $id1.text.equals( "fs" ) )
            {
               numUnit =FVPTimeScale.TimeUnit.FS;
            }
        }
        
        if ( null != $id2.text )
        {
            if ( $id2.text.equals( "ps" ) )
            {
               denoUnit =FVPTimeScale.TimeUnit.PS;
            }
            else if ( $id2.text.equals( "ns" ) )
            {
               denoUnit =FVPTimeScale.TimeUnit.NS;
            }
            else if ( $id2.text.equals( "fs" ) )
            {
               denoUnit  =FVPTimeScale.TimeUnit.FS;
            }
        }
        $line = $num1.line;
        FVPExprBase numerator = null;
        FVPExprBase denominator = null;
        if ( -1 < $num1.text.indexOf("'") ) // check for bitstring e.g. 2'b10 
        {
            numerator = new FVPExprString($line, $num1.text );
        }
        else if ( -1 < $num1.text.indexOf(".") ) // check for bitstring e.g. 2'b10 
        {
            numerator = new FVPExprReal($line, $num1.text );
        }
        else
        {
            Integer intval = 0;
            try
            {
                intval = Integer.parseInt($num1.text);
                numerator = new FVPExprInteger( $line, intval );
            }
            catch(NumberFormatException e)
            {
                LogMessages.instance().print(1890,
                                    LogMessages.MSG_SEVERITY.ERROR,
                                    "Unsupported expression " +
                                    $num1.text + " in file " + 
                                    FVPDescription.instance().getFile() + 
                                    ":" + $line ); 
            }              
        }
        
        if ( -1 < $num2.text.indexOf("'") ) // check for bitstring e.g. 2'b10 
        {
            denominator = new FVPExprString($line, $num2.text );
        }
        else if ( -1 < $num2.text.indexOf(".") ) // check for bitstring e.g. 2'b10 
        {
            denominator = new FVPExprReal($line, $num2.text );
        }
        else
        {
            Integer intval = 0;
            try
            {
                intval = Integer.parseInt($num2.text);
                denominator = new FVPExprInteger( $line, intval );
            }
            catch(NumberFormatException e)
            {
                LogMessages.instance().print(1891,
                                    LogMessages.MSG_SEVERITY.ERROR,
                                    "Unsupported expression " +
                                    $num2.text + " in file " + 
                                    FVPDescription.instance().getFile() + 
                                    ":" + $line ); 
            }              
        }
        $ts = new FVPTimeScale( $line, numerator, numUnit, denominator, denoUnit );
        FVPDescription.instance().addStatement( $ts );
    }
    ;
        

resetall_directive
  :
    '`resetall'
    {
        LogMessages.instance().print(1892,
                               LogMessages.MSG_SEVERITY.WARNING,
                               "Ignoring `resetall directive found in " + 
                               FVPDescription.instance().getFile() + ":" + " ... ");
    }
    ;
        
celldefine_directive
  : 
  '`celldefine'
    {
        LogMessages.instance().print(1893,
                               LogMessages.MSG_SEVERITY.WARNING, 
                               "Ignoring `celldefine directive found in " + 
                               FVPDescription.instance().getFile() + ":" + " ... ");
    } 
  ;

endcelldefine_directive
  : 
  '`endcelldefine'
    {
        LogMessages.instance().print(1894,
                               LogMessages.MSG_SEVERITY.WARNING,
                               "Ignoring `endcelldefine directive found in " + 
                               FVPDescription.instance().getFile() + ":" + " ... ");
    } 
  ;

protect_directive
  : 
  '`protect'
    {
        LogMessages.instance().print(1895,
                               LogMessages.MSG_SEVERITY.WARNING,
                               "Ignoring `protect directive found in " + 
                               FVPDescription.instance().getFile() + ":" + " ... ");
    } 
  ;

endprotect_directive
  : 
  '`endprotect'
    {
        LogMessages.instance().print(1896,
                               LogMessages.MSG_SEVERITY.WARNING, 
                               "Ignoring `endprotect directive found in " + 
                               FVPDescription.instance().getFile() + ":" + " ... ");
    } 
  ;
  
conditional_compilation_directive 
    : 
    '`ifdef'  id1=IDENTIFIER 
    {
        FVPCompilerDirectiveCond dirnode = new FVPCompilerDirectiveCond($id1.line, $id1.text, Directive.IFDEF );
        dirnode.setIdentifier( $id1.text );
        if ( FVPDescription.instance().getRoot().isDefined( $id1.text ) )
        {
            dirnode.setCondTrue(true);
            dirnode.setOneblockalreadyvisited(true);
        }
        FVPDescription.instance().pushCompilerDirective( dirnode );
    }
    |
    '`ifndef' id2=IDENTIFIER 
    {
        FVPCompilerDirectiveCond dirnode = new FVPCompilerDirectiveCond($id2.line, $id2.text, Directive.IFNDEF );
        dirnode.setIdentifier( $id2.text );
        if ( false == FVPDescription.instance().getRoot().isDefined( $id2.text ) )
        {
            dirnode.setCondTrue(true);
            dirnode.setOneblockalreadyvisited(true);
        }
        FVPDescription.instance().pushCompilerDirective( dirnode );
    } 
    |
    {
        false == FVPDescription.instance().getTheLastVisitedDirective().isCondTrue()
    }? =>
    '`elsif'  id3=IDENTIFIER
    {
        FVPCompilerDirectiveCond dirnode = new FVPCompilerDirectiveCond($id3.line, $id3.text, Directive.ELSIF );
        dirnode.setIdentifier( $id3.text );
        if ( FVPDescription.instance().getRoot().isDefined( $id3.text )  )
        {
            dirnode.setCondTrue(true);
            dirnode.setOneblockalreadyvisited(true);
        }
        FVPDescription.instance().pushCompilerDirective( dirnode );  
    }  
    |
    {
       false == FVPDescription.instance().getTheLastVisitedDirective().isCondTrue()
    }? =>
    '`else'
    {
        FVPCompilerDirectiveCond dirnode = new FVPCompilerDirectiveCond($id1.line, $id1.text, Directive.ELSE );
        if ( false == FVPDescription.instance().isOneblockalreadyvisited() )
        {
            dirnode.setCondTrue(true);            
        }
        FVPDescription.instance().pushCompilerDirective( dirnode );
    } 
    |    
    '`endif' 
    {
        FVPDescription.instance().processEndifCompilerDirective();
    }
    |
    '`undef' id4=identifier
    {
        FVPDescription.instance().getRoot().removeDefine( $id4.text );
    }
    | 
    {
        FVPDescription.instance().getTheLastVisitedDirective().isCondTrue()
    }? =>  '`elsif'  id3=IDENTIFIER {FVPDescription.instance().setBlockReadingCompleted();}
    | 
    { 
        FVPDescription.instance().getTheLastVisitedDirective().isCondTrue() 
    }? =>  '`else' {FVPDescription.instance().setBlockReadingCompleted();}
    ;
    
            
line_compiler_directive:
    LINE NUMBER STRING
    {
        LogMessages.instance().print(1897,
                               LogMessages.MSG_SEVERITY.WARNING,
                               "Ignoring line_compiler_directive found in " + 
                               FVPDescription.instance().getFile() + ":" + 
                               $LINE.getLine() + " ... ");
    }
    ;    

udp_body :
    (combinational_body)=> combinational_body 
    {
        LogMessages.instance().print(1898,
                               LogMessages.MSG_SEVERITY.WARNING, 
                               "Ignoring combinational body found in " + 
                               FVPDescription.instance().getFile() );
    }
    |
    sequential_body
    {
        LogMessages.instance().print(1899,
                               LogMessages.MSG_SEVERITY.WARNING, 
                               "Ignoring sequential body found in " + 
                               FVPDescription.instance().getFile() );
    }
    ;        

combinational_body :
    'table' combinational_entry ( combinational_entry )* 'endtable'
    ;
        
combinational_entry :
    level_input_list COLON output_symbol SEMI
    ;

sequential_entry :
    seq_input_list COLON state COLON next_state SEMI
    ;
    
sequential_body :
    ( udp_initial_statement )? 'table'
    sequential_entry ( sequential_entry )* 
    'endtable'
    ;    
        
//udp_initial_statement :
//        'initial' output_terminal_name ASSIGN init_val SEMI
//        ;
        
seq_input_list :
    (level_input_list) => level_input_list 
    |
    edge_input_list
    ;

level_input_list :
    (level_symbol)+
    ;

edge_input_list :
    (options{greedy = true;}:(level_symbol))* 
    edge_indicator (options{greedy = true;}:(level_symbol))*
    ;

edge_indicator :
    LPAREN level_symbol level_symbol RPAREN
    |
    edge_symbol
    ;

state :
    level_symbol
    ;

next_state :
    output_symbol 
    |
    MINUS
    ;

   // Next 3 rules use semantic predicates to determine whether a matched
   // NUMBER or IDENTIFIER is a valid special value in the given context.
   // This kludge avoids having the special values in the Literals table,
   // thus avoiding a lexical conflict.

output_symbol:
    n=NUMBER
    { $n.text.equals("0") || $n.text.equals("1")}?
    |
    i=IDENTIFIER
    { 
        $i.text.equals("x") || $i.text.equals("X")
    }?
    ;

level_symbol:
    QUESTION
    |
    n=NUMBER
    { 
        $n.text.equals("0") || $n.text.equals("1")
    }?
    |
    i=IDENTIFIER
    { 
        $i.text.equals("x") || $i.text.equals("X") ||
        $i.text.equals("b") || $i.text.equals("B") 
    }?
    ;

edge_symbol
    :
    STAR  
    | 
    i=IDENTIFIER
    {
        $i.text.equals("r") || $i.text.equals("R") ||
        $i.text.equals("f") || $i.text.equals("F") ||
        $i.text.equals("p") || $i.text.equals("P") ||
        $i.text.equals("n") || $i.text.equals("N") 
    }?
    ;

udp_instantiation returns [FVPUDPInstance inst]
    @init
    {
        String udpname = null;
        Integer line = 0;
        FVPUDPInstance instance = null;
        FVPDescription desc = FVPDescription.instance();
        FVPBase master = null;
    }
    
    :
    name_of_UDP
    {
        udpname = $name_of_UDP.id;
        line = $name_of_UDP.line;
        master =  desc.getSymbol( udpname );
        if ( null == master )
        {
            master = desc.getRoot().findModuleOrUDP( udpname );
        }
        if ( null == master )
        {
            LogMessages.instance().print(1900,
                                          LogMessages.MSG_SEVERITY.ERROR,
                                          "Missing function or UDP definition for '" + udpname + "' referred at " +
                                          FVPDescription.instance().getFile() + ":" + line + " ...");
        }
    } 
    (drive_strength)? (delay)?  
    inst1=udp_instance
    {
        FVPBase instortfcall = null;
        String instname = null;
        if ( null != master )
        {
            switch( master.getObjType() )
            {
                case MODULE:
                    if ( null != $inst1.id )
                    {
                        instname = $inst1.id.getName();
                    }
                    instortfcall = new FVPModuleInstance( line, instname );
                    ((FVPModuleInstance)instortfcall).setRange( $inst1.range );
                    ((FVPModuleInstance)instortfcall).setMaster((FVPModule)master);
                    FVPDescription.instance().addStatement(    instortfcall );
                    break;
                case UDP:
                    if ( null != $inst1.id )
                    {
                        instname = $inst1.id.getName();
                    }
                    instortfcall = new FVPUDPInstance( line, instname );
                    ((FVPUDPInstance)instortfcall).setRange( $inst1.range );
                    FVPDescription.instance().addStatement ( instortfcall );
                    ((FVPUDPInstance)instortfcall).setMaster((FVPUDP)master);
                    ((FVPUDPInstance)instortfcall).setTerminals( $inst1.terminals );
                    break;    
                case TASK_OR_FUNCTION_DECL:
                     instortfcall = new FVPSeqTaskCall(line, (FVPTaskOrFunctionDecl)master, $inst1.terminals);
                     FVPDescription.instance().addStatement ( instortfcall );
                     break;    
            }
        }
    } 
    (
        COMMA inst2=udp_instance
        {
            FVPBase instortfcall = null;
            String instname = null;
            if ( null != master )
            {            
                switch( master.getObjType() )
                {
                    case MODULE:
                        if ( null != $inst2.id )
                        {
                            instname = $inst2.id.getName();
                        }    
                        instortfcall = new FVPModuleInstance( line, instname );
                        ((FVPModuleInstance)instortfcall).setRange( $inst2.range );
                        ((FVPModuleInstance)instortfcall).setMaster((FVPModule)master);
                        FVPDescription.instance().addStatement(instortfcall);
                        break;
                    case UDP:
                        if ( null != $inst2.id )
                        {
                            instname = $inst2.id.getName();
                        }
                        instortfcall = new FVPUDPInstance( line, instname );
                        ((FVPUDPInstance)instortfcall).setRange( $inst2.range );
                        ((FVPUDPInstance)instortfcall).setMaster((FVPUDP)master);
                        ((FVPUDPInstance)instortfcall).setTerminals( $inst2.terminals );
                        FVPDescription.instance().addStatement(instortfcall);
                        break;    
                    case TASK_OR_FUNCTION_DECL:
                        instortfcall = new FVPSeqTaskCall(line, (FVPTaskOrFunctionDecl)master, $inst2.terminals);
                        FVPBase scope = desc.getCurrentscope();
                        FVPDescription.instance().addStatement(instortfcall);
                        break;        
                }
            }
        }
        
    )* SEMI
    ;

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
// The Verilog scanner
//----------------------------------------------------------------------------


// Operators
AT        
    : 
    '@'
    ;

COLON        
    : 
    ':'
    ;

COMMA        
    : 
    ','
    ;

DOT        
    : 
    '.'
    ;

ASSIGN        
    : 
    '=' 
    ;

MINUS        
    : 
    '-' 
    ;

LBRACK        
    : 
    '[' 
    ;

RBRACK        
    : 
    ']'
    ;

LCURLY        
    : 
    '{'
    ;

RCURLY        
    : 
    '}' 
    ;

LPAREN        
    : 
    '(' 
    ;

RPAREN        
    : 
    ')' 
    ;

POUND        
    : 
    '#'  
    ;

QUESTION    
    : 
    '?'   
    ;

SEMI        
    : 
    ';' 
    ;

PLUS        
    : 
    '+'   
    ;

LNOT        
    : 
    '!'
    ;

BNOT        
    : 
    '~'   
    ;

BAND        
    : 
    '&' 
    ;

RNAND       
    : 
    '~&'
    ;

BOR         
    : 
    '|'   
    ;

RNOR        
    : 
    '~|'  
    ;

BXOR        
    : 
    '^'   
    ;

BXNOR       
    : 
    '~^' 
    | '^~'  
    ;
    
POWER        
    : 
    '**'    
    ;
    
    
STAR        
    : 
    '*'    
    ;

DIV         
    : 
    '/'    
    ;

MOD         
    : 
    '%'    
    ;

EQUAL       
    : '=='   
    ;

NOT_EQ      
    : 
    '!='   
    ;

NOT_EQ_CASE 
    : 
    '!=='  
    ;

EQ_CASE     
    : 
    '==='  
    ;

LAND        
    : 
    '&&'   
    ;

LOR         
    : 
    '||'   
    ;


    // an identifier.  Note that testLiterals is set to true!  This means
    // that after we match the rule, we look in the Literals table to see
    // if it's a literal or really an identifer.

IDENTIFIER 
//      options {testLiterals=true;}
    :
    ('a'..'z'|'A'..'Z'|'_') ('a'..'z'|'A'..'Z'|'_'|'$'|'0'..'9')*
    ;

ESCAPED_IDENTIFIER :
    '\\' (~ '\u0020' )+ ('\u0020'|'\t'|'\n')   
    ;

SYSTEM_TASK_NAME :
    '$' 
    IDENTIFIER
    ;

    // string literals
STRING :
    '"' (~('"'|'\n'))* '"'  
    ;

LINE        
    : 
    '`line' 
    ;

INCLUDE     
    : '`include' 
    ;

TIMESCALE   
    : 
    '`timescale' 
    ;

    // 'compiler' define/macro.
DEFINE
      //options {testLiterals=true;}
    :
    TICK id=IDENTIFIER 
    ;
    
fragment TICK: '`' { setText(""); };

   // a dummy rule to force vocabulary to be all characters (except special
   //   ones that ANTLR uses internally (0 to 2)
fragment
VOCAB :
    '\u0003'..'\u007F'
    ;
//    
//NUMBER :
//    ( (SIZE (WS)*)? BASE (WS)* SIZED_DIGIT ) => SIZED_NUMBER
//    | UNSIZED_NUMBER
//    ;

    
NUMBER :
    ( (SIZE (WS)*)? BASE (WS)* SIZED_DIGIT ) => SIZED_NUMBER
    |
    ( (SIZE (WS)*)? BASE (WS)* (BIT)+ ) => SIZED_NUMBER
    | 
    UNSIZED_NUMBER
    |
    REAL_NUMBER
    ;

fragment
BIT
  :
  (
    'x'
    | 'z'
    | 'X'
    | 'Z'
    | '?'
    | '_'
  )
  ;


//// Baya: 1'b1 and 1'b0 does not match. this is a bug as of today     
//fragment
//SIZED_NUMBER :
//    (SIZE (WS)*)? BASE (WS)* SIZED_DIGIT 
//    (SIZED_DIGIT | '_' )*
//    ;

// Removing the WS, no space is supported as of today, otherwise #10 out1 = does not work
fragment
SIZED_NUMBER :
    (SIZE)? BASE (SIZED_DIGIT | BIT )+
    (SIZED_DIGIT | '_' )*
    ;

fragment
REAL_NUMBER :
    (DIGIT)+'.' (DIGIT)+
    ;


fragment
SIZE :
    (DIGIT)+
    ;

fragment
BASE :
    ('\''('s'|'S')?( 'd' | 'D' | 'h' | 'H' | 'o' | 'O' | 'b' | 'B') )
    ;
    
fragment
SIZED_DIGIT :
    DIGIT | HEXDIGIT
    ;

fragment
UNSIZED_NUMBER :
    DIGIT ( DIGIT | '_' )*
        ;
 
fragment
DIGIT :
        ('0'..'9')
        ;

fragment
HEXDIGIT :
        ('A'..'F'|'a'..'f')
        ;
        
fragment SPACE_OR_TAB
    :    (
            ' '
        |    '\t'
        )+
    ;

WS
    :    SPACE_OR_TAB+
        {$channel=HIDDEN;}
    ;

// The folling rule for newline has been added by Baya as it was not in the orignal grammar file
NEW_LINE
    :
    (('\r''\n') | ('\n'))+
    {$channel=HIDDEN;}
    ;    
ML_COMMENT
    :   '/*' ( options {greedy=false;} : . )* '*/' 
    {
        if ( true == FVPDescription.instance().isStoreComment() )
        { 
            String str = getText(); 
            FVPDescription.instance().addComment( str, getLine() );
        } 
        $channel=HIDDEN;
    }
    ;

SL_COMMENT
    : '//' ( options {greedy=false;} : . )* '\r'? '\n' 
    {
        if ( true == FVPDescription.instance().isStoreComment() )
        { 
            String str = getText(); 
            FVPDescription.instance().addComment( str, getLine() - 1 ); // The line number is +1 of the current comment line
        } 
        $channel=HIDDEN;
    }
    ;
